@@ -2,64 +2,15 @@
 /****** BEGIN USER CONFIGURATION SECTION *****/
 
 /*
- * IMPORTANT: UNCOMMENT THE DEFINES FOR YOUR OPERATIING SYSTEM
+ * IMPORTANT: You should no longer need to edit this file to handle
+ * your operating system. That should be handled and set correctly by
+ * configure now.
  *
  * These are slowly being phased out, but currently
  * not everyone is auto-configured.  Then decide if you
  * wish to change the features that are compiled into nmh.
  */
 
-/*
- *  Solaris 2.x
- *  Irix
- *  OSF/1
- *  HP-UX
- *  AIX
- */
-/* #define SYS5  1 */
-/* #define SVR4  1 */
-
-/*
- *  SunOS 4.1.x
- */
-/* #define BIND     1 */
-/* #define BSD42    1 */
-
-/*
- *  Linux
- */
-/* #define LINUX_STDIO  1 */
-
-/*
- *  FreeBSD 2.x
- *  NetBSD 1.x,
- *  OpenBSD 2.x
- *  BSD/OS 2.x
- */
-/* #define BIND     1 */
-/* #define BSD42    1 */
-/* #define BSD44    1 */
-
-/*
- *  SCO 4.x
- *  SCO 5.x
- *
- * I believe the second `define' is only necessary
- * for SCO 5.x, not SCO 4.x
- */
-/* #define SYS5         1 */
-/* #define SCO_5_STDIO  1 */
-
-/*
- * Define to 1 if you need to make `inc' set-group-id
- * because your mail spool is not world writable.  This
- * will add some extra security checks, although I can't
- * guarantee it is safe.  Also, you will need to change the
- * group and add the setgid bit to `inc' manually after
- * installation.
- */
-/* #define MAILGROUP  1 */
-
 /*
  * Turn on locale (setlocale) support
  */
@@ -137,7 +88,9 @@
 /*
  * If you enable POP support, this is the the port name
  * that nmh will use.  Make sure this is defined in your
- * /etc/services file (or its NIS/NIS+ eqivalent).
+ * /etc/services file (or its NIS/NIS+ equivalent).  If you
+ * are using KPOP, you will probably need to change this
+ * to "kpop".
  */
 #define POPSERVICE "pop3"
 
@@ -145,7 +98,7 @@
  * Define the default creation modes for folders and messages.
  */
 #define DEFAULT_FOLDER_MODE "0700"
-#define DEFAULT_MESSAGE_MODE "0644"
+#define DEFAULT_MESSAGE_MODE "0600"
 
 /*
  * The prefix which is prepended to the name of messages when they
@@ -163,9 +116,35 @@
  */
 #define WAITINT 1
 
+/* The following are autoconfigured, but you may wish to override the
+ * decisions of autoconf (and AC_CANONICAL_SYSTEM) and do your own
+ * thing. If so, you can modify the definitions. The Comments are as
+ * useful as ever. */
+
+/* these three are only used in vmh.c, which currently is not being
+ * built anyway. -Doug */
+/* Defined for Solaris 2.x, Irix, OSF/1, HP-UX, AIX, SCO5 */
+#undef SYS5
+/* Defined for Solaris 2.x, Irix, OSF/1, HP-UX, AIX */
+#undef SVR4
+/* Defined for SunOS 4, FreeBSD, NetBSD, OpenBSD, BSD/OS, Mac OS X/Rhapsody */
+#undef BSD44
+
+/* Defined for SunOS 4, FreeBSD, NetBSD, OpenBSD, BSD/OS, Mac OS X/Rhapsody 
+ * -- does PicoBSD have uname? :) */
+#undef BSD42
+
+/* Defined for SCO5 */
+#undef SCO_5_STDIO
+
+/* This tells nmh to use the Linux _IO_*_ptr defines from libio.h */ 
+#undef LINUX_STDIO
+
+
 /***** END USER CONFIGURATION SECTION *****/
 @TOP@
 
+
 /*
  * Define this if you want SMTP (simple mail transport protocol)
  * support.  When sending mail, instead of passing the message to
@@ -189,6 +168,27 @@
  */
 #undef POP
 
+/*
+ * Define this to compile client-side support for kpop
+ * (kerberized pop) into inc and msgchk.  Do not change this
+ * value manually.  You must run configure with the option
+ * '--with-krb4=PREFIX' to correctly build the kpop client support.
+ */
+#undef KPOP
+
+/*
+ * Define this to "pop" when using Kerberos V4
+ */
+#undef KPOP_PRINCIPAL
+
+/*
+ * Define this to compile support for using Hesiod to locate
+ * pop servers into inc and msgchk.  Do not change this value
+ * manually.  You must run configure with the option
+ * '--with-hesiod=PREFIX' to correctly build Hesiod support.
+ */
+#undef HESIOD
+
 /*
  * Compile in support for the Emacs front-end mh-e.
  */
@@ -226,6 +226,13 @@
  */
 #undef ATTVIBUG
 
+/*
+ * Define to 1 if you need to make `inc' set-group-id because your mail spool is
+ * not world writable.  There are no guarantees as to the safety of doing this,
+ * but this #define will add some extra security checks.
+ */
+#undef MAILGROUP
+
 /* Define ruserpass as _ruserpass if your libraries have a bug *
  * such that it can't find ruserpass, but can find _ruserpass. */
 #undef ruserpass
@@ -238,3 +245,21 @@
 
 /* Define to 1 if struct tm has gmtoff */
 #undef HAVE_TM_GMTOFF
+
+/* Define if your system has sigsetjmp */
+#undef HAVE_SIGSETJMP
+
+/* Define if your system has mkstemp */
+#undef HAVE_MKSTEMP
+
+/* Define if your system has sethostent */
+#undef HAVE_SETHOSTENT
+
+/* Define if your system has gethostbyname */
+#undef HAVE_GETHOSTBYNAME
+
+/* Define if your system has netdb.h */
+#undef HAVE_NETDB_H
+
+/* Define if your system has db1/ndbm.h instead of ndbm.h (ppclinux) */
+#undef HAVE_DB1_NDBM_H
