@@ -2,7 +2,7 @@
 /*
  * addrsbr.h -- definitions for the address parsing system
  *
- * $Id$
+ * $Id: addrsbr.h,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #define	AD_HOST	1		/* getm(): lookup official hostname    */
@@ -27,10 +27,6 @@ struct mailname {
     int m_ingrp;
     char *m_gname;
     char *m_note;
-
-#ifdef MHMTS
-    char *m_aka;
-#endif
 };
 
 #define	adrformat(m) auxformat ((m), 1)
