@@ -2,7 +2,7 @@
 /*
  * mh.h -- main header file for all of nmh
  *
- * $Id$
+ * $Id: mh.h,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/nmh.h>
@@ -295,6 +295,10 @@ extern char *mhlreply;
 extern char *moreproc;
 extern char *msgprot;
 extern char *mshproc;
+extern char *nmhaccessftp;
+extern char *nmhstorage;
+extern char *nmhcache;
+extern char *nmhprivcache;
 extern char *nsequence;
 extern char *packproc;
 extern char *postproc;
