@@ -2,7 +2,7 @@
 /*
  * popsbr.h -- header for POP client subroutines
  *
- * $Id$
+ * $Id: popsbr.h,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #if 0
@@ -12,12 +12,6 @@
 #endif
 #endif
 
-#if defined(RPOP) || defined(APOP)
-int pop_init (char *, char *, char *, int, int);
-#else
-int pop_init (char *, char *, char *, int);
-#endif
-
 #ifdef NNTP
 int pop_set (int, int, int, char *);
 #else
@@ -28,7 +22,8 @@ int pop_set (int, int, int);
 int pop_exists (int (*)());
 #endif
 
-int pop_fd (char *, char *);
+int pop_init (char *, char *, char *, int, int);
+int pop_fd (char *, int, char *, int);
 int pop_stat (int *, int *);
 int pop_retr (int, int (*)());
 int pop_dele (int);
@@ -57,3 +52,11 @@ int pop_last (void);
 int command(const char *, ...);
 int multiline(void);
 #endif
+
+/*
+ * Flags for the various pop authentication methods
+ */
+#define POP_APOP   -1
+#define POP_PASSWD  0
+#define POP_RPOP    1
+#define POP_KPOP    2
