@@ -2,7 +2,7 @@
 /*
  * prototypes.h -- various prototypes
  *
- * $Id$
+ * $Id: prototypes.h,v 1.3 1999/09/12 13:50:11 doug Exp $
  */
 
 /*
@@ -48,7 +48,7 @@ void cpydata (int, int, char *, char *);
 void cpydgst (int, int, char *, char *);
 int decode_rfc2047 (char *, char *);
 void discard (FILE *);
-void done (int);
+int done (int);
 int fdcompare (int, int);
 int folder_addmsg (struct msgs **, char *, int, int, int);
 int folder_delmsgs (struct msgs *, int);
@@ -94,6 +94,7 @@ char *pwd (void);
 char *r1bindex(char *, int);
 void readconfig (struct node **, FILE *, char *, int);
 int refile (char **, char *);
+int ruserpass(char *, char **, char **);
 int remdir (char *);
 int seq_addmsg (struct msgs *, char *, int, int, int);
 int seq_addsel (struct msgs *, char *, int, int);
