@@ -2,7 +2,7 @@
 /*
  * hosts.c -- find out the official name of a host
  *
- * $Id$
+ * $Id: hosts.c,v 1.3 1999/10/15 22:38:56 doug Exp $
  */
 
 /*
@@ -39,23 +39,23 @@ OfficialName (char *name)
     char **r;
     struct host *h;
 
-    for (p = name, q = site; *p; p++, q++)
+    for (p = name, q = site; *p && (q - site < sizeof(site) - 1); p++, q++)
 	*q = isupper (*p) ? tolower (*p) : *p;
-    *q = 0;
+    *q = '\0';
     q = site;
 
     if (!strcasecmp (LocalName(), site))
 	return LocalName();
 
-#ifndef	BIND
+#ifdef	HAVE_SETHOSTENT
     sethostent (1);
 #endif
 
     if ((hp = gethostbyname (q))) {
-	strcpy (buffer, hp->h_name);
+	strncpy (buffer, hp->h_name, sizeof(buffer));
 	return buffer;
     }
-    if (hosts.h_name || init_hs ())
+    if (hosts.h_name || init_hs ()) {
 	for (h = hosts.h_next; h; h = h->h_next)
 	    if (!strcasecmp (h->h_name, q))
 		return h->h_name;
@@ -63,8 +63,9 @@ OfficialName (char *name)
 		for (r = h->h_aliases; *r; r++)
 		    if (!strcasecmp (*r, q))
 			return h->h_name;
+    }
 
-    strcpy (buffer, site);
+    strncpy (buffer, site, sizeof(buffer));
     return buffer;
 }
 
