@@ -2,13 +2,16 @@
 /*
  * sendmail.c -- nmh sendmail interface
  *
- * $Id$
+ * $Id: sendmail.c,v 1.3 1999/07/16 01:17:02 danh Exp $
  */
 
 #include <h/mh.h>
 #include <mts/smtp/smtp.h>
 #include <zotnet/mts/mts.h>
 #include <signal.h>
+#ifdef MPOP
+#include <errno.h>
+#endif
 
 /*
  * This module implements an interface to SendMail very similar
@@ -74,13 +77,9 @@ static char *sm_moreply = "; ";
 
 struct smtp sm_reply;		/* global... */
 
-#ifdef MPOP
-extern int errno;
-#endif
-
 static int doingEHLO;
 
-#define	MAXEHLO	10
+#define	MAXEHLO	20
 char *EHLOkeys[MAXEHLO + 1];
 
 /*
@@ -113,11 +112,12 @@ sm_init (char *client, char *server, int watch, int verbose,
     if (sm_rfp != NULL && sm_wfp != NULL)
 	return RP_OK;
 
-    if (client == NULL || *client == '\0')
+    if (client == NULL || *client == '\0') {
 	if (clientname)
 	    client = clientname;
 	else
 	    client = LocalName();	/* no clientname -> LocalName */
+    }
 
 #ifdef ZMAILER
     if (client == NULL || *client == '\0')
@@ -428,7 +428,7 @@ sm_ierror (char *fmt, ...)
     va_list ap;
 
     va_start(ap, fmt);
-    vsprintf (sm_reply.text, fmt, ap);
+    vsnprintf (sm_reply.text, sizeof(sm_reply.text), fmt, ap);
     va_end(ap);
 
     sm_reply.length = strlen (sm_reply.text);
@@ -446,7 +446,7 @@ smtalk (int time, char *fmt, ...)
     va_list ap;
 
     va_start(ap, fmt);
-    vsprintf (buffer, fmt, ap);
+    vsnprintf (buffer, sizeof(buffer), fmt, ap);
     va_end(ap);
 
     if (sm_debug) {
@@ -456,7 +456,7 @@ smtalk (int time, char *fmt, ...)
 
 #ifdef MPOP
     if (sm_ispool) {
-	char    file[BUFSIZ];
+	char file[BUFSIZ];
 
 	if (strcmp (buffer, ".") == 0)
 	    time = SM_DOT;
@@ -466,18 +466,22 @@ smtalk (int time, char *fmt, ...)
 	        fflush (sm_wfp);
 	        if (ferror (sm_wfp))
 		    return sm_werror ();
-		sprintf (file, "%s%c.bulk", sm_tmpfil,
+		snprintf (file, sizeof(file), "%s%c.bulk", sm_tmpfil,
 				(char) (sm_ispool + 'a' - 1));
 		if (rename (sm_tmpfil, file) == NOTOK) {
-		    char   *bp;
-		    sprintf (bp = sm_reply.text,
-				    "error renaming %s to %s: ",
-				    sm_tmpfil, file);
-		    bp += strlen (bp);
+		    int len;
+		    char *bp;
+
+		    snprintf (sm_reply.text, sizeof(sm_reply.text),
+			"error renaming %s to %s: ", sm_tmpfil, file);
+		    bp = sm_reply.text;
+		    len = strlen (bp);
+		    bp += len;
 		    if ((s = strerror (errno)))
-			strcpy (bp, s);
+			strncpy (bp, s, sizeof(sm_reply.text) - len);
 		    else
-			sprintf (bp, "unknown error %d", errno);
+			snprintf (bp, sizeof(sm_reply.text) - len,
+				"unknown error %d", errno);
 		    sm_reply.length = strlen (sm_reply.text);
 		    sm_reply.code = NOTOK;
 		    return RP_BHST;
@@ -622,10 +626,8 @@ smhear (void)
 	static int at_least_once = 0;
 
 	if (at_least_once) {
-	    char *ep;
-
-	    for (ehlo = EHLOkeys; ep = *ehlo; ehlo++)
-		free (ep);
+	    for (ehlo = EHLOkeys; *ehlo; ehlo++)
+		free (*ehlo);
 	} else {
 	    at_least_once = 1;
 	}
@@ -633,11 +635,13 @@ smhear (void)
 	*(ehlo = EHLOkeys) = NULL;
     }
 
-again: ;
+again:
 
     sm_reply.text[sm_reply.length = 0] = 0;
 
-    rp = sm_reply.text, rc = sizeof sm_reply.text - 1;
+    rp = sm_reply.text;
+    rc = sizeof(sm_reply.text) - 1;
+
     for (more = FALSE; sm_rrecord (bp = buffer, &bc) != NOTOK;) {
 	if (sm_debug) {
 	    printf ("<= %s\n", buffer);
@@ -645,7 +649,7 @@ again: ;
 	}
 
 	if (doingEHLO
-	        && strncmp (buffer, "250", sizeof "250" - 1) == 0
+	        && strncmp (buffer, "250", sizeof("250") - 1) == 0
 	        && (buffer[3] == '-' || doingEHLO == 2)
 	        && buffer[4]) {
 	    if (doingEHLO == 2) {
@@ -681,20 +685,21 @@ again: ;
 	    if (code != sm_reply.code || cont)
 		continue;
 	    more = FALSE;
-	}
-	else {
+	} else {
 	    sm_reply.code = code;
 	    more = cont;
 	    if (bc <= 0) {
-		strcpy (bp = buffer, sm_noreply);
+		strncpy (buffer, sm_noreply, sizeof(buffer));
+		bp = buffer;
 		bc = strlen (sm_noreply);
 	    }
 	}
 	if ((i = min (bc, rc)) > 0) {
 	    strncpy (rp, bp, i);
-	    rp += i, rc -= i;
+	    rp += i;
+	    rc -= i;
 	    if (more && rc > strlen (sm_moreply) + 1) {
-		strcpy (sm_reply.text + rc, sm_moreply);
+		strncpy (sm_reply.text + rc, sm_moreply, sizeof(sm_reply.text) - rc);
 		rc += strlen (sm_moreply);
 	    }
 	}
@@ -795,7 +800,7 @@ rp_string (int code)
 	case RP_BHST: 
 	default: 
 	    text = "BHST";
-	    sprintf (buffer, "[%s] %s", text, sm_reply.text);
+	    snprintf (buffer, sizeof(buffer), "[%s] %s", text, sm_reply.text);
 	    return buffer;
 
 	case RP_PARM: 
@@ -815,7 +820,8 @@ rp_string (int code)
 	    break;
     }
 
-    sprintf (buffer, "[%s] %3d %s", text, sm_reply.code, sm_reply.text);
+    snprintf (buffer, sizeof(buffer), "[%s] %3d %s",
+		text, sm_reply.code, sm_reply.text);
     return buffer;
 }
 
