@@ -2,7 +2,7 @@
 /*
  * addrsbr.c -- parse addresses 822-style
  *
- * $Id$
+ * $Id: addrsbr.c,v 1.2 1999/07/16 01:20:38 danh Exp $
  */
 
 #include <h/mh.h>
@@ -87,7 +87,7 @@ getname (char *addrs)
     if ((ap = getadrx (addrs ? addrs : "")) == NULL)
 	return NULL;
 
-    strcpy (adr, ap->text);
+    strncpy (adr, ap->text, sizeof(adr));
     pers = ap->pers;
     mbox = ap->mbox;
     host = ap->host;
@@ -96,7 +96,7 @@ getname (char *addrs)
     ingrp = ap->ingrp;
     note = ap->note;
     if (ap->err && *ap->err)
-	strcpy (err, ap->err);
+	strncpy (err, ap->err, sizeof(err));
 
     return adr;
 }
@@ -270,35 +270,36 @@ auxformat (struct mailname *mp, int extras)
 
 #ifdef DUMB
 	if (mp->m_nohost)
-	    strcpy (addr, mp->m_mbox ? mp->m_mbox : "");
+	    strncpy (addr, mp->m_mbox ? mp->m_mbox : "", sizeof(addr));
 	else
 #endif /* DUMB */
 
 #ifndef	BANG
 	if (mp->m_type != UUCPHOST)
-	    sprintf (addr, mp->m_host ? "%s%s@%s" : "%s%s",
+	    snprintf (addr, sizeof(addr), mp->m_host ? "%s%s@%s" : "%s%s",
 		empty(mp->m_path), empty(mp->m_mbox), mp->m_host);
 	else
 #endif /* not BANG */
-	    sprintf (addr, "%s!%s", mp->m_host, mp->m_mbox);
+	    snprintf (addr, sizeof(addr), "%s!%s", mp->m_host, mp->m_mbox);
 
     if (!extras)
 	return addr;
 
-    if (mp->m_pers || mp->m_path)
+    if (mp->m_pers || mp->m_path) {
 	if (mp->m_note)
-	    sprintf (buffer, "%s %s <%s>",
+	    snprintf (buffer, sizeof(buffer), "%s %s <%s>",
 		    legal_person (mp->m_pers ? mp->m_pers : mp->m_mbox),
 		    mp->m_note, addr);
 	else
-	    sprintf (buffer, "%s <%s>",
+	    snprintf (buffer, sizeof(buffer), "%s <%s>",
 		    legal_person (mp->m_pers ? mp->m_pers : mp->m_mbox),
 		    addr);
+    }
     else
 	if (mp->m_note)
-	    sprintf (buffer, "%s %s", addr, mp->m_note);
+	    snprintf (buffer, sizeof(buffer), "%s %s", addr, mp->m_note);
 	else
-	    strcpy (buffer, addr);
+	    strncpy (buffer, addr, sizeof(buffer));
 
     return buffer;
 }
@@ -328,9 +329,9 @@ adrsprintf (char *local, char *domain)
 	domain = LocalName ();
 
 #ifndef	BANG
-    sprintf (addr, "%s@%s", local, domain);
+    snprintf (addr, sizeof(addr), "%s@%s", local, domain);
 #else /* BANG */
-    sprintf (addr, "%s!%s", domain, local);
+    snprintf (addr, sizeof(addr), "%s!%s", domain, local);
 #endif /* BANG */
 
     return addr;
@@ -402,7 +403,7 @@ ismymbox (struct mailname *np)
 		    if ((cp = getenv ("MHWDEBUG")) && *cp)
 			fprintf (stderr, "mbox=\"%s\" host=\"%s\" %s\n",
 			    mp->m_mbox, mp->m_host,
-			    snprintb (buffer, BUFSIZ, (unsigned) mp->m_type, WBITS));
+			    snprintb (buffer, sizeof(buffer), (unsigned) mp->m_type, WBITS));
 		}
 	    }
 	    if (oops)
