@@ -2,7 +2,7 @@
 /*
  * context_find.c -- find an entry in the context/profile list
  *
- * $Id$
+ * $Id: context_find.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/mh.h>
@@ -13,7 +13,10 @@ context_find (char *str)
 {
     struct node *np;
 
-    context_read();
+    /* sanity check - check that context has been read */
+    if (defpath == NULL)
+	adios (NULL, "oops, context hasn't been read yet");
+
     for (np = m_defs; np; np = np->n_next)
 	if (!strcasecmp (np->n_name, str))
 	    return (np->n_field);
