@@ -2,7 +2,7 @@
 /*
  * context_read.c -- find and read profile and context files
  *
- * $Id$
+ * $Id: context_read.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/mh.h>
@@ -74,7 +74,7 @@ context_read (void)
 	    }
 	}
     }
-    m_readefs (&m_defs, ib, mh_profile, 0);
+    readconfig (&m_defs, ib, mh_profile, 0);
     fclose (ib);
 
     /*
@@ -82,9 +82,9 @@ context_read (void)
      */
     if ((pp = context_find ("path")) && *pp != '\0') {
 	if (*pp != '/')
-	    snprintf (buf, BUFSIZ, "%s/%s", mypath, pp);
+	    snprintf (buf, sizeof(buf), "%s/%s", mypath, pp);
 	else
-	    strcpy (buf, pp);
+	    strncpy (buf, pp, sizeof(buf));
 	if (stat(buf, &st) == -1) {
 	    if (errno != ENOENT)
 		adios (buf, "error opening");
@@ -105,7 +105,7 @@ context_read (void)
 	cp = context;
     ctxpath = getcpy (m_maildir (cp));
     if ((ib = fopen (ctxpath, "r"))) {
-	m_readefs ((struct node **) 0, ib, cp, 1);
+	readconfig ((struct node **) 0, ib, cp, 1);
 	fclose (ib);
     }
 }
