@@ -2,7 +2,7 @@
 /*
  * context_replace.c -- add/replace an entry in the context/profile list
  *
- * $Id$
+ * $Id: context_replace.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/mh.h>
@@ -13,7 +13,9 @@ context_replace (char *key, char *value)
 {
     register struct node *np;
 
-    context_read();
+    /* sanity check - check that context has been read */
+    if (defpath == NULL)
+	adios (NULL, "oops, context hasn't been read yet");
 
     /*
      * If list is emtpy, allocate head of profile/context list.
