@@ -2,7 +2,7 @@
 /*
  * fmt_compile.c -- "compile" format strings for fmt_scan
  *
- * $Id$
+ * $Id: fmt_compile.c,v 1.2 1999/09/12 13:50:11 doug Exp $
  */
 
 #include <h/mh.h>
@@ -11,6 +11,22 @@
 #include <h/fmt_scan.h>
 #include <h/fmt_compile.h>
 
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef TM_IN_SYS_TIME
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
+/*
+ * hash table for deciding if a component is "interesting"
+ */
+struct comp *wantcomp[128];
+
 static struct format *formatvec;	/* array to hold formats */
 static struct format *next_fp;		/* next free format slot */
 static struct format *fp;		/* current format slot   */
@@ -132,16 +148,20 @@ static struct ftable functable[] = {
      { NULL,         0,		0,		0,		0 }
 };
 
-
+/* Add new component to the hash table */
 #define NEWCOMP(cm,name)\
 	cm = ((struct comp *) calloc(1, sizeof (struct comp)));\
-	cm->c_name = name; ncomp++;\
- 	i = CHASH(name); cm->c_next = wantcomp[i]; wantcomp[i] = cm;
+	cm->c_name = name;\
+	ncomp++;\
+	i = CHASH(name);\
+	cm->c_next = wantcomp[i];\
+	wantcomp[i] = cm;
 
 #define NEWFMT (next_fp++)
 #define NEW(type,fill,wid)\
 	fp=NEWFMT; fp->f_type=(type); fp->f_fill=(fill); fp->f_width=(wid);
 
+/* Add (possibly new) component to the hash table */
 #define ADDC(name)\
 	FINDCOMP(cm, name);\
 	if (!cm) {\
@@ -149,19 +169,22 @@ static struct ftable functable[] = {
 	}\
 	fp->f_comp = cm;
 
-#define LV( type, value )	NEW(type,0,0); fp->f_value = (value);
-#define LS( type, str )		NEW(type,0,0); fp->f_text = (str);
-#define PUTCOMP( comp )		NEW(FT_COMP,0,0); ADDC(comp);
-#define PUTLIT( str )		NEW(FT_LIT,0,0); fp->f_text = (str);
-#define PUTC( c )		NEW(FT_CHAR,0,0); fp->f_char = (c);
+#define LV(type, value)		NEW(type,0,0); fp->f_value = (value);
+#define LS(type, str)		NEW(type,0,0); fp->f_text = (str);
+
+#define PUTCOMP(comp)		NEW(FT_COMP,0,0); ADDC(comp);
+#define PUTLIT(str)		NEW(FT_LIT,0,0); fp->f_text = (str);
+#define PUTC(c)			NEW(FT_CHAR,0,0); fp->f_char = (c);
 
 static char *format_string;
 static char *usr_fstring;	/* for CERROR */
 
 #define CERROR(str) compile_error (str, cp)
 
-
-extern char *getusr();
+/*
+ * external prototypes
+ */
+extern char *getusername(void);
 
 /*
  * static prototypes
@@ -197,18 +220,21 @@ lookup(char *name)
 static void
 compile_error(char *str, char *cp)
 {
-    int errpos = cp - format_string;
-    int errctx = errpos > 20 ? 20 : errpos;
-    int i;
+    int i, errpos, errctx;
 
+    errpos = cp - format_string;
+    errctx = errpos > 20 ? 20 : errpos;
     usr_fstring[errpos] = '\0';
-    for (i = errpos-errctx; i < errpos; i++)
+
+    for (i = errpos-errctx; i < errpos; i++) {
 #ifdef LOCALE
 	if (iscntrl(usr_fstring[i]))
 #else
 	if (usr_fstring[i] < 32)
 #endif
 	    usr_fstring[i] = '_';
+    }
+
     advise(NULL, "\"%s\": format compile error - %s",
 	   &usr_fstring[errpos-errctx], str);
     adios (NULL, "%*s", errctx+1, "^");
@@ -490,7 +516,7 @@ do_func(char *sp)
 	break;
 
     case TF_MYBOX:
-	LS(t->f_type, getusr());
+	LS(t->f_type, getusername());
 	break;
 
     case TF_NOW:
