@@ -2,7 +2,7 @@
 /*
  * fmt_scan.c -- format string interpretation
  *
- * $Id$
+ * $Id: fmt_scan.c,v 1.5 1999/09/26 17:45:56 ruud Exp $
  */
 
 #include <h/mh.h>
@@ -11,7 +11,18 @@
 #include <zotnet/tws/tws.h>
 #include <h/fmt_compile.h>
 
-#define	NFMTS	MAXARGS
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef TM_IN_SYS_TIME
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
+#define	NFMTS MAXARGS
 
 extern char *formataddr ();	/* hook for custom address formatting */
 
@@ -26,8 +37,8 @@ struct mailname fmt_mnull;
  * static prototypes
  */
 static int match (char *, char *);
-static char *get_x400_friendly (char *, char *);
-static int get_x400_comp (char *, char *, char *);
+static char *get_x400_friendly (char *, char *, int);
+static int get_x400_comp (char *, char *, char *, int);
 
 
 /*
@@ -74,38 +85,45 @@ match (char *str, char *sub)
  * macros to format data
  */
 
-#define PUTDF(cp, num, wid, fill) if (cp + wid < ep){\
-		if((i = (num))<0) i = -(num);\
-		if((c = (wid))<0) c = -c;\
-		sp = cp + c;\
-		do {\
-		    *--sp = (i % 10) + '0';\
-		    i /= 10;\
-		} while (i > 0 && sp > cp);\
-		if (i > 0)\
-		    *sp = '?';\
-		else if ((num) < 0 && sp > cp)\
-		    *--sp = '-';\
-		while (sp > cp)\
-		    *--sp = fill;\
-		cp += c;\
-		}
+#define PUTDF(cp, num, wid, fill)\
+	if (cp + wid < ep) {\
+	    if ((i = (num)) < 0)\
+		i = -(num);\
+	    if ((c = (wid)) < 0)\
+		c = -c;\
+	    sp = cp + c;\
+	    do {\
+		*--sp = (i % 10) + '0';\
+		i /= 10;\
+	    } while (i > 0 && sp > cp);\
+	    if (i > 0)\
+		*sp = '?';\
+	    else if ((num) < 0 && sp > cp)\
+		*--sp = '-';\
+	    while (sp > cp)\
+		*--sp = fill;\
+	    cp += c;\
+	    }
 
-#define PUTD(cp, num) if (cp < ep){\
-		if((i = (num))==0) *cp++ = '0';\
-		else {\
-		    if((i = (num))<0) \
-			*cp++ = '-', i = -(num);\
-		    c = 10;\
-		    while (c <= i) \
-			c *= 10;\
-		    while (cp < ep && c > 1) {\
-			c /= 10;\
-			*cp++ = (i / c) + '0';\
-			i %= c;\
-			}\
+#define PUTD(cp, num)\
+	if (cp < ep) {\
+	    if ((i = (num)) == 0)\
+		*cp++ = '0';\
+	    else {\
+		if ((i = (num)) < 0) {\
+		    *cp++ = '-';\
+		    i = -(num);\
 		    }\
-		}
+		c = 10;\
+		while (c <= i) \
+		    c *= 10;\
+		while (cp < ep && c > 1) {\
+		    c /= 10;\
+		    *cp++ = (i / c) + '0';\
+		    i %= c;\
+		    }\
+		}\
+	    }
 
 #ifdef LOCALE
 #define PUTSF(cp, str, wid, fill) {\
@@ -215,7 +233,7 @@ static char *lmonth[] = { "January",  "February","March",   "April",
 			  "September","October", "November","December" };
 
 static char *
-get_x400_friendly (char *mbox, char *buffer)
+get_x400_friendly (char *mbox, char *buffer, int buffer_len)
 {
     char given[BUFSIZ], surname[BUFSIZ];
 
@@ -226,26 +244,26 @@ get_x400_friendly (char *mbox, char *buffer)
     if (*mbox != '/')
 	return NULL;
 
-    if (get_x400_comp (mbox, "/PN=", buffer)) {
-	for (mbox = buffer; mbox = strchr(mbox, '.'); )
+    if (get_x400_comp (mbox, "/PN=", buffer, buffer_len)) {
+	for (mbox = buffer; (mbox = strchr(mbox, '.')); )
 	    *mbox++ = ' ';
 
 	return buffer;
     }
 
-    if (!get_x400_comp (mbox, "/S=", surname))
+    if (!get_x400_comp (mbox, "/S=", surname, sizeof(surname)))
 	return NULL;
 
-    if (get_x400_comp (mbox, "/G=", given))
-	sprintf (buffer, "%s %s", given, surname);
+    if (get_x400_comp (mbox, "/G=", given, sizeof(given)))
+	snprintf (buffer, buffer_len, "%s %s", given, surname);
     else
-	strcpy (buffer, surname);
+	snprintf (buffer, buffer_len, "%s", surname);
 
     return buffer;
 }
 
 static int
-get_x400_comp (char *mbox, char *key, char *buffer)
+get_x400_comp (char *mbox, char *key, char *buffer, int buffer_len)
 {
     int	idx;
     char *cp;
@@ -254,7 +272,7 @@ get_x400_comp (char *mbox, char *key, char *buffer)
 	    || !(cp = strchr(mbox += idx + strlen (key), '/')))
 	return 0;
 
-    sprintf (buffer, "%*.*s", cp - mbox, cp - mbox, mbox);
+    snprintf (buffer, buffer_len, "%*.*s", cp - mbox, cp - mbox, mbox);
     return 1;
 }
 
@@ -446,7 +464,7 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 	    if (str) {
 		    char *xp;
 
-		    strcpy(buffer, str);
+		    strncpy(buffer, str, sizeof(buffer));
 		    str = buffer;
 		    while (isspace(*str))
 			    str++;
@@ -629,9 +647,9 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 	    }
 	    if (fmt->f_type == FT_LS_ADDR)
 		goto unfriendly;
-	    if ((str = mn->m_pers) == NULL)
+	    if ((str = mn->m_pers) == NULL) {
 	        if ((str = mn->m_note)) {
-	            strcpy (buffer, str);
+	            strncpy (buffer, str, sizeof(buffer));
 	            str = buffer;
 	            if (*str == '(')
 	            	str++;
@@ -644,19 +662,22 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 	            	    else
 	            		break;
 	            }
-	        } else if (!(str = get_x400_friendly (mn->m_mbox, buffer))) {
+	        } else if (!(str = get_x400_friendly (mn->m_mbox,
+				buffer, sizeof(buffer)))) {
 	unfriendly: ;
 		  switch (mn->m_type) {
 		    case LOCALHOST:
 			str = mn->m_mbox;
 			break;
 		    case UUCPHOST:
-			sprintf (buffer, "%s!%s", mn->m_host, mn->m_mbox);
+			snprintf (buffer, sizeof(buffer), "%s!%s",
+				mn->m_host, mn->m_mbox);
 			str = buffer;
 			break;
 		    default:
 			if (mn->m_mbox) {
-			    sprintf (buffer, "%s@%s", mn->m_mbox, mn->m_host);
+			    snprintf (buffer, sizeof(buffer), "%s@%s",
+				mn->m_mbox, mn->m_host);
 			    str= buffer;
 			}
 			else
@@ -664,6 +685,7 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 			break;
 		  }
 		}
+	    }
 	    break;
 
 	case FT_LOCALDATE:
@@ -719,24 +741,33 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 		*cp++ = c;
 	    while (len > wid) {
 		/* try to break at a comma; failing that, break at a
-		 * space, failing that, just split the line.
+		 * space.
 		 */
 		lastb = 0; sp = lp + wid;
 		while (sp > lp && (c = *--sp) != ',') {
 		    if (! lastb && isspace(c))
 			lastb = sp - 1;
 		}
-		if (sp == lp)
-		    if (! (sp = lastb))
+		if (sp == lp) {
+		    if (! (sp = lastb)) {
 			sp = lp + wid - 1;
+			while (*sp && *sp != ',' && !isspace(*sp))
+			    sp++;
+			if (*sp != ',')
+			    sp--;
+		    }
+		}
 		len -= sp - lp + 1;
 		while (cp < ep && lp <= sp)
 		    *cp++ = *lp++;
-		*cp++ = '\n';
-		for (i=indent; cp < ep && i > 0; i--)
-		    *cp++ = ' ';
 		while (isspace(*lp))
 		    lp++, len--;
+		if (*lp) {
+		    if (cp < ep)
+			*cp++ = '\n';
+		    for (i=indent; cp < ep && i > 0; i--)
+			*cp++ = ' ';
+		}
 	    }
 	    PUTS (cp, lp);
 	    }
