@@ -3,7 +3,7 @@
  * folder_pack.c -- pack (renumber) the messages in a folder
  *               -- into a contiguous range from 1 to n.
  *
- * $Id$
+ * $Id: folder_pack.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/mh.h>
@@ -44,8 +44,8 @@ folder_pack (struct msgs **mpp, int verbose)
     for (msgnum = mp->lowmsg, hole = 1; msgnum <= mp->hghmsg; msgnum++) {
 	if (does_exist (mp, msgnum)) {
 	    if (msgnum != hole) {
-		strcpy (newmsg, m_name (hole));
-		strcpy (oldmsg, m_name (msgnum));
+		strncpy (newmsg, m_name (hole), sizeof(newmsg));
+		strncpy (oldmsg, m_name (msgnum), sizeof(oldmsg));
 		if (verbose)
 		    printf ("message %s becomes %s\n", oldmsg, newmsg);
 
