@@ -2,7 +2,7 @@
 /*
  * folder_read.c -- initialize folder structure and read folder
  *
- * $Id$
+ * $Id: folder_read.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/mh.h>
@@ -16,12 +16,13 @@
  *    record the numbers of the messages we have seen.
  * 3) Then allocate the array for message attributes and
  *    set the initial flags for all messages we've seen.
+ * 4) Read and initialize the sequence information.
  */
 
 struct msgs *
 folder_read (char *name)
 {
-    int i, j, len, *mi;
+    int msgnum, prefix_len, len, *mi;
     struct msgs *mp;
     struct stat st;
     struct dirent *dp;
@@ -54,7 +55,7 @@ folder_read (char *name)
 
     if (access (name, W_OK) == -1 || st.st_uid != getuid())
 	set_readonly (mp);
-    j = strlen(BACKUP_PREFIX);
+    prefix_len = strlen(BACKUP_PREFIX);
 
     /*
      * Allocate a temporary place to record the
@@ -65,7 +66,7 @@ folder_read (char *name)
 	adios (NULL, "unable to allocate storage");
 
     while ((dp = readdir (dd))) {
-	if ((i = m_atoi (dp->d_name))) {
+	if ((msgnum = m_atoi (dp->d_name))) {
 	    /*
 	     * Check if we need to allocate more
 	     * temporary elements for message names.
@@ -78,14 +79,24 @@ folder_read (char *name)
 		}
 	    }
 
-	    /* Check if it is highest or lowest we've seen */
-	    if (mp->lowmsg == 0 || mp->lowmsg > i)
-		mp->lowmsg = i;
-	    if (mp->hghmsg < i)
-		mp->hghmsg = i;
+	    /* Check if this is the first message we've seen */
+	    if (mp->nummsg == 0) {
+		mp->lowmsg = msgnum;
+		mp->hghmsg = msgnum;
+	    } else {
+		/* Check if this is it the highest or lowest we've seen? */
+		if (msgnum < mp->lowmsg)
+		   mp->lowmsg = msgnum;
+		if (msgnum > mp->hghmsg)
+		   mp->hghmsg = msgnum;
+	    }
+
+	    /*
+	     * Now increment count, and record message
+	     * number in a temporary place for now.
+	     */
+	    mi[mp->nummsg++] = msgnum;
 
-	    /* Now record it in temporary place */
-	    mi[mp->nummsg++] = i;
 	} else {
 	    switch (dp->d_name[0]) {
 		case '.': 
@@ -93,16 +104,18 @@ folder_read (char *name)
 #ifdef MHE
 		case '+': 
 #endif /* MHE */
-#ifdef UCI
-		case '_': 
-		case '#': 
-#endif /* UCI */
 		    continue;
 
 		default: 
-		    if (strcmp(dp->d_name, LINK) == 0
-			    || strncmp(dp->d_name, BACKUP_PREFIX, j) == 0)
+		    /* skip any files beginning with backup prefix */
+		    if (!strncmp (dp->d_name, BACKUP_PREFIX, prefix_len))
 			continue;
+
+		    /* skip the LINK file */
+		    if (!strcmp (dp->d_name, LINK))
+			continue;
+
+		    /* indicate that there are other files in folder */
 		    set_other_files (mp);
 		    continue;
 	    }
@@ -128,18 +141,23 @@ folder_read (char *name)
      * Clear all the flag bits for all the message
      * status entries we just allocated.
      */
-    for (i = mp->lowoff; i <= mp->hghoff; i++)
-	clear_msg_flags (mp, i);
+    for (msgnum = mp->lowoff; msgnum <= mp->hghoff; msgnum++)
+	clear_msg_flags (mp, msgnum);
 
     /*
      * Scan through the array of messages we've seen and
      * setup the initial flags for those messages in the
      * newly allocated mp->msgstats area.
      */
-    for (i = 0; i < mp->nummsg; i++)
-	set_exists (mp, mi[i]);
+    for (msgnum = 0; msgnum < mp->nummsg; msgnum++)
+	set_exists (mp, mi[msgnum]);
 
     free (mi);		/* We don't need this anymore    */
-    seq_read (mp);	/* read the sequence information */
+
+    /*
+     * Read and initialize the sequence information.
+     */
+    seq_read (mp);
+
     return mp;
 }
