@@ -2,12 +2,23 @@
 /*
  * lock.c -- routines to lock/unlock files
  *
- * $Id$
+ * $Id: lock_file.c,v 1.4 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
 #include <h/signals.h>
 
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef TM_IN_SYS_TIME
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 #ifdef HAVE_ERRNO_H
 # include <errno.h>
 #endif
@@ -49,14 +60,14 @@ struct lockinfo {
 };
 
 /*
- * amount of time to wait before
- * updating ctime of lock file
+ * Amount of time to wait before
+ * updating ctime of lock file.
  */
 #define	NSECS 20
 
 /*
- * how old does a lock file need to be
- * before we remove it
+ * How old does a lock file need to be
+ * before we remove it.
  */
 #define RSECS 180
 
@@ -75,11 +86,11 @@ static struct lock *l_top = NULL;
  * static prototypes
  */
 #ifdef KERNEL_LOCKING
-static int lkopen_kernel (char *, int);
+static int lkopen_kernel (char *, int, mode_t);
 #endif
 
 #ifdef DOT_LOCKING
-static int lkopen_dot (char *, int);
+static int lkopen_dot (char *, int, mode_t);
 static int lockit (struct lockinfo *);
 static void lockname (char *, struct lockinfo *, int);
 static void timerON (char *, int);
@@ -94,14 +105,14 @@ static RETSIGTYPE alrmser (int);
  */
 
 int
-lkopen (char *file, int access)
+lkopen (char *file, int access, mode_t mode)
 {
 #ifdef KERNEL_LOCKING
-    return lkopen_kernel(file, access);
+    return lkopen_kernel(file, access, mode);
 #endif
 
 #ifdef DOT_LOCKING
-    return lkopen_dot(file, access);
+    return lkopen_dot(file, access, mode);
 #endif
 }
 
@@ -144,9 +155,9 @@ lkclose (int fd, char *file)
 #endif	
 
 #ifdef DOT_LOCKING
-    lockname (file, &lkinfo, 0);
-    unlink (lkinfo.curlock);
-    timerOFF (fd);
+    lockname (file, &lkinfo, 0);	/* get name of lock file */
+    unlink (lkinfo.curlock);		/* remove lock file      */
+    timerOFF (fd);			/* turn off lock timer   */
 #endif
 
     return (close (fd));
@@ -161,10 +172,15 @@ lkclose (int fd, char *file)
 FILE *
 lkfopen (char *file, char *mode)
 {
-    int fd;
+    int fd, access;
     FILE *fp;
 
-    if ((fd = lkopen (file, strcmp (mode, "r") ? 2 : 0)) == -1)
+    if (strcmp (mode, "r") == 0)
+	access = O_RDONLY;
+    else
+	access = O_RDWR;
+
+    if ((fd = lkopen (file, access, 0)) == -1)
 	return NULL;
 
     if ((fp = fdopen (fd, mode)) == NULL) {
@@ -214,9 +230,9 @@ lkfclose (FILE *fp, char *file)
 #endif
 
 #ifdef DOT_LOCKING
-    lockname (file, &lkinfo, 0);
-    unlink (lkinfo.curlock);
-    timerOFF (fileno(fp));
+    lockname (file, &lkinfo, 0);	/* get name of lock file */
+    unlink (lkinfo.curlock);		/* remove lock file      */
+    timerOFF (fileno(fp));		/* turn off lock timer   */
 #endif
 
     return (fclose (fp));
@@ -230,7 +246,7 @@ lkfclose (FILE *fp, char *file)
  */
 
 static int
-lkopen_kernel (char *file, int access)
+lkopen_kernel (char *file, int access, mode_t mode)
 {
     int fd, i, j;
 
@@ -241,20 +257,23 @@ lkopen_kernel (char *file, int access)
     for (i = 0; i < 5; i++) {
 
 # if defined(LOCKF_LOCKING) || defined(FCNTL_LOCKING)
+	/* remember the original mode */
 	j = access;
+
 	/* make sure we open at the beginning */
 	access &= ~O_APPEND;
+
+	/*
+	 * We MUST have write permission or
+	 * lockf/fcntl() won't work
+	 */
 	if ((access & 03) == O_RDONLY) {
-	    /*
-	     * We MUST have write permission or
-	     * lockf/fcntl() won't work
-	     */
 	    access &= ~O_RDONLY;
 	    access |= O_RDWR;
 	}
 # endif /* LOCKF_LOCKING || FCNTL_LOCKING */
 
-	if ((fd = open (file, access | O_NDELAY)) == -1)
+	if ((fd = open (file, access | O_NDELAY, mode)) == -1)
 	    return -1;
 
 # ifdef FCNTL_LOCKING
@@ -300,22 +319,34 @@ lkopen_kernel (char *file, int access)
  */
 
 static int
-lkopen_dot (char *file, int access)
+lkopen_dot (char *file, int access, mode_t mode)
 {
     int i, fd;
     time_t curtime;
     struct lockinfo lkinfo;
     struct stat st;
 
-    if ((fd = open (file, access)) == -1)
+    /* open the file */
+    if ((fd = open (file, access, mode)) == -1)
 	return -1;
+
+    /*
+     * Get the name of the eventual lock file, as well
+     * as a name for a temporary lock file.
+     */
     lockname (file, &lkinfo, 1);
 
     for (i = 0;;) {
+	/* attempt to create lock file */
 	if (lockit (&lkinfo) == 0) {
+	    /* if successful, turn on timer and return */
 	    timerON (lkinfo.curlock, fd);
 	    return fd;
 	} else {
+	    /*
+	     * Abort locking, if we fail to lock after 5 attempts
+	     * and are never able to stat the lock file.
+	     */
 	    if (stat (lkinfo.curlock, &st) == -1) {
 		if (i++ > 5)
 		    return -1;
@@ -323,10 +354,12 @@ lkopen_dot (char *file, int access)
 	    } else {
 		i = 0;
 		time (&curtime);
-		if (curtime < st.st_ctime + RSECS)
-		    sleep (5);
-		else
+
+		/* check for stale lockfile, else sleep */
+		if (curtime > st.st_ctime + RSECS)
 		    unlink (lkinfo.curlock);
+		else
+		    sleep (5);
 	    }
 	}
     }
@@ -356,7 +389,7 @@ lockit (struct lockinfo *li)
 
 #if 0
     /* write our process id into lock file */
-    sprintf (buffer, "nmh lock: pid %d\n", (int) getpid());
+    snprintf (buffer, sizeof(buffer), "nmh lock: pid %d\n", (int) getpid());
     write(fd, buffer, strlen(buffer) + 1);
 #endif
 
@@ -379,23 +412,29 @@ lockit (struct lockinfo *li)
 static void
 lockname (char *file, struct lockinfo *li, int isnewlock)
 {
+    int bplen, tmplen;
     char *bp, *cp;
 
 #if 0
     struct stat st;
 #endif
 
-    bp = li->curlock;
-    if ((cp = strrchr(file, '/')) == NULL || *++cp == 0)
+    if ((cp = strrchr (file, '/')) == NULL || *++cp == 0)
 	cp = file;
 
+    bp = li->curlock;
+    bplen = 0;
 #ifdef LOCKDIR
-    sprintf (bp, "%s/", lockdir);
-    bp += strlen (bp);
+    snprintf (bp, sizeof(li->curlock), "%s/", lockdir);
+    tmplen = strlen (bp);
+    bp    += tmplen;
+    bplen += tmplen;
 #else
     if (cp != file) {
-	sprintf (bp, "%.*s", cp - file, file);
-	bp += strlen (bp);
+	snprintf (bp, sizeof(li->curlock), "%.*s", cp - file, file);
+	tmplen = strlen (bp);
+	bp    += tmplen;
+	bplen += tmplen;
     }
 #endif
 
@@ -408,28 +447,41 @@ lockname (char *file, struct lockinfo *li, int isnewlock)
     if (stat (file, &st) == -1)
 	return -1;
 
-    sprintf (bp, "LCK%05d.%05d", st.st_dev, st.st_ino);
+    snprintf (bp, sizeof(li->curlock) - bplen, "LCK%05d.%05d",
+	st.st_dev, st.st_ino);
 #endif
 
-    sprintf (bp, "%s.lock", cp);
+    snprintf (bp, sizeof(li->curlock) - bplen, "%s.lock", cp);
 
     /*
      * If this is for a new lock, create a name for
      * the temporary lock file for lockit()
      */
     if (isnewlock) {
-	if ((cp = strrchr(li->curlock, '/')) == NULL || *++cp == 0)
-	    strcpy (li->tmplock, ",LCK.XXXXXX");
+	if ((cp = strrchr (li->curlock, '/')) == NULL || *++cp == 0)
+	    strncpy (li->tmplock, ",LCK.XXXXXX", sizeof(li->tmplock));
 	else
-	    sprintf (li->tmplock, "%.*s,LCK.XXXXXX",
+	    snprintf (li->tmplock, sizeof(li->tmplock), "%.*s,LCK.XXXXXX",
 		     cp - li->curlock, li->curlock);
-	unlink (mktemp (li->tmplock));
+/*
+  Mkstemp work postponed until later -Doug
+#ifdef HAVE_MKSTEMP
+	mkstemp (li->tmplock);
+#else
+*/
+	mktemp (li->tmplock);
+/*
+#endif
+*/
+
+	unlink (li->tmplock);	/* remove any stray */
     }
 }
 
 
 /*
- * Add new lockfile to the list of open lockfiles.
+ * Add new lockfile to the list of open lockfiles
+ * and start the lock file timer.
  */
 
 static void
