@@ -2,7 +2,7 @@
 /*
  * m_convert.c -- parse a message range or sequence and set SELECTED
  *
- * $Id$
+ * $Id: m_convert.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/mh.h>
@@ -257,13 +257,17 @@ m_conv (struct msgs *mp, char *str, int call)
 	    return BADNUM;
     }
 
-    bp = buf;
 #ifdef LOCALE
-    while (isalpha(*cp) || *cp == '.')	/* doesn't enforce lower case */
+    /* doesn't enforce lower case */
+    for (bp = buf; (isalpha(*cp) || *cp == '.')
+		&& (bp - buf < sizeof(buf) - 1); )
 #else
-    while ((*cp >= 'a' && *cp <= 'z') || *cp == '.')
+    for (bp = buf; ((*cp >= 'a' && *cp <= 'z') || *cp == '.')
+		&& (bp - buf < sizeof(buf) - 1); )
 #endif /* LOCALE */
+    {
 	*bp++ = *cp++;
+    }
     *bp++ = '\0';
     delimp = cp;
 
