@@ -2,7 +2,7 @@
 /*
  * m_getfld.c -- read/parse a message
  *
- * $Id$
+ * $Id: m_getfld.c,v 1.4 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
@@ -259,7 +259,7 @@ m_getfld (int state, unsigned char *name, unsigned char *buf,
 		bp = sp = (unsigned char *) iob->_ptr - 1;
 		j = (cnt = iob->_cnt+1) < i ? cnt : i;
 #endif
-		while ((c = *bp++) != ':' && c != '\n' && --j >= 0)
+		while (--j >= 0 && (c = *bp++) != ':' && c != '\n')
 		    *cp++ = c;
 
 		j = bp - sp;
@@ -390,7 +390,8 @@ m_getfld (int state, unsigned char *name, unsigned char *buf,
 		*cp++ = j = *(iob->_ptr + c);
 		c = _filbuf(iob);
 #endif
-		if ((j == '\0' || j == '\n') && c != ' ' && c != '\t') {
+                if (c == EOF || 
+		  ((j == '\0' || j == '\n') && c != ' ' && c != '\t')) {
 		    if (c != EOF) {
 #ifdef LINUX_STDIO
 			--iob->_IO_read_ptr;
@@ -538,7 +539,7 @@ m_unknown(FILE *iob)
 	    ;
 #else /* RPATHS */
 	cp = unixbuf;
-	while ((c = getc (iob)) != '\n')
+	while ((c = getc (iob)) != '\n' && cp - unixbuf < BUFSIZ - 1)
 	    *cp++ = c;
 	*cp = 0;
 #endif /* RPATHS */
@@ -558,7 +559,7 @@ m_unknown(FILE *iob)
     edelim = (unsigned char *)msg_delim+1;
     fdelimlen = c + 1;
     edelimlen = c - 1;
-    strcpy(msg_delim, delimstr);
+    strcpy (msg_delim, delimstr);
     delimend = (unsigned char *)msg_delim + edelimlen;
     if (edelimlen <= 1)
 	adios (NULL, "maildrop delimiter must be at least 2 bytes");
@@ -639,7 +640,7 @@ m_Eom (int c, FILE *iob)
 		break;
 #else /* RPATHS */
 	cp = unixbuf;
-	while ((c = getc (iob)) != '\n' && c >= 0)
+	while ((c = getc (iob)) != '\n' && c >= 0 && cp - unixbuf < BUFSIZ - 1)
 	    *cp++ = c;
 	*cp = 0;
 #endif /* RPATHS */
@@ -662,7 +663,7 @@ m_Eom (int c, FILE *iob)
  */
 
 int
-get_returnpath (char *rp, char *dd)
+get_returnpath (char *rp, int rplen, char *dd, int ddlen)
 {
     char *ap, *bp, *cp, *dp;
 
@@ -688,10 +689,10 @@ get_returnpath (char *rp, char *dd)
     if (cp) {
 	/* return path for UUCP style addressing */
 	dp = strchr (++cp, '\n');
-	sprintf (rp, "%.*s!%.*s\n", dp - cp, cp, bp - ap, ap);
+	snprintf (rp, rplen, "%.*s!%.*s\n", dp - cp, cp, bp - ap, ap);
     } else {
 	/* return path for standard domain addressing */
-	sprintf (rp, "%.*s\n", bp - ap, ap);
+	snprintf (rp, rplen, "%.*s\n", bp - ap, ap);
     }
 
     /*
@@ -702,7 +703,7 @@ get_returnpath (char *rp, char *dd)
 	bp++;
 
     /* Now get delivery date from envelope */
-    sprintf (dd, "%.*s\n", 24, bp);
+    snprintf (dd, ddlen, "%.*s\n", 24, bp);
 
     unixbuf[0] = 0;
     return 1;
