@@ -2,15 +2,17 @@
 /*
  * m_msgdef.c -- some defines for sbr/m_getfld.c
  *
- * $Id$
+ * $Id: m_msgdef.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/mh.h>
 
-int msg_count = 0;	/* disgusting hack for "inc" so it can
-			 * know how many characters were stuffed
-			 * in the buffer on the last call (see
-			 * comments in uip/scansbr.c) */
+/*
+ * disgusting hack for "inc" so it can know how many characters
+ * were stuffed in the buffer on the last call (see comments
+ * in uip/scansbr.c)
+ */
+int msg_count = 0;
 
 int msg_style = MS_DEFAULT;
 
