@@ -2,7 +2,7 @@
 /*
  * m_name.c -- return a message number as a string
  *
- * $Id$
+ * $Id: m_name.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/mh.h>
@@ -16,6 +16,6 @@ m_name (int num)
     if (num <= 0)
 	return "?";
 
-    sprintf (name, "%d", num);
+    snprintf (name, sizeof(name), "%d", num);
     return name;
 }
