@@ -2,7 +2,7 @@
 /*
  * makedir.c -- make a directory
  *
- * $Id$
+ * $Id: makedir.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 /*
@@ -28,7 +28,7 @@ makedir (char *dir)
     fflush(stdout);
 
     if (getuid () == geteuid ()) {
-	    c = strcpy(path, dir);     
+	    c = strncpy(path, dir, sizeof(path));     
 
 	    while ((c = strchr((c + 1), '/')) != NULL) {	
 		    *c = (char)0;
