@@ -2,19 +2,18 @@
 /*
  * peekc.c -- peek at the next character in a stream
  *
- * $Id$
+ * $Id: peekc.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/mh.h>
 
 
 int
-peekc(FILE *ib)
+peekc(FILE *fp)
 {
     register int c;
 
-    c = getc(ib);
-    ungetc(c, ib);
-
+    c = getc(fp);
+    ungetc(c, fp);
     return c;
 }
