@@ -2,7 +2,7 @@
 /*
  * print_sw.c -- print switches
  *
- * $Id$
+ * $Id: print_sw.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/mh.h>
@@ -18,15 +18,18 @@ print_sw (char *substr, struct swit *swp, char *prefix)
 
     len = strlen(substr);
     for (; swp->sw; swp++) {
-	if (!*substr ||		/* null matches all strings */
-		(ssequal (substr, swp->sw) && len >= swp->minchars)) {
+	/* null matches all strings */
+	if (!*substr || (ssequal (substr, swp->sw) && len >= swp->minchars)) {
 	    optno = 0;
-	    if ((sp = (&swp[1])->sw)) /* next switch */
+	    /* next switch */
+	    if ((sp = (&swp[1])->sw)) {
 		if (!*substr && sp[0] == 'n' && sp[1] == 'o' &&
 			strcmp (&sp[2], swp->sw) == 0 && (
 			((&swp[1])->minchars == 0 && swp->minchars == 0) ||
 			((&swp[1])->minchars == (swp->minchars) + 2)))
 		    optno++;
+	    }
+
 	    if (swp->minchars > 0) {
 		cp = buf;
 		*cp++ = '(';
@@ -39,10 +42,10 @@ print_sw (char *substr, struct swit *swp, char *prefix)
 		*cp++ = ')';
 		while ((*cp++ = *cp1++));
 		printf ("  %s%s\n", prefix, buf);
-	    }
-	    else
+	    } else {
 		if (!swp->minchars)
 		    printf(optno ? "  %s[no]%s\n" : "  %s%s\n", prefix, swp->sw);
+	    }
 	    if (optno)
 		swp++;	/* skip -noswitch */
 	}
