@@ -2,7 +2,7 @@
 /*
  * remdir.c -- remove a directory
  *
- * $Id$
+ * $Id: remdir.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/mh.h>
@@ -14,7 +14,7 @@ remdir (char *dir)
     context_save();	/* save the context file */
     fflush(stdout);
 
-    if (rmdir(dir) == NOTOK) {
+    if (rmdir(dir) == -1) {
 	admonish (dir, "unable to remove directory");
 	return 0;
     }
