@@ -1,9 +1,9 @@
 
 /*
- * seq_list.c -- get all messages in a sequence and return them
- *            -- as a space separated list of message ranges
+ * seq_list.c -- Get all messages in a sequence and return them
+ *            -- as a space separated list of message ranges.
  *
- * $Id$
+ * $Id: seq_list.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/mh.h>
@@ -22,9 +22,7 @@ seq_list(struct msgs *mp, char *seqname)
     int i, j, seqnum;
     char *bp;
 
-    /*
-     * on first invocation, allocate initial buffer space.
-     */
+    /* On first invocation, allocate initial buffer space */
     if (!buffer) {
 	len = MAXBUFFER;
 	if (!(buffer = malloc ((size_t) len)))
@@ -32,9 +30,10 @@ seq_list(struct msgs *mp, char *seqname)
     }
 
     /*
-     * Special processing for "cur" sequence.
-     * We assume that "cur" sequence and mp->curmsg
-     * are in sync - see seq_add.c
+     * Special processing for "cur" sequence.  We assume that the
+     * "cur" sequence and mp->curmsg are in sync (see seq_add.c).
+     * This is returned, even if message doesn't exist or the
+     * folder is empty.
      */
     if (!strcmp (current, seqname)) {
 	if (mp->curmsg) {	
@@ -44,7 +43,11 @@ seq_list(struct msgs *mp, char *seqname)
 	    return (NULL);
     }
 
-    /* get the index of the sequence */
+    /* If the folder is empty, just return NULL */
+    if (mp->nummsg == 0)
+	return NULL;
+
+    /* Get the index of the sequence */
     if ((seqnum = seq_getnum (mp, seqname)) == -1)
 	return NULL;
 
@@ -83,10 +86,10 @@ seq_list(struct msgs *mp, char *seqname)
 
 	sprintf(bp, "%s", m_name(i));
 	bp += strlen(bp);
-	j = i;			/* remember beginning of message range */
+	j = i;			/* Remember beginning of message range */
 
 	/*
-	 * scan to the end of this message range
+	 * Scan to the end of this message range
 	 */
 	for (++i; i <= mp->hghmsg && does_exist(mp, i) && in_sequence(mp, seqnum, i);
 	     ++i)
