@@ -3,7 +3,7 @@
  * seq_save.c -- 1) synchronize sequences
  *            -- 2) save public sequences
  *
- * $Id$
+ * $Id: seq_save.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 #include <h/mh.h>
@@ -27,27 +27,30 @@ seq_save (struct msgs *mp)
     FILE *fp;
     sigset_t set, oset;
 
+    /* sanity check - check that context has been read */
+    if (defpath == NULL)
+	adios (NULL, "oops, context hasn't been read yet");
+
+    /* check if sequence information has changed */
     if (!(mp->msgflags & SEQMOD))
 	return;
     mp->msgflags &= ~SEQMOD;
 
-    context_read ();
     fp = NULL;
-
-    flags = mp->msgflags;
+    flags = mp->msgflags;	/* record folder flags */
 
     /*
      * If no mh-sequences file is defined, or if a mh-sequences file
      * is defined but empty (*mh_seq == '\0'), then pretend folder
      * is readonly.  This will force all sequences to be private.
      */
-    if (mh_seq && *mh_seq != '\0')
-	snprintf (seqfile, PATH_MAX, "%s/%s", mp->foldpath, mh_seq);
-    else
+    if (mh_seq == NULL || *mh_seq == '\0')
 	set_readonly (mp);
+    else
+	snprintf (seqfile, sizeof(seqfile), "%s/%s", mp->foldpath, mh_seq);
 
     for (i = 0; mp->msgattrs[i]; i++) {
-	snprintf (attr, BUFSIZ, "atr-%s-%s", mp->msgattrs[i], mp->foldpath);
+	snprintf (attr, sizeof(attr), "atr-%s-%s", mp->msgattrs[i], mp->foldpath);
 
 	/* get space separated list of sequence ranges */
 	if (!(cp = seq_list(mp, mp->msgattrs[i]))) {
@@ -56,7 +59,7 @@ seq_save (struct msgs *mp)
 	}
 
 	if (is_readonly(mp) || is_seq_private(mp, i)) {
-priv:;
+priv:
 	    /*
 	     * sequence is private
 	     */
