@@ -2,12 +2,13 @@
 /*
  * ali.c -- list nmh mail aliases
  *
- * $Id$
+ * $Id: ali.c,v 1.3 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
 #include <h/addrsbr.h>
 #include <h/aliasbr.h>
+#include <zotnet/mts/mts.h>
 
 /*
  * maximum number of names
@@ -54,22 +55,20 @@ main (int argc, char **argv)
 {
     int i, vecp = 0, inverted = 0, list = 0;
     int noalias = 0, normalize = AD_NHST;
-    char *cp, **ap, **argp, buf[100];
-    char *vec[NVEC], *arguments[MAXARGS];
+    char *cp, **ap, **argp, buf[BUFSIZ];
+    char *vec[NVEC], **arguments;
     struct aka *ak;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
+
+    /* read user profile/context */
+    context_read();
+
     mts_init (invo_name);
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -82,7 +81,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [switches] aliases ...", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches] aliases ...",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -165,7 +165,7 @@ main (int argc, char **argv)
 	}
     }
 
-    done (0);
+    return done (0);
 }
 
 void
