@@ -2,7 +2,7 @@
 /*
  * aliasbr.c -- new aliasing mechanism
  *
- * $Id$
+ * $Id: aliasbr.c,v 1.2 1999/07/16 01:16:37 danh Exp $
  */
 
 #include <h/mh.h>
@@ -250,23 +250,23 @@ akerror (int i)
 
     switch (i) {
 	case AK_NOFILE: 
-	    sprintf (buffer, "unable to read '%s'", akerrst);
+	    snprintf (buffer, sizeof(buffer), "unable to read '%s'", akerrst);
 	    break;
 
 	case AK_ERROR: 
-	    sprintf (buffer, "error in line '%s'", akerrst);
+	    snprintf (buffer, sizeof(buffer), "error in line '%s'", akerrst);
 	    break;
 
 	case AK_LIMIT: 
-	    sprintf (buffer, "out of memory while on '%s'", akerrst);
+	    snprintf (buffer, sizeof(buffer), "out of memory while on '%s'", akerrst);
 	    break;
 
 	case AK_NOGROUP: 
-	    sprintf (buffer, "no such group as '%s'", akerrst);
+	    snprintf (buffer, sizeof(buffer), "no such group as '%s'", akerrst);
 	    break;
 
 	default: 
-	    sprintf (buffer, "unknown error (%d)", i);
+	    snprintf (buffer, sizeof(buffer), "unknown error (%d)", i);
 	    break;
     }
 
@@ -450,11 +450,12 @@ getalias (char *addrs)
     if (*qp == ',')
 	*qp++ = 0;
     for (cp = qp, qp--; qp > pp; qp--)
-	if (*qp != 0)
+	if (*qp != 0) {
 	    if (isspace (*qp))
 		*qp = 0;
 	    else
 		break;
+	}
 
     return pp;
 }
@@ -582,7 +583,7 @@ seek_home (char *name)
      * The only place where there might be problems.
      * This assumes that ALL usernames are kept in lowercase.
      */
-    for (c = name, c1 = lname; *c; c++, c1++) {
+    for (c = name, c1 = lname; *c && (c1 - lname < sizeof(lname) - 1); c++, c1++) {
         if (isalpha(*c) && isupper(*c))
 	    *c1 = tolower (*c);
 	else
