@@ -2,11 +2,18 @@
 /*
  * anno.c -- annotate messages
  *
- * $Id$
+ * $Id: anno.c,v 1.2 1999/07/16 01:34:56 danh Exp $
  */
 
 #include <h/mh.h>
 
+/*
+ * We allocate space for messages (msgs array)
+ * this number of elements at a time.
+ */
+#define MAXMSGS  256
+
+
 static struct swit switches[] = {
 #define	COMPSW	0
     { "component field", 0 },
@@ -37,25 +44,32 @@ int
 main (int argc, char **argv)
 {
     int inplace = 1, datesw = 1;
-    int msgp = 0, msgnum;
+    int nummsgs, maxmsgs, msgnum;
     char *cp, *maildir, *comp = NULL;
-    char *text = NULL, *folder = NULL, buf[100], **ap;
-    char **argp, *arguments[MAXARGS], *msgs[MAXARGS];
+    char *text = NULL, *folder = NULL, buf[BUFSIZ];
+    char **argp, **arguments, **msgs;
     struct msgs *mp;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
+    /*
+     * Allocate the initial space to record message
+     * names and ranges.
+     */
+    nummsgs = 0;
+    maxmsgs = MAXMSGS;
+    if (!(msgs = (char **) malloc ((size_t) (maxmsgs * sizeof(*msgs)))))
+	adios (NULL, "unable to allocate storage");
+
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
@@ -66,7 +80,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [+folder] [msgs] [switches]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -108,9 +123,20 @@ main (int argc, char **argv)
 	    else
 		folder = path (cp + 1, *cp == '+' ? TFOLDER : TSUBCWF);
 	} else {
-	    msgs[msgp++] = cp;
+	    /*
+	     * Check if we need to allocate more space
+	     * for message name/ranges.
+	     */
+	    if (nummsgs >= maxmsgs) {
+		maxmsgs += MAXMSGS;
+		if (!(msgs = (char **) realloc (msgs,
+			(size_t) (maxmsgs * sizeof(*msgs)))))
+		    adios (NULL, "unable to reallocate msgs storage");
+	    }
+	    msgs[nummsgs++] = cp;
 	}
     }
+
 #ifdef UCI
     if (strcmp(invo_name, "fanno") == 0)	/* ugh! */
 	datesw = 0;
@@ -118,8 +144,8 @@ main (int argc, char **argv)
 
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
-    if (!msgp)
-	msgs[msgp++] = "cur";
+    if (!nummsgs)
+	msgs[nummsgs++] = "cur";
     if (!folder)
 	folder = getfolder (1);
     maildir = m_maildir (folder);
@@ -131,11 +157,12 @@ main (int argc, char **argv)
     if (!(mp = folder_read (folder)))
 	adios (NULL, "unable to read folder %s", folder);
 
-    if (mp->hghmsg == 0)
+    /* check for empty folder */
+    if (mp->nummsg == 0)
 	adios (NULL, "no messages in %s", folder);
 
     /* parse all the message ranges/sequences and set SELECTED */
-    for (msgnum = 0; msgnum < msgp; msgnum++)
+    for (msgnum = 0; msgnum < nummsgs; msgnum++)
 	if (!m_convert (mp, msgs[msgnum]))
 	    done (1);
 
@@ -151,7 +178,7 @@ main (int argc, char **argv)
     seq_save (mp);	/* synchronize message sequences */
     folder_free (mp);	/* free folder/message structure */
     context_save ();	/* save the context file         */
-    done (0);
+    return done (0);
 }
 
 
