@@ -2,7 +2,7 @@
 /*
  * annosbr.c -- prepend annotation to messages
  *
- * $Id$
+ * $Id: annosbr.c,v 1.1.1.1 1999/04/30 18:08:35 doug Exp $
  */
 
 #include <h/mh.h>
@@ -23,7 +23,8 @@ annotate (char *file, char *comp, char *text, int inplace, int datesw)
 {
     int i, fd;
 
-    if ((fd = lkopen (file, O_RDWR)) == NOTOK) {
+    /* open and lock the file to be annotated */
+    if ((fd = lkopen (file, O_RDWR, 0)) == NOTOK) {
 	switch (errno) {
 	    case ENOENT: 
 		break;
@@ -52,7 +53,7 @@ annosbr (int fd, char *file, char *comp, char *text, int inplace, int datesw)
 
     mode = fstat (fd, &st) != NOTOK ? (st.st_mode & 0777) : m_gmprot ();
 
-    strcpy (tmpfil, m_scratch (file, "annotate"));
+    strncpy (tmpfil, m_scratch (file, "annotate"), sizeof(tmpfil));
 
     if ((tmp = fopen (tmpfil, "w")) == NULL) {
 	admonish (tmpfil, "unable to create");
@@ -87,7 +88,7 @@ annosbr (int fd, char *file, char *comp, char *text, int inplace, int datesw)
 	close (tmpfd);
 	unlink (tmpfil);
     } else {
-	strcpy (buffer, m_backup (file));
+	strncpy (buffer, m_backup (file), sizeof(buffer));
 	if (rename (file, buffer) == NOTOK) {
 	    switch (errno) {
 		case ENOENT:	/* unlinked early - no annotations */
