@@ -2,12 +2,13 @@
 /*
  * ap.c -- parse addresses 822-style
  *
- * $Id$
+ * $Id: ap.c,v 1.3 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
 #include <h/addrsbr.h>
 #include <h/fmt_scan.h>
+#include <zotnet/mts/mts.h>
 
 #define	NADDRS	100
 
@@ -55,21 +56,19 @@ main (int argc, char **argv)
     int addrp = 0, normalize = AD_HOST;
     int width = 0, status = 0;
     char *cp, *form = NULL, *format = NULL, *nfs;
-    char buf[80], **ap, **argp;
-    char *arguments[MAXARGS], *addrs[NADDRS];
+    char buf[BUFSIZ], **argp;
+    char **arguments, *addrs[NADDRS];
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
+
+    /* read user profile/context */
+    context_read();
+
     mts_init (invo_name);
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -83,7 +82,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [switches] addrs ...", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches] addrs ...",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -147,7 +147,7 @@ main (int argc, char **argv)
     for (addrp = 0; addrs[addrp]; addrp++)
 	status += process (addrs[addrp], width, normalize);
 
-    done (status);
+    return done (status);
 }
 
 struct pqpair {
