@@ -2,7 +2,7 @@
 /*
  * comp.c -- compose a message
  *
- * $Id$
+ * $Id: comp.c,v 1.2 1999/07/16 01:33:58 danh Exp $
  */
 
 #include <h/mh.h>
@@ -72,7 +72,7 @@ main (int argc, char **argv)
     char *cp, *cwd, *maildir, *dfolder = NULL;
     char *ed = NULL, *file = NULL, *form = NULL;
     char *folder = NULL, *msg = NULL, buf[BUFSIZ];
-    char drft[BUFSIZ], **ap, **argp, *arguments[MAXARGS];
+    char drft[BUFSIZ], **argp, **arguments;
     struct msgs *mp = NULL;
     struct stat st;
 
@@ -80,13 +80,11 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -99,7 +97,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [+folder] [msg] [switches]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [+folder] [msg] [switches]",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -182,7 +181,10 @@ main (int argc, char **argv)
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
 
-    /* Check if we are using a draft folder */
+    /*
+     * Check if we are using a draft folder
+     * and have specified a message in it.
+     */
     if ((dfolder || context_find ("Draft-Folder")) && !folder && msg && !file) {
 	file = msg;
 	msg = NULL;
@@ -207,7 +209,8 @@ main (int argc, char **argv)
 	if (!(mp = folder_read (folder)))
 	    adios (NULL, "unable to read folder %s", folder);
 
-	if (mp->hghmsg == 0)
+	/* check for empty folder */
+	if (mp->nummsg == 0)
 	    adios (NULL, "no messages in %s", folder);
 
 	/* parse the message range/sequence/name and set SELECTED */
@@ -235,7 +238,7 @@ main (int argc, char **argv)
     }
 
 try_it_again:
-    strcpy (drft, m_draft (dfolder, file, use, &isdf));
+    strncpy (drft, m_draft (dfolder, file, use, &isdf), sizeof(drft));
 
     /*
      * Check if we have an existing draft
@@ -299,5 +302,5 @@ edit_it:
     if (nwhat)
 	done (0);
     what_now (ed, nedit, use, drft, NULL, 0, NULLMP, NULL, 0, cwd);
-    done (1);
+    return done (1);
 }
