@@ -2,7 +2,7 @@
 /*
  * conflict.c -- check for conflicts in mail system
  *
- * $Id$
+ * $Id: conflict.c,v 1.2 1999/07/16 01:33:36 danh Exp $
  */
 
 #include <h/mh.h>
@@ -60,8 +60,8 @@ int
 main (int argc, char **argv)
 {
     int	akp = 0, dp = 0;
-    char *cp, **argp = argv + 1;
-    char buf[80], *akv[50];
+    char *cp, **argp, **arguments;
+    char buf[BUFSIZ], *akv[50];
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
@@ -71,7 +71,10 @@ main (int argc, char **argv)
     /* foil search of user profile/context */
     if (context_foil (NULL) == -1)
 	done (1);
+
     mts_init (invo_name);
+    arguments = getarguments (invo_name, argc, argv, 0);
+    argp = arguments;
 
     while ((cp = *argp++)) {
 	if (*cp == '-') {
@@ -83,7 +86,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [switches] [aliasfiles ...]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches] [aliasfiles ...]",
+			invo_name);
 		    print_help (buf, switches, 0);
 		    done (1);
 		case VERSIONSW:
@@ -130,7 +134,7 @@ main (int argc, char **argv)
 #endif	/* UCI */
     maildrops ();
 
-    done (0);
+    return done (0);
 }
 
 
@@ -199,7 +203,7 @@ grp_names (void)
 		maxgroups += NGRPS;
 		if (!(grps = (char **) realloc(grps,
 			(size_t) (maxgroups * sizeof(*grps)))))
-		    adios (NULL, "unable to rellocate group name storage");
+		    adios (NULL, "unable to reallocate group name storage");
 	    }
 	    grps[numgroups++] = getcpy (gr->gr_name);
 	}
