@@ -2,7 +2,7 @@
 /*
  * dist.c -- re-distribute a message
  *
- * $Id$
+ * $Id: dist.c,v 1.2 1999/07/16 01:33:15 danh Exp $
  */
 
 #include <h/mh.h>
@@ -73,8 +73,8 @@ main (int argc, char **argv)
     int nwhat = 0, i, in, isdf = 0, out;
     char *cp, *cwd, *maildir, *msgnam, *dfolder = NULL;
     char *dmsg = NULL, *ed = NULL, *file = NULL, *folder = NULL;
-    char *form = NULL, *msg = NULL, buf[100], drft[BUFSIZ];
-    char **ap, **argp, *arguments[MAXARGS];
+    char *form = NULL, *msg = NULL, buf[BUFSIZ], drft[BUFSIZ];
+    char **argp, **arguments;
     struct msgs *mp = NULL;
     struct stat st;
 
@@ -82,13 +82,11 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -101,7 +99,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [+folder] [msg] [switches]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [+folder] [msg] [switches]",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -202,7 +201,7 @@ main (int argc, char **argv)
     }
 
 try_it_again:
-    strcpy (drft, m_draft (dfolder, dmsg, NOUSE, &isdf));
+    strncpy (drft, m_draft (dfolder, dmsg, NOUSE, &isdf), sizeof(drft));
 
     /* Check if draft already exists */
     if (stat (drft, &st) != NOTOK) {
@@ -255,9 +254,13 @@ try_it_again:
 
 	if (chdir (maildir) == NOTOK)
 	    adios (maildir, "unable to change directory to");
+
+	/* read folder and create message structure */
 	if (!(mp = folder_read (folder)))
 	    adios (NULL, "unable to read folder %s", folder);
-	if (mp->hghmsg == 0)
+
+	/* check for empty folder */
+	if (mp->nummsg == 0)
 	    adios (NULL, "no messages in %s", folder);
 
 	/* parse the message range/sequence/name and set SELECTED */
@@ -284,5 +287,5 @@ try_it_again:
 	done (0);
     what_now (ed, nedit, NOUSE, drft, msgnam, 1, mp,
 	anot ? "Resent" : NULL, inplace, cwd);
-    done (1);
+    return done (1);
 }
