@@ -2,7 +2,7 @@
 /*
  * dp.c -- parse dates 822-style
  *
- * $Id$
+ * $Id: dp.c,v 1.2 1999/07/16 01:32:44 danh Exp $
  */
 
 #include <h/mh.h>
@@ -50,20 +50,18 @@ main (int argc, char **argv)
 {
     int datep = 0, width = 0, status = 0;
     char *cp, *form = NULL, *format = NULL, *nfs;
-    char buf[80], **ap, **argp, *arguments[MAXARGS];
+    char buf[BUFSIZ], **argp, **arguments;
     char *dates[NDATES];
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -76,7 +74,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [switches] dates ...", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches] dates ...",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -133,7 +132,7 @@ main (int argc, char **argv)
 	status += process (dates[datep], width);
 
     context_save ();	/* save the context file */
-    done (status);
+    return done (status);
 }
 
 
