@@ -1,8 +1,8 @@
 
 /*
- * dropsbr.c -- write to a mailbox
+ * dropsbr.c -- create/read/manipulate mail drops
  *
- * $Id$
+ * $Id: dropsbr.c,v 1.1.1.1 1999/04/30 18:08:35 doug Exp $
  */
 
 #include <h/nmh.h>
@@ -36,61 +36,33 @@ extern int errno;
 /*
  * static prototypes
  */
-static int mbx_create (char *, uid_t, gid_t, mode_t);
-static int mbx_chk (int);
+static int mbx_chk_mbox (int);
+static int mbx_chk_mmdf (int);
 static int map_open (char *, int *, int);
 
 
+/*
+ * Main entry point to open/create and lock
+ * a file or maildrop.
+ */
+
 int
 mbx_open (char *file, int mbx_style, uid_t uid, gid_t gid, mode_t mode)
 {
-    int clear, fd;
-
-    if ((fd = mbx_Xopen (file, uid, gid, mode, &clear)) == NOTOK)
-	return fd;
-
-    if (!clear)
-	switch (mbx_style) {
-	    case MMDF_FORMAT: 
-	    default: 
-		if (mbx_chk (fd) == NOTOK) {
-		    close (fd);
-		    return NOTOK;
-		}
-		break;
-
-	    case MBOX_FORMAT: 
-		if (lseek (fd, (off_t) 0, SEEK_END) == (off_t) NOTOK) {
-		    close (fd);
-		    return NOTOK;
-		}
-		break;
-	}
-
-    return fd;
-}
-
-
-int
-mbx_Xopen (char *file, uid_t uid, gid_t gid, mode_t mode, int *clear)
-{
-    register int j;
-    int	count, fd;
+    int j, count, fd;
     struct stat st;
 
-    for (*clear = 0, count = 4, j = 0; count > 0; count--) {
-	if ((fd = lkopen (file, 6)) == NOTOK) {
-	    switch (errno) {
-		case ENOENT: 
-		    if (mbx_create (file, uid, gid, mode) == NOTOK)
-			return NOTOK;
-		    (*clear)++;
-		    break;
+    j = 0;
 
+    /* attempt to open and lock file */
+    for (count = 4; count > 0; count--) {
+	if ((fd = lkopen (file, O_RDWR | O_CREAT | O_NONBLOCK, mode)) == NOTOK) {
+	    switch (errno) {
 #if defined(FCNTL_LOCKING) || defined(LOCKF_LOCKING)
 		case EACCES:
 		case EAGAIN:
 #endif
+
 #ifdef FLOCK_LOCKING
 		case EWOULDBLOCK:
 #endif
@@ -100,45 +72,93 @@ mbx_Xopen (char *file, uid_t uid, gid_t gid, mode_t mode, int *clear)
 		    break;
 
 		default: 
+		    /* just return error */
 		    return NOTOK;
 	    }
-	} else {
-	    *clear = fstat (fd, &st) != NOTOK && st.st_size == (off_t) 0;
-	    break;
 	}
+
+	/* good file descriptor */
+	break;
     }
 
     errno = j;
+
+    /*
+     * Return if we still failed after 4 attempts,
+     * or we just want to skip the sanity checks.
+     */
+    if (fd == NOTOK || mbx_style == OTHER_FORMAT)
+	return fd;
+
+    /*
+     * Do sanity checks on maildrop.
+     */
+    if (fstat (fd, &st) == NOTOK) {
+	/*
+	 * The stat failed.  So we make sure file
+	 * has right ownership/modes
+	 */
+	chown (file, uid, gid);
+	chmod (file, mode);
+    } else if (st.st_size > (off_t) 0) {
+	int status;
+
+	/* check the maildrop */
+	switch (mbx_style) {
+	    case MMDF_FORMAT: 
+	    default: 
+		status = mbx_chk_mmdf (fd);
+		break;
+
+	    case MBOX_FORMAT: 
+		status = mbx_chk_mbox (fd);
+		break;
+	}
+
+	/* if error, attempt to close it */
+	if (status == NOTOK) {
+	    close (fd);
+	    return NOTOK;
+	}
+    }
+
     return fd;
 }
 
 
-static int
-mbx_create (char *file, uid_t uid, gid_t gid, mode_t mode)
-{
-    int fd;
+/*
+ * Check/prepare MBOX style maildrop for appending.
+ */
 
-    if ((fd = creat (file, 0600)) == NOTOK)
+static int
+mbx_chk_mbox (int fd)
+{
+    /* just seek to the end */
+    if (lseek (fd, (off_t) 0, SEEK_END) == (off_t) NOTOK)
 	return NOTOK;
 
-    close (fd);
-    chown (file, uid, gid);
-    chmod (file, mode);
     return OK;
 }
 
 
+/*
+ * Check/prepare MMDF style maildrop for appending.
+ */
+
 static int
-mbx_chk (int fd)
+mbx_chk_mmdf (int fd)
 {
     size_t count;
     char ldelim[BUFSIZ];
 
     count = strlen (mmdlm2);
 
-    if (lseek (fd, (off_t) (-count), SEEK_END) == (off_t) NOTOK
-	    || read (fd, ldelim, count) != count)
+    /* casting -count to off_t, seem to break FreeBSD 2.2.6 */
+    if (lseek (fd, (long) (-count), SEEK_END) == (off_t) NOTOK)
 	return NOTOK;
+    if (read (fd, ldelim, count) != count)
+	return NOTOK;
+
     ldelim[count] = 0;
 
     if (strcmp (ldelim, mmdlm2)
@@ -170,7 +190,7 @@ mbx_read (FILE *fp, long pos, struct drop **drops, int noisy)
     ld2 = (long) strlen (mmdlm2);
 
     fseek (fp, pos, SEEK_SET);
-    for (ep = (dp = pp) + len - 1; fgets (buffer, sizeof buffer, fp);) {
+    for (ep = (dp = pp) + len - 1; fgets (buffer, sizeof(buffer), fp);) {
 	size = 0;
 	if (strcmp (buffer, mmdlm1) == 0)
 	    pos += ld1, dp->d_start = (long) pos;
@@ -181,7 +201,7 @@ mbx_read (FILE *fp, long pos, struct drop **drops, int noisy)
 		    size++;
 	}
 
-	while (fgets (buffer, sizeof buffer, fp) != NULL)
+	while (fgets (buffer, sizeof(buffer), fp) != NULL)
 	    if (strcmp (buffer, mmdlm2) == 0)
 		break;
 	    else {
@@ -203,7 +223,7 @@ mbx_read (FILE *fp, long pos, struct drop **drops, int noisy)
 	    register int    curlen = dp - pp;
 
 	    cp = (struct drop *) realloc ((char *) pp,
-		                    (size_t) (len += MAXFOLDER) * sizeof *pp);
+		                    (size_t) (len += MAXFOLDER) * sizeof(*pp));
 	    if (cp == NULL) {
 		if (noisy)
 		    admonish (NULL, "unable to allocate drop storage");
@@ -240,7 +260,7 @@ mbx_write(char *mailbox, int md, FILE *fp, int id, long last,
     size = 0;
 
     fseek (fp, pos, SEEK_SET);
-    while (fgets (buffer, sizeof buffer, fp) && (pos < stop)) {
+    while (fgets (buffer, sizeof(buffer), fp) && (pos < stop)) {
 	i = strlen (buffer);
 	for (j = 0; (j = stringdex (mmdlm1, buffer)) >= 0; buffer[j]++)
 	    continue;
@@ -266,6 +286,10 @@ mbx_write(char *mailbox, int md, FILE *fp, int id, long last,
 }
 
 
+/*
+ * Append message to end of file or maildrop.
+ */
+
 int
 mbx_copy (char *mailbox, int mbx_style, int md, int fd,
           int mapping, char *text, int noisy)
@@ -355,12 +379,13 @@ mbx_copy (char *mailbox, int mbx_style, int md, int fd,
 			char tmpbuffer[BUFSIZ];
 			char *tp, *ep, *fp;
 
-			strcpy(tmpbuffer, buffer);
+			strncpy(tmpbuffer, buffer, sizeof(tmpbuffer));
 			ep = tmpbuffer + 13;
 			if (!(fp = strchr(ep + 1, ' ')))
 			    fp = strchr(ep + 1, '\n');
 			tp = dctime(dlocaltimenow());
-			sprintf (buffer, "From %.*s  %s", fp - ep, ep, tp);
+			snprintf (buffer, sizeof(buffer), "From %.*s  %s",
+				fp - ep, ep, tp);
 		    } else if (!strncmp (buffer, "X-Envelope-From:", 16)) {
 			/*
 			 * Change the "X-Envelope-From:" field
@@ -369,9 +394,9 @@ mbx_copy (char *mailbox, int mbx_style, int md, int fd,
 			char tmpbuffer[BUFSIZ];
 			char *ep;
 
-			strcpy(tmpbuffer, buffer);
+			strncpy(tmpbuffer, buffer, sizeof(tmpbuffer));
 			ep = tmpbuffer + 17;
-			sprintf (buffer, "From %s", ep);
+			snprintf (buffer, sizeof(buffer), "From %s", ep);
 		    } else if (strncmp (buffer, "From ", 5)) {
 			/*
 			 * If there is already a "From " line,
@@ -380,10 +405,10 @@ mbx_copy (char *mailbox, int mbx_style, int md, int fd,
 			char tmpbuffer[BUFSIZ];
 			char *tp, *ep;
 
-			strcpy(tmpbuffer, buffer);
+			strncpy(tmpbuffer, buffer, sizeof(tmpbuffer));
 			ep = "nobody@nowhere";
 			tp = dctime(dlocaltimenow());
-			sprintf (buffer, "From %s  %s", ep, tp);
+			snprintf (buffer, sizeof(buffer), "From %s  %s", ep, tp);
 			strcat (buffer, tmpbuffer);
 		    }
 		}
@@ -447,6 +472,10 @@ mbx_size (int md, off_t start, off_t stop)
 }
 
 
+/*
+ * Close and unlock file/maildrop.
+ */
+
 int
 mbx_close (char *mailbox, int md)
 {
@@ -469,9 +498,10 @@ map_name (char *file)
     if ((dp = strchr(cp = r1bindex (file, '/'), '.')) == NULL)
 	dp = cp + strlen (cp);
     if (cp == file)
-	sprintf (buffer, ".%.*s%s", dp - cp, cp, ".map");
+	snprintf (buffer, sizeof(buffer), ".%.*s%s", dp - cp, cp, ".map");
     else
-	sprintf (buffer, "%.*s.%.*s%s", cp - file, file, dp - cp, cp, ".map");
+	snprintf (buffer, sizeof(buffer), "%.*s.%.*s%s",
+		cp - file, file, dp - cp, cp, ".map");
 
     return buffer;
 }
@@ -493,16 +523,16 @@ map_read (char *file, long pos, struct drop **drops, int noisy)
     }
 
     msgp = mp->d_id;
-    dp = (struct drop *) calloc ((size_t) (msgp + 1), sizeof *dp);
+    dp = (struct drop *) calloc ((size_t) (msgp + 1), sizeof(*dp));
     if (dp == NULL) {
 	close (md);
 	return 0;
     }
 
-    memcpy((char *) dp, (char *) mp, sizeof *dp);
+    memcpy((char *) dp, (char *) mp, sizeof(*dp));
 
     lseek (md, (off_t) sizeof(*mp), SEEK_SET);
-    if ((i = read (md, (char *) (dp + 1), msgp * sizeof *dp)) < sizeof *dp) {
+    if ((i = read (md, (char *) (dp + 1), msgp * sizeof(*dp))) < sizeof(*dp)) {
 	i = 0;
 	free ((char *) dp);
     } else {
@@ -522,7 +552,7 @@ map_read (char *file, long pos, struct drop **drops, int noisy)
 
     close (md);
 
-    return (i / sizeof *dp);
+    return (i / sizeof(*dp));
 }
 
 
@@ -573,7 +603,7 @@ map_write (char *mailbox, int md, int id, long last, off_t start,
 		    if (dp->d_start == start)
 			dp->d_id = id;
 		    lseek (fd, (off_t) (++d1.d_id * sizeof(*dp)), SEEK_SET);
-		    if (write (fd, (char *) dp, sizeof *dp) != sizeof *dp) {
+		    if (write (fd, (char *) dp, sizeof(*dp)) != sizeof(*dp)) {
 			if (noisy)
 			    admonish (file, "write error");
 			mbx_close (file, fd);
@@ -594,7 +624,7 @@ map_write (char *mailbox, int md, int id, long last, off_t start,
 	dp->d_start = start;
 	dp->d_stop = stop;
 	lseek (fd, (off_t) (++d1.d_id * sizeof(*dp)), SEEK_SET);
-	if (write (fd, (char *) dp, sizeof *dp) != sizeof *dp) {
+	if (write (fd, (char *) dp, sizeof(*dp)) != sizeof(*dp)) {
 	    if (noisy)
 		admonish (file, "write error");
 	    mbx_close (file, fd);
@@ -608,7 +638,7 @@ map_write (char *mailbox, int md, int id, long last, off_t start,
     dp->d_stop = lseek (md, (off_t) 0, SEEK_CUR);
 
     lseek (fd, (off_t) 0, SEEK_SET);
-    if (write (fd, (char *) dp, sizeof *dp) != sizeof *dp) {
+    if (write (fd, (char *) dp, sizeof(*dp)) != sizeof(*dp)) {
 	if (noisy)
 	    admonish (file, "write error");
 	mbx_close (file, fd);
@@ -628,7 +658,7 @@ map_open (char *file, int *clear, int md)
     struct stat st;
 
     mode = fstat (md, &st) != NOTOK ? (mode_t) (st.st_mode & 0777) : m_gmprot ();
-    return mbx_Xopen (file, st.st_uid, st.st_gid, mode, clear);
+    return mbx_open (file, OTHER_FORMAT, st.st_uid, st.st_gid, mode);
 }
 
 
@@ -639,7 +669,7 @@ map_chk (char *file, int fd, struct drop *dp, long pos, int noisy)
     struct drop d, tmpd;
     register struct drop *dl;
 
-    if (read (fd, (char *) &tmpd, sizeof *dp) != sizeof *dp) {
+    if (read (fd, (char *) &tmpd, sizeof(*dp)) != sizeof(*dp)) {
 #ifdef notdef
 	admonish (NULL, "%s: missing or partial index", file);
 #endif /* notdef */
@@ -678,7 +708,7 @@ map_chk (char *file, int fd, struct drop *dp, long pos, int noisy)
     dl = &d;
     count = (long) strlen (mmdlm2);
     lseek (fd, (off_t) (dp->d_id * sizeof(*dp)), SEEK_SET);
-    if (read (fd, (char *) dl, sizeof *dl) != sizeof *dl
+    if (read (fd, (char *) dl, sizeof(*dl)) != sizeof(*dl)
 	    || (ntohl(dl->d_stop) != dp->d_stop
 		&& ntohl(dl->d_stop) + count != dp->d_stop)) {
 	if (noisy)
