@@ -13,7 +13,7 @@
  * makes no warranty about the software, its performance or its conformity to
  * any specification.
  *
- *  $Id$
+ *  $Id: flist.c,v 1.3 1999/07/16 01:32:28 danh Exp $
  */
 
 #include <h/mh.h>
@@ -21,6 +21,13 @@
 #define FALSE   0
 #define TRUE    1
 
+/*
+ * We allocate space to record the names of folders
+ * (foldersToDo array), this number of elements at a time.
+ */
+#define MAXFOLDERS  100
+
+
 static struct swit switches[] = {
 #define SEQSW           0
     { "sequence name", 0 },
@@ -56,12 +63,12 @@ static struct swit switches[] = {
 };
 
 struct Folder {
-    char *name;		/* name of folder */
+    char *name;			/* name of folder */
     int priority;
-    int private;	/* is the sequence, public or private   */
-    int nMsgs;		/* number of messages in folder         */
-    int nSeq;		/* number of messages in given sequence */
-    int error;		/* error == 1 for unreadable folder     */
+    int error;			/* error == 1 for unreadable folder     */
+    int nMsgs;			/* number of messages in folder         */
+    int nSeq[NUMATTRS];		/* number of messages in each sequence  */
+    int private[NUMATTRS];	/* is given sequence, public or private */
 };
 
 static struct Folder *orders = NULL;
@@ -71,8 +78,14 @@ static struct Folder *folders = NULL;
 static int nFolders = 0;
 static int nFoldersAlloced = 0;
 
+/* info on folders to search */
 static char **foldersToDo;
-static int nFoldersToDo;
+static int numfolders;
+static int maxfolders;
+
+/* info on sequences to search for */
+static char *sequencesToDo[NUMATTRS];
+static int numsequences;
 
 static int all        = FALSE;  /* scan all folders in top level?           */
 static int alphaOrder = FALSE;	/* want alphabetical order only             */
@@ -81,9 +94,8 @@ static int showzero   = TRUE;	/* show folders even if no messages in seq? */
 static int Total      = TRUE;	/* display info on number of messages in    *
 				 * sequence found, and total num messages   */
 
-static char *sequence;		/* name of sequence we are searching for    */
-static char curfolder[BUFSIZ];	/* name of the current folder               */
-static char *nmhdir;		/* base nmh mail directory                  */
+static char curfolder[BUFSIZ];	/* name of the current folder */
+static char *nmhdir;		/* base nmh mail directory    */
 
 /*
  * Type for a compare function for qsort.  This keeps
@@ -111,16 +123,18 @@ static void do_readonly_folders(void);
 int
 main(int argc, char **argv)
 {
-    char *cp, **ap;
-    char **argp, **lastArg;
-    char *arguments[MAXARGS];
-    char buf[100];
+    char *cp, **argp;
+    char **arguments;
+    char buf[BUFSIZ];
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex(argv[0], '/');
 
+    /* read user profile/context */
+    context_read();
+
     /*
      * If program was invoked with name ending
      * in `s', then add switch `-all'.
@@ -128,18 +142,19 @@ main(int argc, char **argv)
     if (argv[0][strlen (argv[0]) - 1] == 's')
 	all = TRUE;
 
-    if ((cp = context_find(invo_name)) != NULL) {
-	ap = brkstring(cp = getcpy(cp), " ", "\n");
-	ap = copyip(ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    lastArg = copyip(argv + 1, ap);
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
-    argc = lastArg - argp;
-    foldersToDo = (char **) malloc(argc * sizeof(char *));
-    nFoldersToDo = 0;
 
+    /* allocate the initial space to record the folder names */
+    numfolders = 0;
+    maxfolders = MAXFOLDERS;
+    if (!(foldersToDo = (char **) malloc ((size_t) (maxfolders * sizeof(*foldersToDo)))))
+	adios (NULL, "unable to allocate folder storage");
+
+    /* no sequences yet */
+    numsequences = 0;
+
+    /* parse arguments */
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch(++cp, switches)) {
@@ -150,7 +165,8 @@ main(int argc, char **argv)
 		adios(NULL, "-%s unknown", cp);
 
 	    case HELPSW:
-		sprintf(buf, "%s [+folder1 [+folder2 ...]][switches]", invo_name);
+		snprintf(buf, sizeof(buf), "%s [+folder1 [+folder2 ...]][switches]",
+			invo_name);
 		print_help(buf, switches, 1);
 		done(1);
 	    case VERSIONSW:
@@ -160,7 +176,11 @@ main(int argc, char **argv)
 	    case SEQSW:
 		if (!(cp = *argp++) || *cp == '-')
 		    adios (NULL, "missing argument to %s", argp[-2]);
-		sequence = cp;
+
+		/* check if too many sequences specified */
+		if (numsequences >= NUMATTRS)
+		    adios (NULL, "too many sequences (more than %d) specified", NUMATTRS);
+		sequencesToDo[numsequences++] = cp;
 		break;
 
 	    case ALLSW:
@@ -202,30 +222,63 @@ main(int argc, char **argv)
 		break;
 	    }
 	} else {
-	    if (*cp == '+')
-		++cp;
-	    foldersToDo[nFoldersToDo++] = cp;
+	    /*
+	     * Check if we need to allocate more space
+	     * for folder names.
+	     */
+	    if (numfolders >= maxfolders) {
+		maxfolders += MAXFOLDERS;
+		if (!(foldersToDo = (char **) realloc (foldersToDo,
+			(size_t) (maxfolders * sizeof(*foldersToDo)))))
+		    adios (NULL, "unable to reallocate folder name storage");
+	    }
+	    if (*cp == '+' || *cp == '@') {
+		foldersToDo[numfolders++] =
+		    path (cp + 1, *cp == '+' ? TFOLDER : TSUBCWF);
+	    } else
+		foldersToDo[numfolders++] = cp;
 	}
     }
 
     if (!context_find ("path"))
         free (path ("./", TFOLDER));
 
-    strcpy(curfolder, getfolder(1));	/* get current folder     */
-    nmhdir = m_maildir ("");		/* get nmh base directory */
+    /* get current folder */
+    strncpy (curfolder, getfolder(1), sizeof(curfolder));
+
+    /* get nmh base directory */
+    nmhdir = m_maildir ("");
 
     /*
-     * If we didn't specify a sequence, we search
-     * for the "Unseen-Sequence" sequence (if defined).
+     * If we didn't specify any sequences, we search
+     * for the "Unseen-Sequence" profile entry and use
+     * all the sequences defined there.  We check to
+     * make sure that the Unseen-Sequence entry doesn't
+     * contain more than NUMATTRS sequences.
      */
-    if (!sequence && !(sequence = context_find(usequence)))
-	adios (NULL, "no default sequence in which to search");
+    if (numsequences == 0) {
+	if ((cp = context_find(usequence)) && *cp) {
+	    char **ap, *dp;
+
+	    dp = getcpy(cp);
+	    ap = brkstring (dp, " ", "\n");
+	    for (; ap && *ap; ap++) {
+		if (numsequences >= NUMATTRS)
+		    adios (NULL, "too many sequences (more than %d) in %s profile entry",
+			   NUMATTRS, usequence);
+		else
+		    sequencesToDo[numsequences++] = *ap;
+	    }
+	} else {
+	    adios (NULL, "no sequence specified or %s profile entry found", usequence);
+	}
+    }
 
     GetFolderOrder();
     ScanFolders();
     qsort(folders, nFolders, sizeof(struct Folder), (qsort_comp) CompareFolders);
     PrintFolders();
-    done (0);
+    return done (0);
 }
 
 /*
@@ -276,9 +329,9 @@ ScanFolders(void)
     if (chdir (nmhdir) == NOTOK)
 	adios (nmhdir, "unable to change directory to");
 
-    if (nFoldersToDo > 0) {
+    if (numfolders > 0) {
 	/* Update context */
-	strcpy (curfolder, foldersToDo[nFoldersToDo - 1]);
+	strncpy (curfolder, foldersToDo[numfolders - 1], sizeof(curfolder));
 	context_replace (pfolder, curfolder);/* update current folder */
 	context_save ();                     /* save the context file */
 
@@ -287,7 +340,7 @@ ScanFolders(void)
 	 * then also scan the 1st level subfolders under
 	 * each given folder.
 	 */
-	for (i = 0; i < nFoldersToDo; ++i)
+	for (i = 0; i < numfolders; ++i)
 	    BuildFolderList(foldersToDo[i], all ? 1 : 0);
     } else {
 	if (all) {
@@ -379,10 +432,10 @@ BuildFolderListRecurse(char *dirName, struct stat *s, int searchdepth)
 	}
 	if (dp->d_name[0] == '.')
 	    continue;
-	strcpy(name, base);
+	strncpy (name, base, sizeof(name) - 2);
 	if (*base)
 	    strcat(name, "/");
-	strcat(name, dp->d_name);
+	strncat(name, dp->d_name, sizeof(name) - strlen(name) - 1);
 	if ((stat(name, &st) != -1) && S_ISDIR(st.st_mode)) {
 	    /*
 	     * Check if this was really a symbolic link pointing
@@ -400,26 +453,21 @@ BuildFolderListRecurse(char *dirName, struct stat *s, int searchdepth)
 }
 
 /*
- * Add this folder to our list, counting the
- * number of messages in given sequence and
- * total number of messages.
+ * Add this folder to our list, counting the total number of
+ * messages and the number of messages in each sequence.
  */
 
 int
 AddFolder(char *name, int force)
 {
-    int msgnum, seqnum, nSeq;
+    int i, msgnum, nonzero;
+    int seqnum[NUMATTRS], nSeq[NUMATTRS];
     struct Folder *f;
     struct msgs *mp;
 
-    /*
-     * read folder and create message structure
-     */
+    /* Read folder and create message structure */
     if (!(mp = folder_read (name))) {
-	/*
-	 * Oops, error occurred.  Just
-	 * record it and continue.
-	 */
+	/* Oops, error occurred.  Record it and continue. */
 	AllocFolders(&folders, &nFoldersAlloced, nFolders + 1);
 	f = &folders[nFolders++];
 	f->name = getcpy(name);
@@ -428,26 +476,46 @@ AddFolder(char *name, int force)
 	return 0;
     }
 
-    nSeq = 0;
+    for (i = 0; i < numsequences; i++) {
+	/* Convert sequences to their sequence numbers */
+	if (sequencesToDo[i])
+	    seqnum[i] = seq_getnum(mp, sequencesToDo[i]);
+	else
+	    seqnum[i] = -1;
 
-    /* Count the number of messages in the sequence */
-    if (sequence && ((seqnum = seq_getnum(mp, sequence)) != -1)) {
-	for (msgnum = mp->lowmsg; msgnum <= mp->hghmsg; msgnum++) {
-	    if (in_sequence(mp, seqnum, msgnum))
-		nSeq++;
+	/* Now count messages in this sequence */
+	nSeq[i] = 0;
+	if (mp->nummsg > 0 && seqnum[i] != -1) {
+	    for (msgnum = mp->lowmsg; msgnum <= mp->hghmsg; msgnum++) {
+		if (in_sequence(mp, seqnum[i], msgnum))
+		    nSeq[i]++;
+	    }
 	}
     }
 
-    /* Save folder/message information in Folder struct */
-    if (nSeq > 0 || force) {
+    /* Check if any of the sequence checks were nonzero */
+    nonzero = 0;
+    for (i = 0; i < numsequences; i++) {
+	if (nSeq[i] > 0) {
+	    nonzero = 1;
+	    break;
+	}
+    }
+
+    if (nonzero || force) {
+	/* save general folder information */
 	AllocFolders(&folders, &nFoldersAlloced, nFolders + 1);
 	f = &folders[nFolders++];
 	f->name = getcpy(name);
-	f->private = (seqnum != -1) ? is_seq_private(mp, seqnum) : 0;
 	f->nMsgs = mp->nummsg;
-	f->nSeq = nSeq;
 	f->error = 0;
 	f->priority = AssignPriority(f->name);
+
+	/* record the sequence information */
+	for (i = 0; i < numsequences; i++) {
+	    f->nSeq[i] = nSeq[i];
+	    f->private[i] = (seqnum[i] != -1) ? is_seq_private(mp, seqnum[i]) : 0;
+	}
     }
 
     folder_free (mp);	/* free folder/message structure */
@@ -462,12 +530,12 @@ void
 PrintFolders(void)
 {
     char tmpname[BUFSIZ];
-    int i, len, maxlen = 0;
+    int i, j, len, has_private = 0;
+    int maxfolderlen = 0, maxseqlen = 0;
     int maxnum = 0, maxseq = 0;
-    int has_private = 0;
 
     if (!Total) {
-	for (i = 0; i < nFolders; ++i)
+	for (i = 0; i < nFolders; i++)
 	    printf("%s\n", folders[i].name);
 	return;
     }
@@ -478,55 +546,74 @@ PrintFolders(void)
     for (i = 0; i < nFolders; ++i) {
 	/* find the length of longest folder name */
 	len = strlen(folders[i].name);
-	if (len > maxlen)
-	    maxlen = len;
+	if (len > maxfolderlen)
+	    maxfolderlen = len;
 
 	/* If folder had error, skip the rest */
 	if (folders[i].error)
 	    continue;
 
-	/* find the maximum number of messages in sequence */
-	if (folders[i].nSeq > maxseq)
-	    maxseq = folders[i].nSeq;
-
 	/* find the maximum total messages */
 	if (folders[i].nMsgs > maxnum)
 	    maxnum = folders[i].nMsgs;
 
-	/* check if this sequence is private in any of the folders */
-	if (folders[i].private)
-	    has_private = 1;
+	for (j = 0; j < numsequences; j++) {
+	    /* find maximum width of sequence name */
+	    len = strlen (sequencesToDo[j]);
+	    if ((folders[i].nSeq[j] > 0 || showzero) && (len > maxseqlen))
+		maxseqlen = len;
+
+	    /* find the maximum number of messages in sequence */
+	    if (folders[i].nSeq[j] > maxseq)
+		maxseq = folders[i].nSeq[j];
+
+	    /* check if this sequence is private in any of the folders */
+	    if (folders[i].private[j])
+		has_private = 1;
+	}
     }
 
-    for (i = 0; i < nFolders; ++i) {
-	/* Add `+' to end of name of current folder */
-	if (strcmp(curfolder, folders[i].name))
-	    sprintf(tmpname, "%s", folders[i].name);
-	else
-	    sprintf(tmpname, "%s+", folders[i].name);
+    /* Now print all the folder/sequence information */
+    for (i = 0; i < nFolders; i++) {
+	for (j = 0; j < numsequences; j++) {
+	    if (folders[i].nSeq[j] > 0 || showzero) {
+		/* Add `+' to end of name of current folder */
+		if (strcmp(curfolder, folders[i].name))
+		    snprintf(tmpname, sizeof(tmpname), "%s", folders[i].name);
+		else
+		    snprintf(tmpname, sizeof(tmpname), "%s+", folders[i].name);
 
-	if (folders[i].error) {
-	    printf("%-*s is unreadable\n", maxlen+1, tmpname);
-	    continue;
+		if (folders[i].error) {
+		    printf("%-*s is unreadable\n", maxfolderlen+1, tmpname);
+		    continue;
+		}
+
+		printf("%-*s has %*d in sequence %-*s%s; out of %*d\n",
+		       maxfolderlen+1, tmpname,
+		       num_digits(maxseq), folders[i].nSeq[j],
+		       maxseqlen, sequencesToDo[j],
+		       !has_private ? "" : folders[i].private[j] ? " (private)" : "          ",
+		       num_digits(maxnum), folders[i].nMsgs);
+	    }
 	}
-
-	printf("%-*s has %*d in sequence %s%s; out of %*d\n",
-		maxlen+1, tmpname,
-		num_digits(maxseq), folders[i].nSeq,
-		sequence,
-		!has_private ? "" : folders[i].private ? " (private)" : "          ",
-		num_digits(maxnum), folders[i].nMsgs);
     }
 }
 
 /*
- * Calculate the number of digits in positive integer
+ * Calculate the number of digits in a nonnegative integer
  */
 static int
 num_digits (int n)
 {
     int ndigits = 0;
 
+    /* Sanity check */
+    if (n < 0)
+	adios (NULL, "oops, num_digits called with negative value");
+
+    if (n == 0)
+	return 1;
+
     while (n) {
 	n /= 10;
 	ndigits++;
@@ -559,10 +646,10 @@ AllocFolders(struct Folder **f, int *nfa, int n)
 	return;
     if (*f == NULL) {
 	*nfa = 10;
-	*f = (struct Folder *) malloc(*nfa * (sizeof(struct Folder)));
+	*f = (struct Folder *) malloc (*nfa * (sizeof(struct Folder)));
     } else {
 	*nfa *= 2;
-	*f = (struct Folder *) realloc(*f, *nfa * (sizeof(struct Folder)));
+	*f = (struct Folder *) realloc (*f, *nfa * (sizeof(struct Folder)));
     }
 }
 
@@ -610,10 +697,13 @@ do_readonly_folders (void)
     char atrcur[BUFSIZ];
     register struct node *np;
 
-    sprintf (atrcur, "atr-%s-", current);
+    /* sanity check - check that context has been read */
+    if (defpath == NULL)
+	adios (NULL, "oops, context hasn't been read yet");
+
+    snprintf (atrcur, sizeof(atrcur), "atr-%s-", current);
     atrlen = strlen (atrcur);
 
-    context_read ();
     for (np = m_defs; np; np = np->n_next)
         if (ssequal (atrcur, np->n_name)
                 && !ssequal (nmhdir, np->n_name + atrlen))
