@@ -2,7 +2,7 @@
 /*
  * fmtdump.c -- compile format file and dump out instructions
  *
- * $Id$
+ * $Id: fmtdump.c,v 1.3 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
@@ -24,7 +24,7 @@ static struct swit switches[] = {
 
 /* for assignlabel */
 static struct format *lvec[128];
-static lused = 0;
+static int lused = 0;
 
 /*
  * static prototypes
@@ -44,21 +44,18 @@ main (int argc, char **argv)
 {
     int ncomps;
     char *cp, *form = NULL, *format = NULL;
-    char buf[100], **ap, **argp;
-    char *nfs, *arguments[MAXARGS];
+    char buf[BUFSIZ], *nfs, **argp, **arguments;
     struct format *fmt;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -71,7 +68,7 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [switches]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches]", invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -104,7 +101,7 @@ main (int argc, char **argv)
     ncomps = fmt_compile(nfs, &fmt);
 
     fmt_dump(fmt);
-    done(0);
+    return done(0);
 }
 
 static void
