@@ -4,7 +4,7 @@
  *             -- push/pop a folder onto/from the folder stack
  *             -- list the folder stack
  *
- * $Id$
+ * $Id: folder.c,v 1.2 1999/07/16 01:31:56 danh Exp $
  */
 
 #include <h/mh.h>
@@ -132,7 +132,7 @@ main (int argc, char **argv)
     int printsw = 0, listsw = 0;
     int pushsw = 0, popsw = 0;
     char *cp, *dp, *msg = NULL, *argfolder = NULL;
-    char **ap, **argp, buf[100], *arguments[MAXARGS];
+    char **ap, **argp, buf[BUFSIZ], **arguments;
     struct stat st;
 
 #ifdef LOCALE
@@ -140,6 +140,9 @@ main (int argc, char **argv)
 #endif
     invo_name = r1bindex (argv[0], '/');
 
+    /* read user profile/context */
+    context_read();
+
     /*
      * If program was invoked with name ending
      * in `s', then add switch `-all'.
@@ -147,13 +150,7 @@ main (int argc, char **argv)
     if (argv[0][strlen (argv[0]) - 1] == 's')
 	all = 1;
 
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -166,7 +163,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [+folder] [msg] [switches]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [+folder] [msg] [switches]",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -372,10 +370,10 @@ main (int argc, char **argv)
 	    if (msg)
 		admonish (NULL, "no folder given for message %s", msg);
 	    readonly_folders (); /* do any readonly folders */
-	    strcpy (folder, (cp = context_find (pfolder)) ? cp : "");
+	    strncpy (folder, (cp = context_find (pfolder)) ? cp : "", sizeof(folder));
 	    dodir (".");
 	} else {
-	    strcpy (folder, argfolder);
+	    strncpy (folder, argfolder, sizeof(folder));
 	    if (get_folder_info (argfolder, msg)) {
 		context_replace (pfolder, argfolder);/* update current folder */
 		context_save ();		     /* save the context file */
@@ -388,13 +386,13 @@ main (int argc, char **argv)
 		dodir (folder);
 	}
     } else {
-	strcpy (folder, argfolder ? argfolder : getfolder (1));
+	strncpy (folder, argfolder ? argfolder : getfolder (1), sizeof(folder));
 
 	/*
 	 * Check if folder exists.  If not, then see if
 	 * we should create it, or just exit.
 	 */
-	if (stat (strcpy (buf, m_maildir (folder)), &st) == -1) {
+	if (stat (strncpy (buf, m_maildir (folder), sizeof(buf)), &st) == -1) {
 	    if (errno != ENOENT)
 		adios (buf, "error on folder");
 	    if (fcreat == 0) {
@@ -423,7 +421,7 @@ main (int argc, char **argv)
     print_folders();
 
     context_save ();	/* save the context file */
-    done (0);
+    return done (0);
 }
 
 /*
@@ -444,7 +442,7 @@ dodir (char *dir)
     if (chdir (nmhdir) == NOTOK)
 	adios (nmhdir, "unable to change directory to");
 
-    addir (strcpy (buffer, dir));
+    addir (strncpy (buffer, dir, sizeof(buffer)));
 
     for (i = start; i < foldp; i++) {
 	get_folder_info (folds[i], NULL);
@@ -575,7 +573,7 @@ print_folders (void)
 	    maxcurmsg = fi[i].curmsg;
 
 	/* check for empty folders */
-	if (fi[i].hghmsg == 0)
+	if (fi[i].nummsg == 0)
 	    hasempty = 1;
     }
     nummsgdigits = num_digits (maxnummsg);
@@ -609,9 +607,9 @@ print_folders (void)
 
 	    /* Add `+' to end of name, if folder is current */
 	    if (strcmp (folder, fi[i].name))
-		sprintf (tmpname, "%s", fi[i].name);
+		snprintf (tmpname, sizeof(tmpname), "%s", fi[i].name);
 	    else
-		sprintf (tmpname, "%s+", fi[i].name);
+		snprintf (tmpname, sizeof(tmpname), "%s+", fi[i].name);
 
 	    if (fi[i].error) {
 		printf ("%-*s is unreadable\n", maxlen+1, tmpname);
@@ -621,7 +619,7 @@ print_folders (void)
 	    printf ("%-*s ", maxlen+1, tmpname);
 
 	    curprinted = 0; /* remember if we print cur */
-	    if (fi[i].hghmsg == 0) {
+	    if (fi[i].nummsg == 0) {
 		printf ("has %*s messages%*s",
 			nummsgdigits, "no",
 			fi[i].others ? lowmsgdigits + hghmsgdigits + 5 : 0, "");
@@ -658,13 +656,20 @@ print_folders (void)
 }
 
 /*
- * Return the number of digits needed for a positive integer
+ * Calculate the number of digits in a nonnegative integer
  */
 int
 num_digits (int n)
 {
     int ndigits = 0;
 
+    /* Sanity check */
+    if (n < 0)
+	adios (NULL, "oops, num_digits called with negative value");
+
+    if (n == 0)
+	return 1;
+
     while (n) {
 	n /= 10;
 	ndigits++;
@@ -809,10 +814,13 @@ readonly_folders (void)
     char atrcur[BUFSIZ];
     register struct node *np;
 
-    sprintf (atrcur, "atr-%s-", current);
+    /* sanity check - check that context has been read */
+    if (defpath == NULL)
+	adios (NULL, "oops, context hasn't been read yet");
+
+    snprintf (atrcur, sizeof(atrcur), "atr-%s-", current);
     atrlen = strlen (atrcur);
 
-    context_read ();
     for (np = m_defs; np; np = np->n_next)
 	if (ssequal (atrcur, np->n_name)
 		&& !ssequal (nmhdir, np->n_name + atrlen))
