@@ -2,7 +2,7 @@
 /*
  * forw.c -- forward a message, or group of messages.
  *
- * $Id$
+ * $Id: forw.c,v 1.2 1999/07/16 01:31:41 danh Exp $
  */
 
 #include <h/mh.h>
@@ -125,8 +125,8 @@ main (int argc, char **argv)
     char *cp, *cwd, *maildir, *dfolder = NULL;
     char *dmsg = NULL, *digest = NULL, *ed = NULL;
     char *file = NULL, *filter = NULL, *folder = NULL;
-    char *form = NULL, buf[100], value[10], **ap;
-    char **argp, *arguments[MAXARGS], *msgs[MAXARGS];
+    char *form = NULL, buf[BUFSIZ], value[10];
+    char **argp, **arguments, *msgs[MAXARGS];
     struct stat st;
 
 #ifdef	MHE
@@ -137,13 +137,11 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -156,7 +154,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [+folder] [msgs] [switches]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -297,13 +296,13 @@ main (int argc, char **argv)
 try_it_again:
 
 #ifdef MHE
-    strcpy (drft, buildsw ? m_maildir ("draft")
-			  : m_draft (dfolder, NULL, NOUSE, &isdf));
+    strncpy (drft, buildsw ? m_maildir ("draft")
+			  : m_draft (dfolder, NULL, NOUSE, &isdf), sizeof(drft));
 
     /* Check if a draft already exists */
     if (!buildsw && stat (drft, &st) != NOTOK) {
 #else
-    strcpy (drft, m_draft (dfolder, dmsg, NOUSE, &isdf));
+    strncpy (drft, m_draft (dfolder, dmsg, NOUSE, &isdf), sizeof(drft));
 
     /* Check if a draft already exists */
     if (stat (drft, &st) != NOTOK) {
@@ -351,9 +350,13 @@ try_it_again:
 
 	if (chdir (maildir) == NOTOK)
 	    adios (maildir, "unable to change directory to");
+
+	/* read folder and create message structure */
 	if (!(mp = folder_read (folder)))
 	    adios (NULL, "unable to read folder %s", folder);
-	if (mp->hghmsg == 0)
+
+	/* check for empty folder */
+	if (mp->nummsg == 0)
 	    adios (NULL, "no messages in %s", folder);
 
 	/* parse all the message ranges/sequences and set SELECTED */
@@ -371,7 +374,7 @@ try_it_again:
      */
     if (digest) {
 	if (issue == 0) {
-	    sprintf (buf, IFORMAT, digest);
+	    snprintf (buf, sizeof(buf), IFORMAT, digest);
 	    if (volume == 0
 		    && (cp = context_find (buf))
 		    && ((issue = atoi (cp)) < 0))
@@ -379,9 +382,9 @@ try_it_again:
 	    issue++;
 	}
 	if (volume == 0)
-	    sprintf (buf, VFORMAT, digest);
-	    if ((cp = context_find (buf)) == NULL || (volume = atoi (cp)) <= 0)
-		volume = 1;
+	    snprintf (buf, sizeof(buf), VFORMAT, digest);
+	if ((cp = context_find (buf)) == NULL || (volume = atoi (cp)) <= 0)
+	    volume = 1;
 	if (!form)
 	    form = digestcomps;
 	in = build_form (form, digest, volume, issue);
@@ -426,11 +429,11 @@ try_it_again:
 	close (out);
 
 	if (digest) {
-	    sprintf (buf, IFORMAT, digest);
-	    sprintf (value, "%d", issue);
+	    snprintf (buf, sizeof(buf), IFORMAT, digest);
+	    snprintf (value, sizeof(value), "%d", issue);
 	    context_replace (buf, getcpy (value));
-	    sprintf (buf, VFORMAT, digest);
-	    sprintf (value, "%d", volume);
+	    snprintf (buf, sizeof(buf), VFORMAT, digest);
+	    snprintf (value, sizeof(value), "%d", volume);
 	    context_replace (buf, getcpy (value));
 	}
 
@@ -444,7 +447,7 @@ try_it_again:
 	done (0);
     what_now (ed, nedit, NOUSE, drft, NULL, 0, mp,
 	anot ? "Forwarded" : NULL, inplace, cwd);
-    done (1);
+    return done (1);
 }
 
 
@@ -489,9 +492,11 @@ mhl_draft (int out, char *digest, int volume, int issue,
 		vec[i++] = "-digest";
 		vec[i++] = digest;
 		vec[i++] = "-issue";
-		sprintf(buf1, "%d", issue); vec[i++] = buf1;
+		snprintf (buf1, sizeof(buf1), "%d", issue);
+		vec[i++] = buf1;
 		vec[i++] = "-volume";
-		sprintf(buf2, "%d", volume); vec[i++] = buf2;
+		snprintf (buf2, sizeof(buf2), "%d", volume);
+		vec[i++] = buf2;
 	    }
 
 	    /*
@@ -507,7 +512,14 @@ mhl_draft (int out, char *digest, int volume, int issue,
 	    if (mp->numsel >= MAXARGS - i)
 		adios (NULL, "more than %d messages for %s exec",
 			vec[0], MAXARGS - i);
-	    for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++)
+
+	    /*
+	     * Now add the message names to filter.  We can only
+	     * handle about 995 messages (because vec is fixed size),
+	     * but that should be plenty.
+	     */
+	    for (msgnum = mp->lowsel; msgnum <= mp->hghsel && i < sizeof(vec) - 1;
+			msgnum++)
 		if (is_selected (mp, msgnum))
 		    vec[i++] = getcpy (m_name (msgnum));
 	    vec[i] = NULL;
@@ -537,22 +549,37 @@ static void
 copy_draft (int out, char *digest, char *file, int volume, int issue, int dashstuff)
 {
     int fd,i, msgcnt, msgnum;
+    int len, buflen;
     register char *bp, *msgnam;
     char buffer[BUFSIZ];
 
     msgcnt = 1;
     for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
 	if (is_selected (mp, msgnum)) {
-	    if (digest)
-		strcpy (buffer, msgnum == mp->lowsel ? delim3 : delim4);
-	    else {
-		strcpy (bp = buffer, "\n-------"), bp += strlen (bp);
-		if (msgnum == mp->lowsel)
-		    sprintf (bp, " Forwarded Message%s", mp->numsel > 1 ? "s" : "");
-		else
-		    sprintf (bp, " Message %d", msgcnt);
-		bp += strlen (bp);
-		strcpy (bp, "\n\n");
+	    if (digest) {
+		strncpy (buffer, msgnum == mp->lowsel ? delim3 : delim4,
+			sizeof(buffer));
+	    } else {
+		/* Get buffer ready to go */
+		bp = buffer;
+		buflen = sizeof(buffer);
+
+		strncpy (bp, "\n-------", buflen);
+		len = strlen (bp);
+		bp += len;
+		buflen -= len;
+
+		if (msgnum == mp->lowsel) {
+		    snprintf (bp, buflen, " Forwarded Message%s",
+			mp->numsel > 1 ? "s" : "");
+		} else {
+		    snprintf (bp, buflen, " Message %d", msgcnt);
+		}
+		len = strlen (bp);
+		bp += len;
+		buflen -= len;
+
+		strncpy (bp, "\n\n", buflen);
 	    }
 	    write (out, buffer, strlen (buffer));
 
@@ -576,15 +603,16 @@ copy_draft (int out, char *digest, char *file, int volume, int issue, int dashst
     }
 
     if (digest) {
-	strcpy (buffer, delim4);
+	strncpy (buffer, delim4, sizeof(buffer));
     } else {
-	sprintf (buffer, "\n------- End of Forwarded Message%s\n\n",
+	snprintf (buffer, sizeof(buffer), "\n------- End of Forwarded Message%s\n\n",
 		mp->numsel > 1 ? "s" : "");
     }
     write (out, buffer, strlen (buffer));
 
     if (digest) {
-	sprintf (buffer, "End of %s Digest [Volume %d Issue %d]\n", digest, volume, issue);
+	snprintf (buffer, sizeof(buffer), "End of %s Digest [Volume %d Issue %d]\n",
+		digest, volume, issue);
 	i = strlen (buffer);
 	for (bp = buffer + i; i > 1; i--)
 	    *bp++ = '*';
@@ -605,12 +633,12 @@ copy_mime_draft (int out)
     int msgnum;
     char buffer[BUFSIZ];
 
-    sprintf (buffer, "#forw [forwarded message%s] +%s",
+    snprintf (buffer, sizeof(buffer), "#forw [forwarded message%s] +%s",
 	mp->numsel == 1 ? "" : "s", mp->foldpath);
     write (out, buffer, strlen (buffer));
     for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++)
 	if (is_selected (mp, msgnum)) {
-	    sprintf (buffer, " %s", m_name (msgnum));
+	    snprintf (buffer, sizeof(buffer), " %s", m_name (msgnum));
 	    write (out, buffer, strlen (buffer));
 	}
     write (out, "\n", 1);
@@ -632,6 +660,8 @@ build_form (char *form, char *digest, int volume, int issue)
     /* Get new format string */
     nfs = new_fs (form, NULL, NULL);
     fmtsize = strlen (nfs) + 256;
+
+    /* Compile format string */
     fmt_compile (nfs, &fmt);
 
     FINDCOMP (cptr, "digest");
@@ -647,7 +677,7 @@ build_form (char *form, char *digest, int volume, int issue)
     dat[3] = fmtsize;
     dat[4] = 0;
 
-    strcpy (tmpfil, m_tmpfil (invo_name));
+    strncpy (tmpfil, m_tmpfil (invo_name), sizeof(tmpfil));
     if ((tmp = fopen (tmpfil, "w+")) == NULL)
 	adios (tmpfil, "unable to create");
     unlink (tmpfil);
