@@ -2,7 +2,7 @@
 /*
  * inc.c -- incorporate messages from a maildrop into a folder
  *
- * $Id$
+ * $Id: inc.c,v 1.2 1999/07/16 01:13:05 danh Exp $
  */
 
 #ifdef MAILGROUP
@@ -25,10 +25,6 @@
 # include <h/popsbr.h>
 #endif
 
-#ifdef KPOP
-# include <krb.h>
-#endif
-
 #ifdef HESIOD
 # include <hesiod.h>
 #endif
@@ -148,7 +144,7 @@ static int return_gid;
 char *map_name(char *);
 
 #ifdef POP
-void done(int);
+int done(int);
 static int pop_action(char *);
 static int pop_pack(char *);
 static int map_count(void);
@@ -160,14 +156,12 @@ main (int argc, char **argv)
 {
     int chgflag = 1, trnflag = 1;
     int noisy = 1, width = 0, locked = 0;
-    int rpop = 0, i, hghnum, msgnum;
+    int rpop, i, hghnum, msgnum;
     char *cp, *maildir, *folder = NULL;
     char *format = NULL, *form = NULL;
-    char *newmail, *host = NULL;
+    char *newmail, *host = NULL, *user = NULL;
     char *audfile = NULL, *from = NULL;
-    char buf[100], **ap;
-    char **argp, *nfs, *arguments[MAXARGS];
-    char *user = NULL;
+    char buf[BUFSIZ], **argp, *nfs, **arguments;
     struct msgs *mp;
     struct stat st, s1;
     FILE *in, *aud = NULL;
@@ -190,7 +184,13 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
+
+    /* read user profile/context */
+    context_read();
+
     mts_init (invo_name);
+    arguments = getarguments (invo_name, argc, argv, 1);
+    argp = arguments;
 
 #ifdef POP
 # ifdef HESIOD
@@ -218,14 +218,11 @@ main (int argc, char **argv)
 	snoop++;
 #endif /* POP */
 
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
-    argp = arguments;
+#ifdef KPOP
+    rpop = 1;
+#else
+    rpop = 0;
+#endif
 
     while ((cp = *argp++)) {
 	if (*cp == '-') {
@@ -237,7 +234,7 @@ main (int argc, char **argv)
 		adios (NULL, "-%s unknown", cp);
 
 	    case HELPSW: 
-		snprintf (buf, 100, "%s [+folder] [switches]", invo_name);
+		snprintf (buf, sizeof(buf), "%s [+folder] [switches]", invo_name);
 		print_help (buf, switches, 1);
 		done (1);
 	    case VERSIONSW:
@@ -404,13 +401,8 @@ main (int argc, char **argv)
 	/*
 	 * initialize POP connection
 	 */
-# if defined(RPOP) || defined(APOP)
 	if (pop_init (host, user, pass, snoop, rpop) == NOTOK)
 	    adios (NULL, "%s", response);
-# else
-	if (pop_init (host, user, pass, snoop) == NOTOK)
-	    adios (NULL, "%s", response);
-# endif
 
 	/* Check if there are any messages */
 	if (pop_stat (&nmsgs, &nbytes) == NOTOK)
@@ -475,7 +467,7 @@ main (int argc, char **argv)
 	adios (NULL, "unable to read folder %s", folder);
 
 #ifdef POP
- go_to_it: ;
+go_to_it:
 #endif /* POP */
 
     if (inc_type == INC_FILE) {
@@ -856,7 +848,7 @@ main (int argc, char **argv)
     seq_setunseen (mp, 0);	/* set the Unseen-Sequence */
     seq_save (mp);		/* synchronize sequences   */
     context_save ();		/* save the context file   */
-    done (0);
+    return done (0);
 }
 
 
@@ -894,13 +886,14 @@ cpymsg (FILE *in, FILE *out)
 
 
 #ifdef POP
-void
+int
 done (int status)
 {
     if (packfile && pd != NOTOK)
 	mbx_close (packfile, pd);
 
     exit (status);
+    return 1;  /* dead code to satisfy the compiler */
 }
 
 static int
@@ -908,6 +901,7 @@ pop_action (char *s)
 {
     fprintf (pf, "%s\n", s);
     stop += strlen (s) + 1;
+    return 0;  /* Is return value used?  This was missing before 1999-07-15. */
 }
 
 static int
@@ -916,13 +910,14 @@ pop_pack (char *s)
     int j;
     char buffer[BUFSIZ];
 
-    snprintf (buffer, BUFSIZ, "%s\n", s);
+    snprintf (buffer, sizeof(buffer), "%s\n", s);
     for (j = 0; (j = stringdex (mmdlm1, buffer)) >= 0; buffer[j]++)
 	continue;
     for (j = 0; (j = stringdex (mmdlm2, buffer)) >= 0; buffer[j]++)
 	continue;
     fputs (buffer, pf);
     size += strlen (buffer) + 1;
+    return 0;  /* Is return value used?  This was missing before 1999-07-15. */
 }
 
 static int
