@@ -2,7 +2,7 @@
 /*
  * install-mh.c -- initialize the nmh environment of a new user
  *
- * $Id$
+ * $Id: install-mh.c,v 1.2 1999/07/16 01:31:15 danh Exp $
  */
 
 #include <h/mh.h>
@@ -190,7 +190,7 @@ query:
 	    fprintf (out, "%s: %s\n", np->n_name, np->n_field);
     }
     fclose (out);
-    done (0);
+    return done (0);
 }
 
 
