@@ -4,11 +4,18 @@
  *        -- delete messages (s) from sequences in given folder
  *        -- list sequences in given folder
  *
- * $Id$
+ * $Id: mark.c,v 1.2 1999/07/16 01:30:45 danh Exp $
  */
 
 #include <h/mh.h>
 
+/*
+ * We allocate space for messages (msgs array)
+ * this number of elements at a time.
+ */
+#define MAXMSGS  256
+
+
 static struct swit switches[] = {
 #define	ADDSW               0
     { "add", 0 },
@@ -47,81 +54,93 @@ main (int argc, char **argv)
 {
     int addsw = 0, deletesw = 0, debugsw = 0;
     int listsw = 0, publicsw = -1, zerosw = 0;
-    int seqp = 0, msgp = 0, msgnum;
-    char *cp, *maildir, *folder = NULL, buf[100];
-    char **ap, **argp, *arguments[MAXARGS];
-    char *seqs[NUMATTRS + 1], *msgs[MAXARGS];
+    int seqp = 0, msgnum, nummsgs, maxmsgs;
+    char *cp, *maildir, *folder = NULL, buf[BUFSIZ];
+    char **argp, **arguments;
+    char *seqs[NUMATTRS + 1], **msgs;
     struct msgs *mp;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
+    /*
+     * Allocate the initial space to record message
+     * names, ranges, and sequences.
+     */
+    nummsgs = 0;
+    maxmsgs = MAXMSGS;
+    if (!(msgs = (char **) malloc ((size_t) (maxmsgs * sizeof(*msgs)))))
+	adios (NULL, "unable to allocate storage");
+
+    /*
+     * Parse arguments
+     */
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
-		case AMBIGSW: 
-		    ambigsw (cp, switches);
-		    done (1);
-		case UNKWNSW: 
-		    adios (NULL, "-%s unknown\n", cp);
+	    case AMBIGSW: 
+		ambigsw (cp, switches);
+		done (1);
+	    case UNKWNSW: 
+		adios (NULL, "-%s unknown\n", cp);
 
-		case HELPSW: 
-		    sprintf (buf, "%s [+folder] [msgs] [switches]", invo_name);
-		    print_help (buf, switches, 1);
-		    done (1);
-		case VERSIONSW:
-		    print_version(invo_name);
-		    done (1);
+	    case HELPSW: 
+		snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
+			  invo_name);
+		print_help (buf, switches, 1);
+		done (1);
+	    case VERSIONSW:
+		print_version(invo_name);
+		done (1);
 
-		case ADDSW: 
-		    addsw++;
-		    deletesw = listsw = 0;
-		    continue;
-		case DELSW: 
-		    deletesw++;
-		    addsw = listsw = 0;
-		    continue;
-		case LSTSW: 
-		    listsw++;
-		    addsw = deletesw = 0;
-		    continue;
+	    case ADDSW: 
+		addsw++;
+		deletesw = listsw = 0;
+		continue;
+	    case DELSW: 
+		deletesw++;
+		addsw = listsw = 0;
+		continue;
+	    case LSTSW: 
+		listsw++;
+		addsw = deletesw = 0;
+		continue;
 
-		case SEQSW: 
-		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
-		    if (seqp < NUMATTRS)
-			seqs[seqp++] = cp;
-		    else
-			adios (NULL, "only %d sequences allowed!", NUMATTRS);
-		    continue;
+	    case SEQSW: 
+		if (!(cp = *argp++) || *cp == '-')
+		    adios (NULL, "missing argument to %s", argp[-2]);
 
-		case PUBLSW: 
-		    publicsw = 1;
-		    continue;
-		case NPUBLSW: 
-		    publicsw = 0;
-		    continue;
+		/* check if too many sequences specified */
+		if (seqp >= NUMATTRS)
+		    adios (NULL, "too many sequences (more than %d) specified", NUMATTRS);
+		seqs[seqp++] = cp;
+		continue;
 
-		case DEBUGSW: 
-		    debugsw++;
-		    continue;
+	    case PUBLSW: 
+		publicsw = 1;
+		continue;
+	    case NPUBLSW: 
+		publicsw = 0;
+		continue;
 
-		case ZEROSW: 
-		    zerosw++;
-		    continue;
-		case NZEROSW: 
-		    zerosw = 0;
-		    continue;
+	    case DEBUGSW: 
+		debugsw++;
+		continue;
+
+	    case ZEROSW: 
+		zerosw++;
+		continue;
+	    case NZEROSW: 
+		zerosw = 0;
+		continue;
 	    }
 	}
 	if (*cp == '+' || *cp == '@') {
@@ -130,7 +149,17 @@ main (int argc, char **argv)
 	    else
 		folder = path (cp + 1, *cp == '+' ? TFOLDER : TSUBCWF);
 	} else {
-	    msgs[msgp++] = cp;
+	    /*
+	     * Check if we need to allocate more space
+	     * for message names/ranges/sequences.
+	     */
+	    if (nummsgs >= maxmsgs) {
+		maxmsgs += MAXMSGS;
+		if (!(msgs = (char **) realloc (msgs,
+						(size_t) (maxmsgs * sizeof(*msgs)))))
+		    adios (NULL, "unable to reallocate msgs storage");
+	    }
+	    msgs[nummsgs++] = cp;
 	}
     }
 
@@ -148,8 +177,8 @@ main (int argc, char **argv)
 
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
-    if (!msgp)
-	msgs[msgp++] = listsw ? "all" :"cur";
+    if (!nummsgs)
+	msgs[nummsgs++] = listsw ? "all" :"cur";
     if (!folder)
 	folder = getfolder (1);
     maildir = m_maildir (folder);
@@ -165,12 +194,12 @@ main (int argc, char **argv)
     if (debugsw)
 	print_debug(mp);
 
-    /* no messages in folder */
-    if (mp->hghmsg == 0)
+    /* check for empty folder */
+    if (mp->nummsg == 0)
 	adios (NULL, "no messages in %s", folder);
 
     /* parse all the message ranges/sequences and set SELECTED */
-    for (msgnum = 0; msgnum < msgp; msgnum++)
+    for (msgnum = 0; msgnum < nummsgs; msgnum++)
 	if (!m_convert (mp, msgs[msgnum]))
 	    done (1);
 
@@ -183,7 +212,7 @@ main (int argc, char **argv)
      */
     if (seqp == 0 && (addsw || deletesw))
 	adios (NULL, "-%s requires at least one -sequence argument",
-		addsw ? "add" : "delete");
+	       addsw ? "add" : "delete");
     seqs[seqp] = NULL;
 
     /* Adding messages to sequences */
@@ -220,7 +249,7 @@ main (int argc, char **argv)
     context_replace (pfolder, folder);	/* update current folder         */
     context_save ();			/* save the context file         */
     folder_free (mp);			/* free folder/message structure */
-    done (0);
+    return done (0);
 }
 
 
@@ -236,9 +265,11 @@ print_debug (struct msgs *mp)
     printf ("mypath        = %s\n", mypath);
     printf ("defpath       = %s\n", defpath);
     printf ("ctxpath       = %s\n", ctxpath);
-    printf ("context flags = %s\n", snprintb (buf, 100, (unsigned) ctxflags, DBITS));
+    printf ("context flags = %s\n", snprintb (buf, sizeof(buf),
+		(unsigned) ctxflags, DBITS));
     printf ("foldpath      = %s\n", mp->foldpath);
-    printf ("folder flags  = %s\n\n", snprintb(buf, 100, (unsigned) mp->msgflags, FBITS));
+    printf ("folder flags  = %s\n\n", snprintb(buf, sizeof(buf),
+		(unsigned) mp->msgflags, FBITS));
     printf ("lowmsg=%d hghmsg=%d nummsg=%d curmsg=%d\n",
 	mp->lowmsg, mp->hghmsg, mp->nummsg, mp->curmsg);
     printf ("lowsel=%d hghsel=%d numsel=%d\n",
@@ -257,10 +288,11 @@ seq_printdebug (struct msgs *mp)
     int msgnum;
     char buf[100];
 
+    printf ("\n");
     for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
 	if (is_selected (mp, msgnum))
 	    printf ("%*d: %s\n", DMAXFOLDER, msgnum,
-		snprintb (buf, 100, (unsigned) mp->msgstats[msgnum - mp->lowoff],
-		seq_bits (mp)));
+		snprintb (buf, sizeof(buf),
+		(unsigned) mp->msgstats[msgnum - mp->lowoff], seq_bits (mp)));
     }
 }
