@@ -2,7 +2,7 @@
 /*
  * mhbuild.c -- expand/translate MIME composition files
  *
- * $Id$
+ * $Id: mhbuild.c,v 1.2 1999/07/16 01:25:05 danh Exp $
  */
 
 #include <h/mh.h>
@@ -363,12 +363,11 @@ main (int argc, char **argv)
     unlink_outfile = 0;
 
     free_content (ct);
-    done (0);
-    /* NOT REACHED */
+    return done (0);
 }
 
 
-void
+int
 done (int status)
 {
     /*
@@ -381,4 +380,5 @@ done (int status)
 	unlink (outfile);
 
     exit (status);
+    return 1;  /* dead code to satisfy the compiler */
 }
