@@ -3,7 +3,7 @@
  * mhlistsbr.c -- routines to list information about the
  *             -- contents of MIME messages
  *
- * $Id$
+ * $Id: mhlistsbr.c,v 1.1.1.1 1999/04/30 18:08:35 doug Exp $
  */
 
 #include <h/mh.h>
@@ -13,11 +13,16 @@
 #include <signal.h>
 #include <zotnet/mts/mts.h>
 #include <zotnet/tws/tws.h>
-#include <h/mhn.h>
+#include <h/mime.h>
 #include <h/mhparse.h>
 
 extern int errno;
 
+/* mhmisc.c */
+int part_ok (CT, int);
+int type_ok (CT, int);
+void flush_errors (void);
+
 /*
  * prototypes
  */
@@ -25,11 +30,6 @@ void list_all_messages (CT *, int, int, int, int);
 int list_switch (CT, int, int, int, int);
 int list_content (CT, int, int, int, int);
 
-/* mhmisc.c */
-int part_ok (CT, int);
-int type_ok (CT, int);
-void flush_errors (void);
-
 /*
  * static prototypes
  */
@@ -162,7 +162,8 @@ list_content (CT ct, int toplevel, int realsize, int verbose, int debug)
 
     printf (toplevel > 0 ? LSTFMT2a : toplevel < 0 ? "part " : "     ",
 	    atoi (r1bindex (empty (ct->c_file), '/')));
-    sprintf (buffer, "%s/%s", empty (ci->ci_type), empty (ci->ci_subtype));
+    snprintf (buffer, sizeof(buffer), "%s/%s", empty (ci->ci_type),
+		empty (ci->ci_subtype));
     printf (LSTFMT2b, empty (ct->c_partno), buffer);
 
     if (ct->c_cesizefnx && realsize)
@@ -213,7 +214,7 @@ list_content (CT ct, int toplevel, int realsize, int verbose, int debug)
 
 	dp = trimcpy (cp = add (ci->ci_comment, NULL));
 	free (cp);
-	sprintf (buffer, "(%s)", dp);
+	snprintf (buffer, sizeof(buffer), "(%s)", dp);
 	free (dp);
 	printf (LSTFMT2d2, buffer);
     }
