@@ -2,12 +2,12 @@
 /*
  * mhparse.c -- misc routines to process MIME messages
  *
- * $Id$
+ * $Id: mhmisc.c,v 1.2 1999/07/16 01:09:46 danh Exp $
  */
 
 #include <h/mh.h>
 #include <errno.h>
-#include <h/mhn.h>
+#include <h/mime.h>
 #include <h/mhparse.h>
 
 extern int errno;
@@ -64,7 +64,7 @@ type_ok (CT ct, int sP)
     if (ntype == 0 || (ct->c_type == CT_MULTIPART && (sP || ct->c_subtype)))
 	return 1;
 
-    sprintf (buffer, "%s/%s", ci->ci_type, ci->ci_subtype);
+    snprintf (buffer, sizeof(buffer), "%s/%s", ci->ci_type, ci->ci_subtype);
     for (ap = types; *ap; ap++)
 	if (!strcasecmp (*ap, ci->ci_type) || !strcasecmp (*ap, buffer))
 	    return 1;
@@ -94,7 +94,7 @@ make_intermediates (char *file)
 {
     char *cp;
 
-    for (cp = file + 1; cp = strchr(cp, '/'); cp++) {
+    for (cp = file + 1; (cp = strchr(cp, '/')); cp++) {
 	struct stat st;
 
 	*cp = '\0';
@@ -127,58 +127,90 @@ losing_directory:
 }
 
 
+/*
+ * Construct error message for content
+ */
+
 void
 content_error (char *what, CT ct, char *fmt, ...)
 {
     va_list arglist;
-    int	i;
-    char *bp;
-    char buffer[BUFSIZ];
+    int	i, len, buflen;
+    char *bp, buffer[BUFSIZ];
     CI ci;
 
     bp = buffer;
+    buflen = sizeof(buffer);
 
     if (userrs && invo_name && *invo_name) {
-	sprintf (bp, "%s: ", invo_name);
-	bp += strlen (bp);
+	snprintf (bp, buflen, "%s: ", invo_name);
+	len = strlen (bp);
+	bp += len;
+	buflen -= len;
     }
 
     va_start (arglist, fmt);
-    vsprintf (bp, fmt, arglist);
-    bp += strlen (bp);
+
+    vsnprintf (bp, buflen, fmt, arglist);
+    len = strlen (bp);
+    bp += len;
+    buflen -= len;
+
     ci = &ct->c_ctinfo;
 
     if (what) {
 	char *s;
 
 	if (*what) {
-	    sprintf (bp, " %s: ", what);
-	    bp += strlen (bp);
+	    snprintf (bp, buflen, " %s: ", what);
+	    len = strlen (bp);
+	    bp += len;
+	    buflen -= len;
 	}
+
 	if ((s = strerror (errno)))
-	    sprintf (bp, "%s", s);
+	    snprintf (bp, buflen, "%s", s);
 	else
-	    sprintf (bp, "Error %d", errno);
-	bp += strlen (bp);
+	    snprintf (bp, buflen, "Error %d", errno);
+
+	len = strlen (bp);
+	bp += len;
+	buflen -= len;
     }
 
     i = strlen (invo_name) + 2;
-    sprintf (bp, "\n%*.*s(content %s/%s", i, i, "", ci->ci_type, ci->ci_subtype);
-    bp += strlen (bp);
+
+    /* Now add content type and subtype */
+    snprintf (bp, buflen, "\n%*.*s(content %s/%s", i, i, "",
+	ci->ci_type, ci->ci_subtype);
+    len = strlen (bp);
+    bp += len;
+    buflen -= len;
+
+    /* Now add the message/part number */
     if (ct->c_file) {
-	sprintf (bp, " in message %s", ct->c_file);
-	bp += strlen (bp);
+	snprintf (bp, buflen, " in message %s", ct->c_file);
+	len = strlen (bp);
+	bp += len;
+	buflen -= len;
+
 	if (ct->c_partno) {
-	    sprintf (bp, ", part %s", ct->c_partno);
-	    bp += strlen (bp);
+	    snprintf (bp, buflen, ", part %s", ct->c_partno);
+	    len = strlen (bp);
+	    bp += len;
+	    buflen -= len;
 	}
     }
-    sprintf (bp, ")");
-    bp += strlen (bp);
+
+    snprintf (bp, buflen, ")");
+    len = strlen (bp);
+    bp += len;
+    buflen -= len;
 
     if (userrs) {
 	*bp++ = '\n';
 	*bp = '\0';
+	buflen--;
 
 	errs = add (buffer, errs);
     } else {
