@@ -2,7 +2,7 @@
 /*
  * mhn.c -- display, list, cache, or store the contents of MIME messages
  *
- * $Id$
+ * $Id: mhn.c,v 1.2 1999/07/16 01:24:25 danh Exp $
  */
 
 #include <h/mh.h>
@@ -13,13 +13,21 @@
 #include <signal.h>
 #include <zotnet/mts/mts.h>
 #include <zotnet/tws/tws.h>
-#include <h/mhn.h>
+#include <h/mime.h>
 #include <h/mhparse.h>
+#include <h/mhcachesbr.h>
 
 #ifdef HAVE_SYS_WAIT_H
 # include <sys/wait.h>
 #endif
 
+/*
+ * We allocate space for message names (msgs array)
+ * this number of elements at a time.
+ */
+#define MAXMSGS  256
+
+
 static struct swit switches[] = {
 #define	AUTOSW                  0
     { "auto", 0 },
@@ -118,27 +126,18 @@ static struct swit switches[] = {
     { NULL, 0 }
 };
 
-static struct swit caches[] = {
-#define	CACHE_NEVER    0
-    { "never", 0 },
-#define	CACHE_PRIVATE  1
-    { "private", 0 },
-#define	CACHE_PUBLIC   2
-    { "public", 0 },
-#define	CACHE_ASK      3
-    { "ask", 0 },
-    { NULL, 0 }
-};
 
 extern int errno;
 
 /* mhparse.c */
 extern int checksw;
+extern char *tmp;	/* directory to place temp files */
+
+/* mhcachesbr.c */
 extern int rcachesw;
 extern int wcachesw;
 extern char *cache_public;
 extern char *cache_private;
-extern char *tmp;	/* directory to place temp files */
 
 /* mhshowsbr.c */
 extern int pausesw;
@@ -182,11 +181,8 @@ static int storesw = 0;
 
 #define	quitser	pipeser
 
-/*
- * prototypes
- */
+/* mhparse.c */
 CT parse_mime (char *);
-void cache_content (CT);
 
 /* mhmisc.c */
 int part_ok (CT, int);
@@ -203,6 +199,9 @@ void list_all_messages (CT *, int, int, int, int);
 /* mhstoresbr.c */
 void store_all_messages (CT *);
 
+/* mhcachesbr.c */
+void cache_all_messages (CT *);
+
 /* mhfree.c */
 void free_content (CT);
 
@@ -216,10 +215,10 @@ int
 main (int argc, char **argv)
 {
     int sizesw = 1, headsw = 1;
-    int msgp = 0, msgnum, *icachesw;
+    int nummsgs, maxmsgs, msgnum, *icachesw;
     char *cp, *file = NULL, *folder = NULL;
-    char *maildir, buf[100], **ap, **argp;
-    char *arguments[MAXARGS], *msgs[MAXARGS];
+    char *maildir, buf[100], **argp;
+    char **arguments, **msgs;
     struct msgs *mp = NULL;
     CT ct, *ctp;
     FILE *fp;
@@ -228,15 +227,25 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
+    /*
+     * Allocate the initial space to record message
+     * names, ranges, and sequences.
+     */
+    nummsgs = 0;
+    maxmsgs = MAXMSGS;
+    if (!(msgs = (char **) malloc ((size_t) (maxmsgs * sizeof(*msgs)))))
+	adios (NULL, "unable to allocate storage");
+
+    /*
+     * Parse arguments
+     */
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
@@ -247,7 +256,8 @@ main (int argc, char **argv)
 		adios (NULL, "-%s unknown", cp);
 
 	    case HELPSW: 
-		sprintf (buf, "%s [+folder] [msgs] [switches]", invo_name);
+		snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
+			invo_name);
 		print_help (buf, switches, 1);
 		done (1);
 	    case VERSIONSW:
@@ -431,7 +441,17 @@ do_cache:
 	    else
 		folder = path (cp + 1, *cp == '+' ? TFOLDER : TSUBCWF);
 	} else {
-	    msgs[msgp++] = cp;
+	    /*
+	     * Check if we need to allocate more space
+	     * for message names/ranges/sequences.
+	     */
+	    if (nummsgs >= maxmsgs) {
+		maxmsgs += MAXMSGS;
+		if (!(msgs = (char **) realloc (msgs,
+			(size_t) (maxmsgs * sizeof(*msgs)))))
+		    adios (NULL, "unable to reallocate msgs storage");
+	    }
+	    msgs[nummsgs++] = cp;
 	}
     }
 
@@ -452,7 +472,7 @@ do_cache:
      */
     if ((cp = getenv ("MHN"))) {
 	if ((fp = fopen (cp, "r"))) {
-	    m_readefs ((struct node **) 0, fp, cp, 0);
+	    readconfig ((struct node **) 0, fp, cp, 0);
 	    fclose (fp);
 	} else {
 	    admonish ("", "unable to read $MHN profile (%s)", cp);
@@ -463,18 +483,16 @@ do_cache:
      * Read the standard profile setup
      */
     if ((fp = fopen (cp = etcpath ("mhn.defaults"), "r"))) {
-	m_readefs ((struct node **) 0, fp, cp, 0);
+	readconfig ((struct node **) 0, fp, cp, 0);
 	fclose (fp);
     }
 
     /* Check for public cache location */
-    sprintf (buf, "%s-cache", invo_name);
-    if ((cache_public = context_find (buf)) && *cache_public != '/')
+    if ((cache_public = context_find (nmhcache)) && *cache_public != '/')
 	cache_public = NULL;
 
     /* Check for private cache location */
-    sprintf (buf, "%s-private-cache", invo_name);
-    if (!(cache_private = context_find (buf)))
+    if (!(cache_private = context_find (nmhprivcache)))
 	cache_private = ".cache";
     cache_private = getcpy (m_maildir (cache_private));
 
@@ -488,8 +506,7 @@ do_cache:
      * then store temporary files there.  Else we
      * store them in standard nmh directory.
      */
-    sprintf (buf, "%s-storage", invo_name);
-    if ((cp = context_find (buf)) && *cp)
+    if ((cp = context_find (nmhstorage)) && *cp)
 	tmp = concat (cp, "/", invo_name, NULL);
     else
 	tmp = add (m_maildir (invo_name), NULL);
@@ -506,9 +523,9 @@ do_cache:
 
 	if (showsw || storesw || cachesw)
 	    adios (NULL, "cannot use -build with -show, -store, -cache");
-	if (msgp < 1)
+	if (nummsgs < 1)
 	    adios (NULL, "need to specify a %s composition file", invo_name);
-	if (msgp > 1)
+	if (nummsgs > 1)
 	    adios (NULL, "only one %s composition file at a time", invo_name);
 
 	vecp = 0;
@@ -535,7 +552,7 @@ do_cache:
     /*
      * Process a mhn composition file (old MH style)
      */
-    if (msgp == 1 && !folder && !npart && !cachesw
+    if (nummsgs == 1 && !folder && !npart && !cachesw
 	&& !showsw && !storesw && !ntype && !file
 	&& (cp = getenv ("mhdraft"))
 	&& strcmp (cp, msgs[0]) == 0) {
@@ -564,7 +581,7 @@ do_cache:
 	_exit (-1);
     }
 
-    if (file && msgp)
+    if (file && nummsgs)
 	adios (NULL, "cannot specify msg and file at same time!");
 
     /*
@@ -581,21 +598,25 @@ do_cache:
 	/*
 	 * message(s) are coming from a folder
 	 */
-	if (!msgp)
-	    msgs[msgp++] = "cur";
+	if (!nummsgs)
+	    msgs[nummsgs++] = "cur";
 	if (!folder)
 	    folder = getfolder (1);
 	maildir = m_maildir (folder);
 
 	if (chdir (maildir) == NOTOK)
 	    adios (maildir, "unable to change directory to");
+
+	/* read folder and create message structure */
 	if (!(mp = folder_read (folder)))
 	    adios (NULL, "unable to read folder %s", folder);
-	if (mp->hghmsg == 0)
+
+	/* check for empty folder */
+	if (mp->nummsg == 0)
 	    adios (NULL, "no messages in %s", folder);
 
 	/* parse all the message ranges/sequences and set SELECTED */
-	for (msgnum = 0; msgnum < msgp; msgnum++)
+	for (msgnum = 0; msgnum < nummsgs; msgnum++)
 	    if (!m_convert (mp, msgs[msgnum]))
 		done (1);
 	seq_setprev (mp);	/* set the previous-sequence */
@@ -663,22 +684,8 @@ do_cache:
     /*
      * Cache the message content
      */
-    if (cachesw) {
-	for (ctp = cts; *ctp; ctp++) {
-	    ct = *ctp;
-	    if (type_ok (ct, 1)) {
-		cache_content (ct);
-		if (ct->c_fp) {
-		    fclose (ct->c_fp);
-		    ct->c_fp = NULL;
-		}
-		if (ct->c_ceclosefnx)
-		    (*ct->c_ceclosefnx) (ct);
-	    }
-	}
-
-	flush_errors ();
-    }
+    if (cachesw)
+	cache_all_messages (cts);
 
     /*
      * Show the message content
@@ -701,8 +708,7 @@ do_cache:
 	context_save ();		  /* save the context file  */
     }
 
-    done (0);
-    /* NOTREACHED */
+    return done (0);
 }
 
 
@@ -721,7 +727,7 @@ pipeser (int i)
 }
 
 
-void
+int
 done (int status)
 {
     CT *ctp;
@@ -731,4 +737,5 @@ done (int status)
 	    free_content (*ctp);
 
     exit (status);
+    return 1;  /* dead code to satisfy the compiler */
 }
