@@ -5,7 +5,7 @@
  * Originally contributed by
  * Jeffrey C Honig <Jeffrey_C_Honig@cornell.edu>
  *
- * $Id$
+ * $Id: mhparam.c,v 1.2 1999/07/16 01:29:28 danh Exp $
  */
 
 #include <h/mh.h>
@@ -83,17 +83,15 @@ main(int argc, char **argv)
     int i, compp = 0, missed = 0;
     int all = 0, debug = 0;
     int components = -1;
-    char *cp, buf[100], **ap, **argp;
-    char *arguments[MAXARGS], *comps[MAXARGS];
+    char *cp, buf[BUFSIZ], **argp;
+    char **arguments, *comps[MAXARGS];
 
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -106,8 +104,9 @@ main(int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [profile-components] [switches]", invo_name);
-		    print_help (buf, switches);
+		    snprintf (buf, sizeof(buf), "%s [profile-components] [switches]",
+			invo_name);
+		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
 		    print_version(invo_name);
@@ -144,7 +143,6 @@ main(int argc, char **argv)
 		   components ? "" : "no");
 
 	/* print all entries in context/profile list */
-	context_read();
 	for (np = m_defs; np; np = np->n_next)
 	    printf("%s: %s\n", np->n_name, np->n_field);
 
@@ -179,7 +177,7 @@ main(int argc, char **argv)
 	}
     }
     
-    done (missed);
+    return done (missed);
 }
 
 
@@ -189,7 +187,7 @@ p_find(char *str)
     struct proc *ps;
 
     for (ps = procs; ps->p_name; ps++)
-	if (uleq (ps->p_name, str))
+	if (!strcasecmp (ps->p_name, str))
 	    return (*ps->p_field);
 
     return NULL;
