@@ -2,7 +2,7 @@
 /*
  * mhparse.c -- routines to parse the contents of MIME messages
  *
- * $Id$
+ * $Id: mhparse.c,v 1.4 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
@@ -14,24 +14,13 @@
 #include <signal.h>
 #include <zotnet/mts/mts.h>
 #include <zotnet/tws/tws.h>
-#include <h/mhn.h>
+#include <h/mime.h>
 #include <h/mhparse.h>
 
 #ifdef HAVE_SYS_WAIT_H
 # include <sys/wait.h>
 #endif
 
-static struct swit caches[] = {
-#define	CACHE_NEVER    0
-    { "never", 0 },
-#define	CACHE_PRIVATE  1
-    { "private", 0 },
-#define	CACHE_PUBLIC   2
-    { "public", 0 },
-#define	CACHE_ASK      3
-    { "ask", 0 },
-    { NULL, 0 }
-};
 
 extern int errno;
 extern int debugsw;
@@ -40,18 +29,11 @@ extern int endian;	/* mhmisc.c     */
 
 extern pid_t xpid;	/* mhshowsbr.c  */
 
-int checksw = 0;	/* check Content-MD5 field */
-
 /* cache policies */
-int rcachesw = CACHE_ASK;
-int wcachesw = CACHE_ASK;
+extern int rcachesw;	/* mhcachesbr.c */
+extern int wcachesw;	/* mhcachesbr.c */
 
-/*
- * Location of public and private cache.  These must
- * be set before these routines are called.
- */
-char *cache_public;
-char *cache_private;
+int checksw = 0;	/* check Content-MD5 field */
 
 /*
  * Directory to place temp files.  This must
@@ -113,16 +95,13 @@ static struct k2v SubApplication[] = {
     { NULL,           APPLICATION_UNKNOWN }	/* this one must be last! */
 };
 
-/*
- * prototypes
- */
-int pidcheck (int);
-CT parse_mime (char *);
-void cache_content (CT);
 
 /* ftpsbr.c */
 int ftp_get (char *, char *, char *, char *, char *, char *, int, int);
 
+/* mhcachesbr.c */
+int find_cache (CT, int, int *, char *, char *, int);
+
 /* mhmisc.c */
 int part_ok (CT, int);
 int type_ok (CT, int);
@@ -133,15 +112,23 @@ void content_error (char *, CT, char *, ...);
 void free_content (CT);
 void free_encoding (CT, int);
 
+/*
+ * prototypes
+ */
+int pidcheck (int);
+CT parse_mime (char *);
+
 /*
  * static prototypes
  */
 static CT get_content (FILE *, char *, int);
+static int add_header (CT, char *, char *);
 static int get_ctinfo (char *, CT);
 static int get_comment (CT, char **, int);
 static int InitGeneric (CT);
 static int InitText (CT);
 static int InitMultiPart (CT);
+static void reverse_parts (CT);
 static int InitMessage (CT);
 static int params_external (CT, int);
 static int InitApplication (CT);
@@ -161,9 +148,6 @@ static int InitFTP (CT);
 static int openFTP (CT, char **);
 static int InitMail (CT);
 static int openMail (CT, char **);
-static int find_cache (CT, int, int *, char *, char *);
-static int find_cache_aux (int, char *, char *, char *);
-static int find_cache_aux2 (char *, char *, char *);
 static int readDigest (CT, char *);
 
 /*
@@ -221,8 +205,7 @@ pidcheck (int status)
 
     fflush (stdout);
     fflush (stderr);
-    done (1);
-    /* NOTREACHED */
+    return done (1);
 }
 
 
@@ -314,8 +297,11 @@ get_content (FILE *in, char *file, int toplevel)
 {
     int compnum, state;
     char buf[BUFSIZ], name[NAMESZ];
+    char *np, *vp;
     CT ct;
+    HF hp;
 
+    /* allocate the content structure */
     if (!(ct = (CT) calloc (1, sizeof(*ct))))
 	adios (NULL, "out of memory");
 
@@ -324,7 +310,8 @@ get_content (FILE *in, char *file, int toplevel)
     ct->c_begin = ftell (ct->c_fp) + 1;
 
     /*
-     * Read the content headers
+     * Parse the header fields for this
+     * content into a linked list.
      */
     for (compnum = 1, state = FLD;;) {
 	switch (state = m_getfld (state, name, buf, sizeof(buf), in)) {
@@ -333,224 +320,20 @@ get_content (FILE *in, char *file, int toplevel)
 	case FLDEOF:
 	    compnum++;
 
-	    /* Get MIME-Version field */
-	    if (!strcasecmp (name, VRSN_FIELD)) {
-		int ucmp;
-		char c, *cp, *dp;
+	    /* get copies of the buffers */
+	    np = add (name, NULL);
+	    vp = add (buf, NULL);
 
-		cp = add (buf, NULL);
-		while (state == FLDPLUS) {
-		    state = m_getfld (state, name, buf, sizeof(buf), in);
-		    cp = add (buf, cp);
-		}
-
-		if (ct->c_vrsn) {
-		    advise (NULL, "message %s has multiple %s: fields (%s)",
-			    ct->c_file, VRSN_FIELD, dp = trimcpy (cp));
-		    free (dp);
-		    free (cp);
-		    goto out;
-		}
-
-		ct->c_vrsn = cp;
-		while (isspace (*cp))
-		    cp++;
-		for (dp = strchr(cp, '\n'); dp; dp = strchr(dp, '\n'))
-		    *dp++ = ' ';
-		for (dp = cp + strlen (cp) - 1; dp >= cp; dp--)
-		    if (!isspace (*dp))
-			break;
-		*++dp = '\0';
-		if (debugsw)
-		    fprintf (stderr, "%s: %s\n", VRSN_FIELD, cp);
-
-		if (*cp == '(' && get_comment (ct, &cp, 0) == NOTOK)
-		    goto out;
-
-		for (dp = cp; istoken (*dp); dp++)
-		    continue;
-		c = *dp, *dp = '\0';
-		ucmp = !strcasecmp (cp, VRSN_VALUE);
-		*dp = c;
-		if (!ucmp)
-		    admonish (NULL,
-			      "message %s has unknown value for %s: field (%s)",
-			      ct->c_file, VRSN_FIELD, cp);
-		goto got_header;
-	    }
-
-	    /* Get Content-Type field */
-	    if (!strcasecmp (name, TYPE_FIELD)) {
-		char *cp;
-		struct str2init *s2i;
-		CI ci = &ct->c_ctinfo;
-
-		cp = add (buf, NULL);
-		while (state == FLDPLUS) {
-		    state = m_getfld (state, name, buf, sizeof(buf), in);
-		    cp = add (buf, cp);
-		}
-
-		/* Check if we've already seen a Content-Type header */
-		if (ct->c_ctline) {
-		    char *dp = trimcpy (cp);
-
-		    advise (NULL, "message %s has multiple %s: fields (%s)",
-			    ct->c_file, TYPE_FIELD, dp);
-		    free (dp);
-		    free (cp);
-		    goto out;
-		}
-
-		/* Parse the Content-Type field */
-		if (get_ctinfo (cp, ct) == NOTOK)
-		    goto out;
-
-		/*
-		 * Set the Init function and the internal
-		 * flag for this content type.
-		 */
-		for (s2i = str2cts; s2i->si_key; s2i++)
-		    if (!strcasecmp (ci->ci_type, s2i->si_key))
-			break;
-		if (!s2i->si_key && !uprf (ci->ci_type, "X-"))
-		    s2i++;
-		ct->c_type = s2i->si_val;
-		ct->c_ctinitfnx = s2i->si_init;
-		goto got_header;
-	    }
-
-	    /* Get Content-Transfer-Encoding field */
-	    if (!strcasecmp (name, ENCODING_FIELD)) {
-		char *cp, *dp;
-		char c;
-		struct str2init *s2i;
-
-		cp = add (buf, NULL);
-		while (state == FLDPLUS) {
-		    state = m_getfld (state, name, buf, sizeof(buf), in);
-		    cp = add (buf, cp);
-		}
-
-		/*
-		 * Check if we've already seen the
-		 * Content-Transfer-Encoding field
-		 */
-		if (ct->c_celine) {
-		    advise (NULL, "message %s has multiple %s: fields (%s)",
-			    ct->c_file, ENCODING_FIELD, dp = trimcpy (cp));
-		    free (dp);
-		    free (cp);
-		    goto out;
-		}
-
-		ct->c_celine = cp;	/* Save copy of this field */
-		while (isspace (*cp))
-		    cp++;
-		for (dp = cp; istoken (*dp); dp++)
-		    continue;
-		c = *dp;
-		*dp = '\0';
-
-		/*
-		 * Find the internal flag and Init function
-		 * for this transfer encoding.
-		 */
-		for (s2i = str2ces; s2i->si_key; s2i++)
-		    if (!strcasecmp (cp, s2i->si_key))
-			break;
-		if (!s2i->si_key && !uprf (cp, "X-"))
-		    s2i++;
-		*dp = c;
-		ct->c_encoding = s2i->si_val;
-
-		/* Call the Init function for this encoding */
-		if (s2i->si_init && (*s2i->si_init) (ct) == NOTOK)
-		    goto out;
-		goto got_header;
-	    }
-
-	    /* Get Content-ID field */
-	    if (!strcasecmp (name, ID_FIELD)) {
-		ct->c_id = add (buf, ct->c_id);
-		while (state == FLDPLUS) {
-		    state = m_getfld (state, name, buf, sizeof(buf), in);
-		    ct->c_id = add (buf, ct->c_id);
-		}
-		goto got_header;
-	    }
-
-	    /* Get Content-Description field */
-	    if (!strcasecmp (name, DESCR_FIELD)) {
-		ct->c_descr = add (buf, ct->c_descr);
-		while (state == FLDPLUS) {
-		    state = m_getfld (state, name, buf, sizeof(buf), in);
-		    ct->c_descr = add (buf, ct->c_descr);
-		}
-		goto got_header;
-	    }
-
-	    /* Get Content-MD5 field */
-	    if (!strcasecmp (name, MD5_FIELD)) {
-		char *cp, *dp, *ep;
-
-		cp = add (buf, NULL);
-		while (state == FLDPLUS) {
-		    state = m_getfld (state, name, buf, sizeof(buf), in);
-		    cp = add (buf, cp);
-		}
-
-		if (!checksw) {
-		    free (cp);
-		    goto got_header;
-		}
-
-		if (ct->c_digested) {
-		    advise (NULL, "message %s has multiple %s: fields (%s)",
-			    ct->c_file, MD5_FIELD, dp = trimcpy (cp));
-		    free (dp);
-		    free (cp);
-		    goto out;
-		}
-
-		ep = cp;
-		while (isspace (*cp))
-		    cp++;
-		for (dp = strchr(cp, '\n'); dp; dp = strchr(dp, '\n'))
-		    *dp++ = ' ';
-		for (dp = cp + strlen (cp) - 1; dp >= cp; dp--)
-		    if (!isspace (*dp))
-			break;
-		*++dp = '\0';
-		if (debugsw)
-		    fprintf (stderr, "%s: %s\n", MD5_FIELD, cp);
-
-		if (*cp == '(' && get_comment (ct, &cp, 0) == NOTOK) {
-		    free (ep);
-		    goto out;
-		}
-
-		for (dp = cp; *dp && !isspace (*dp); dp++)
-		    continue;
-		*dp = '\0';
-
-		readDigest (ct, cp);
-		free (ep);
-		ct->c_digested++;
-		goto got_header;
-	    }
-
-#if 0
-	    if (uprf (name, XXX_FIELD_PRF))
-		advise (NULL, "unknown field (%s) in message %s",
-			name, ct->c_file);
-	    /* and fall... */
-#endif
-
-	    while (state == FLDPLUS)
+	    /* if necessary, get rest of field */
+	    while (state == FLDPLUS) {
 		state = m_getfld (state, name, buf, sizeof(buf), in);
+		vp = add (buf, vp);	/* add to previous value */
+	    }
 
-got_header:
+	    /* Now add the header data to the list */
+	    add_header (ct, np, vp);
+
+	    /* continue, if this isn't the last header field */
 	    if (state != FLDEOF) {
 		ct->c_begin = ftell (in) + 1;
 		continue;
@@ -573,9 +356,181 @@ got_header:
 	default:
 	    adios (NULL, "getfld() returned %d", state);
 	}
+
+	/* break out of the loop */
 	break;
     }
 
+    /*
+     * Read the content headers.  We will parse the
+     * MIME related header fields into their various
+     * structures and set internal flags related to
+     * content type/subtype, etc.
+     */
+
+    hp = ct->c_first_hf;	/* start at first header field */
+    while (hp) {
+	/* Get MIME-Version field */
+	if (!strcasecmp (hp->name, VRSN_FIELD)) {
+	    int ucmp;
+	    char c, *cp, *dp;
+
+	    if (ct->c_vrsn) {
+		advise (NULL, "message %s has multiple %s: fields",
+			ct->c_file, VRSN_FIELD);
+		goto next_header;
+	    }
+	    ct->c_vrsn = add (hp->value, NULL);
+
+	    /* Now, cleanup this field */
+	    cp = ct->c_vrsn;
+
+	    while (isspace (*cp))
+		cp++;
+	    for (dp = strchr(cp, '\n'); dp; dp = strchr(dp, '\n'))
+		*dp++ = ' ';
+	    for (dp = cp + strlen (cp) - 1; dp >= cp; dp--)
+		if (!isspace (*dp))
+		    break;
+	    *++dp = '\0';
+	    if (debugsw)
+		fprintf (stderr, "%s: %s\n", VRSN_FIELD, cp);
+
+	    if (*cp == '(' && get_comment (ct, &cp, 0) == NOTOK)
+		goto out;
+
+	    for (dp = cp; istoken (*dp); dp++)
+		continue;
+	    c = *dp;
+	    *dp = '\0';
+	    ucmp = !strcasecmp (cp, VRSN_VALUE);
+	    *dp = c;
+	    if (!ucmp) {
+		admonish (NULL, "message %s has unknown value for %s: field (%s)",
+		ct->c_file, VRSN_FIELD, cp);
+	    }
+	}
+	else if (!strcasecmp (hp->name, TYPE_FIELD)) {
+	/* Get Content-Type field */
+	    struct str2init *s2i;
+	    CI ci = &ct->c_ctinfo;
+
+	    /* Check if we've already seen a Content-Type header */
+	    if (ct->c_ctline) {
+		advise (NULL, "message %s has multiple %s: fields",
+			ct->c_file, TYPE_FIELD);
+		goto next_header;
+	    }
+
+	    /* Parse the Content-Type field */
+	    if (get_ctinfo (hp->value, ct) == NOTOK)
+		goto out;
+
+	    /*
+	     * Set the Init function and the internal
+	     * flag for this content type.
+	     */
+	    for (s2i = str2cts; s2i->si_key; s2i++)
+		if (!strcasecmp (ci->ci_type, s2i->si_key))
+		    break;
+	    if (!s2i->si_key && !uprf (ci->ci_type, "X-"))
+		s2i++;
+	    ct->c_type = s2i->si_val;
+	    ct->c_ctinitfnx = s2i->si_init;
+	}
+	else if (!strcasecmp (hp->name, ENCODING_FIELD)) {
+	/* Get Content-Transfer-Encoding field */
+	    char c, *cp, *dp;
+	    struct str2init *s2i;
+
+	    /*
+	     * Check if we've already seen the
+	     * Content-Transfer-Encoding field
+	     */
+	    if (ct->c_celine) {
+		advise (NULL, "message %s has multiple %s: fields",
+			ct->c_file, ENCODING_FIELD);
+		goto next_header;
+	    }
+
+	    /* get copy of this field */
+	    ct->c_celine = cp = add (hp->value, NULL);
+
+	    while (isspace (*cp))
+		cp++;
+	    for (dp = cp; istoken (*dp); dp++)
+		continue;
+	    c = *dp;
+	    *dp = '\0';
+
+	    /*
+	     * Find the internal flag and Init function
+	     * for this transfer encoding.
+	     */
+	    for (s2i = str2ces; s2i->si_key; s2i++)
+		if (!strcasecmp (cp, s2i->si_key))
+		    break;
+	    if (!s2i->si_key && !uprf (cp, "X-"))
+		s2i++;
+	    *dp = c;
+	    ct->c_encoding = s2i->si_val;
+
+	    /* Call the Init function for this encoding */
+	    if (s2i->si_init && (*s2i->si_init) (ct) == NOTOK)
+		goto out;
+	}
+	else if (!strcasecmp (hp->name, MD5_FIELD)) {
+	/* Get Content-MD5 field */
+	    char *cp, *dp, *ep;
+
+	    if (!checksw)
+		goto next_header;
+
+	    if (ct->c_digested) {
+		advise (NULL, "message %s has multiple %s: fields",
+			ct->c_file, MD5_FIELD);
+		goto next_header;
+	    }
+
+	    ep = cp = add (hp->value, NULL);	/* get a copy */
+
+	    while (isspace (*cp))
+		cp++;
+	    for (dp = strchr(cp, '\n'); dp; dp = strchr(dp, '\n'))
+		*dp++ = ' ';
+	    for (dp = cp + strlen (cp) - 1; dp >= cp; dp--)
+		if (!isspace (*dp))
+		    break;
+	    *++dp = '\0';
+	    if (debugsw)
+		fprintf (stderr, "%s: %s\n", MD5_FIELD, cp);
+
+	    if (*cp == '(' && get_comment (ct, &cp, 0) == NOTOK) {
+		free (ep);
+		goto out;
+	    }
+
+	    for (dp = cp; *dp && !isspace (*dp); dp++)
+		continue;
+	    *dp = '\0';
+
+	    readDigest (ct, cp);
+	    free (ep);
+	    ct->c_digested++;
+	}
+	else if (!strcasecmp (hp->name, ID_FIELD)) {
+	/* Get Content-ID field */
+	    ct->c_id = add (hp->value, ct->c_id);
+	}
+	else if (!strcasecmp (hp->name, DESCR_FIELD)) {
+	/* Get Content-Description field */
+	    ct->c_descr = add (hp->value, ct->c_descr);
+	}
+
+next_header:
+	hp = hp->next;	/* next header field */
+    }
+
     /*
      * Check if we saw a Content-Type field.
      * If not, then assign a default value for
@@ -616,6 +571,37 @@ out:
 }
 
 
+/*
+ * small routine to add header field to list
+ */
+
+static int
+add_header (CT ct, char *name, char *value)
+{
+    HF hp;
+
+    /* allocate header field structure */
+    if (!(hp = malloc (sizeof(*hp))))
+	adios (NULL, "out of memory");
+
+    /* link data into header structure */
+    hp->name = name;
+    hp->value = value;
+    hp->next = NULL;
+
+    /* link header structure into the list */
+    if (ct->c_first_hf == NULL) {
+	ct->c_first_hf = hp;		/* this is the first */
+	ct->c_last_hf = hp;
+    } else {
+	ct->c_last_hf->next = hp;	/* add it to the end */
+	ct->c_last_hf = hp;
+    }
+
+    return 0;
+}
+
+
 /*
  * Parse Content-Type line and fill in the
  * information of the CTinfo structure.
@@ -742,9 +728,11 @@ magic_skip:
 	    return OK;
 	}
 
+	/* down case the attribute name */
 	for (dp = cp; istoken (*dp); dp++)
 	    if (isalpha(*dp) && isupper (*dp))
 		*dp = tolower (*dp);
+
 	for (up = dp; isspace (*dp);)
 	    dp++;
 	if (dp == cp || *dp != '=') {
@@ -758,7 +746,10 @@ magic_skip:
 	*vp = '\0';
 	for (dp++; isspace (*dp);)
 	    dp++;
+
+	/* now add the attribute value */
 	ci->ci_values[ap - ci->ci_attrs] = vp = *ap + (dp - cp);
+
 	if (*dp == '"') {
 	    for (cp = ++dp, dp = vp;;) {
 		switch (c = *cp++) {
@@ -882,7 +873,8 @@ invalid:
 /*
  * CONTENTS
  *
- * Handles content types audio, image, and video
+ * Handles content types audio, image, and video.
+ * There's not much to do right here.
  */
 
 static int
@@ -943,7 +935,7 @@ InitText (CT ct)
      * terminal or display method.
      */
     if (!check_charset (chset, strlen (chset))) {
-	sprintf (buffer, "%s-charset-%s", invo_name, chset);
+	snprintf (buffer, sizeof(buffer), "%s-charset-%s", invo_name, chset);
 	if ((cp = context_find (buffer)))
 	    ct->c_termproc = getcpy (cp);
     }
@@ -998,17 +990,21 @@ InitMultiPart (CT ct)
 	    break;
 	}
     }
+
+    /* complain if boundary parameter is missing */
     if (!*ap) {
 	advise (NULL,
 		"a \"boundary\" parameter is mandatory for \"%s/%s\" type in message %s's %s: field",
 		ci->ci_type, ci->ci_subtype, ct->c_file, TYPE_FIELD);
 	return NOTOK;
     }
-    
+
+    /* allocate primary structure for multipart info */
     if ((m = (struct multipart *) calloc (1, sizeof(*m))) == NULL)
 	adios (NULL, "out of memory");
     ct->c_ctparams = (void *) m;
 
+    /* check if boundary parameter contains only whitespace characters */
     for (cp = bp; isspace (*cp); cp++)
 	continue;
     if (!*cp) {
@@ -1016,10 +1012,14 @@ InitMultiPart (CT ct)
 		ci->ci_type, ci->ci_subtype, ct->c_file, TYPE_FIELD);
 	return NOTOK;
     }
+
+    /* remove trailing whitespace from boundary parameter */
     for (cp = bp, dp = cp + strlen (cp) - 1; dp > cp; dp--)
 	if (!isspace (*dp))
 	    break;
     *++dp = '\0';
+
+    /* record boundary separators */
     m->mp_start = concat (bp, "\n", NULL);
     m->mp_stop = concat (bp, "--\n", NULL);
 
@@ -1085,8 +1085,7 @@ end_part:
 	p->c_end = ct->c_end;
 
 	if (p->c_begin >= p->c_end) {
-	    for (next = &m->mp_parts;
-		     *next != part;
+	    for (next = &m->mp_parts; *next != part;
 		     next = &((*next)->mp_next))
 		continue;
 	    *next = NULL;
@@ -1096,38 +1095,21 @@ end_part:
     }
 
 last_part:
-    if (ct->c_subtype == MULTI_ALTERNATE && m->mp_parts->mp_next) {
-	int i;
-	struct part **base, **bmp;
-
-	i = 0;
-	for (part = m->mp_parts; part; part = part->mp_next)
-	    i++;
-	if ((base = (struct part **) calloc ((size_t) (i + 1), sizeof(*base)))
-	        == NULL)
-	    adios (NULL, "out of memory");
-	bmp = base;
-	for (part = m->mp_parts; part; part = part->mp_next)
-	    *bmp++ = part;
-	*bmp = NULL;
-
-	next = &m->mp_parts;
-	for (bmp--; bmp >= base; bmp--) {
-	    part = *bmp;
-	    *next = part, next = &part->mp_next;
-	}
-	*next = NULL;
-
-	free ((char *) base);
-    }
+    /* reverse the order of the parts for multipart/alternative */
+    if (ct->c_subtype == MULTI_ALTERNATE)
+	reverse_parts (ct);
 
+    /*
+     * label all subparts with part number, and
+     * then initialize the content of the subpart.
+     */
     {
 	int partnum;
 	char *pp;
 	char partnam[BUFSIZ];
 
 	if (ct->c_partno) {
-	    sprintf (partnam, "%s.", ct->c_partno);
+	    snprintf (partnam, sizeof(partnum), "%s.", ct->c_partno);
 	    pp = partnam + strlen (partnam);
 	} else {
 	    pp = partnam;
@@ -1140,6 +1122,7 @@ last_part:
 	    sprintf (pp, "%d", partnum);
 	    p->c_partno = add (partnam, NULL);
 
+	    /* initialize the content of the subparts */
 	    if (p->c_ctinitfnx && (*p->c_ctinitfnx) (p) == NOTOK) {
 		fclose (ct->c_fp);
 		ct->c_fp = NULL;
@@ -1154,6 +1137,52 @@ last_part:
 }
 
 
+/*
+ * reverse the order of the parts of a multipart
+ */
+
+static void
+reverse_parts (CT ct)
+{
+    int i;
+    struct multipart *m;
+    struct part **base, **bmp, **next, *part;
+
+    m = (struct multipart *) ct->c_ctparams;
+
+    /* if only one part, just return */
+    if (!m->mp_parts || !m->mp_parts->mp_next)
+	return;
+
+    /* count number of parts */
+    i = 0;
+    for (part = m->mp_parts; part; part = part->mp_next)
+	i++;
+
+    /* allocate array of pointers to the parts */
+    if (!(base = (struct part **) calloc ((size_t) (i + 1), sizeof(*base))))
+	adios (NULL, "out of memory");
+    bmp = base;
+
+    /* point at all the parts */
+    for (part = m->mp_parts; part; part = part->mp_next)
+	*bmp++ = part;
+    *bmp = NULL;
+
+    /* reverse the order of the parts */
+    next = &m->mp_parts;
+    for (bmp--; bmp >= base; bmp--) {
+	part = *bmp;
+	*next = part;
+	next = &part->mp_next;
+    }
+    *next = NULL;
+
+    /* free array of pointers */
+    free ((char *) base);
+}
+
+
 /*
  * MESSAGE
  */
@@ -1164,9 +1193,9 @@ InitMessage (CT ct)
     struct k2v *kv;
     CI ci = &ct->c_ctinfo;
 
-    if (ct->c_encoding != CE_7BIT) {
+    if ((ct->c_encoding != CE_7BIT) && (ct->c_encoding != CE_8BIT)) {
 	admonish (NULL,
-		  "\"%s/%s\" type in message %s should be encoded in 7bit",
+		  "\"%s/%s\" type in message %s should be encoded in 7bit or 8bit",
 		  ci->ci_type, ci->ci_subtype, ct->c_file);
 	return NOTOK;
     }
@@ -1194,6 +1223,7 @@ InitMessage (CT ct)
 		    adios (NULL, "out of memory");
 		ct->c_ctparams = (void *) p;
 
+		/* scan for parameters "id", "number", and "total" */
 		for (ap = ci->ci_attrs, ep = ci->ci_values; *ap; ap++, ep++) {
 		    if (!strcasecmp (*ap, "id")) {
 			p->pm_partid = add (*ep, NULL);
@@ -1609,12 +1639,14 @@ openBase64 (CT ct, char **file)
 		default:
 		    if (isspace (*cp))
 			break;
-		    if (skip
-			|| (*cp & 0x80)
+		    if (skip || (*cp & 0x80)
 			|| (value = b642nib[*cp & 0x7f]) > 0x3f) {
-			if (debugsw)
-			    fprintf (stderr, "*cp=0x%x pos=%ld skip=%d\n", *cp,
-				     lseek (fd, (off_t)0, SEEK_CUR) - (ep - cp), skip);
+			if (debugsw) {
+			    fprintf (stderr, "*cp=0x%x pos=%ld skip=%d\n",
+				*cp,
+				(long) (lseek (fd, (off_t) 0, SEEK_CUR) - (ep - cp)),
+				skip);
+			}
 			content_error (NULL, ct,
 				       "invalid BASE64 encoding -- continuing");
 			continue;
@@ -1974,7 +2006,7 @@ open7Bit (CT ct, char **file)
 	    putc (';', ce->ce_fp);
 	    len++;
 
-	    sprintf (buffer, "%s=\"%s\"", *ap, *ep);
+	    snprintf (buffer, sizeof(buffer), "%s=\"%s\"", *ap, *ep);
 
 	    if (len + 1 + (cc = strlen (buffer)) >= CPERLIN) {
 		fputs ("\n\t", ce->ce_fp);
@@ -2079,7 +2111,8 @@ openExternal (CT ct, CT cb, CE ce, char **file, int *fd)
 	goto ready_already;
     }
 
-    if (find_cache (ct, rcachesw, (int *) 0, cb->c_id, cachefile) != NOTOK) {
+    if (find_cache (ct, rcachesw, (int *) 0, cb->c_id,
+		cachefile, sizeof(cachefile)) != NOTOK) {
 	if ((ce->ce_fp = fopen (cachefile, "r"))) {
 	    ce->ce_file = getcpy (cachefile);
 	    ce->ce_unlink = 0;
@@ -2141,8 +2174,8 @@ openFile (CT ct, char **file)
     }
 
     if ((!e->eb_permission || strcasecmp (e->eb_permission, "read-write"))
-	    && find_cache (NULL, wcachesw, &cachetype,
-			   e->eb_content->c_id, cachefile) != NOTOK) {
+	    && find_cache (NULL, wcachesw, &cachetype, e->eb_content->c_id,
+		cachefile, sizeof(cachefile)) != NOTOK) {
 	int mask;
 	FILE *fp;
 
@@ -2193,6 +2226,7 @@ static int
 openFTP (CT ct, char **file)
 {
     int	cachetype, caching, fd;
+    int len, buflen;
     char *bp, *ftp, *user, *pass;
     char buffer[BUFSIZ], cachefile[BUFSIZ];
     struct exbody *e;
@@ -2203,8 +2237,7 @@ openFTP (CT ct, char **file)
     e  = ct->c_ctexbody;
     ce = ct->c_cefile;
 
-    sprintf (buffer, "%s-access-ftp", invo_name);
-    if ((ftp = context_find (buffer)) && !*ftp)
+    if ((ftp = context_find (nmhaccessftp)) && !*ftp)
 	ftp = NULL;
 
 #ifndef BUILTIN_FTP
@@ -2236,27 +2269,49 @@ openFTP (CT ct, char **file)
 	xpid = 0;
     }
 
+    /* Get the buffer ready to go */
     bp = buffer;
-    sprintf (bp, "Retrieve %s", e->eb_name);
-    bp += strlen (bp);
+    buflen = sizeof(buffer);
+
+    /*
+     * Construct the query message for user
+     */
+    snprintf (bp, buflen, "Retrieve %s", e->eb_name);
+    len = strlen (bp);
+    bp += len;
+    buflen -= len;
+
     if (e->eb_partno) {
-	sprintf (bp, " (content %s)", e->eb_partno);
-	bp += strlen (bp);
+	snprintf (bp, buflen, " (content %s)", e->eb_partno);
+	len = strlen (bp);
+	bp += len;
+	buflen -= len;
     }
-    sprintf (bp, "\n    using %sFTP from site %s",
+
+    snprintf (bp, buflen, "\n    using %sFTP from site %s",
 		    e->eb_flags ? "anonymous " : "", e->eb_site);
-    bp += strlen (bp);
+    len = strlen (bp);
+    bp += len;
+    buflen -= len;
+
     if (e->eb_size > 0) {
-	sprintf (bp, " (%lu octets)", e->eb_size);
-	bp += strlen (bp);
+	snprintf (bp, buflen, " (%lu octets)", e->eb_size);
+	len = strlen (bp);
+	bp += len;
+	buflen -= len;
     }
-    sprintf (bp, "? ");
+    snprintf (bp, buflen, "? ");
+
+    /*
+     * Now, check the answer
+     */
     if (!getanswer (buffer))
 	return NOTOK;
 
     if (e->eb_flags) {
 	user = "anonymous";
-	sprintf (pass = buffer, "%s@%s", getusername (), LocalName ());
+	snprintf (buffer, sizeof(buffer), "%s@%s", getusername (), LocalName ());
+	pass = buffer;
     } else {
 	ruserpass (e->eb_site, &username, &password);
 	user = username;
@@ -2267,8 +2322,8 @@ openFTP (CT ct, char **file)
     caching = 0;
     cachefile[0] = '\0';
     if ((!e->eb_permission || strcasecmp (e->eb_permission, "read-write"))
-	    && find_cache (NULL, wcachesw, &cachetype,
-			   e->eb_content->c_id, cachefile) != NOTOK) {
+	    && find_cache (NULL, wcachesw, &cachetype, e->eb_content->c_id,
+		cachefile, sizeof(cachefile)) != NOTOK) {
 	if (*file == NULL) {
 	    ce->ce_unlink = 0;
 	    caching = 1;
@@ -2344,7 +2399,7 @@ losing_ftp:
 	    goto losing_ftp;
 #endif
 
-    if (cachefile[0])
+    if (cachefile[0]) {
 	if (caching)
 	    chmod (cachefile, cachetype ? m_gmprot () : 0444);
 	else {
@@ -2358,7 +2413,7 @@ losing_ftp:
 
 		fseek (gp, 0L, SEEK_SET);
 
-		while ((cc = fread (buffer, sizeof(*buffer), sizeof(buffer), gp))
+		while ((cc= fread (buffer, sizeof(*buffer), sizeof(buffer), gp))
 		           > 0)
 		    fwrite (buffer, sizeof(*buffer), cc, fp);
 		fflush (fp);
@@ -2376,6 +2431,7 @@ losing_ftp:
 	    }
 	    umask (mask);
 	}
+    }
 
     fseek (ce->ce_fp, 0L, SEEK_SET);
     *file = ce->ce_file;
@@ -2398,6 +2454,7 @@ static int
 openMail (CT ct, char **file)
 {
     int	child_id, fd, i, vecp;
+    int len, buflen;
     char *bp, buffer[BUFSIZ], *vec[7];
     struct exbody *e = ct->c_ctexbody;
     CE ce = ct->c_cefile;
@@ -2425,16 +2482,28 @@ openMail (CT ct, char **file)
 	xpid = 0;
     }
 
+    /* Get buffer ready to go */
     bp = buffer;
-    sprintf (bp, "Retrieve content");
-    bp += strlen (bp);
+    buflen = sizeof(buffer);
+
+    /* Now, construct query message */
+    snprintf (bp, buflen, "Retrieve content");
+    len = strlen (bp);
+    bp += len;
+    buflen -= len;
+
     if (e->eb_partno) {
-	sprintf (bp, " %s", e->eb_partno);
-	bp += strlen (bp);
+	snprintf (bp, buflen, " %s", e->eb_partno);
+	len = strlen (bp);
+	bp += len;
+	buflen -= len;
     }
-    sprintf (bp, " by asking %s\n\n%s\n? ",
+
+    snprintf (bp, buflen, " by asking %s\n\n%s\n? ",
 		    e->eb_server,
 		    e->eb_subject ? e->eb_subject : e->eb_body);
+
+    /* Now, check answer */
     if (!getanswer (buffer))
 	return NOTOK;
 
@@ -2489,255 +2558,6 @@ openMail (CT ct, char **file)
     return fileno (ce->ce_fp);
 }
 
-/*
- * CACHE
- */
-
-static int
-find_cache (CT ct, int policy, int *writing, char *id, char *buffer)
-{
-    int	status = NOTOK;
-
-    if (id == NULL)
-	return NOTOK;
-    id = trimcpy (id);
-
-    if (debugsw)
-	fprintf (stderr, "find_cache %s(%d) %s %s\n",
-		 caches[policy].sw, policy, writing ? "writing" : "reading",
-		 id);
-
-    switch (policy) {
-	case CACHE_NEVER:
-	default:
-	    break;
-
-	case CACHE_ASK:
-	case CACHE_PUBLIC:
-	    if (cache_private
-		    && !writing
-		    && find_cache_aux (writing ? 2 : 0, cache_private, id,
-				       buffer) == OK) {
-		if (access (buffer, R_OK) != NOTOK) {
-got_private:
-		    if (writing)
-			*writing = 1;
-got_it:
-		    status = OK;
-		    break;
-		}
-	    }
-	    if (cache_public
-		    && find_cache_aux (writing ? 1 : 0, cache_public, id,
-				       buffer) == OK) {
-		if (writing || access (buffer, R_OK) != NOTOK) {
-		    if (writing)
-			*writing = 0;
-		    goto got_it;
-		}
-	    }
-	    break;
-
-	case CACHE_PRIVATE:
-	    if (cache_private
-		    && find_cache_aux (writing ? 2 : 0, cache_private, id,
-				       buffer) == OK) {
-		if (writing || access (buffer, R_OK) != NOTOK)
-		    goto got_private;
-	    }
-	    break;
-
-    }
-
-    if (status == OK && policy == CACHE_ASK) {
-	char   *bp,
-		query[BUFSIZ];
-
-	if (xpid) {
-	    if (xpid < 0)
-		xpid = -xpid;
-	    pidcheck (pidwait (xpid, NOTOK));
-	    xpid = 0;
-	}
-
-	bp = query;
-	if (writing)
-	    sprintf (bp, "Make cached, publically-accessible copy");
-	else {
-	    struct stat st;
-
-	    sprintf (bp, "Use cached copy");
-	    bp += strlen (bp);
-	    if (ct->c_partno) {
-		sprintf (bp, " of content %s", ct->c_partno);
-		bp += strlen (bp);
-	    }
-	    stat (buffer, &st);
-	    sprintf (bp, " (size %lu octets)",
-			    (unsigned long) st.st_size);
-	}
-	bp += strlen (bp);
-	sprintf (bp, "\n    in file %s? ", buffer);
-	if (!getanswer (query))
-	    status = NOTOK;
-    }
-    if (status == OK && writing) {
-	if (*writing && strchr(buffer, '/'))
-	    make_intermediates (buffer);
-	unlink (buffer);
-    }
-
-    free (id);
-    return status;
-}
-
-
-static int
-find_cache_aux (int writing, char *directory, char *id, char *buffer)
-{
-    int	mask, usemap;
-    char mapfile[BUFSIZ], mapname[BUFSIZ];
-    FILE *fp;
-    static int partno, pid;
-    static time_t clock = 0;
-
-#ifdef BSD42
-    usemap = strchr(id, '/') ? 1 : 0;
-#else
-    usemap = 1;
-#endif
-
-    if (debugsw)
-	fprintf (stderr, "find_cache_aux %s usemap=%d\n", directory, usemap);
-
-    sprintf (mapfile, "%s/cache.map", directory);
-    if (find_cache_aux2 (mapfile, id, mapname) == OK)
-	goto done_map;
-
-    if (!writing) {
-	if (usemap)
-	    return NOTOK;
-
-use_raw:
-	sprintf (buffer, "%s/%s", directory, id);
-	return OK;
-    }
-
-    if (!usemap && access (mapfile, W_OK) == NOTOK)
-	goto use_raw;
-
-    if (clock != 0) {
-	time_t now;
-	
-	time (&now);
-	if (now > clock)
-	    clock = 0;
-    }
-    else
-	pid = getpid ();
-    if (clock == 0) {
-	time (&clock);
-	partno = 0;
-    }
-    else
-	if (partno > 0xff) {
-	    clock++;
-	    partno = 0;
-	}
-
-    sprintf (mapname, "%08x%04x%02x",
-		(unsigned int) (clock & 0xffffffff),
-		(unsigned int) (pid & 0xffff),
-		(unsigned int) (partno++ & 0xff));
-
-    if (debugsw)
-	fprintf (stderr, "creating mapping %s->%s\n", mapname, id);
-
-    make_intermediates (mapfile);
-    mask = umask (writing == 2 ? 0077 : 0);
-    if (!(fp = lkfopen (mapfile, "a")) && errno == ENOENT) {
-	int fd;
-
-	if ((fd = creat (mapfile, 0666)) != NOTOK) {
-	    close (fd);
-	    fp = lkfopen (mapfile, "a");
-	}
-    }
-    umask (mask);
-    if (!fp)
-	return NOTOK;
-    fprintf (fp, "%s: %s\n", mapname, id);
-    lkfclose (fp, mapfile);
-
-done_map:
-    if (*mapname == '/')
-	strcpy (buffer, mapname);
-    else
-	sprintf (buffer, "%s/%s", directory, mapname);
-    if (debugsw)
-	fprintf (stderr, "use %s\n", buffer);
-
-    return OK;
-}
-
-
-static int
-find_cache_aux2 (char *mapfile, char *id, char *mapname)
-{
-    int	state;
-    char buf[BUFSIZ], name[NAMESZ];
-    FILE *fp;
-
-    if (!(fp = lkfopen (mapfile, "r")))
-	return NOTOK;
-
-    for (state = FLD;;) {
-	int result;
-	char *cp, *dp;
-
-	switch (state = m_getfld (state, name, buf, sizeof(buf), fp)) {
-	    case FLD:
-	    case FLDPLUS:
-	    case FLDEOF:
-	        strcpy (mapname, name);
-		if (state != FLDPLUS)
-		    cp = buf;
-		else {
-		    cp = add (buf, NULL);
-		    while (state == FLDPLUS) {
-			state = m_getfld (state, name, buf, sizeof(buf), fp);
-			cp = add (buf, cp);
-		    }
-		}
-		dp = trimcpy (cp);
-		if (cp != buf)
-		    free (cp);
-		if (debugsw)
-		    fprintf (stderr, "compare %s to %s <- %s\n", id, dp,
-			     mapname);
-		result = strcmp (id, dp);
-		free (dp);
-		if (result == 0) {
-		    lkfclose (fp, mapfile);
-		    return OK;
-		}
-		if (state != FLDEOF)
-		    continue;
-		/* else fall... */
-
-	    case BODY:
-	    case BODYEOF:
-	    case FILEEOF:
-	    default:
-		break;
-	}
-	break;
-    }
-
-    lkfclose (fp, mapfile);
-    return NOTOK;
-}
-
 
 static int
 readDigest (CT ct, char *cp)
@@ -2818,89 +2638,3 @@ invalid_digest:
 
     return OK;
 }
-
-
-/*
- * Entry point to cache content from external sources.
- */
-
-void
-cache_content (CT ct)
-{
-    int	cachetype;
-    char *file, cachefile[BUFSIZ];
-    CE ce = ct->c_cefile;
-
-    if (!ct->c_id) {
-	advise (NULL, "no %s: field in %s", ID_FIELD, ct->c_file);
-	return;
-    }
-
-    if (!ce) {
-	advise (NULL, "unable to decode %s", ct->c_file);
-	return;
-    }
-
-    if (ct->c_ceopenfnx == openMail) {
-	advise (NULL, "a radish may no know Greek, but I do...");
-	return;
-    }
-
-    if (find_cache (NULL, wcachesw != CACHE_NEVER ? wcachesw : CACHE_ASK,
-		    &cachetype, ct->c_id, cachefile)
-	    == NOTOK) {
-	advise (NULL, "unable to cache %s's contents", ct->c_file);
-	return;
-    }
-    if (wcachesw != CACHE_NEVER && wcachesw != CACHE_ASK) {
-	fflush (stdout);
-	fprintf (stderr, "caching message %s as file %s\n", ct->c_file,
-		 cachefile);
-    }
-
-    if (ce->ce_file) {
-	int mask = umask (cachetype ? ~m_gmprot () : 0222);
-	FILE *fp;
-
-	if (debugsw)
-	    fprintf (stderr, "caching by copying %s...\n", ce->ce_file);
-
-	file = NULL;
-	if ((*ct->c_ceopenfnx) (ct, &file) == NOTOK)
-	    goto reset_umask;
-
-	if ((fp = fopen (cachefile, "w"))) {
-	    int	cc;
-	    char buffer[BUFSIZ];
-	    FILE *gp = ce->ce_fp;
-
-	    fseek (gp, 0L, SEEK_SET);
-
-	    while ((cc = fread (buffer, sizeof(*buffer), sizeof(buffer), gp))
-		       > 0)
-		fwrite (buffer, sizeof(*buffer), cc, fp);
-	    fflush (fp);
-
-	    if (ferror (gp)) {
-		admonish (ce->ce_file, "error reading");
-		unlink (cachefile);
-	    } else {
-		if (ferror (fp)) {
-		    admonish (cachefile, "error writing");
-		    unlink (cachefile);
-		}
-	    }
-	    fclose (fp);
-	} else
-	    content_error (cachefile, ct, "unable to fopen for writing");
-reset_umask:
-	umask (mask);
-    } else {
-	if (debugsw)
-	    fprintf (stderr, "in place caching...\n");
-
-	file = cachefile;
-	if ((*ct->c_ceopenfnx) (ct, &file) != NOTOK)
-	    chmod (cachefile, cachetype ? m_gmprot () : 0444);
-    }
-}
