@@ -2,7 +2,7 @@
 /*
  * mhpath.c -- print full pathnames of nmh messages and folders
  *
- * $Id$
+ * $Id: mhpath.c,v 1.2 1999/07/16 01:29:46 danh Exp $
  */
 
 #include <h/mh.h>
@@ -17,40 +17,45 @@ static struct swit switches[] = {
 
 /*
  * Add space for message/sequence names
- * by NUMMSGS at a time.
+ * by MAXMSGS at a time.
  */
-#define NUMMSGS 512
+#define MAXMSGS 256
 
 
 int
 main(int argc, char **argv)
 {
-    int i, maxmsgs, msgp = 0;
+    int i, maxmsgs, nummsgs;
     char *cp, *maildir, *folder = NULL;
-    char **ap, **argp, **msgs;
-    char *arguments[MAXARGS], buf[100];
+    char **argp, **msgs;
+    char **arguments, buf[BUFSIZ];
     struct msgs *mp;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name)) != NULL) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
-    /* Allocate initial space to record message/sequence names */
-    maxmsgs = NUMMSGS;
-    if ((msgs = malloc (maxmsgs * sizeof(char *))) == NULL)
-	adios (NULL, "unable to allocate storage for message/sequence names");
+    /*
+     * Allocate initial space to record message
+     * and sequence names
+     */
+    nummsgs = 0;
+    maxmsgs = MAXMSGS;
+    if (!(msgs = (char **) malloc ((size_t) (maxmsgs * sizeof(*msgs)))))
+	adios (NULL, "unable to allocate storage");
 
+    /*
+     * Parse arguments
+     */
     while ((cp = *argp++)) {
-	if (*cp == '-')
+	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
 		case AMBIGSW: 
 		    ambigsw (cp, switches);
@@ -59,26 +64,32 @@ main(int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [+folder] [msgs] [switches]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
 		    print_version(invo_name);
 		    done (1);
 	    }
+	}
 	if (*cp == '+' || *cp == '@') {
 	    if (folder)
 		adios (NULL, "only one folder at a time!");
 	    else
 		folder = path (cp + 1, *cp == '+' ? TFOLDER : TSUBCWF);
 	} else {
-	    /* if necessary, reallocate space for message/sequence names */
-	    if (msgp >= maxmsgs) {
-		maxmsgs += NUMMSGS;
-		if ((msgs = realloc (msgs, maxmsgs * sizeof(char *))) == NULL)
-		    adios (NULL, "unable to re-allocate storage for message names");
+	    /*
+	     * if necessary, reallocate space for
+	     * message/sequence names
+	     */
+	    if (nummsgs >= maxmsgs) {
+		maxmsgs += MAXMSGS;
+		if (!(msgs = (char **) realloc (msgs,
+			(size_t) (maxmsgs * sizeof(*msgs)))))
+		    adios (NULL, "unable to reallocate msgs storage ");
 	    }
-	    msgs[msgp++] = cp;
+	    msgs[nummsgs++] = cp;
 	}
     }
 
@@ -90,7 +101,7 @@ main(int argc, char **argv)
     maildir = m_maildir (folder);
 
     /* If no messages are given, print folder pathname */
-    if (!msgp) {
+    if (!nummsgs) {
 	printf ("%s\n", maildir);
 	done (0);
     }
@@ -119,7 +130,7 @@ main(int argc, char **argv)
     mp->msgflags |= ALLOW_NEW;	/* allow the "new" sequence */
 
     /* parse all the message ranges/sequences and set SELECTED */
-    for (i = 0; i < msgp; i++)
+    for (i = 0; i < nummsgs; i++)
 	if (!m_convert (mp, msgs[i]))
 	    done (1);
 
@@ -133,5 +144,5 @@ main(int argc, char **argv)
     seq_save (mp);	/* synchronize message sequences */
     context_save ();	/* save the context file         */
     folder_free (mp);	/* free folder/message structure */
-    done (0);
+    return done (0);
 }
