@@ -2,7 +2,7 @@
 /*
  * mhshowsbr.c -- routines to display the contents of MIME messages
  *
- * $Id$
+ * $Id: mhshowsbr.c,v 1.3 1999/09/26 17:45:56 ruud Exp $
  */
 
 #include <h/mh.h>
@@ -14,7 +14,7 @@
 #include <signal.h>
 #include <zotnet/mts/mts.h>
 #include <zotnet/tws/tws.h>
-#include <h/mhn.h>
+#include <h/mime.h>
 #include <h/mhparse.h>
 
 #ifdef HAVE_SYS_WAIT_H
@@ -48,10 +48,11 @@ pid_t xpid = 0;
 
 static sigjmp_buf intrenv;
 
-/*
- * prototypes
- */
-int SOprintf (char *, ...);   /* from termsbr.c */
+
+/* termsbr.c */
+int SOprintf (char *, ...);
+
+/* mhparse.c */
 int pidcheck (int);
 
 /* mhmisc.c */
@@ -64,6 +65,9 @@ void flush_errors (void);
 int list_switch (CT, int, int, int, int);
 int list_content (CT, int, int, int, int);
 
+/*
+ * prototypes
+ */
 void show_all_messages (CT *);
 int show_content_aux (CT, int, int, char *, char *);
 
@@ -293,12 +297,13 @@ show_content (CT ct, int serial, int alternate)
     CI ci = &ct->c_ctinfo;
 
     /* Check for mhn-show-type/subtype */
-    sprintf (buffer, "%s-show-%s/%s", invo_name, ci->ci_type, ci->ci_subtype);
+    snprintf (buffer, sizeof(buffer), "%s-show-%s/%s",
+		invo_name, ci->ci_type, ci->ci_subtype);
     if ((cp = context_find (buffer)) && *cp != '\0')
 	return show_content_aux (ct, serial, alternate, cp, NULL);
 
     /* Check for mhn-show-type */
-    sprintf (buffer, "%s-show-%s", invo_name, ci->ci_type);
+    snprintf (buffer, sizeof(buffer), "%s-show-%s", invo_name, ci->ci_type);
     if ((cp = context_find (buffer)) && *cp != '\0')
 	return show_content_aux (ct, serial, alternate, cp, NULL);
 
@@ -320,10 +325,9 @@ show_content (CT ct, int serial, int alternate)
 int
 show_content_aux (CT ct, int serial, int alternate, char *cp, char *cracked)
 {
-    int fd, xstdin;
-    int	xlist, xpause, xtty;
-    char *bp, *file;
-    char buffer[BUFSIZ];
+    int fd, len, buflen;
+    int	xstdin, xlist, xpause, xtty;
+    char *bp, *file, buffer[BUFSIZ];
     CI ci = &ct->c_ctinfo;
 
     if (!ct->c_ceopenfnx) {
@@ -345,12 +349,17 @@ show_content_aux (CT ct, int serial, int alternate, char *cp, char *cracked)
     xtty   = 0;
 
     if (cracked) {
-	strcpy (buffer, cp);
+	strncpy (buffer, cp, sizeof(buffer));
 	goto got_command;
     }
-    buffer[0] = '\0';
 
-    for (bp = buffer; *cp; cp++) {
+    /* get buffer ready to go */
+    bp = buffer;
+    bp[0] = '\0';
+    buflen = sizeof(buffer);
+
+    /* Now parse display string */
+    for ( ; *cp; cp++) {
 	if (*cp == '%') {
 	    switch (*++cp) {
 	    case 'a':
@@ -360,8 +369,10 @@ show_content_aux (CT ct, int serial, int alternate, char *cp, char *cracked)
 		char *s = "";
 
 		for (ap = ci->ci_attrs, ep = ci->ci_values; *ap; ap++, ep++) {
-		    sprintf (bp, "%s%s=\"%s\"", s, *ap, *ep);
-		    bp += strlen (bp);
+		    snprintf (bp, buflen, "%s%s=\"%s\"", s, *ap, *ep);
+		    len = strlen (bp);
+		    bp += len;
+		    buflen -= len;
 		    s = " ";
 		}
 	    }
@@ -372,7 +383,8 @@ show_content_aux (CT ct, int serial, int alternate, char *cp, char *cracked)
 		if (ct->c_descr) {
 		    char *s;
 
-		    strcpy (bp, s = trimcpy (ct->c_descr));
+		    s = trimcpy (ct->c_descr);
+		    strncpy (bp, s, buflen);
 		    free (s);
 		}
 		break;
@@ -390,7 +402,7 @@ show_content_aux (CT ct, int serial, int alternate, char *cp, char *cracked)
 
 	    case 'f':
 		/* insert filename containing content */
-		sprintf (bp, "%s", file);
+		snprintf (bp, buflen, "%s", file);
 		break;
 
 	    case 'p':
@@ -405,7 +417,7 @@ show_content_aux (CT ct, int serial, int alternate, char *cp, char *cracked)
 
 	    case 's':
 		/* insert subtype of content */
-		strcpy (bp, ci->ci_subtype);
+		strncpy (bp, ci->ci_subtype, buflen);
 		break;
 
 	    case '%':
@@ -415,13 +427,17 @@ show_content_aux (CT ct, int serial, int alternate, char *cp, char *cracked)
 	    default:
 		*bp++ = *--cp;
 		*bp = '\0';
+		buflen--;
 		continue;
 	    }
-	    bp += strlen (bp);
+	    len = strlen (bp);
+	    bp += len;
+	    buflen -= len;
 	} else {
 raw:
 	*bp++ = *cp;
 	*bp = '\0';
+	buflen--;
 	}
     }
 
@@ -429,8 +445,8 @@ raw:
     if (ct->c_termproc) {
 	char term[BUFSIZ];
 
-	strcpy (term, buffer);
-	sprintf (buffer, ct->c_termproc, term);
+	strncpy (term, buffer, sizeof(term));
+	snprintf (buffer, sizeof(buffer), ct->c_termproc, term);
     }
 
 got_command:
@@ -493,14 +509,15 @@ show_content_aux2 (CT ct, int serial, int alternate, char *cracked, char *buffer
 		read (fileno (stdout), prompt, sizeof(prompt));
 	    }
 	    SIGNAL (SIGINT, istat);
-	    if (intr != OK) {
+	    if (intr != OK || prompt[0] == 'n') {
 		(*ct->c_ceclosefnx) (ct);
 		return (alternate ? DONE : NOTOK);
 	    }
+	    if (prompt[0] == 'q') done(OK);
 	}
     }
 
-    sprintf (exec, "exec %s", buffer);
+    snprintf (exec, sizeof(exec), "exec %s", buffer);
 
     vec[0] = "/bin/sh";
     vec[1] = "-c";
@@ -556,12 +573,13 @@ show_text (CT ct, int serial, int alternate)
     CI ci = &ct->c_ctinfo;
 
     /* Check for mhn-show-type/subtype */
-    sprintf (buffer, "%s-show-%s/%s", invo_name, ci->ci_type, ci->ci_subtype);
+    snprintf (buffer, sizeof(buffer), "%s-show-%s/%s",
+		invo_name, ci->ci_type, ci->ci_subtype);
     if ((cp = context_find (buffer)) && *cp != '\0')
 	return show_content_aux (ct, serial, alternate, cp, NULL);
 
     /* Check for mhn-show-type */
-    sprintf (buffer, "%s-show-%s", invo_name, ci->ci_type);
+    snprintf (buffer, sizeof(buffer), "%s-show-%s", invo_name, ci->ci_type);
     if ((cp = context_find (buffer)) && *cp != '\0')
 	return show_content_aux (ct, serial, alternate, cp, NULL);
 
@@ -570,7 +588,7 @@ show_text (CT ct, int serial, int alternate)
      * if it is not a text part of a multipart/alternative
      */
     if (!alternate || ct->c_subtype == TEXT_PLAIN) {
-	sprintf (buffer, "%%p%s '%%F'", progsw ? progsw :
+	snprintf (buffer, sizeof(buffer), "%%p%s '%%F'", progsw ? progsw :
 		moreproc && *moreproc ? moreproc : "more");
 	cp = (ct->c_showproc = add (buffer, NULL));
 	return show_content_aux (ct, serial, alternate, cp, NULL);
@@ -591,12 +609,13 @@ show_multi (CT ct, int serial, int alternate)
     CI ci = &ct->c_ctinfo;
 
     /* Check for mhn-show-type/subtype */
-    sprintf (buffer, "%s-show-%s/%s", invo_name, ci->ci_type, ci->ci_subtype);
+    snprintf (buffer, sizeof(buffer), "%s-show-%s/%s",
+		invo_name, ci->ci_type, ci->ci_subtype);
     if ((cp = context_find (buffer)) && *cp != '\0')
 	return show_multi_aux (ct, serial, alternate, cp);
 
     /* Check for mhn-show-type */
-    sprintf (buffer, "%s-show-%s", invo_name, ci->ci_type);
+    snprintf (buffer, sizeof(buffer), "%s-show-%s", invo_name, ci->ci_type);
     if ((cp = context_find (buffer)) && *cp != '\0')
 	return show_multi_aux (ct, serial, alternate, cp);
 
@@ -716,11 +735,12 @@ show_multi_internal (CT ct, int serial, int alternate)
 	for (part = m->mp_parts; part; part = part->mp_next) {
 	    p = part->mp_part;
 
-	    if (p->c_pid > OK)
+	    if (p->c_pid > OK) {
 		if (kill (p->c_pid, 0) == NOTOK)
 		    p->c_pid = 0;
 		else
 		    kids++;
+	    }
 	}
 
 	while (kids > 0 && (pid = wait (&status)) != NOTOK) {
@@ -762,9 +782,9 @@ out:
 static int
 show_multi_aux (CT ct, int serial, int alternate, char *cp)
 {
+    int len, buflen;
     int xlist, xpause, xtty;
-    char *bp, *file;
-    char buffer[BUFSIZ];
+    char *bp, *file, buffer[BUFSIZ];
     struct multipart *m = (struct multipart *) ct->c_ctparams;
     struct part *part;
     CI ci = &ct->c_ctinfo;
@@ -796,9 +816,14 @@ show_multi_aux (CT ct, int serial, int alternate, char *cp)
     xlist     = 0;
     xpause    = 0;
     xtty      = 0;
-    buffer[0] = '\0';
 
-    for (bp = buffer; *cp; cp++) {
+    /* get buffer ready to go */
+    bp = buffer;
+    bp[0] = '\0';
+    buflen = sizeof(buffer);
+
+    /* Now parse display string */
+    for ( ; *cp; cp++) {
 	if (*cp == '%') {
 	    switch (*++cp) {
 	    case 'a':
@@ -808,8 +833,10 @@ show_multi_aux (CT ct, int serial, int alternate, char *cp)
 		char *s = "";
 
 		for (ap = ci->ci_attrs, ep = ci->ci_values; *ap; ap++, ep++) {
-		    sprintf (bp, "%s%s=\"%s\"", s, *ap, *ep);
-		    bp += strlen (bp);
+		    snprintf (bp, buflen, "%s%s=\"%s\"", s, *ap, *ep);
+		    len = strlen (bp);
+		    bp += len;
+		    buflen -= len;
 		    s = " ";
 		}
 	    }
@@ -820,7 +847,8 @@ show_multi_aux (CT ct, int serial, int alternate, char *cp)
 		if (ct->c_descr) {
 		    char *s;
 
-		    strcpy (bp, s = trimcpy (ct->c_descr));
+		    s = trimcpy (ct->c_descr);
+		    strncpy (bp, s, buflen);
 		    free (s);
 		}
 		break;
@@ -843,8 +871,10 @@ show_multi_aux (CT ct, int serial, int alternate, char *cp)
 		for (part = m->mp_parts; part; part = part->mp_next) {
 		    p = part->mp_part;
 
-		    sprintf (bp, "%s'%s'", s, p->c_storage);
-		    bp += strlen (bp);
+		    snprintf (bp, buflen, "%s'%s'", s, p->c_storage);
+		    len = strlen (bp);
+		    bp += len;
+		    buflen -= len;
 		    s = " ";
 		}
 	    }
@@ -862,7 +892,7 @@ show_multi_aux (CT ct, int serial, int alternate, char *cp)
 
 	    case 's':
 		/* insert subtype of content */
-		strcpy (bp, ci->ci_subtype);
+		strncpy (bp, ci->ci_subtype, buflen);
 		break;
 
 	    case '%':
@@ -872,13 +902,17 @@ show_multi_aux (CT ct, int serial, int alternate, char *cp)
 	    default:
 		*bp++ = *--cp;
 		*bp = '\0';
+		buflen--;
 		continue;
 	    }
-	    bp += strlen (bp);
+	    len = strlen (bp);
+	    bp += len;
+	    buflen -= len;
 	} else {
 raw:
 	*bp++ = *cp;
 	*bp = '\0';
+	buflen--;
 	}
     }
 
@@ -886,8 +920,8 @@ raw:
     if (ct->c_termproc) {
 	char term[BUFSIZ];
 
-	strcpy (term, buffer);
-	sprintf (buffer, ct->c_termproc, term);
+	strncpy (term, buffer, sizeof(term));
+	snprintf (buffer, sizeof(buffer), ct->c_termproc, term);
     }
 
     return show_content_aux2 (ct, serial, alternate, NULL, buffer,
@@ -906,12 +940,13 @@ show_message_rfc822 (CT ct, int serial, int alternate)
     CI ci = &ct->c_ctinfo;
 
     /* Check for mhn-show-type/subtype */
-    sprintf (buffer, "%s-show-%s/%s", invo_name, ci->ci_type, ci->ci_subtype);
+    snprintf (buffer, sizeof(buffer), "%s-show-%s/%s",
+		invo_name, ci->ci_type, ci->ci_subtype);
     if ((cp = context_find (buffer)) && *cp != '\0')
 	return show_content_aux (ct, serial, alternate, cp, NULL);
 
     /* Check for mhn-show-type */
-    sprintf (buffer, "%s-show-%s", invo_name, ci->ci_type);
+    snprintf (buffer, sizeof(buffer), "%s-show-%s", invo_name, ci->ci_type);
     if ((cp = context_find (buffer)) && *cp != '\0')
 	return show_content_aux (ct, serial, alternate, cp, NULL);
 
