@@ -2,7 +2,7 @@
 /*
  * mhstore.c -- store the contents of MIME messages
  *
- * $Id$
+ * $Id: mhstore.c,v 1.2 1999/07/16 01:23:41 danh Exp $
  */
 
 #include <h/mh.h>
@@ -407,8 +407,7 @@ do_cache:
 	context_save ();		  /* save the context file  */
     }
 
-    done (0);
-    /* NOTREACHED */
+    return done (0);
 }
 
 
@@ -427,7 +426,7 @@ pipeser (int i)
 }
 
 
-void
+int
 done (int status)
 {
     CT *ctp;
@@ -437,4 +436,5 @@ done (int status)
 	    free_content (*ctp);
 
     exit (status);
+    return 1;  /* dead code to satisfy the compiler */
 }
