@@ -2,7 +2,7 @@
 /*
  * mhstoresbr.c -- routines to save/store the contents of MIME messages
  *
- * $Id$
+ * $Id: mhstoresbr.c,v 1.3 1999/09/26 17:45:56 ruud Exp $
  */
 
 #include <h/mh.h>
@@ -14,7 +14,7 @@
 #include <signal.h>
 #include <zotnet/mts/mts.h>
 #include <zotnet/tws/tws.h>
-#include <h/mhn.h>
+#include <h/mime.h>
 #include <h/mhparse.h>
 
 extern int errno;
@@ -43,10 +43,6 @@ static char *dir;
  */
 typedef int (*qsort_comp) (const void *, const void *);
 
-/*
- * prototypes
- */
-void store_all_messages (CT *);
 
 /* mhmisc.c */
 int part_ok (CT, int);
@@ -57,6 +53,11 @@ void flush_errors (void);
 /* mhshowsbr.c */
 int show_content_aux (CT, int, int, char *, char *);
 
+/*
+ * prototypes
+ */
+void store_all_messages (CT *);
+
 /*
  * static prototypes
  */
@@ -72,7 +73,7 @@ static int store_content (CT, CT);
 static int output_content_file (CT, int);
 static int check_folder (char *);
 static int output_content_folder (char *, char *);
-static int parse_format_string (CT, char *, char *, char *);
+static int parse_format_string (CT, char *, char *, int, char *);
 static void get_storeproc (CT);
 static int copy_some_headers (FILE *, CT);
 
@@ -86,21 +87,18 @@ void
 store_all_messages (CT *cts)
 {
     CT ct, *ctp;
-    char buffer[BUFSIZ], *cp;
+    char *cp;
 
     /*
      * Check for the directory in which to
      * store any contents.
      */
-    if (autosw) {
+    if (autosw)
+	dir = getcpy (cwd);
+    else if ((cp = context_find (nmhstorage)) && *cp)
+	dir = getcpy (cp);
+    else
 	dir = getcpy (cwd);
-    } else {
-	sprintf (buffer, "%s-storage", invo_name);
-	if ((cp = context_find (buffer)) && *cp)
-	    dir = getcpy (cp);
-	else
-	    dir = getcpy (cwd);
-    }
 
     for (ctp = cts; *ctp; ctp++) {
 	ct = *ctp;
@@ -220,7 +218,7 @@ store_application (CT ct)
      * attribute/value pairs which specify if this a tar file.
      */
     if (!ct->c_storeproc && ct->c_subtype == APPLICATION_OCTETS) {
-	int tarP = 0, zP = 0;
+	int tarP = 0, zP = 0, gzP = 0;
 
 	for (ap = ci->ci_attrs, ep = ci->ci_values; *ap; ap++, ep++) {
 	    /* check for "type=tar" attribute */
@@ -238,19 +236,30 @@ store_application (CT ct)
 		zP = 1;
 		continue;
 	    }
+	    /* check for "conversions=gzip" attribute */
+	    if ((!strcasecmp (*ap, "conversions") || !strcasecmp (*ap, "x-conversions"))
+		&& (!strcasecmp (*ep, "gzip") || !strcasecmp (*ep, "x-gzip"))) {
+		gzP = 1;
+		continue;
+	    }
 	}
 
 	if (tarP) {
 	    ct->c_showproc = add (zP ? "%euncompress | tar tvf -"
-				  : "%etar tvf -", NULL);
-	    if (!ct->c_storeproc)
+				  : (gzP ? "%egzip -dc | tar tvf -"
+				     : "%etar tvf -"), NULL);
+	    if (!ct->c_storeproc) {
 		if (autosw) {
 		    ct->c_storeproc = add (zP ? "| uncompress | tar xvpf -"
-					   : "| tar xvpf -", NULL);
+					   : (gzP ? "| gzip -dc | tar xvpf -"
+					      : "| tar xvpf -"), NULL);
 		    ct->c_umask = 0022;
 		} else {
-		    ct->c_storeproc = add (zP ? "%m%P.tar.Z" : "%m%P.tar", NULL);
+		    ct->c_storeproc= add (zP ? "%m%P.tar.Z"
+				          : (gzP ? "%m%P.tar.gz"
+					     : "%m%P.tar"), NULL);
 		}
+	    }
 	}
     }
 
@@ -530,9 +539,10 @@ store_content (CT ct, CT p)
     if ((cp = ct->c_storeproc) == NULL || *cp == '\0') {
 	CI ci = &ct->c_ctinfo;
 
-	sprintf (buffer, "%s-store-%s/%s", invo_name, ci->ci_type, ci->ci_subtype);
+	snprintf (buffer, sizeof(buffer), "%s-store-%s/%s",
+		invo_name, ci->ci_type, ci->ci_subtype);
 	if ((cp = context_find (buffer)) == NULL || *cp == '\0') {
-	    sprintf (buffer, "%s-store-%s", invo_name, ci->ci_type);
+	    snprintf (buffer, sizeof(buffer), "%s-store-%s", invo_name, ci->ci_type);
 	    if ((cp = context_find (buffer)) == NULL || *cp == '\0') {
 		cp = ct->c_type == CT_MESSAGE ? "+" : "%m%P.%s";
 	    }
@@ -570,9 +580,10 @@ store_content (CT ct, CT p)
     }
 
     /*
-     * Parse and expand the storage formatting string.
+     * Parse and expand the storage formatting string
+     * in `cp' into `buffer'.
      */
-    parse_format_string (ct, cp, buffer, dir);
+    parse_format_string (ct, cp, buffer, sizeof(buffer), dir);
 
     /*
      * If formatting begins with '|' or '!', then pass
@@ -925,11 +936,13 @@ output_content_folder (char *folder, char *filename)
 
 /*
  * Parse and expand the storage formatting string
+ * pointed to by "cp" into "buffer".
  */
 
 static int
-parse_format_string (CT ct, char *cp, char *buffer, char *dir)
+parse_format_string (CT ct, char *cp, char *buffer, int buflen, char *dir)
 {
+    int len;
     char *bp;
     CI ci = &ct->c_ctinfo;
 
@@ -937,13 +950,13 @@ parse_format_string (CT ct, char *cp, char *buffer, char *dir)
      * If storage string is "-", just copy it, and
      * return (send content to standard output).
      */
-    if (!cp[1]) {
-	sprintf (buffer, "-");
+    if (cp[0] == '-' && cp[1] == '\0') {
+	strncpy (buffer, cp, buflen);
 	return 0;
     }
 
-    buffer[0] = '\0';
     bp = buffer;
+    bp[0] = '\0';
 
     /*
      * If formatting string is a pathname that doesn't
@@ -951,8 +964,10 @@ parse_format_string (CT ct, char *cp, char *buffer, char *dir)
      * appropriate directory.
      */
     if (*cp != '/' && *cp != '|' && *cp != '!') {
-	sprintf (buffer, "%s/", dir[1] ? dir : "");
-	bp += strlen (bp);
+	snprintf (bp, buflen, "%s/", dir[1] ? dir : "");
+	len = strlen (bp);
+	bp += len;
+	buflen -= len;
     }
 
     for (; *cp; cp++) {
@@ -968,6 +983,7 @@ parse_format_string (CT ct, char *cp, char *buffer, char *dir)
 		    if (buffer[0] != '|' && buffer[0] != '!') {
 			*bp++ = *--cp;
 			*bp = '\0';
+			buflen--;
 			continue;
 		    } else {
 			char **ap, **ep;
@@ -975,8 +991,10 @@ parse_format_string (CT ct, char *cp, char *buffer, char *dir)
 
 			for (ap = ci->ci_attrs, ep = ci->ci_values;
 			         *ap; ap++, ep++) {
-			    sprintf (bp, "%s%s=\"%s\"", s, *ap, *ep);
-			    bp += strlen (bp);
+			    snprintf (bp, buflen, "%s%s=\"%s\"", s, *ap, *ep);
+			    len = strlen (bp);
+			    bp += len;
+			    buflen -= len;
 			    s = " ";
 			}
 		    }
@@ -984,29 +1002,29 @@ parse_format_string (CT ct, char *cp, char *buffer, char *dir)
 
 		case 'm':
 		    /* insert message number */
-		    sprintf (bp, "%s", r1bindex (ct->c_file, '/'));
+		    snprintf (bp, buflen, "%s", r1bindex (ct->c_file, '/'));
 		    break;
 
 		case 'P':
 		    /* insert part number with leading dot */
 		    if (ct->c_partno)
-			sprintf (bp, ".%s", ct->c_partno);
+			snprintf (bp, buflen, ".%s", ct->c_partno);
 		    break;
 
 		case 'p':
 		    /* insert part number withouth leading dot */
 		    if (ct->c_partno)
-			strcpy (bp, ct->c_partno);
+			strncpy (bp, ct->c_partno, buflen);
 		    break;
 
 		case 't':
 		    /* insert content type */
-		    strcpy (bp, ci->ci_type);
+		    strncpy (bp, ci->ci_type, buflen);
 		    break;
 
 		case 's':
 		    /* insert content subtype */
-		    strcpy (bp, ci->ci_subtype);
+		    strncpy (bp, ci->ci_subtype, buflen);
 		    break;
 
 		case '%':
@@ -1016,14 +1034,20 @@ parse_format_string (CT ct, char *cp, char *buffer, char *dir)
 		default:
 		    *bp++ = *--cp;
 		    *bp = '\0';
+		    buflen--;
 		    continue;
 	    }
-	    bp += strlen (bp);
+
+	    /* Advance bp and decrement buflen */
+	    len = strlen (bp);
+	    bp += len;
+	    buflen -= len;
 
 	} else {
 raw:
 	    *bp++ = *cp;
 	    *bp = '\0';
+	    buflen--;
 	}
     }
 
@@ -1077,56 +1101,23 @@ get_storeproc (CT ct)
 static int
 copy_some_headers (FILE *out, CT ct)
 {
-    int	state;
-    char buf[BUFSIZ], name[NAMESZ];
-    FILE *in;
+    HF hp;
 
-    if ((in = fopen (ct->c_file, "r")) == NULL) {
-	advise (ct->c_file, "unable to open for reading");
-	return NOTOK;
+    hp = ct->c_first_hf;	/* start at first header field */
+
+    while (hp) {
+	/*
+	 * A few of the header fields of the enclosing
+	 * messages are not copied.
+	 */
+	if (!uprf (hp->name, XXX_FIELD_PRF)
+		&& strcasecmp (hp->name, VRSN_FIELD)
+		&& strcasecmp (hp->name, "Subject")
+		&& strcasecmp (hp->name, "Encrypted")
+		&& strcasecmp (hp->name, "Message-ID"))
+	    fprintf (out, "%s:%s", hp->name, hp->value);
+	hp = hp->next;	/* next header field */
     }
 
-    for (state = FLD;;) {
-	switch (state = m_getfld (state, name, buf, sizeof(buf), in)) {
-	    case FLD:
-	    case FLDPLUS:
-	    case FLDEOF:
-		/*
-		 * A few of the header fields of the enclosing
-		 * messages are not copied.
-		 */
-		if (uprf (name, XXX_FIELD_PRF)
-			|| !strcasecmp (name, VRSN_FIELD)
-			|| !strcasecmp (name, "Subject")
-			|| !strcasecmp (name, "Encrypted")
-			|| !strcasecmp (name, "Message-ID")) {
-		    while (state == FLDPLUS)
-			state = m_getfld (state, name, buf, sizeof(buf), in);
-		    continue;
-		}
-
-		fprintf (out, "%s:%s", name, buf);
-		while (state == FLDPLUS) {
-		    state = m_getfld (state, name, buf, sizeof(buf), in);
-		    fputs (buf, out);
-		}
-		if (state != FLDEOF)
-		    continue;
-		/* else fall... */
-	   case BODY:
-	   case BODYEOF:
-	   case FILEEOF:
-		break;
-
-	   case LENERR:
-	   case FMTERR:
-	   default:
-		break;
-	}
-
-	break;
-    }
-
-    fclose (in);
     return OK;
 }
