@@ -2,7 +2,7 @@
 /*
  * msgchk.c -- check for mail
  *
- * $Id$
+ * $Id: msgchk.c,v 1.2 1999/07/16 01:03:34 danh Exp $
  */
 
 #include <h/mh.h>
@@ -66,6 +66,12 @@ static struct swit switches[] = {
     { NULL, 0 }
 };
 
+/*
+ * Maximum numbers of users we can check (plus
+ * one for the NULL vector at the end).
+ */
+#define MAXVEC  51
+
 #define	NT_NONE	0x0
 #define	NT_MAIL	0x1
 #define	NT_NMAI	0x2
@@ -95,12 +101,11 @@ int
 main (int argc, char **argv)
 {
     int datesw = 1, notifysw = NT_ALL;
-    int rpop = 0, status = 0;
+    int rpop, status = 0;
     int snoop = 0, vecp = 0;
     uid_t uid;
-    char *cp, *host = NULL, buf[80], **ap;
-    char **argp, *arguments[MAXARGS], *vec[50];
-    char *user;
+    char *cp, *host = NULL, *user, buf[BUFSIZ];
+    char **argp, **arguments, *vec[MAXVEC];
     struct passwd *pw;
 
 #ifdef HESIOD
@@ -111,28 +116,31 @@ main (int argc, char **argv)
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
+    invo_name = r1bindex (argv[0], '/');
 
+    /* read user profile/context */
+    context_read();
+
+    mts_init (invo_name);
     uid = getuid ();
     user = getusername();
-    invo_name = r1bindex (argv[0], '/');
-    mts_init (invo_name);
+
+    arguments = getarguments (invo_name, argc, argv, 1);
+    argp = arguments;
 
 #ifdef POP
     if ((cp = getenv ("MHPOPDEBUG")) && *cp)
 	snoop++;
 #endif
 
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
-    argp = arguments;
+#ifdef KPOP
+    rpop = 1;
+#else
+    rpop = 0;
+#endif
 
     while ((cp = *argp++)) {
-	if (*cp == '-')
+	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
 		case AMBIGSW: 
 		    ambigsw (cp, switches);
@@ -141,7 +149,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    snprintf (buf, 80, "%s [switches] [users ...]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches] [users ...]",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -173,7 +182,10 @@ main (int argc, char **argv)
 		case USERSW: 
 		    if (!(cp = *argp++) || *cp == '-')
 			adios (NULL, "missing argument to %s", argp[-2]);
-		    vec[vecp++] = cp;
+		    if (vecp >= MAXVEC-1)
+			adios (NULL, "you can only check %d users at a time", MAXVEC-1);
+		    else
+			vec[vecp++] = cp;
 		    continue;
 
 		case APOPSW: 
@@ -194,7 +206,11 @@ main (int argc, char **argv)
 		    snoop++;
 		    continue;
 	    }
-	vec[vecp++] = cp;
+	}
+	if (vecp >= MAXVEC-1)
+	    adios (NULL, "you can only check %d users at a time", MAXVEC-1);
+	else
+	    vec[vecp++] = cp;
     }
 
 #ifdef POP
@@ -267,7 +283,7 @@ main (int argc, char **argv)
     }		/* host == NULL */
 #endif
 
-    done (status);
+    return done (status);
 }
 
 
@@ -299,6 +315,8 @@ donote (char *cp, int ntflag)
 	case NNMAISW: 
 	    return NT_NMAI;
     }
+
+    return 0; /* Before 1999-07-15, garbage was returned if control got here. */
 }
 
 
@@ -309,7 +327,7 @@ checkmail (char *user, char *home, int datesw, int notifysw, int personal)
     char buffer[BUFSIZ];
     struct stat st;
 
-    snprintf (buffer, BUFSIZ, "%s/%s", mmdfldir[0] ? mmdfldir : home, mmdflfil[0] ? mmdflfil : user);
+    snprintf (buffer, sizeof(buffer), "%s/%s", mmdfldir[0] ? mmdfldir : home, mmdflfil[0] ? mmdflfil : user);
     if (datesw) {
 	st.st_size = 0;
 	st.st_atime = st.st_mtime = 0;
@@ -369,13 +387,8 @@ remotemail (char *host, char *user, int rpop, int notifysw, int personal, int sn
     else
 	ruserpass (host, &user, &pass);
 
-#if defined(RPOP) || defined(APOP)
     /* open the POP connection */
     if (pop_init (host, user, pass, snoop, rpop) == NOTOK
-#else
-    /* open the POP connection */
-    if (pop_init (host, user, pass, snoop) == NOTOK
-#endif
 	    || pop_stat (&nmsgs, &nbytes) == NOTOK	/* check for messages  */
 	    || pop_quit () == NOTOK) {			/* quit POP connection */
 	advise (NULL, "%s", response);
