@@ -2,7 +2,7 @@
 /*
  * msh.c -- The nmh shell
  *
- * $Id$
+ * $Id: msh.c,v 1.2 1999/07/16 01:02:21 danh Exp $
  */
 
 /*
@@ -236,8 +236,7 @@ main (int argc, char **argv)
 {
     int	id = 0, scansw = 0, vmh1 = 0, vmh2 = 0;
     char *cp, *file = NULL, *folder = NULL;
-    char **ap, **argp, buf[80];
-    char *arguments[MAXARGS];
+    char **argp, **arguments, buf[BUFSIZ];
 #ifdef BPOP
     int	pmsh1 = 0, pmsh2 = 0;
 #endif
@@ -246,14 +245,12 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
+
+    /* read user profile/context */
+    context_read();
+
     mts_init (invo_name);
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+    arguments = getarguments (invo_name, argc,argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -266,7 +263,7 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [switches] file", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches] file", invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -362,7 +359,7 @@ main (int argc, char **argv)
 	file = "./msgbox";
     if (file && folder)
 	adios (NULL, "use a file or a folder, not both");
-    strcpy (myfilter, etcpath (mhlforward));
+    strncpy (myfilter, etcpath (mhlforward), sizeof(myfilter));
 #ifdef FIOCLEX
     if (pfd > 1)
 	ioctl (pfd, FIOCLEX, NULL);
@@ -411,7 +408,7 @@ main (int argc, char **argv)
 
     m_reset ();
     
-    done (0);
+    return done (0);
 }
 
 
@@ -486,7 +483,7 @@ msh (int scansw)
     register struct Cmd *cmdp;
     static int once_only = ADVCMD;
 
-    sprintf (prompt, myprompt, invo_name);
+    snprintf (prompt, sizeof(prompt), myprompt, invo_name);
     cmdp = &typein;
 
     for (;;) {
@@ -499,8 +496,7 @@ msh (int scansw)
 		rcdone ();
 		return;
 	    }
-	}
-	else {
+	} else {
 	    check_folder (scansw);
 	    if ((i = getargs (prompt, mshcmds, cmdp)) == EOF) {
 		putchar ('\n');
@@ -535,12 +531,13 @@ msh (int scansw)
 	    case SHOWCMD: 
 	    case SCANCMD: 
 	    case SORTCMD: 
-		if ((cp = context_find (cmd_name)) != NULL) {
-		    ap = brkstring (cp = getcpy (cp), " ", "\n");
-		    ap = copyip (ap, vec);
-		}
-		else
+		if ((cp = context_find (cmd_name))) {
+		    cp = getcpy (cp);
+		    ap = brkstring (cp, " ", "\n");
+		    ap = copyip (ap, vec, MAXARGS);
+		} else {
 		    ap = vec;
+		}
 		break;
 
 	    default: 
@@ -548,7 +545,7 @@ msh (int scansw)
 		ap = vec;
 		break;
 	}
-	copyip (cmdp->args + 1, ap);
+	copyip (cmdp->args + 1, ap, MAXARGS);
 
 	m_init ();
 
@@ -687,9 +684,13 @@ fsetup (char *folder)
     maildir = m_maildir (folder);
     if (chdir (maildir) == NOTOK)
 	padios (maildir, "unable to change directory to");
+
+    /* read folder and create message structure */
     if (!(mp = folder_read (folder)))
 	padios (NULL, "unable to read folder %s", folder);
-    if (mp->hghmsg == 0)
+
+    /* check for empty folder */
+    if (mp->nummsg == 0)
 	padios (NULL, "no messages in %s", folder);
 
     mode = m_gmprot ();
@@ -726,7 +727,7 @@ setup (char *file)
 
 #ifdef BPOP
     if (pmsh) {
-	strcpy (tmpfil, m_tmpfil (invo_name));
+	strncpy (tmpfil, m_tmpfil (invo_name), sizeof(tmpfil));
 	if ((fp = fopen (tmpfil, "w+")) == NULL)
 	    padios (tmpfil, "unable to create");
 	unlink (tmpfil);
@@ -991,7 +992,7 @@ msh_ready (int msgnum, int full)
 	if (Msgs[msgnum].m_top == 0)
 	    padios (NULL, "msh_ready (%d, %d) botch", msgnum, full);
 	if (!full) {
-	    strcpy (tmpfil, m_tmpfil (invo_name));
+	    strncpy (tmpfil, m_tmpfil (invo_name), sizeof(tmpfil));
 	    if ((yp = fopen (tmpfil, "w+")) == NULL)
 		padios (tmpfil, "unable to create");
 	    unlink (tmpfil);
@@ -1126,7 +1127,7 @@ scanrange (int low, int hgh)
 {
     char buffer[BUFSIZ];
 
-    sprintf (buffer, "%d-%d", low, hgh);
+    snprintf (buffer, sizeof(buffer), "%d-%d", low, hgh);
     scanstring (buffer);
 }
 
@@ -1136,12 +1137,16 @@ scanstring (char *arg)
 {
     char *cp, **ap, *vec[MAXARGS];
 
-    if ((cp = context_find (cmd_name = "scan")) != NULL) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, vec);
-    }
-    else
+    /*
+     * This should be replace with a call to getarguments()
+     */
+    if ((cp = context_find (cmd_name = "scan"))) {
+	cp = getcpy (cp);
+	ap = brkstring (cp, " ", "\n");
+	ap = copyip (ap, vec, MAXARGS);
+    } else {
 	ap = vec;
+    }
     *ap++ = arg;
     *ap = NULL;
     m_init ();
@@ -1213,14 +1218,14 @@ readid (int msgnum)
 
     zp = msh_ready (msgnum, 0);
     for (state = FLD;;)
-	switch (state = m_getfld (state, name, buf, sizeof buf, zp)) {
+	switch (state = m_getfld (state, name, buf, sizeof(buf), zp)) {
 	    case FLD: 
 	    case FLDEOF: 
 	    case FLDPLUS: 
 		if (!strcasecmp (name, BBoard_ID)) {
 		    bp = getcpy (buf);
 		    while (state == FLDPLUS) {
-			state = m_getfld (state, name, buf, sizeof buf, zp);
+			state = m_getfld (state, name, buf, sizeof(buf), zp);
 			bp = add (buf, bp);
 		    }
 		    i = atoi (bp);
@@ -1231,7 +1236,7 @@ readid (int msgnum)
 			continue;
 		}
 		while (state == FLDPLUS)
-		    state = m_getfld (state, name, buf, sizeof buf, zp);
+		    state = m_getfld (state, name, buf, sizeof(buf), zp);
 		if (state != FLDEOF)
 		    continue;
 
@@ -1305,7 +1310,7 @@ write_ids (void)
 		    break;
 	    }
 
-    sprintf (buffer, "%d %d\n", i, Msgs[mp->hghmsg].m_bboard_id);
+    snprintf (buffer, sizeof(buffer), "%d %d\n", i, Msgs[mp->hghmsg].m_bboard_id);
     write (pfd, buffer, sizeof(buffer));
     close (pfd);
     pfd = NOTOK;
@@ -1363,7 +1368,7 @@ quit (void)
     cp = concat ("Update file \"", mp->foldpath, "\"? ", NULL);
     if (!getanswer (cp))
 	goto release;
-    strcpy (tmpfil, m_backup (mp->foldpath));
+    strncpy (tmpfil, m_backup (mp->foldpath), sizeof(tmpfil));
     if ((md = mbx_open (tmpfil, mbx_style, st.st_uid, st.st_gid, mode)) == NOTOK) {
 	advise (tmpfil, "unable to open");
 	goto release;
@@ -1381,8 +1386,8 @@ quit (void)
     if (rename (tmpfil, mp->foldpath) == NOTOK)
 	admonish (mp->foldpath, "unable to rename %s to", tmpfil);
     else {
-	strcpy (map1, map_name (tmpfil));
-	strcpy (map2, map_name (mp->foldpath));
+	strncpy (map1, map_name (tmpfil), sizeof(map1));
+	strncpy (map2, map_name (mp->foldpath), sizeof(map2));
 
 	if (rename (map1, map2) == NOTOK) {
 	    admonish (map2, "unable to rename %s to", map1);
@@ -1544,9 +1549,10 @@ parse (char *buffer, struct Cmd *cmdp)
     cmdp->direction = STDIO;
     cmdp->stream = NULL;
 
-    for (cp = buffer; c = *cp; cp++)
+    for (cp = buffer; (c = *cp); cp++) {
 	if (!isspace (c))
 	    break;
+    }
     if (c == '\0') {
 	if (vmh)
 	    fmt2peer (RC_EOF, "null command");
@@ -1609,7 +1615,7 @@ parse (char *buffer, struct Cmd *cmdp)
 		    cp++;
 		}
 		cmdp->redirect = pp + 1;/* sigh */
-		for (; c = *cp; cp++)
+		for (; (c = *cp); cp++)
 		    if (!isspace (c))
 			break;
 		if (c == 0) {
@@ -1662,7 +1668,7 @@ expand (char *redirect)
 	    return NOTOK;
 	}
 
-    sprintf (path, "%s/%s", pp, cp ? cp : "");
+    snprintf (path, sizeof(path), "%s/%s", pp, cp ? cp : "");
     strcpy (redirect, path);
     return OK;
 }
@@ -1930,6 +1936,8 @@ pINI (void)
 	    fmt2peer (RC_ERR, "pINI protocol screw-up");
 	    done (1);		/* NOTREACHED */
     }
+
+    return 1;  /* dead code to satisfy the compiler */
 }
 
 
@@ -2061,11 +2069,12 @@ pQRY2 (void)
     for (msgnum = mp->curmsg; msgnum <= mp->hghmsg; msgnum++)
 	if (does_exist (mp, msgnum))
 	    i++;
-    if (i-- > 0)
+    if (i-- > 0) {
 	if (topcur)
 	    k = i >= k ? 1 : k - i;
 	else
 	    k -= i > j ? j : i;
+    }
 
     i = j = 0;
     n = 1;
@@ -2286,6 +2295,8 @@ ttyR (struct Cmd *cmdp)
 	    fmt2peer (RC_ERR, "pTTY protocol screw-up");
 	    done (1);		/* NOTREACHED */
     }
+
+    return 1;  /* dead code to satisfy compiler */
 }
 
 
@@ -2308,7 +2319,7 @@ winN (struct Cmd *cmdp, int n, int eof)
 
     vmhpid = OK;
 
-    sprintf (buffer, "%d", n);
+    snprintf (buffer, sizeof(buffer), "%d", n);
     switch (str2rc (RC_WIN, buffer, rc)) {
 	case RC_ACK: 
 	    break;
@@ -2467,7 +2478,7 @@ winX (int n)
     if (!fmsh)
 	fseek (fp, 0L, SEEK_SET);
 
-    sprintf (buffer, "%d", n);
+    snprintf (buffer, sizeof(buffer), "%d", n);
     switch (str2rc (RC_WIN, buffer, rc)) {
 	case RC_ACK: 
 	    break;
