@@ -2,7 +2,7 @@
 /*
  * mshcmds.c -- command handlers in msh
  *
- * $Id$
+ * $Id: mshcmds.c,v 1.5 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
@@ -68,6 +68,8 @@ static int process (int, char *, int, char **);
 static void copy_message (int, FILE *);
 static void copy_digest (int, FILE *);
 
+/* from mhlsbr.c */
+int mhlsbr (int, char **, FILE *(*)());
 
 void
 forkcmd (char **args, char *pgm)
@@ -691,7 +693,7 @@ foldcmd (char **args)
 		    packsw = 0;
 		    continue;
 	    }
-	if (*cp == '+' || *cp == '@')
+	if (*cp == '+' || *cp == '@') {
 	    if (folder) {
 		advise (NULL, "only one folder at a time!\n");
 		return;
@@ -699,6 +701,7 @@ foldcmd (char **args)
 	    else
 		folder = fmsh ? path (cp + 1, *cp == '+' ? TFOLDER : TSUBCWF)
 			    : cp + 1;
+	}
 	else
 	    if (msg) {
 		advise (NULL, "only one message at a time!\n");
@@ -932,7 +935,16 @@ forwcmd (char **args)
 
 					/* foil search of .mh_profile */
     snprintf (buf, sizeof(buf), "%sXXXXXX", invo_name);
+/*
+  Mkstemp work postponed until later -Doug
+#ifdef HAVE_MKSTEMP
+    vec[0] = (char *)mkstemp (buf);
+#else
+*/
     vec[0] = (char *)mktemp (buf);
+/*
+#endif
+*/
     vec[vecp++] = "-file";
     vec[vecp] = NULL;
     if (!msgp)
@@ -1188,7 +1200,7 @@ markcmd (char **args)
 	}
     }
 
-    if (!addsw && !deletesw && !listsw)
+    if (!addsw && !deletesw && !listsw) {
 	if (seqp)
 	    addsw++;
 	else
@@ -1201,6 +1213,7 @@ markcmd (char **args)
 		if (!msgp)
 		    msgs[msgp++] = "all";
 	    }
+    }
 
     if (!msgp)
 	msgs[msgp++] = listsw ? "all" :"cur";
@@ -1445,7 +1458,7 @@ static struct swit packswit[] = {
     { NULL, 0 }
 };
 
-static mbx_style = MMDF_FORMAT;
+static int mbx_style = MMDF_FORMAT;
 
 void
 packcmd (char **args)
