@@ -2,7 +2,7 @@
 /*
  * packf.c -- pack a nmh folder into a file
  *
- * $Id$
+ * $Id: packf.c,v 1.2 1999/07/16 01:22:56 danh Exp $
  */
 
 #include <h/mh.h>
@@ -10,6 +10,13 @@
 #include <h/dropsbr.h>
 #include <errno.h>
 
+/*
+ * We allocate space for messages (msgs array)
+ * this number of elements at a time.
+ */
+#define MAXMSGS  256
+
+
 static struct swit switches[] = {
 #define FILESW         0
     { "file name", 0 },
@@ -36,9 +43,9 @@ char *file = NULL;
 int
 main (int argc, char **argv)
 {
-    int msgp = 0, fd, msgnum;
-    char *cp, *maildir, *msgnam, *folder = NULL, buf[100];
-    char **ap, **argp, *arguments[MAXARGS], *msgs[MAXARGS];
+    int nummsgs, maxmsgs, fd, msgnum;
+    char *cp, *maildir, *msgnam, *folder = NULL, buf[BUFSIZ];
+    char **argp, **arguments, **msgs;
     struct msgs *mp;
     struct stat st;
 
@@ -46,15 +53,24 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
+    /* Allocate the initial space to record message
+     * names and ranges.
+     */
+    nummsgs = 0;
+    maxmsgs = MAXMSGS;
+    if (!(msgs = (char **) malloc ((size_t) (maxmsgs * sizeof(*msgs)))))
+	adios (NULL, "unable to allocate storage");
+
+    /*
+     * Parse arguments
+     */
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
@@ -65,8 +81,9 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [+folder] [msgs] [switches]", invo_name);
-		    print_help (buf, switches);
+		    snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
+			invo_name);
+		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
 		    print_version(invo_name);
@@ -94,13 +111,28 @@ main (int argc, char **argv)
 		adios (NULL, "only one folder at a time!");
 	    folder = path (cp + 1, *cp == '+' ? TFOLDER : TSUBCWF);
 	} else {
-	    msgs[msgp++] = cp;
+	    /*
+	     * Check if we need to allocate more space
+	     * for message name/ranges.
+	     */
+	    if (nummsgs >= maxmsgs) {
+		maxmsgs += MAXMSGS;
+		if (!(msgs = (char **) realloc (msgs,
+			(size_t) (maxmsgs * sizeof(*msgs)))))
+		    adios (NULL, "unable to reallocate msgs storage");
+	    }
+	    msgs[nummsgs++] = cp;
 	}
     }
 
     if (!file)
 	file = "./msgbox";
     file = path (file, TFILE);
+
+    /*
+     * Check if file to be created (or appended to)
+     * exists.  If not, ask for confirmation.
+     */
     if (stat (file, &st) == NOTOK) {
 	if (errno != ENOENT)
 	    adios (file, "error on file");
@@ -112,8 +144,11 @@ main (int argc, char **argv)
 
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
-    if (!msgp)
-	msgs[msgp++] = "all";
+
+    /* default is to pack whole folder */
+    if (!nummsgs)
+	msgs[nummsgs++] = "all";
+
     if (!folder)
 	folder = getfolder (1);
     maildir = m_maildir (folder);
@@ -125,15 +160,17 @@ main (int argc, char **argv)
     if (!(mp = folder_read (folder)))
 	adios (NULL, "unable to read folder %s", folder);
 
-    if (mp->hghmsg == 0)
+    /* check for empty folder */
+    if (mp->nummsg == 0)
 	adios (NULL, "no messages in %s", folder);
 
     /* parse all the message ranges/sequences and set SELECTED */
-    for (msgnum = 0; msgnum < msgp; msgnum++)
+    for (msgnum = 0; msgnum < nummsgs; msgnum++)
 	if (!m_convert (mp, msgs[msgnum]))
 	    done (1);
     seq_setprev (mp);	/* set the previous-sequence */
 
+    /* open and lock new maildrop file */
     if ((md = mbx_open(file, mbx_style, getuid(), getgid(), m_gmprot())) == NOTOK)
 	adios (file, "unable to open");
 
@@ -150,6 +187,8 @@ main (int argc, char **argv)
 
 	    close (fd);
 	}
+
+    /* close and unlock maildrop file */
     mbx_close (file, md);
 
     context_replace (pfolder, folder);	/* update current folder         */
@@ -158,12 +197,13 @@ main (int argc, char **argv)
     seq_save (mp);
     context_save ();			/* save the context file         */
     folder_free (mp);			/* free folder/message structure */
-    done (0);
+    return done (0);
 }
 
-void
+int
 done (int status)
 {
     mbx_close (file, md);
     exit (status);
+    return 1;  /* dead code to satisfy the compiler */
 }
