@@ -2,13 +2,20 @@
 /*
  * pick.c -- search for messages by content
  *
- * $Id$
+ * $Id: pick.c,v 1.2 1999/07/16 01:22:40 danh Exp $
  */
 
 #include <h/mh.h>
 #include <zotnet/tws/tws.h>
 #include <h/picksbr.h>
 
+/*
+ * We allocate space for messages (msgs array)
+ * this number of elements at a time.
+ */
+#define MAXMSGS  256
+
+
 static struct swit switches[] = {
 #define	ANDSW                   0
     { "and", 0 },
@@ -67,11 +74,11 @@ static int listsw = 0;
 int
 main (int argc, char **argv)
 {
-    int publicsw = -1, zerosw = 1, msgp = 0;
-    int seqp = 0, vecp = 0, lo, hi, msgnum;
+    int publicsw = -1, zerosw = 1, seqp = 0, vecp = 0;
+    int nummsgs, maxmsgs, lo, hi, msgnum;
     char *maildir, *folder = NULL, buf[100];
-    char *cp, **ap, **argp, *arguments[MAXARGS];
-    char *msgs[MAXARGS], *seqs[NUMATTRS + 1], *vec[MAXARGS];
+    char *cp, **argp, **arguments;
+    char **msgs, *seqs[NUMATTRS + 1], *vec[MAXARGS];
     struct msgs *mp;
     register FILE *fp;
 
@@ -79,15 +86,22 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
+    /*
+     * Allocate the initial space to record message
+     * names, ranges, and sequences.
+     */
+    nummsgs = 0;
+    maxmsgs = MAXMSGS;
+    if (!(msgs = (char **) malloc ((size_t) (maxmsgs * sizeof(*msgs)))))
+	adios (NULL, "unable to allocate storage");
+
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    if (*++cp == '-') {
@@ -95,84 +109,97 @@ main (int argc, char **argv)
 		goto pattern;
 	    }
 	    switch (smatch (cp, switches)) {
-		case AMBIGSW: 
-		    ambigsw (cp, switches);
-		    done (1);
-		case UNKWNSW: 
-		    adios (NULL, "-%s unknown", cp);
+	    case AMBIGSW: 
+		ambigsw (cp, switches);
+		done (1);
+	    case UNKWNSW: 
+		adios (NULL, "-%s unknown", cp);
 
-		case HELPSW: 
-		    sprintf (buf, "%s [+folder] [msgs] [switches]", invo_name);
-		    print_help (buf, switches, 1);
-		    listsw = 0;	/* HACK */
-		    done (1);
-		case VERSIONSW:
-		    print_version(invo_name);
-		    done (1);
+	    case HELPSW: 
+		snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
+			  invo_name);
+		print_help (buf, switches, 1);
+		listsw = 0;	/* HACK */
+		done (1);
+	    case VERSIONSW:
+		print_version(invo_name);
+		done (1);
 
-		case CCSW: 
-		case DATESW: 
-		case FROMSW: 
-		case SUBJSW: 
-		case TOSW: 
-		case DATFDSW: 
-		case AFTRSW: 
-		case BEFRSW: 
-		case SRCHSW: 
-		    vec[vecp++] = --cp;
-pattern:
-		    if (!(cp = *argp++))/* allow -xyz arguments */
-			adios (NULL, "missing argument to %s", argp[-2]);
-		    vec[vecp++] = cp;
-		    continue;
-		case OTHRSW: 
-		    adios (NULL, "internal error!");
+	    case CCSW: 
+	    case DATESW: 
+	    case FROMSW: 
+	    case SUBJSW: 
+	    case TOSW: 
+	    case DATFDSW: 
+	    case AFTRSW: 
+	    case BEFRSW: 
+	    case SRCHSW: 
+		vec[vecp++] = --cp;
+	    pattern:
+		if (!(cp = *argp++))/* allow -xyz arguments */
+		    adios (NULL, "missing argument to %s", argp[-2]);
+		vec[vecp++] = cp;
+		continue;
+	    case OTHRSW: 
+		adios (NULL, "internal error!");
 
-		case ANDSW:
-		case ORSW:
-		case NOTSW:
-		case LBRSW:
-		case RBRSW:
-		    vec[vecp++] = --cp;
-		    continue;
+	    case ANDSW:
+	    case ORSW:
+	    case NOTSW:
+	    case LBRSW:
+	    case RBRSW:
+		vec[vecp++] = --cp;
+		continue;
 
-		case SEQSW: 
-		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
-		    if (seqp < NUMATTRS)
-			seqs[seqp++] = cp;
-		    else
-			adios (NULL, "only %d sequences allowed!", NUMATTRS);
-		    listsw = 0;
-		    continue;
-		case PUBLSW: 
-		    publicsw = 1;
-		    continue;
-		case NPUBLSW: 
-		    publicsw = 0;
-		    continue;
-		case ZEROSW: 
-		    zerosw++;
-		    continue;
-		case NZEROSW: 
-		    zerosw = 0;
-		    continue;
+	    case SEQSW: 
+		if (!(cp = *argp++) || *cp == '-')
+		    adios (NULL, "missing argument to %s", argp[-2]);
 
-		case LISTSW: 
-		    listsw++;
-		    continue;
-		case NLISTSW: 
-		    listsw = 0;
-		    continue;
+		/* check if too many sequences specified */
+		if (seqp >= NUMATTRS)
+		    adios (NULL, "too many sequences (more than %d) specified", NUMATTRS);
+		seqs[seqp++] = cp;
+		listsw = 0;
+		continue;
+	    case PUBLSW: 
+		publicsw = 1;
+		continue;
+	    case NPUBLSW: 
+		publicsw = 0;
+		continue;
+	    case ZEROSW: 
+		zerosw++;
+		continue;
+	    case NZEROSW: 
+		zerosw = 0;
+		continue;
+
+	    case LISTSW: 
+		listsw++;
+		continue;
+	    case NLISTSW: 
+		listsw = 0;
+		continue;
 	    }
 	}
-	if (*cp == '+' || *cp == '@')
+	if (*cp == '+' || *cp == '@') {
 	    if (folder)
 		adios (NULL, "only one folder at a time!");
 	    else
 		folder = path (cp + 1, *cp == '+' ? TFOLDER : TSUBCWF);
-	else
-	    msgs[msgp++] = cp;
+	} else {
+	    /*
+	     * Check if we need to allocate more space
+	     * for message name/ranges/sequences.
+	     */
+	    if (nummsgs >= maxmsgs) {
+		maxmsgs += MAXMSGS;
+		if (!(msgs = (char **) realloc (msgs,
+						(size_t) (maxmsgs * sizeof(*msgs)))))
+		    adios (NULL, "unable to reallocate msgs storage");
+	    }
+	    msgs[nummsgs++] = cp;
+	}
     }
     vec[vecp] = NULL;
 
@@ -181,10 +208,10 @@ pattern:
 
     /*
      * If we didn't specify which messages to search,
-     * then searcht the whole folder.
+     * then search the whole folder.
      */
-    if (!msgp)
-	msgs[msgp++] = "all";
+    if (!nummsgs)
+	msgs[nummsgs++] = "all";
 
     if (!folder)
 	folder = getfolder (1);
@@ -197,11 +224,12 @@ pattern:
     if (!(mp = folder_read (folder)))
 	adios (NULL, "unable to read folder %s", folder);
 
-    if (mp->hghmsg == 0)
+    /* check for empty folder */
+    if (mp->nummsg == 0)
 	adios (NULL, "no messages in %s", folder);
 
     /* parse all the message ranges/sequences and set SELECTED */
-    for (msgnum = 0; msgnum < msgp; msgnum++)
+    for (msgnum = 0; msgnum < nummsgs; msgnum++)
 	if (!m_convert (mp, msgs[msgnum]))
 	    done (1);
     seq_setprev (mp);	/* set the previous-sequence */
@@ -222,7 +250,11 @@ pattern:
     lo = mp->lowsel;
     hi = mp->hghsel;
 
-    for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++)
+    /*
+     * Scan through all the SELECTED messages and check for a
+     * match.  If the message does not match, then unselect it.
+     */
+    for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
 	if (is_selected (mp, msgnum)) {
 	    if ((fp = fopen (cp = m_name (msgnum), "r")) == NULL)
 		admonish (cp, "unable to read message");
@@ -239,6 +271,7 @@ pattern:
 	    if (fp)
 		fclose (fp);
 	}
+    }
 
     mp->lowsel = lo;
     mp->hghsel = hi;
@@ -270,14 +303,15 @@ pattern:
     seq_save (mp);			/* synchronize message sequences */
     context_save ();			/* save the context file         */
     folder_free (mp);			/* free folder/message structure */
-    done (0);
+    return done (0);
 }
 
 
-void
+int
 done (int status)
 {
     if (listsw && status && !isatty (fileno (stdout)))
 	printf ("0\n");
     exit (status);
+    return 1;  /* dead code to satisfy the compiler */
 }
