@@ -2,13 +2,24 @@
 /*
  * picksbr.c -- routines to help pick along...
  *
- * $Id$
+ * $Id: picksbr.c,v 1.3 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
 #include <zotnet/tws/tws.h>
 #include <h/picksbr.h>
 
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef TM_IN_SYS_TIME
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 static struct swit parswit[] = {
 #define	PRAND                   0
     { "and", 0 },
@@ -386,7 +397,7 @@ exp3 (void)
 	    }
 	    n = newnexus (GREPaction);
 	    n->n_header = 1;
-	    sprintf (buffer, "^%s[ \t]*:.*%s", dp, cp);
+	    snprintf (buffer, sizeof(buffer), "^%s[ \t]*:.*%s", dp, cp);
 	    dp = buffer;
 	    goto pattern;
 
@@ -641,14 +652,15 @@ plist
 		ebp = ibuf + strlen (ibuf);
 	    }
 	    c = *p2++;
-	    if (lf && c != '\n')
+	    if (lf && c != '\n') {
 		if (c != ' ' && c != '\t') {
 		    --p2;
 		    break;
 		}
 		else
 		    lf = 0;
-	    if (c == '\n')
+	    }
+	    if (c == '\n') {
 		if (body)
 		    break;
 		else {
@@ -659,6 +671,7 @@ plist
 		    lf++;
 		    c = ' ';
 		}
+	    }
 	    if (c && p1 < &linebuf[LBSIZE - 1])
 		*p1++ = c;
 	}
@@ -819,21 +832,21 @@ tws_parse (char *ap, int isafter)
     if ((ts = dlocaltimenow ()) == NULL)
 	return NULL;
 
-    sprintf (buffer, "%s %s", ap, dtwszone (ts));
+    snprintf (buffer, sizeof(buffer), "%s %s", ap, dtwszone (ts));
     if ((tw = dparsetime (buffer)) != NULL)
 	return tw;
 
-    sprintf (buffer, "%s %02d:%02d:%02d %s", ap,
+    snprintf (buffer, sizeof(buffer), "%s %02d:%02d:%02d %s", ap,
 	    ts->tw_hour, ts->tw_min, ts->tw_sec, dtwszone (ts));
     if ((tw = dparsetime (buffer)) != NULL)
 	return tw;
 
-    sprintf (buffer, "%02d %s %04d %s",
+    snprintf (buffer, sizeof(buffer), "%02d %s %04d %s",
 	    ts->tw_mday, tw_moty[ts->tw_mon], ts->tw_year, ap);
     if ((tw = dparsetime (buffer)) != NULL)
 	return tw;
 
-    sprintf (buffer, "%02d %s %04d %s %s",
+    snprintf (buffer, sizeof(buffer), "%02d %s %04d %s %s",
 	    ts->tw_mday, tw_moty[ts->tw_mon], ts->tw_year,
 	    ap, dtwszone (ts));
     if ((tw = dparsetime (buffer)) != NULL)
@@ -851,19 +864,19 @@ tws_special (char *ap)
     register struct tws *tw;
 
     time (&clock);
-    if (uleq (ap, "Today"))
+    if (!strcasecmp (ap, "today"))
 	return dlocaltime (&clock);
-    if (uleq (ap, "Yesterday")) {
+    if (!strcasecmp (ap, "yesterday")) {
 	clock -= (long) (60 * 60 * 24);
 	return dlocaltime (&clock);
     }
-    if (uleq (ap, "Tomorrow")) {
+    if (!strcasecmp (ap, "tomorrow")) {
 	clock += (long) (60 * 60 * 24);
 	return dlocaltime (&clock);
     }
 
     for (i = 0; tw_ldotw[i]; i++)
-	if (uleq (ap, tw_ldotw[i]))
+	if (!strcasecmp (ap, tw_ldotw[i]))
 	    break;
     if (tw_ldotw[i]) {
 	if ((tw = dlocaltime (&clock)) == NULL)
@@ -904,7 +917,7 @@ plist
 		    state = m_getfld (state, name, buf, sizeof buf, fp);
 		    bp = add (buf, bp);
 		}
-		if (uleq (name, n->n_datef))
+		if (!strcasecmp (name, n->n_datef))
 		    break;
 		if (state != FLDEOF)
 		    continue;
