@@ -2,7 +2,7 @@
 /*
  * popi.c -- POP initiator for MPOP
  *
- * $Id$
+ * $Id: popi.c,v 1.2 1999/07/16 01:29:08 danh Exp $
  */
 
 #include <h/mh.h>
@@ -91,24 +91,23 @@ main (int argc, char **argv)
     int	autosw = 1, noisy = 1, rpop;
     char *cp, *maildir, *folder = NULL, *form = NULL;
     char *format = NULL, *host = NULL, *user = NULL;
-    char *pass = NULL, buf[100], **ap, **argp;
-    char *arguments[MAXARGS];
+    char *pass = NULL, buf[BUFSIZ], **argp;
+    char **arguments;
     struct stat st;
 
     invo_name = r1bindex (argv[0], '/');
+
+    /* read user profile/context */
+    context_read();
+
     mts_init (invo_name);
+    arguments = getarguments (invo_name, argc, argv, 1);
+    argp = arguments;
+
     if (pophost && *pophost)
 	host = pophost;
     if ((cp = getenv ("MHPOPDEBUG")) && *cp)
 	snoop++;
-    if ((cp = context_find (invo_name)) != NULL) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    }
-    else
-	ap = arguments;
-    copyip (argv + 1, ap);
-    argp = arguments;
 
     rpop = getuid() && !geteuid();
 
@@ -122,7 +121,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [+folder] [switches]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [+folder] [switches]",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -209,7 +209,7 @@ main (int argc, char **argv)
 	setuid (getuid ());
 	ruserpass (host, &user, &pass);
     }
-    sprintf (mailname, "PO box for %s@%s", user, host);
+    snprintf (mailname, sizeof(mailname), "PO box for %s@%s", user, host);
 
     if (pop_init (host, user, pass, snoop, rpop) == NOTOK)
 	adios (NULL, "%s", response);
@@ -238,6 +238,7 @@ main (int argc, char **argv)
 
     if (chdir (maildir) == NOTOK)
 	adios (maildir, "unable to change directory to");
+
     if (!(mp = folder_read (folder)))
 	adios (NULL, "unable to read folder %s", folder);
 
@@ -254,9 +255,7 @@ main (int argc, char **argv)
     seq_setunseen (mp, 0);		/* set the Unseen-Sequence */
     seq_save (mp);
     context_save ();			/* save the context file   */
-    done (0);
-
-    /* NOTREACHED */
+    return done (0);
 }
 
 
@@ -483,7 +482,7 @@ msh (void)
     int	child_id, vecp;
     char buf1[BUFSIZ], buf2[BUFSIZ], *vec[9];
 
-    if (pop_fd (buf1, buf2) == NOTOK)
+    if (pop_fd (buf1, sizeof(buf1), buf2, sizeof(buf2)) == NOTOK)
 	adios (NULL, "%s", response);
 
     vecp = 0;
