@@ -2,7 +2,7 @@
 /*
  * popsbr.c -- POP client subroutines
  *
- * $Id$
+ * $Id: popsbr.c,v 1.3 1999/07/16 01:11:24 danh Exp $
  */
 
 #include <h/mh.h>
@@ -22,12 +22,11 @@
 #include <h/popsbr.h>
 #include <h/signals.h>
 #include <signal.h>
+#include <errno.h>
 
 #define	TRM	"."
 #define	TRMLEN	(sizeof TRM - 1)
 
-extern int errno;
-
 static int poprint = 0;
 static int pophack = 0;
 
@@ -55,7 +54,7 @@ static char xtnd_name[512];	/* INCREDIBLE HACK!! */
  * static prototypes
  */
 #if !defined(NNTP) && defined(APOP)
-static char *pop_auth (char *, char *)
+static char *pop_auth (char *, char *);
 #endif
 
 #if defined(NNTP) || !defined(MPOP)
@@ -74,33 +73,40 @@ static int putline (char *, FILE *);
 static char *
 pop_auth (char *user, char *pass)
 {
-    register char *cp, *lp;
-    register unsigned char *dp;
-    unsigned char *ep, digest[16];
+    int len, buflen;
+    char *cp, *lp;
+    unsigned char *dp, *ep, digest[16];
     MD5_CTX mdContext;
     static char buffer[BUFSIZ];
 
     if ((cp = strchr (response, '<')) == NULL
 	    || (lp = strchr (cp, '>')) == NULL) {
-	sprintf (buffer, "APOP not available: %s", response);
-	strcpy (response, buffer);
+	snprintf (buffer, sizeof(buffer), "APOP not available: %s", response);
+	strncpy (response, buffer, sizeof(response));
 	return NULL;
     }
 
     *++lp = NULL;
-    sprintf (buffer, "%s%s", cp, pass);
+    snprintf (buffer, sizeof(buffer), "%s%s", cp, pass);
 
     MD5Init (&mdContext);
     MD5Update (&mdContext, (unsigned char *) buffer,
 	       (unsigned int) strlen (buffer));
     MD5Final (digest, &mdContext);
 
-    sprintf (cp = buffer, "%s ", user);
-    cp += strlen (cp);
-    for (ep = (dp = digest) + sizeof digest / sizeof digest[0];
-	     dp < ep;
-	     cp += 2)
-	sprintf (cp, "%02x", *dp++ & 0xff);
+    cp = buffer;
+    buflen = sizeof(buffer);
+
+    snprintf (cp, buflen, "%s ", user);
+    len = strlen (cp);
+    cp += len;
+    buflen -= len;
+
+    for (ep = (dp = digest) + sizeof(digest) / sizeof(digest[0]); dp < ep; ) {
+	snprintf (cp, buflen, "%02x", *dp++ & 0xff);
+	cp += 2;
+	buflen -= 2;
+    }
     *cp = NULL;
 
     return buffer;
@@ -108,39 +114,28 @@ pop_auth (char *user, char *pass)
 #endif	/* !NNTP && APOP */
 
 
-#if defined(RPOP) || defined(APOP)
 int
 pop_init (char *host, char *user, char *pass, int snoop, int rpop)
-#else
-int
-pop_init (char *host, char *user, char *pass, int snoop)
-#endif
 {
     int fd1, fd2;
     char buffer[BUFSIZ];
 
 #ifdef APOP
     int apop;
-#endif
 
-#if !defined(RPOP) && !defined(APOP)
-    int	rpop = 0;
-#endif
-
-#ifdef APOP
     if ((apop = rpop) < 0)
 	rpop = 0;
 #endif
 
 #ifndef NNTP
 # ifndef KPOP
-    if ((fd1 = client (host, "tcp", POPSERVICE, rpop, response)) == NOTOK)
+    if ((fd1 = client (host, "tcp", POPSERVICE, rpop, response, sizeof(response))) == NOTOK)
 # else	/* KPOP */
-    sprintf (buffer, "%s/%s", POPSERVICE, "kpop");
-    if ((fd1 = client (host, "tcp", buffer, rpop, response)) == NOTOK)
+    snprintf (buffer, sizeof(buffer), "%s/%s", KPOP_PRINCIPAL, POPSERVICE);
+    if ((fd1 = client (host, "tcp", buffer, rpop, response, sizeof(response))) == NOTOK)
 # endif
 #else	/* NNTP */
-    if ((fd1 = client (host, "tcp", "nntp", rpop, response)) == NOTOK)
+    if ((fd1 = client (host, "tcp", "nntp", rpop, response, sizeof(response))) == NOTOK)
 #endif
 	return NOTOK;
 
@@ -148,9 +143,11 @@ pop_init (char *host, char *user, char *pass, int snoop)
 	char *s;
 
 	if ((s = strerror(errno)))
-	    sprintf (response, "unable to dup connection descriptor: %s", s);
+	    snprintf (response, sizeof(response),
+		"unable to dup connection descriptor: %s", s);
 	else
-	    sprintf (response, "unable to dup connection descriptor: unknown error");
+	    snprintf (response, sizeof(response),
+		"unable to dup connection descriptor: unknown error");
 	close (fd1);
 	return NOTOK;
     }
@@ -195,9 +192,9 @@ pop_init (char *host, char *user, char *pass, int snoop)
 		return OK;
 	    }
 #endif
-	    strcpy (buffer, response);
+	    strncpy (buffer, response, sizeof(buffer));
 	    command ("QUIT");
-	    strcpy (response, buffer);
+	    strncpy (response, buffer, sizeof(response));
 				/* and fall */
 
 	case NOTOK: 
@@ -222,13 +219,15 @@ pop_set (int in, int out, int snoop)
 {
 
 #ifdef NNTP
-    if (myname && *myname)
-	strcpy (xtnd_name, myname);	/* interface from bbc to msh */
+    if (myname && *myname) {
+	/* interface from bbc to msh */
+	strncpy (xtnd_name, myname, sizeof(xtnd_name));
+    }
 #endif	/* NNTP */
 
     if ((input = fdopen (in, "r")) == NULL
 	    || (output = fdopen (out, "w")) == NULL) {
-	strcpy (response, "fdopen failed on connection descriptor");
+	strncpy (response, "fdopen failed on connection descriptor", sizeof(response));
 	if (input)
 	    fclose (input);
 	else
@@ -244,10 +243,10 @@ pop_set (int in, int out, int snoop)
 
 
 int
-pop_fd (char *in, char *out)
+pop_fd (char *in, int inlen, char *out, int outlen)
 {
-    sprintf (in, "%d", fileno (input));
-    sprintf (out, "%d", fileno (output));
+    snprintf (in, inlen, "%d", fileno (input));
+    snprintf (out, outlen, "%d", fileno (output));
     return OK;
 }
 
@@ -396,7 +395,7 @@ traverse (int (*action)(), const char *fmt, ...)
 {
     int result;
     va_list ap;
-    char buffer[sizeof response];
+    char buffer[sizeof(response)];
 
     va_start(ap, fmt);
     result = vcommand (fmt, ap);
@@ -404,7 +403,7 @@ traverse (int (*action)(), const char *fmt, ...)
 
     if (result == NOTOK)
 	return NOTOK;
-    strcpy (buffer, response);
+    strncpy (buffer, response, sizeof(buffer));
 
     for (;;)
 	switch (multiline ()) {
@@ -412,7 +411,7 @@ traverse (int (*action)(), const char *fmt, ...)
 		return NOTOK;
 
 	    case DONE: 
-		strcpy (response, buffer);
+		strncpy (response, buffer, sizeof(response));
 		return OK;
 
 	    case OK: 
@@ -478,12 +477,12 @@ pop_xtnd (int (*action)(), char *fmt, ...)
     va_start(ap, fmt);
 #ifndef NNTP
     /* needs to be fixed... va_end needs to be added */
-    sprintf (buffer, "XTND %s", fmt);
+    snprintf (buffer, sizeof(buffer), "XTND %s", fmt);
     result = traverse (action, buffer, a, b, c, d);
     va_end(ap);
     return result;
 #else /* NNTP */
-    sprintf (buffer, fmt, a, b, c, d);
+    snprintf (buffer, sizeof(buffer), fmt, a, b, c, d);
     ap = brkstring (buffer, " ", "\n");	/* a hack, i know... */
 
     if (!strcasecmp(ap[0], "x-bboards")) {	/* XTND "X-BBOARDS group */
@@ -494,7 +493,7 @@ pop_xtnd (int (*action)(), char *fmt, ...)
 	return OK;
     }
     if (!strcasecmp (ap[0], "archive") && ap[1]) {
-	sprintf (xtnd_name, "%s", ap[1]);		/* save the name */
+	snprintf (xtnd_name, sizeof(xtnd_name), "%s", ap[1]);	/* save the name */
 	xtnd_last = 0;
 	xtnd_first = 1;		/* setup to fail in pop_stat */
 	return OK;
@@ -502,11 +501,12 @@ pop_xtnd (int (*action)(), char *fmt, ...)
     if (!strcasecmp (ap[0], "bboards")) {
 
 	if (ap[1]) {			/* XTND "BBOARDS group" */
-	    sprintf (xtnd_name, "%s", ap[1]);		/* save the name */
+	    snprintf (xtnd_name, sizeof(xtnd_name), "%s", ap[1]);	/* save the name */
 	    if (command("GROUP %s", xtnd_name) == NOTOK)
 		return NOTOK;
 
-	    strcpy (buffer, response);	/* action must ignore extra args */
+	    /* action must ignore extra args */
+	    strncpy (buffer, response, sizeof(buffer));
 	    ap = brkstring (response, " ", "\n");/* "211 nart first last g" */
 	    xtnd_first = atoi (ap[2]);
 	    xtnd_last  = atoi (ap[3]);
@@ -568,8 +568,8 @@ vcommand (const char *fmt, va_list ap)
 {
     char *cp, buffer[BUFSIZ];
 
-    vsprintf (buffer, fmt, ap);
-    if (poprint)
+    vsnprintf (buffer, sizeof(buffer), fmt, ap);
+    if (poprint) {
 	if (pophack) {
 	    if ((cp = strchr (buffer, ' ')))
 		*cp = 0;
@@ -580,6 +580,7 @@ vcommand (const char *fmt, va_list ap)
 	}
 	else
 	    fprintf (stderr, "---> %s\n", buffer);
+    }
 
     if (putline (buffer, output) == NOTOK)
 	return NOTOK;
@@ -625,10 +626,10 @@ multiline (void)
 	if (buffer[TRMLEN] == 0)
 	    return DONE;
 	else
-	    strcpy (response, buffer + TRMLEN);
+	    strncpy (response, buffer + TRMLEN, sizeof(response));
     }
     else
-	strcpy (response, buffer);
+	strncpy (response, buffer, sizeof(response));
 
     return OK;
 }
@@ -645,11 +646,11 @@ getline (char *s, int n, FILE *iop)
 	if ((*p++ = c) == '\n')
 	    break;
     if (ferror (iop) && c != EOF) {
-	strcpy (response, "error on connection");
+	strncpy (response, "error on connection", sizeof(response));
 	return NOTOK;
     }
     if (c == EOF && p == s) {
-	strcpy (response, "connection closed by foreign host");
+	strncpy (response, "connection closed by foreign host", sizeof(response));
 	return DONE;
     }
     *p = 0;
@@ -668,7 +669,7 @@ putline (char *s, FILE *iop)
     fprintf (iop, "%s\r\n", s);
     fflush (iop);
     if (ferror (iop)) {
-	strcpy (response, "lost connection");
+	strncpy (response, "lost connection", sizeof(response));
 	return NOTOK;
     }
 
