@@ -2,7 +2,7 @@
 /*
  * post.c -- enter messages into the mail transport system
  *
- * $Id$
+ * $Id: post.c,v 1.4 1999/09/13 18:20:28 doug Exp $
  */
 
 #include <h/mh.h>
@@ -11,7 +11,7 @@
 #include <h/addrsbr.h>
 #include <h/aliasbr.h>
 #include <h/dropsbr.h>
-#include <h/mhn.h>
+#include <h/mime.h>
 
 #include <zotnet/tws/tws.h>
 #include <zotnet/mts/mts.h>
@@ -20,6 +20,17 @@
 #include <setjmp.h>
 #include <signal.h>
 
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef TM_IN_SYS_TIME
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 #ifdef MMDFMTS
 # include <mts/mmdf/util.h>
 # include <mts/mmdf/mmdf.h>
@@ -298,7 +309,7 @@ int
 main (int argc, char **argv)
 {
     int state, compnum, dashstuff = 0;
-    char *cp, *msg = NULL, **argp = argv + 1;
+    char *cp, *msg = NULL, **argp, **arguments;
     char buf[BUFSIZ], name[NAMESZ];
     FILE *in, *out;
 
@@ -307,11 +318,13 @@ main (int argc, char **argv)
 #endif
     invo_name = r1bindex (argv[0], '/');
 
-    /* foil search of use profile/context */
+    /* foil search of user profile/context */
     if (context_foil (NULL) == -1)
 	done (1);
 
     mts_init (invo_name);
+    arguments = getarguments (invo_name, argc, argv, 0);
+    argp = arguments;
 
 #if defined(MMDFMTS) && defined(MMDFII)
     mmdf_init (invo_name);
@@ -327,7 +340,7 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [switches] file", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches] file", invo_name);
 		    print_help (buf, switches, 0);
 		    done (1);
 		case VERSIONSW:
@@ -531,9 +544,10 @@ main (int argc, char **argv)
 	    if ((out = fopen (fill_in ? fill_in : "/dev/null", "w")) == NULL)
 		adios ("/dev/null", "unable to open");
 	} else {
-	    strcpy (tmpfil, m_scratch ("", m_maildir (invo_name)));
+	    strncpy (tmpfil, m_scratch ("", m_maildir (invo_name)),
+		sizeof(tmpfil));
 	    if ((out = fopen (tmpfil, "w")) == NULL) {
-		strcpy (tmpfil, m_tmpfil (invo_name));
+		strncpy (tmpfil, m_tmpfil (invo_name), sizeof(tmpfil));
 		if ((out = fopen (tmpfil, "w")) == NULL)
 		    adios (tmpfil, "unable to create");
 	    }
@@ -629,7 +643,7 @@ main (int argc, char **argv)
     if (verbose)
 	printf (partno ? "Partial Message #%s Processed\n" : "Message Processed\n",
 		partno);
-    done (0);
+    return done (0);
 }
 
 
@@ -687,7 +701,7 @@ putfmt (char *name, char *str, FILE *out)
 
 	if ((cp = strrchr(str, '\n')))
 	    *cp = 0;
-	for (cp = pp = str; cp = strchr(pp, ','); pp = cp) {
+	for (cp = pp = str; (cp = strchr(pp, ',')); pp = cp) {
 	    *cp++ = 0;
 	    insert_fcc (hdr, pp);
 	}
@@ -701,7 +715,7 @@ putfmt (char *name, char *str, FILE *out)
     }
 
     tmpaddrs.m_next = NULL;
-    for (count = 0; cp = getname (str); count++)
+    for (count = 0; (cp = getname (str)); count++)
 	if ((mp = getm (cp, NULL, 0, AD_HOST, NULL))) {
 	    if (tmpaddrs.m_next)
 		np->m_next = mp;
@@ -728,7 +742,8 @@ putfmt (char *name, char *str, FILE *out)
     }
 
     nameoutput = linepos = 0;
-    sprintf (namep, "%s%s", !fill_in && (hdr->flags & HMNG) ? "Original-" : "", name);
+    snprintf (namep, sizeof(namep), "%s%s",
+		!fill_in && (hdr->flags & HMNG) ? "Original-" : "", name);
 
     for (grp = 0, mp = tmpaddrs.m_next; mp; mp = np)
 	if (mp->m_nohost) {	/* also used to test (hdr->flags & HTRY) */
@@ -800,15 +815,16 @@ start_headers (void)
     mygid = getgid ();
     time (&tclock);
 
-    strcpy (from, adrsprintf (NULL, NULL));
+    strncpy (from, adrsprintf (NULL, NULL), sizeof(from));
+    strncpy (myhost, LocalName (), sizeof(myhost));
 
-    strcpy (myhost, LocalName ());
     for (cp = myhost; *cp; cp++)
 	*cp = uptolow (*cp);
 
     if ((cp = getfullname ()) && *cp) {
-	strcpy (sigbuf, cp);
-	sprintf (signature, "%s <%s>", sigbuf, adrsprintf (NULL, NULL));
+	strncpy (sigbuf, cp, sizeof(sigbuf));
+	snprintf (signature, sizeof(signature), "%s <%s>",
+		sigbuf, adrsprintf (NULL, NULL));
 	if ((cp = getname (signature)) == NULL)
 	    adios (NULL, "getname () failed -- you lose extraordinarily big");
 	if ((mp = getm (cp, NULL, 0, AD_HOST, NULL)) == NULL)
@@ -817,7 +833,7 @@ start_headers (void)
 	while (getname (""))
 	    continue;
     } else {
-	strcpy (signature, adrsprintf (NULL, NULL));
+	strncpy (signature, adrsprintf (NULL, NULL), sizeof(signature));
     }
 }
 
@@ -839,7 +855,7 @@ finish_headers (FILE *out)
 	    fprintf (out, "Date: %s\n", dtime (&tclock, 0));
 	    if (msgid)
 		fprintf (out, "Message-ID: <%d.%ld@%s>\n",
-			(int) getpid (), tclock, LocalName ());
+			(int) getpid (), (long) tclock, LocalName ());
 	    if (msgflags & MFRM)
 		fprintf (out, "Sender: %s\n", from);
 	    else
@@ -870,7 +886,7 @@ finish_headers (FILE *out)
 	    fprintf (out, "Resent-Date: %s\n", dtime (&tclock, 0));
 	    if (msgid)
 		fprintf (out, "Resent-Message-ID: <%d.%ld@%s>\n",
-			(int) getpid (), tclock, LocalName ());
+			(int) getpid (), (long) tclock, LocalName ());
 	    if (msgflags & MRFM)
 		fprintf (out, "Resent-Sender: %s\n", from);
 	    else
@@ -911,7 +927,7 @@ putadr (char *name, char *aka, struct mailname *mp, FILE *out, unsigned int flag
 
     if (mp->m_mbox == NULL || ((flags & HTRY) && !insert (mp)))
 	return 0;
-    if (!fill_in && (flags & (HBCC | HDCC)) || mp->m_ingrp)
+    if ((!fill_in && (flags & (HBCC | HDCC))) || mp->m_ingrp)
 	return 1;
 
     if (!nameoutput) {
@@ -922,22 +938,25 @@ putadr (char *name, char *aka, struct mailname *mp, FILE *out, unsigned int flag
     if (*aka && mp->m_type != UUCPHOST && !mp->m_pers)
 	mp->m_pers = getcpy (aka);
     if (format) {
-	if (mp->m_gname && !fill_in)
-	    sprintf (cp = buffer, "%s;", mp->m_gname);
-	else
+	if (mp->m_gname && !fill_in) {
+	    snprintf (buffer, sizeof(buffer), "%s;", mp->m_gname);
+	    cp = buffer;
+	} else {
 	    cp = adrformat (mp);
-    }
-    else
+	}
+    } else {
 	cp = mp->m_text;
+    }
     len = strlen (cp);
 
-    if (linepos != nameoutput)
+    if (linepos != nameoutput) {
 	if (len + linepos + 2 > outputlinelen)
 	    fprintf (out, ",\n%*s", linepos = nameoutput, "");
 	else {
 	    fputs (", ", out);
 	    linepos += 2;
 	}
+    }
 
     fputs (cp, out);
     linepos += len;
@@ -965,7 +984,7 @@ putgrp (char *name, char *group, FILE *out, unsigned int flags)
     cp = fill_in ? group : concat (group, ";", NULL);
     len = strlen (cp);
 
-    if (linepos > nameoutput)
+    if (linepos > nameoutput) {
 	if (len + linepos + 2 > outputlinelen) {
 	    fprintf (out, ",\n%*s", nameoutput, "");
 	    linepos = nameoutput;
@@ -974,6 +993,7 @@ putgrp (char *name, char *group, FILE *out, unsigned int flags)
 	    fputs (", ", out);
 	    linepos += 2;
 	}
+    }
 
     fputs (cp, out);
     linepos += len;
@@ -1065,7 +1085,7 @@ annoaux (struct mailname *mp)
     int i;
     char buffer[BUFSIZ];
 
-    sprintf (buffer, "%s\n", adrformat (mp));
+    snprintf (buffer, sizeof(buffer), "%s\n", adrformat (mp));
     i = strlen (buffer);
 
     return (write (pfd, buffer, i) == i ? OK : NOTOK);
@@ -1103,7 +1123,7 @@ make_bcc_file (int dashstuff)
     char *vec[6];
     FILE *out;
 
-    strcpy (bccfil, m_tmpfil ("bccs"));
+    strncpy (bccfil, m_tmpfil ("bccs"), sizeof(bccfil));
     if ((out = fopen (bccfil, "w")) == NULL)
 	adios (bccfil, "unable to create");
     chmod (bccfil, 0600);
@@ -1111,7 +1131,7 @@ make_bcc_file (int dashstuff)
     fprintf (out, "Date: %s\n", dtime (&tclock, 0));
     if (msgid)
 	fprintf (out, "Message-ID: <%d.%ld@%s>\n",
-		(int) getpid (), tclock, LocalName ());
+		(int) getpid (), (long) tclock, LocalName ());
     fprintf (out, "From: %s\n", signature);
     if (subject)
 	fprintf (out, "Subject: %s", subject);
@@ -1423,19 +1443,19 @@ do_an_address (struct mailname *lp, int talk)
 	case LOCALHOST: 
 	    mbox = lp->m_mbox;
 	    host = lp->m_host;
-	    strcpy (addr, mbox);
+	    strncpy (addr, mbox, sizeof(addr));
 	    break;
 
 	case UUCPHOST: 
 	    mbox = auxformat (lp, 0);
 	    host = NULL;
-	    sprintf (addr, "%s!%s", lp->m_host, lp->m_mbox);
+	    snprintf (addr, sizeof(addr), "%s!%s", lp->m_host, lp->m_mbox);
 	    break;
 
 	default: 		/* let SendMail decide if the host is bad  */
 	    mbox = lp->m_mbox;
 	    host = lp->m_host;
-	    sprintf (addr, "%s at %s", mbox, host);
+	    snprintf (addr, sizeof(addr), "%s at %s", mbox, host);
 	    break;
     }
 
@@ -1634,7 +1654,7 @@ do_an_address (struct mailname *lp, int talk)
 	case LOCALHOST: 
 	    mbox = lp->m_mbox;
 	    host = LocalName ();
-	    strcpy (addr, mbox);
+	    strncpy (addr, mbox, sizeof(addr));
 	    break;
 
 	case UUCPHOST: 
@@ -1647,7 +1667,7 @@ do_an_address (struct mailname *lp, int talk)
 	default: 		/* let MMDF decide if the host is bad */
 	    mbox = lp->m_mbox;
 	    host = lp->m_host;
-	    sprintf (addr, "%s at %s", mbox, host);
+	    snprintf (addr, sizeof(addr), "%s at %s", mbox, host);
 	    break;
     }
 
@@ -1881,7 +1901,7 @@ fcc (char *file, char *folder)
 
 	case OK: 
 	    /* see if we need to add `+' */
-	    sprintf (fold, "%s%s",
+	    snprintf (fold, sizeof(fold), "%s%s",
 		    *folder == '+' || *folder == '@' ? "" : "+", folder);
 
 	    /* now exec the fileproc */
@@ -1947,7 +1967,7 @@ err_abrt (int code, char *fmt, ...)
     char buffer[BUFSIZ];
     va_list ap;
 
-    sprintf (buffer, "[%s]", rp_valstr (code));
+    snprintf (buffer, sizeof(buffer), "[%s]", rp_valstr (code));
 
     va_start(ap, fmt);
     advertise (buffer, NULL, fmt, ap);
