@@ -2,7 +2,7 @@
 /*
  * prompter.c -- simple prompting editor front-end
  *
- * $Id$
+ * $Id: prompter.c,v 1.2 1999/07/16 01:28:48 danh Exp $
  */
 
 #include <h/mh.h>
@@ -102,21 +102,19 @@ main (int argc, char **argv)
     int doteof = 0, fdi, fdo, i, state;
     char *cp, *drft = NULL, *erasep = NULL;
     char *killp = NULL, name[NAMESZ], field[BUFSIZ];
-    char buffer[BUFSIZ], tmpfil[BUFSIZ], **ap;
-    char *arguments[MAXARGS], **argp;
+    char buffer[BUFSIZ], tmpfil[BUFSIZ];
+    char **arguments, **argp;
     FILE *in, *out;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++))
@@ -129,8 +127,9 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buffer, "%s [switches] file", invo_name);
-		    print_help (buffer, switches);
+		    snprintf (buffer, sizeof(buffer), "%s [switches] file",
+			invo_name);
+		    print_help (buffer, switches, 1);
 		    done (1);
 		case VERSIONSW:
 		    print_version(invo_name);
@@ -183,12 +182,14 @@ main (int argc, char **argv)
     if ((in = fopen (drft, "r")) == NULL)
 	adios (drft, "unable to open");
 
-    strcpy (tmpfil, m_tmpfil (invo_name));
+    strncpy (tmpfil, m_tmpfil (invo_name), sizeof(tmpfil));
     if ((out = fopen (tmpfil, "w")) == NULL)
 	adios (tmpfil, "unable to create");
     chmod (tmpfil, 0600);
 
-    /* Are we changing the kill or erase character? */
+    /*
+     * Are we changing the kill or erase character?
+     */
     if (killp || erasep) {
 #ifdef HAVE_TERMIOS_H
 	cc_t save_erase, save_kill;
@@ -246,14 +247,23 @@ main (int argc, char **argv)
     sigint = 0;
     SIGNAL2 (SIGINT, intrser);
 
+    /*
+     * Loop through the lines of the draft skeleton.
+     */
     for (state = FLD;;) {
 	switch (state = m_getfld (state, name, field, sizeof(field), in)) {
 	    case FLD: 
 	    case FLDEOF: 
 	    case FLDPLUS: 
+		/*
+		 * Check if the value of field contains anything
+		 * other than space or tab.
+		 */
 		for (cp = field; *cp; cp++)
 		    if (*cp != ' ' && *cp != '\t')
 			break;
+
+		/* If so, just add header line to draft */
 		if (*cp++ != '\n' || *cp != 0) {
 		    printf ("%s:%s", name, field);
 		    fprintf (out, "%s:%s", name, field);
@@ -264,11 +274,12 @@ main (int argc, char **argv)
 			fprintf (out, "%s", field);
 		    }
 		} else {
+		    /* Else, get value of header field */
 		    printf ("%s: ", name);
 		    fflush (stdout);
 		    i = getln (field, sizeof(field));
 		    if (i == -1) {
-abort: ;
+abort:
 			if (killp || erasep) {
 #ifdef HAVE_TERMIOS_H
 			    tcsetattr(0, TCSADRAIN, &tio);
@@ -295,7 +306,8 @@ abort: ;
 			    goto abort;
 		    }
 		}
-		if (state == FLDEOF) {/* moby hack */
+
+		if (state == FLDEOF) {	/* moby hack */
 		    fprintf (out, "--------\n");
 		    printf ("--------\n");
 		    if (!body)
@@ -337,7 +349,7 @@ abort: ;
 		    else
 			printf ("\n--------Enter additional text\n\n");
 		}
-no_body: ;
+no_body:
 		fflush (stdout);
 		for (;;) {
 		    getln (field, sizeof(field));
@@ -385,7 +397,7 @@ no_body: ;
     unlink (tmpfil);
 
     context_save ();	/* save the context file */
-    done (0);
+    return done (0);
 }
 
 
