@@ -2,13 +2,14 @@
 /*
  * rcvdist.c -- asynchronously redistribute messages
  *
- * $Id$
+ * $Id: rcvdist.c,v 1.3 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
 #include <h/fmt_scan.h>
 #include <h/rcvmail.h>
 #include <zotnet/tws/tws.h>
+#include <zotnet/mts/mts.h>
 
 static struct swit switches[] = {
 #define	FORMSW       0
@@ -28,7 +29,7 @@ static char tmpfil[BUFSIZ] = "";
  * prototypes
  */
 static void rcvdistout (FILE *, char *, char *);
-void done (int);
+int done (int);
 
 
 int
@@ -36,22 +37,20 @@ main (int argc, char **argv)
 {
     pid_t child_id;
     int i, vecp = 1;
-    char *addrs = NULL, *cp, *form = NULL, buf[100];
-    char **ap, **argp, *arguments[MAXARGS], *vec[MAXARGS];
+    char *addrs = NULL, *cp, *form = NULL, buf[BUFSIZ];
+    char **argp, **arguments, *vec[MAXARGS];
     register FILE *fp;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
+
+    /* read user profile/context */
+    context_read();
+
     mts_init (invo_name);
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -65,8 +64,9 @@ main (int argc, char **argv)
 		    continue;
 
 		case HELPSW: 
-		    sprintf (buf, "%s [switches] [switches for postproc] address ...",
-			    invo_name);
+		    snprintf (buf, sizeof(buf),
+			"%s [switches] [switches for postproc] address ...",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -87,12 +87,12 @@ main (int argc, char **argv)
 	    invo_name);
 
     umask (~m_gmprot ());
-    strcpy (tmpfil, m_tmpfil (invo_name));
+    strncpy (tmpfil, m_tmpfil (invo_name), sizeof(tmpfil));
     if ((fp = fopen (tmpfil, "w+")) == NULL)
 	adios (tmpfil, "unable to create");
     cpydata (fileno (stdin), fileno (fp), "message", tmpfil);
     fseek (fp, 0L, SEEK_SET);
-    strcpy (drft, m_tmpfil (invo_name));
+    strncpy (drft, m_tmpfil (invo_name), sizeof(drft));
     rcvdistout (fp, form, addrs);
     fclose (fp);
 
@@ -118,7 +118,8 @@ main (int argc, char **argv)
 	default: 
 	    done (pidXwait(child_id, postproc));
     }
-/* NOTREACHED */
+
+    return 0;  /* dead code to satisfy the compiler */
 }
 
 /* very similar to routine in replsbr.c */
@@ -206,13 +207,14 @@ rcvdistout (FILE *inb, char *form, char *addrs)
 			    }
 			    else {
 				i = strlen (cp = cptr->c_text) - 1;
-				if (cp[i] == '\n')
+				if (cp[i] == '\n') {
 				    if (cptr->c_type & CT_ADDR) {
 					cp[i] = 0;
 					cp = add (",\n\t", cp);
 				    }
 				    else
 					cp = add ("\t", cp);
+				}
 				cptr->c_text = add (tmpbuf, cp);
 			    }
 			    while (state == FLDPLUS) {
@@ -253,7 +255,7 @@ finished: ;
     fclose (out);
 
     free (scanl);
-    for (nxtbuf = compbuffers, i = ncomps; cptr = *savecomp++; nxtbuf++, i--)
+    for (nxtbuf = compbuffers, i = ncomps; (cptr = *savecomp++); nxtbuf++, i--)
 	free (cptr->c_text);
     while (i-- > 0)
         free (*nxtbuf++);
@@ -262,7 +264,7 @@ finished: ;
 }
 
 
-void
+int
 done (int status)
 {
     if (backup[0])
@@ -273,4 +275,5 @@ done (int status)
 	unlink (tmpfil);
 
     exit (status ? RCV_MBX : RCV_MOK);
+    return 1;  /* dead code to satisfy the compiler */
 }
