@@ -2,7 +2,7 @@
 /*
  * rcvpack.c -- append message to a file
  *
- * $Id$
+ * $Id: rcvpack.c,v 1.2 1999/07/16 01:28:27 danh Exp $
  */
 
 #include <h/mh.h>
@@ -33,24 +33,22 @@ int
 main (int argc, char **argv)
 {
     int md;
-    char *cp, *file = NULL, buf[100];
-    char **ap, **argp;
-    char *arguments[MAXARGS];
+    char *cp, *file = NULL, buf[BUFSIZ];
+    char **argp, **arguments;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
+
+    /* read user profile/context */
+    context_read();
+
     mts_init (invo_name);
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
+    /* parse arguments */
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
@@ -61,7 +59,7 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [switches] file", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches] file", invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -87,16 +85,19 @@ main (int argc, char **argv)
 
     rewind (stdin);
 
+    /* open and lock the file */
     if ((md = mbx_open (file, mbx_style, getuid(), getgid(), m_gmprot())) == NOTOK)
 	done (RCV_MBX);
 
+    /* append the message */
     if (mbx_copy (file, mbx_style, md, fileno(stdin), 1, NULL, 0) == NOTOK) {
 	mbx_close (file, md);
 	done (RCV_MBX);
     }
 
+    /* close and unlock the file */
     if (mbx_close (file, md) == NOTOK)
 	done (RCV_MBX);
 
-    done (RCV_MOK);
+    return done (RCV_MOK);
 }
