@@ -2,7 +2,7 @@
 /*
  * rcvstore.c -- asynchronously add mail to a folder
  *
- * $Id$
+ * $Id: rcvstore.c,v 1.3 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
@@ -10,6 +10,7 @@
 #include <h/signals.h>
 #include <errno.h>
 #include <signal.h>
+#include <zotnet/mts/mts.h>
 
 static struct swit switches[] = {
 #define CRETSW         0
@@ -51,8 +52,8 @@ main (int argc, char **argv)
     int publicsw = -1, zerosw = 0;
     int create = 1, unseensw = 1;
     int fd, msgnum, seqp = 0;
-    char *cp, *maildir, *folder = NULL, buf[100];
-    char **ap, **argp, *arguments[MAXARGS], *seqs[NUMATTRS+1];
+    char *cp, *maildir, *folder = NULL, buf[BUFSIZ];
+    char **argp, **arguments, *seqs[NUMATTRS+1];
     struct msgs *mp;
     struct stat st;
 
@@ -60,70 +61,70 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
+
+    /* read user profile/context */
+    context_read();
+
     mts_init (invo_name);
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
+    /* parse arguments */
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
-		case AMBIGSW: 
-		    ambigsw (cp, switches);
-		    done (1);
-		case UNKWNSW: 
-		    adios (NULL, "-%s unknown", cp);
+	    case AMBIGSW: 
+		ambigsw (cp, switches);
+		done (1);
+	    case UNKWNSW: 
+		adios (NULL, "-%s unknown", cp);
 
-		case HELPSW: 
-		    sprintf (buf, "%s [+folder] [switches]", invo_name);
-		    print_help (buf, switches, 1);
-		    done (1);
-		case VERSIONSW:
-		    print_version(invo_name);
-		    done (1);
+	    case HELPSW: 
+		snprintf (buf, sizeof(buf), "%s [+folder] [switches]",
+			  invo_name);
+		print_help (buf, switches, 1);
+		done (1);
+	    case VERSIONSW:
+		print_version(invo_name);
+		done (1);
 
-		case SEQSW: 
-		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument name to %s",
-				argp[-2]);
-		    if (seqp < NUMATTRS)
-			seqs[seqp++] = cp;
-		    else
-			adios (NULL, "only %d sequences allowed!", NUMATTRS);
-		    continue;
+	    case SEQSW: 
+		if (!(cp = *argp++) || *cp == '-')
+		    adios (NULL, "missing argument name to %s", argp[-2]);
 
-		case UNSEENSW:
-		    unseensw = 1;
-		    continue;
-		case NUNSEENSW:
-		    unseensw = 0;
-		    continue;
+		/* check if too many sequences specified */
+		if (seqp >= NUMATTRS)
+		    adios (NULL, "too many sequences (more than %d) specified", NUMATTRS);
+		seqs[seqp++] = cp;
+		continue;
 
-		case PUBSW: 
-		    publicsw = 1;
-		    continue;
-		case NPUBSW: 
-		    publicsw = 0;
-		    continue;
+	    case UNSEENSW:
+		unseensw = 1;
+		continue;
+	    case NUNSEENSW:
+		unseensw = 0;
+		continue;
 
-		case ZEROSW: 
-		    zerosw++;
-		    continue;
-		case NZEROSW: 
-		    zerosw = 0;
-		    continue;
+	    case PUBSW: 
+		publicsw = 1;
+		continue;
+	    case NPUBSW: 
+		publicsw = 0;
+		continue;
 
-		case CRETSW: 
-		    create++;
-		    continue;
-		case NCRETSW: 
-		    create = 0;
-		    continue;
+	    case ZEROSW: 
+		zerosw++;
+		continue;
+	    case NZEROSW: 
+		zerosw = 0;
+		continue;
+
+	    case CRETSW: 
+		create++;
+		continue;
+	    case NCRETSW: 
+		create = 0;
+		continue;
 	    }
 	}
 	if (*cp == '+' || *cp == '@') {
@@ -218,16 +219,17 @@ main (int argc, char **argv)
     unlink (tmpfilenam);	/* remove temporary file                  */
     tmpfilenam = NULL;
 
-    done (0);
+    return done (0);
 }
 
 /*
  * Clean up and exit
  */
-void
+int
 done(int status)
 {
     if (tmpfilenam && *tmpfilenam)
 	unlink (tmpfilenam);
     exit (status);
+    return 1;  /* dead code to satisfy the compiler */
 }
