@@ -2,7 +2,7 @@
 /*
  * rcvtty.c -- a rcvmail program (a lot like rcvalert) handling IPC ttys
  *
- * $Id$
+ * $Id: rcvtty.c,v 1.5 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
@@ -10,6 +10,7 @@
 #include <h/rcvmail.h>
 #include <h/scansbr.h>
 #include <zotnet/tws/tws.h>
+#include <zotnet/mts/mts.h>
 #include <signal.h>
 #include <fcntl.h>
 
@@ -77,9 +78,8 @@ int
 main (int argc, char **argv)
 {
     int md, vecp = 0;
-    char *cp, *user, buf[100], **ap;
-    char **argp, *arguments[MAXARGS], *vec[MAXARGS];
-    char tty[BUFSIZ];
+    char *cp, *user, buf[BUFSIZ], tty[BUFSIZ];
+    char **argp, **arguments, *vec[MAXARGS];
     struct utmp ut;
     register FILE *uf;
 
@@ -87,14 +87,12 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
+
+    /* read user profile/context */
+    context_read();
+
     mts_init (invo_name);
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -108,7 +106,7 @@ main (int argc, char **argv)
 		    continue;
 
 		case HELPSW: 
-		    sprintf (buf, "%s [command ...]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [command ...]", invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -170,6 +168,7 @@ main (int argc, char **argv)
 
     fclose (uf);
     exit (RCV_MOK);
+    return 0;  /* dead code to satisfy the compiler */
 }
 
 
@@ -192,10 +191,14 @@ message_fd (char **vec)
     char tmpfil[BUFSIZ];
     struct stat st;
 
-    unlink (mktemp (strcpy (tmpfil, "/tmp/rcvttyXXXXX")));
+#ifdef HAVE_MKSTEMP
+    fd = mkstemp (strncpy (tmpfil, "/tmp/rcvttyXXXXX", sizeof(tmpfil)));
+#else
+    unlink (mktemp (strncpy (tmpfil, "/tmp/rcvttyXXXXX", sizeof(tmpfil))));
     if ((fd = open (tmpfil, O_RDWR | O_CREAT | O_TRUNC, 0600)) == NOTOK)
 	return header_fd ();
     unlink (tmpfil);
+#endif
 
     if ((child_id = vfork()) == NOTOK) {
 	/* fork error */
@@ -242,6 +245,7 @@ message_fd (char **vec)
     setpgid ((pid_t) 0, getpid ());	/* put in own process group */
     execvp (vec[0], vec);
     _exit (-1);
+    return 1;  /* dead code to satisfy compiler */
 }
 
 
@@ -251,7 +255,7 @@ header_fd (void)
     int fd;
     char *nfs, tmpfil[BUFSIZ];
 
-    strcpy (tmpfil, m_tmpfil (invo_name));
+    strncpy (tmpfil, m_tmpfil (invo_name), sizeof(tmpfil));
     if ((fd = open (tmpfil, O_RDWR | O_CREAT | O_TRUNC, 0600)) == NOTOK)
 	return NOTOK;
     unlink (tmpfil);
@@ -278,7 +282,7 @@ alert (char *tty, int md)
     char buffer[BUFSIZ], ttyspec[BUFSIZ];
     struct stat st;
 
-    sprintf (ttyspec, "/dev/%s", tty);
+    snprintf (ttyspec, sizeof(ttyspec), "/dev/%s", tty);
 
     /*
      * The mask depends on whether we are checking for
