@@ -3,13 +3,20 @@
  * refile.c -- move or link message(s) from a source folder
  *          -- into one or more destination folders
  *
- * $Id$
+ * $Id: refile.c,v 1.2 1999/07/16 01:28:07 danh Exp $
  */
 
 #include <h/mh.h>
 #include <fcntl.h>
 #include <errno.h>
 
+/*
+ * We allocate spaces for messages (msgs array)
+ * this number of elements at a time.
+ */
+#define MAXMSGS  256
+
+
 static struct swit switches[] = {
 #define	DRAFTSW          0
     { "draft", 0 },
@@ -21,17 +28,21 @@ static struct swit switches[] = {
     { "preserve", 0 },
 #define	NPRESSW          4
     { "nopreserve", 0 },
-#define	SRCSW            5
+#define UNLINKSW         5
+    { "unlink", 0 },
+#define NUNLINKSW        6
+    { "nounlink", 0 },
+#define	SRCSW            7
     { "src +folder", 0 },
-#define	FILESW           6
+#define	FILESW           8
     { "file file", 0 },
-#define	RPROCSW          7
+#define	RPROCSW          9
     { "rmmproc program", 0 },
-#define	NRPRCSW          8
+#define	NRPRCSW         10
     { "normmproc", 0 },
-#define VERSIONSW        9
+#define VERSIONSW       11
     { "version", 0 },
-#define	HELPSW          10
+#define	HELPSW          12
     { "help", 4 },
     { NULL, 0 }
 };
@@ -58,13 +69,12 @@ int
 main (int argc, char **argv)
 {
     int	linkf = 0, preserve = 0, filep = 0;
-    int foldp = 0, msgp = 0, isdf = 0;
-    int i, msgnum;
-    char *cp, *folder = NULL, buf[100];
-    char **ap, **argp, *arguments[MAXARGS];
+    int foldp = 0, isdf = 0, unlink_msgs = 0;
+    int i, msgnum, nummsgs, maxmsgs;
+    char *cp, *folder = NULL, buf[BUFSIZ];
+    char **argp, **arguments, **msgs;
     char *filevec[NFOLDERS + 2];
     char **files = &filevec[1];	   /* leave room for remove_files:vec[0] */
-    char *msgs[MAXARGS];
     struct st_fold folders[NFOLDERS + 1];
     struct msgs *mp;
 
@@ -72,84 +82,112 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
+    /*
+     * Allocate the initial space to record message
+     * names, ranges, and sequences.
+     */
+    nummsgs = 0;
+    maxmsgs = MAXMSGS;
+    if (!(msgs = (char **) malloc ((size_t) (maxmsgs * sizeof(*msgs)))))
+	adios (NULL, "unable to allocate storage");
+
+    /*
+     * Parse arguments
+     */
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
-		case AMBIGSW: 
-		    ambigsw (cp, switches);
-		    done (1);
-		case UNKWNSW: 
-		    adios (NULL, "-%s unknown\n", cp);
+	    case AMBIGSW: 
+		ambigsw (cp, switches);
+		done (1);
+	    case UNKWNSW: 
+		adios (NULL, "-%s unknown\n", cp);
 
-		case HELPSW: 
-		    sprintf (buf, "%s [msgs] [switches] +folder ...", invo_name);
-		    print_help (buf, switches, 1);
-		    done (1);
-		case VERSIONSW:
-		    print_version(invo_name);
-		    done (1);
+	    case HELPSW: 
+		snprintf (buf, sizeof(buf), "%s [msgs] [switches] +folder ...",
+			  invo_name);
+		print_help (buf, switches, 1);
+		done (1);
+	    case VERSIONSW:
+		print_version(invo_name);
+		done (1);
 
-		case LINKSW: 
-		    linkf++;
-		    continue;
-		case NLINKSW: 
-		    linkf = 0;
-		    continue;
+	    case LINKSW: 
+		linkf++;
+		continue;
+	    case NLINKSW: 
+		linkf = 0;
+		continue;
 
-		case PRESSW: 
-		    preserve++;
-		    continue;
-		case NPRESSW: 
-		    preserve = 0;
-		    continue;
+	    case PRESSW: 
+		preserve++;
+		continue;
+	    case NPRESSW: 
+		preserve = 0;
+		continue;
 
-		case SRCSW: 
-		    if (folder)
-			adios (NULL, "only one source folder at a time!");
-		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
-		    folder = path (*cp == '+' || *cp == '@' ? cp + 1 : cp,
-				   *cp != '@' ? TFOLDER : TSUBCWF);
-		    continue;
-		case DRAFTSW:
-		    if (filep > NFOLDERS)
-			adios (NULL, "only %d files allowed!", NFOLDERS);
-		    isdf = 0;
-		    files[filep++] = getcpy (m_draft (NULL, NULL, 1, &isdf));
-		    continue;
-		case FILESW: 
-		    if (filep > NFOLDERS)
-			adios (NULL, "only %d files allowed!", NFOLDERS);
-		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
-		    files[filep++] = path (cp, TFILE);
-		    continue;
+	    case UNLINKSW:
+		unlink_msgs++;
+		continue;
+	    case NUNLINKSW:
+		unlink_msgs = 0;
+		continue;
 
-		case RPROCSW: 
-		    if (!(rmmproc = *argp++) || *rmmproc == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
-		    continue;
-		case NRPRCSW: 
-		    rmmproc = NULL;
-		    continue;
+	    case SRCSW: 
+		if (folder)
+		    adios (NULL, "only one source folder at a time!");
+		if (!(cp = *argp++) || *cp == '-')
+		    adios (NULL, "missing argument to %s", argp[-2]);
+		folder = path (*cp == '+' || *cp == '@' ? cp + 1 : cp,
+			       *cp != '@' ? TFOLDER : TSUBCWF);
+		continue;
+	    case DRAFTSW:
+		if (filep > NFOLDERS)
+		    adios (NULL, "only %d files allowed!", NFOLDERS);
+		isdf = 0;
+		files[filep++] = getcpy (m_draft (NULL, NULL, 1, &isdf));
+		continue;
+	    case FILESW: 
+		if (filep > NFOLDERS)
+		    adios (NULL, "only %d files allowed!", NFOLDERS);
+		if (!(cp = *argp++) || *cp == '-')
+		    adios (NULL, "missing argument to %s", argp[-2]);
+		files[filep++] = path (cp, TFILE);
+		continue;
+
+	    case RPROCSW: 
+		if (!(rmmproc = *argp++) || *rmmproc == '-')
+		    adios (NULL, "missing argument to %s", argp[-2]);
+		continue;
+	    case NRPRCSW: 
+		rmmproc = NULL;
+		continue;
 	    }
 	}
 	if (*cp == '+' || *cp == '@') {
 	    if (foldp > NFOLDERS)
 		adios (NULL, "only %d folders allowed!", NFOLDERS);
 	    folders[foldp++].f_name =
-		    path (cp + 1, *cp == '+' ? TFOLDER : TSUBCWF);
+		path (cp + 1, *cp == '+' ? TFOLDER : TSUBCWF);
 	} else {
-	    msgs[msgp++] = cp;
+	    /*
+	     * Check if we need to allocate more space
+	     * for message names, ranges, and sequences.
+	     */
+	    if (nummsgs >= maxmsgs) {
+		maxmsgs += MAXMSGS;
+		if (!(msgs = (char **) realloc (msgs,
+						(size_t) (maxmsgs * sizeof(*msgs)))))
+		    adios (NULL, "unable to reallocate msgs storage");
+	    }
+	    msgs[nummsgs++] = cp;
 	}
     }
 
@@ -159,7 +197,7 @@ main (int argc, char **argv)
 	adios (NULL, "no folder specified");
 
 #ifdef WHATNOW
-    if (!msgp && !foldp && !filep && (cp = getenv ("mhdraft")) && *cp)
+    if (!nummsgs && !foldp && !filep && (cp = getenv ("mhdraft")) && *cp)
 	files[filep++] = cp;
 #endif /* WHATNOW */
 
@@ -167,7 +205,7 @@ main (int argc, char **argv)
      * We are refiling a file to the folders
      */
     if (filep > 0) {
-	if (folder || msgp)
+	if (folder || nummsgs)
 	    adios (NULL, "use -file or some messages, not both");
 	opnfolds (folders, foldp);
 	for (i = 0; i < filep; i++)
@@ -179,23 +217,25 @@ main (int argc, char **argv)
 	done (0);
     }
 
-    if (!msgp)
-	msgs[msgp++] = "cur";
+    if (!nummsgs)
+	msgs[nummsgs++] = "cur";
     if (!folder)
 	folder = getfolder (1);
-    strcpy (maildir, m_maildir (folder));
+    strncpy (maildir, m_maildir (folder), sizeof(maildir));
 
     if (chdir (maildir) == NOTOK)
 	adios (maildir, "unable to change directory to");
 
-    /* read source folder */
+    /* read source folder and create message structure */
     if (!(mp = folder_read (folder)))
 	adios (NULL, "unable to read folder %s", folder);
-    if (mp->hghmsg == 0)
+
+    /* check for empty folder */
+    if (mp->nummsg == 0)
 	adios (NULL, "no messages in %s", folder);
 
     /* parse the message range/sequence/name and set SELECTED */
-    for (msgnum = 0; msgnum < msgp; msgnum++)
+    for (msgnum = 0; msgnum < nummsgs; msgnum++)
 	if (!m_convert (mp, msgs[msgnum]))
 	    done (1);
     seq_setprev (mp);	/* set the previous-sequence */
@@ -227,20 +267,20 @@ main (int argc, char **argv)
 
     /* If -nolink, then "remove" messages from source folder */
     if (!linkf) {
-	folder_delmsgs (mp, 0);
+	folder_delmsgs (mp, unlink_msgs);
     }
 
     clsfolds (folders, foldp);
 
     if (mp->hghsel != mp->curmsg
-	    && (mp->numsel != mp->nummsg || linkf))
+	&& (mp->numsel != mp->nummsg || linkf))
 	seq_setcur (mp, mp->hghsel);
     seq_save (mp);	/* synchronize message sequences */
 
     context_replace (pfolder, folder);	/* update current folder   */
     context_save ();			/* save the context file   */
     folder_free (mp);			/* free folder structure   */
-    done (0);
+    return done (0);
 }
 
 
@@ -260,7 +300,7 @@ opnfolds (struct st_fold *folders, int nfolders)
 
     for (fp = folders, ep = folders + nfolders; fp < ep; fp++) {
 	chdir (m_maildir (""));
-	strcpy (nmaildir, m_maildir (fp->f_name));
+	strncpy (nmaildir, m_maildir (fp->f_name), sizeof(nmaildir));
 
 	if (stat (nmaildir, &st) == NOTOK) {
 	    if (errno != ENOENT)
