@@ -2,7 +2,7 @@
 /*
  * repl.c -- reply to a message
  *
- * $Id$
+ * $Id: repl.c,v 1.3 1999/10/17 17:39:23 ruud Exp $
  */
 
 #include <h/mh.h>
@@ -108,9 +108,9 @@ static struct swit aqrl[] = {
     { NULL, 0 }
 };
 
-short ccto = 1;		/* global for replsbr */
-short cccc = 1;
-short ccme = 1;
+short ccto = 0;		/* global for replsbr */
+short cccc = 0;
+short ccme = 0;
 short querysw = 0;
 
 short outputlinelen = OUTPUTLINELEN;
@@ -177,9 +177,11 @@ main (int argc, char **argv)
 
 		case GROUPSW:
 		    groupreply++;
+		    ccto = cccc = ccme = 1;
 		    continue;
 		case NGROUPSW:
 		    groupreply = 0;
+		    ccto = cccc = ccme = 0;
 		    continue;
 
 		case ANNOSW: 
@@ -430,7 +432,7 @@ try_it_again:
 	done (0);
     what_now (ed, nedit, NOUSE, drft, msg, 0, mp,
 	    anot ? "Replied" : NULL, inplace, cwd);
-    done (1);
+    return done (1);
 }
 
 void
