@@ -2,7 +2,7 @@
 /*
  * rmf.c -- remove a folder
  *
- * $Id$
+ * $Id: rmf.c,v 1.2 1999/07/16 01:27:31 danh Exp $
  */
 
 #include <h/mh.h>
@@ -31,19 +31,17 @@ main (int argc, char **argv)
 {
     int defolder = 0, interactive = -1;
     char *cp, *folder = NULL, newfolder[BUFSIZ];
-    char buf[100], **ap, **argp, *arguments[MAXARGS];
+    char buf[BUFSIZ], **argp, **arguments;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -56,7 +54,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [+folder] [switches]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [+folder] [switches]",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -99,9 +98,9 @@ main (int argc, char **argv)
 	if (cp > newfolder)
 	    *cp = '\0';
 	else
-	    strcpy (newfolder, getfolder(0));
+	    strncpy (newfolder, getfolder(0), sizeof(newfolder));
     } else {
-	strcpy (newfolder, getfolder(0));
+	strncpy (newfolder, getfolder(0), sizeof(newfolder));
     }
 
     if (interactive) {
@@ -116,7 +115,7 @@ main (int argc, char **argv)
 	context_replace (pfolder, newfolder);	/* update current folder */
     }
     context_save ();	/* save the context file */
-    done (0);
+    return done (0);
 }
 
 static int
@@ -134,7 +133,8 @@ rmf (char *folder)
 		break;		/* fall otherwise */
 
 	case NOTOK: 
-	    sprintf (cur, "atr-%s-%s", current, m_mailpath (folder));
+	    snprintf (cur, sizeof(cur), "atr-%s-%s",
+			current, m_mailpath (folder));
 	    if (!context_del (cur)) {
 		printf ("[+%s de-referenced]\n", folder);
 		return OK;
@@ -213,13 +213,16 @@ rma (char *folder)
     register char *cp;
     register struct node *np, *pp;
 
+    /* sanity check - check that context has been read */
+    if (defpath == NULL)
+	adios (NULL, "oops, context hasn't been read yet");
+
     alen = strlen ("atr-");
     plen = strlen (cp = m_mailpath (folder)) + 1;
-    context_read ();
 
     /*
      * Search context list for keys that look like
-     * "atr-something-foldername", and remove them.
+     * "atr-something-folderpath", and remove them.
      */
     for (np = m_defs, pp = NULL; np; np = np->n_next) {
 	if (ssequal ("atr-", np->n_name)
