@@ -2,13 +2,14 @@
 /*
  * scan.c -- display a one-line "scan" listing of folder or messages
  *
- * $Id$
+ * $Id: scan.c,v 1.3 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
 #include <h/fmt_scan.h>
 #include <h/scansbr.h>
 #include <zotnet/tws/tws.h>
+#include <zotnet/mts/mts.h>
 #include <errno.h>
 
 /*
@@ -347,5 +348,5 @@ main (int argc, char **argv)
     if (clearflag)
 	clear_screen ();
 
-    done (0);
+    return done (0);
 }
