@@ -2,7 +2,7 @@
 /*
  * scansbr.c -- routines to help scan along...
  *
- * $Id$
+ * $Id: scansbr.c,v 1.1.1.1 1999/04/30 18:08:35 doug Exp $
  */
 
 #include <h/mh.h>
@@ -26,7 +26,7 @@
 #define MAXSCANL 256		/* longest possible scan line */
 
 /*
- * buffer size for content part of header fields.  We want this
+ * Buffer size for content part of header fields.  We want this
  * to be large enough so that we don't do a lot of extra FLDPLUS
  * calls on m_getfld but small enough so that we don't snarf
  * the entire message body when we're only going to display 30
@@ -95,7 +95,9 @@ scan (FILE *inb, int innum, int outnum, char *nfs, int width, int curflg,
 	if (outnum)
 	    umask(~m_gmprot());
 
+	/* Compile format string */
 	ncomps = fmt_compile (nfs, &fmt) + 1;
+
 #ifdef JLR
 	fmt_top = fmt;
 #endif	/* JLR */
@@ -167,7 +169,8 @@ scan (FILE *inb, int innum, int outnum, char *nfs, int width, int curflg,
 	 * Add the Return-Path and Delivery-Date
 	 * header fields to message.
 	 */
-	if (get_returnpath (returnpath, deliverydate)) {
+	if (get_returnpath (returnpath, sizeof(returnpath),
+		deliverydate, sizeof(deliverydate))) {
 	    FPUTS ("Return-Path: ");
 	    FPUTS (returnpath);
 	    FPUTS ("Delivery-Date: ");
