@@ -2,7 +2,7 @@
 /*
  * send.c -- send a composed message
  *
- * $Id$
+ * $Id: send.c,v 1.2 1999/07/16 01:26:37 danh Exp $
  */
 
 #include <h/mh.h>
@@ -120,7 +120,7 @@ main (int argc, char **argv)
     int isdf = 0, mime = 0;
     int msgnum, status;
     char *cp, *dfolder = NULL, *maildir = NULL;
-    char buf[100], **ap, **argp, *arguments[MAXARGS];
+    char buf[BUFSIZ], **ap, **argp, **arguments;
     char *msgs[MAXARGS], *vec[MAXARGS];
     struct msgs *mp;
     struct stat st;
@@ -132,13 +132,11 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     vec[vecp++] = "-library";
@@ -154,7 +152,7 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown\n", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [file] [switches]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [file] [switches]", invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -321,7 +319,8 @@ main (int argc, char **argv)
 	if (!(mp = folder_read (dfolder)))
 	    adios (NULL, "unable to read folder %s", dfolder);
 
-	if (mp->hghmsg == 0)
+	/* check for empty folder */
+	if (mp->nummsg == 0)
 	    adios (NULL, "no messages in %s", dfolder);
 
 	/* parse all the message ranges/sequences and set SELECTED */
@@ -350,7 +349,7 @@ go_to_it:
 	    m_putenv ("SIGNATURE", cp);
 #ifdef UCI
 	else {
-	    sprintf (buf, "%s/.signature", mypath);
+	    snprintf (buf, sizeof(buf), "%s/.signature", mypath);
 	    if ((fp = fopen (buf, "r")) != NULL
 		&& fgets (buf, sizeof buf, fp) != NULL) {
 		    fclose (fp);
@@ -429,5 +428,5 @@ go_to_it:
     }
 
     context_save ();	/* save the context file */
-    done (status);
+    return done (status);
 }
