@@ -2,7 +2,7 @@
 /*
  * sendsbr.c -- routines to help WhatNow/Send along
  *
- * $Id$
+ * $Id: sendsbr.c,v 1.3 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
@@ -10,17 +10,28 @@
 #include <setjmp.h>
 #include <signal.h>
 #include <fcntl.h>
-#include <h/mhn.h>
+#include <h/mime.h>
+
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef TM_IN_SYS_TIME
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
 
 int debugsw = 0;		/* global */
-int forwsw = 1;
+int forwsw  = 1;
 int inplace = 1;
-int pushsw = 0;
+int pushsw  = 0;
 int splitsw = -1;
-int unique = 0;
-int verbsw = 0;
+int unique  = 0;
+int verbsw  = 0;
 
-char *altmsg = NULL;		/*  .. */
+char *altmsg   = NULL;		/*  .. */
 char *annotext = NULL;
 char *distfile = NULL;
 
@@ -30,8 +41,8 @@ static jmp_buf env;
 /*
  * external prototypes
  */
-int sendsbr (char **, int, char *, struct stat *);
-void done (int);
+int sendsbr (char **, int, char *, struct stat *, int);
+int done (int);
 char *getusername (void);
 
 /*
@@ -41,7 +52,7 @@ static void alert (char *, int);
 static int tmp_fd (void);
 static void anno (int, struct stat *);
 static void annoaux (int);
-static int splitmsg (char **, int, char *, struct stat *);
+static int splitmsg (char **, int, char *, struct stat *, int);
 static int sendaux (char **, int, char *, struct stat *);
 
 
@@ -50,25 +61,41 @@ static int sendaux (char **, int, char *, struct stat *);
  */
 
 int
-sendsbr (char **vec, int vecp, char *drft, struct stat *st)
+sendsbr (char **vec, int vecp, char *drft, struct stat *st, int rename_drft)
 {
     int status;
+    char buffer[BUFSIZ], file[BUFSIZ];
     struct stat sts;
 
     armed++;
     switch (setjmp (env)) {
     case OK: 
+	/*
+	 * If given -push and -unique (which is undocumented), then
+	 * rename the draft file.  I'm not quite sure why.
+	 */
+	if (pushsw && unique) {
+	    if (rename (drft, strncpy (file, m_scratch (drft, invo_name), sizeof(file)))
+		    == NOTOK)
+		adios (file, "unable to rename %s to", drft);
+	    drft = file;
+	}
+
 	/*
 	 * Check if we need to split the message into
 	 * multiple messages of type "message/partial".
 	 */
 	if (splitsw >= 0 && !distfile && stat (drft, &sts) != NOTOK
 		&& sts.st_size >= CPERMSG) {
-	    status = splitmsg (vec, vecp, drft, st) ? NOTOK : OK;
+	    status = splitmsg (vec, vecp, drft, st, splitsw) ? NOTOK : OK;
 	} else {
-	    splitsw = -1;
 	    status = sendaux (vec, vecp, drft, st) ? NOTOK : OK;
 	}
+
+	/* rename the original draft */
+	if (rename_drft && status == OK &&
+		rename (drft, strncpy (buffer, m_backup (drft), sizeof(buffer))) == NOTOK)
+	    advise (buffer, "unable to rename %s to", drft);
 	break;
 
     default: 
@@ -90,12 +117,13 @@ sendsbr (char **vec, int vecp, char *drft, struct stat *st)
  */
 
 static int
-splitmsg (char **vec, int vecp, char *drft, struct stat *st)
+splitmsg (char **vec, int vecp, char *drft, struct stat *st, int delay)
 {
     int	compnum, nparts, partno, state, status;
     long pos, start;
     time_t clock;
     char *cp, *dp, buffer[BUFSIZ], msgid[BUFSIZ];
+    char subject[BUFSIZ];
     char name[NAMESZ], partnum[BUFSIZ];
     FILE *in;
 
@@ -104,6 +132,11 @@ splitmsg (char **vec, int vecp, char *drft, struct stat *st)
 
     cp = dp = NULL;
     start = 0L;
+
+    /*
+     * Scan through the message and examine the various header fields,
+     * as well as locate the beginning of the message body.
+     */
     for (compnum = 1, state = FLD;;) {
 	switch (state = m_getfld (state, name, buffer, sizeof(buffer), in)) {
 	    case FLD:
@@ -111,29 +144,50 @@ splitmsg (char **vec, int vecp, char *drft, struct stat *st)
 	    case FLDEOF:
 	        compnum++;
 
-		if (!strcasecmp (name, VRSN_FIELD)
-		        || !strcasecmp (name, "Encrypted")
-		        || !strcasecmp (name, "Message-ID")) {
+		/*
+		 * This header field is discarded.
+		 */
+		if (!strcasecmp (name, "Message-ID")) {
 		    while (state == FLDPLUS)
-			state = m_getfld (state, name, buffer, sizeof(buffer),
-					  in);
+			state = m_getfld (state, name, buffer, sizeof(buffer), in);
+		} else if (uprf (name, XXX_FIELD_PRF)
+			|| !strcasecmp (name, VRSN_FIELD)
+			|| !strcasecmp (name, "Subject")
+		        || !strcasecmp (name, "Encrypted")) {
+		    /*
+		     * These header fields are copied to the enclosed
+		     * header of the first message in the collection
+		     * of message/partials.  For the "Subject" header
+		     * field, we also record it, so that a modified
+		     * version of it, can be copied to the header
+		     * of each messsage/partial in the collection.
+		     */
+		    if (!strcasecmp (name, "Subject")) {
+			size_t sublen;
+
+			strncpy (subject, buffer, BUFSIZ);
+			sublen = strlen (subject);
+			if (sublen > 0 && subject[sublen - 1] == '\n')
+			    subject[sublen - 1] = '\0';
+		    }
+
+		    dp = add (concat (name, ":", buffer, NULL), dp);
+		    while (state == FLDPLUS) {
+			state = m_getfld (state, name, buffer, sizeof(buffer), in);
+			dp = add (buffer, dp);
+		    }
 		} else {
-		    if (uprf (name, XXX_FIELD_PRF)) {
-			dp = add (concat (name, ":", buffer, NULL), dp);
-			while (state == FLDPLUS) {
-			    state = m_getfld (state, name, buffer,
-					      sizeof(buffer), in);
-			    dp = add (buffer, dp);
-			}
-		    } else {
-			cp = add (concat (name, ":", buffer, NULL), cp);
-			while (state == FLDPLUS) {
-			    state = m_getfld (state, name, buffer,
-					      sizeof(buffer), in);
-			    cp = add (buffer, cp);
-			}
+		    /*
+		     * These header fields are copied to the header of
+		     * each message/partial in the collection.
+		     */
+		    cp = add (concat (name, ":", buffer, NULL), cp);
+		    while (state == FLDPLUS) {
+			state = m_getfld (state, name, buffer, sizeof(buffer), in);
+			cp = add (buffer, cp);
 		    }
 		}
+
 		if (state != FLDEOF) {
 		    start = ftell (in) + 1;
 		    continue;
@@ -147,8 +201,7 @@ splitmsg (char **vec, int vecp, char *drft, struct stat *st)
 
 	   case LENERR:
 	   case FMTERR:
-		adios (NULL, "message format error in component #%d",
-		       compnum);
+		adios (NULL, "message format error in component #%d", compnum);
 
 	   default:
 		adios (NULL, "getfld () returned %d", state);
@@ -177,7 +230,6 @@ splitmsg (char **vec, int vecp, char *drft, struct stat *st)
 	    free (dp);
 
 	fclose (in);
-	splitsw = -1;
 	return sendaux (vec, vecp, drft, st);
     }
 
@@ -189,31 +241,38 @@ splitmsg (char **vec, int vecp, char *drft, struct stat *st)
 
     vec[vecp++] = "-partno";
     vec[vecp++] = partnum;
-    if (splitsw == 0)
+    if (delay == 0)
 	vec[vecp++] = "-queued";
 
     time (&clock);
-    sprintf (msgid, "<%d.%ld@%s>", (int) getpid(),
-		(long) clock, LocalName());
+    snprintf (msgid, sizeof(msgid), "<%d.%ld@%s>",
+		(int) getpid(), (long) clock, LocalName());
 
     fseek (in, start, SEEK_SET);
     for (partno = 1; partno <= nparts; partno++) {
 	char tmpdrf[BUFSIZ];
 	FILE *out;
 
-	strcpy (tmpdrf, m_scratch (drft, invo_name));
+	strncpy (tmpdrf, m_scratch (drft, invo_name), sizeof(tmpdrf));
 	if ((out = fopen (tmpdrf, "w")) == NULL)
 	    adios (tmpdrf, "unable to open for writing");
 	chmod (tmpdrf, 0600);
 
+	/*
+	 * Output the header fields
+	 */
 	fputs (cp, out);
+	fprintf (out, "Subject: %s (part %d of %d)\n", subject, partno, nparts);
 	fprintf (out, "%s: %s\n", VRSN_FIELD, VRSN_VALUE);
-	fprintf (out,
-		 "%s: message/partial; id=\"%s\"; number=%d; total=%d\n",
-		 TYPE_FIELD, msgid, partno, nparts);
-	fprintf (out, "%s: part %d of %d\n\n", DESCR_FIELD, partno,
-		 nparts);
+	fprintf (out, "%s: message/partial; id=\"%s\";\n", TYPE_FIELD, msgid);
+	fprintf (out, "\tnumber=%d; total=%d\n", partno, nparts);
+	fprintf (out, "%s: part %d of %d\n\n", DESCR_FIELD, partno, nparts);
 
+	/*
+	 * If this is the first in the collection, output the
+	 * header fields we are encapsulating at the beginning
+	 * of the body of the first message.
+	 */
 	if (partno == 1) {
 	    if (dp)
 		fputs (dp, out);
@@ -248,22 +307,27 @@ splitmsg (char **vec, int vecp, char *drft, struct stat *st)
 	    printf ("\n");
 	    fflush (stdout);
 	}
-	if (splitsw > 0 && 1 < partno && partno < nparts) {
+
+	/* Pause here, if a delay is specified */
+	if (delay > 0 && 1 < partno && partno <= nparts) {
 	    if (!pushsw) {
 		printf ("pausing %d seconds before sending part %d...\n",
-			splitsw, partno);
+			delay, partno);
 		fflush (stdout);
 	    }
-
-	    sleep ((unsigned) splitsw);
+	    sleep ((unsigned int) delay);
 	}
 
-	sprintf (partnum, "%d", partno);
+	snprintf (partnum, sizeof(partnum), "%d", partno);
 	status = sendaux (vec, vecp, tmpdrf, st);
 	unlink (tmpdrf);
 	if (status != OK)
 	    break;
 
+	/*
+	 * This is so sendaux will only annotate
+	 * the altmsg the first time it is called.
+	 */
 	annotext = NULL;
     }
 
@@ -271,12 +335,7 @@ splitmsg (char **vec, int vecp, char *drft, struct stat *st)
     if (dp)
 	free (dp);
 
-    fclose (in);
-
-    if (status == OK &&
-	    rename (drft, strcpy (buffer, m_backup (drft))) == NOTOK)
-	advise (buffer, "unable to rename %s to", drft);
-
+    fclose (in);	/* close the draft */
     return status;
 }
 
@@ -291,22 +350,16 @@ sendaux (char **vec, int vecp, char *drft, struct stat *st)
 {
     pid_t child_id;
     int i, status, fd, fd2;
-    char backup[BUFSIZ], buf[BUFSIZ], file[BUFSIZ];
+    char backup[BUFSIZ], buf[BUFSIZ];
 
     fd = pushsw ? tmp_fd () : NOTOK;
     fd2 = NOTOK;
 
-    if (pushsw && unique) {
-	if (rename (drft, strcpy (file, m_scratch (drft, invo_name)))
-		== NOTOK)
-	    adios (file, "unable to rename %s to", drft);
-	drft = file;
-    }
     vec[vecp++] = drft;
     if (annotext) {
 	if ((fd2 = tmp_fd ()) != NOTOK) {
 	    vec[vecp++] = "-idanno";
-	    sprintf (buf, "%d", fd2);
+	    snprintf (buf, sizeof(buf), "%d", fd2);
 	    vec[vecp++] = buf;
 	} else {
 	    admonish (NULL, "unable to create file for annotation list");
@@ -347,12 +400,9 @@ sendaux (char **vec, int vecp, char *drft, struct stat *st)
 	/*
 	 * parent process -- wait for it
 	 */
-	if ((status = pidwait(child_id, NOTOK)) == 0) {
+	if ((status = pidwait(child_id, NOTOK)) == OK) {
 	    if (annotext && fd2 != NOTOK)
 		anno (fd2, st);
-	    if (splitsw < 0 &&
-	        rename (drft, strcpy (buf, m_backup (drft))) == NOTOK)
-		advise (buf, "unable to rename %s to", drft);
 	} else {
 	    /*
 	     * If postproc failed, and we have good fd (which means
@@ -411,15 +461,15 @@ alert (char *file, int out)
 		    admonish (file, "unable to re-open");
 		} else {
 		    lseek (out, (off_t) 0, SEEK_END);
-		    strcpy (buf, "\nMessage not delivered to anyone.\n");
+		    strncpy (buf, "\nMessage not delivered to anyone.\n", sizeof(buf));
 		    write (out, buf, strlen (buf));
-		    strcpy (buf, "\n------- Unsent Draft\n\n");
+		    strncpy (buf, "\n------- Unsent Draft\n\n", sizeof(buf));
 		    write (out, buf, strlen (buf));
 		    cpydgst (in, out, file, "temporary file");
 		    close (in);
-		    strcpy (buf, "\n------- End of Unsent Draft\n");
+		    strncpy (buf, "\n------- End of Unsent Draft\n", sizeof(buf));
 		    write (out, buf, strlen (buf));
-		    if (rename (file, strcpy (buf, m_backup (file))) == NOTOK)
+		    if (rename (file, strncpy (buf, m_backup (file), sizeof(buf))) == NOTOK)
 			admonish (buf, "unable to rename %s to", file);
 		}
 	    }
@@ -427,7 +477,7 @@ alert (char *file, int out)
 	    dup2 (out, fileno (stdin));
 	    close (out);
 	    /* create subject for error notification */
-	    sprintf (buf, "send failed on %s",
+	    snprintf (buf, sizeof(buf), "send failed on %s",
 			forwsw ? "enclosed draft" : file);
 
 	    execlp (mailproc, r1bindex (mailproc, '/'), getusername (),
@@ -448,7 +498,7 @@ tmp_fd (void)
     int fd;
     char tmpfil[BUFSIZ];
 
-    strcpy (tmpfil, m_tmpfil (invo_name));
+    strncpy (tmpfil, m_tmpfil (invo_name), sizeof(tmpfil));
     if ((fd = open (tmpfil, O_RDWR | O_CREAT | O_TRUNC, 0600)) == NOTOK)
 	return NOTOK;
     if (debugsw)
@@ -460,6 +510,7 @@ tmp_fd (void)
     return fd;
 }
 
+
 static void
 anno (int fd, struct stat *st)
 {
@@ -538,7 +589,9 @@ annoaux (int fd)
 	    admonish (NULL, "unable to read folder %s");
 	return;
     }
-    if (mp->hghmsg == 0) {
+
+    /* check for empty folder */
+    if (mp->nummsg == 0) {
 	if (debugsw)
 	    admonish (NULL, "no messages in %s", folder);
 	goto oops;
@@ -598,11 +651,12 @@ oops:
 }
 
 
-void
+int
 done (int status)
 {
     if (armed)
 	longjmp (env, status ? status : NOTOK);
 
     exit (status);
+    return 1;  /* dead code to satisfy the compiler */
 }
