@@ -2,11 +2,11 @@
 /*
  * show.c -- show/list messages
  *
- * $Id$
+ * $Id: show.c,v 1.4 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
-#include <h/mhn.h>
+#include <h/mime.h>
 
 static struct swit switches[] = {
 #define CHECKMIMESW          0
@@ -49,6 +49,9 @@ static struct swit switches[] = {
  */
 static int is_nontext(char *);
 
+/* prototype from mhlsbr.c */
+int mhl (int, char **);
+
 #define	SHOW  0
 #define	NEXT  1
 #define	PREV  2
@@ -61,7 +64,7 @@ main (int argc, char **argv)
     int nshow = 0, checkmime = 1, mime;
     int vecp = 1, procp = 1, isdf = 0, mode = SHOW, msgnum;
     char *cp, *maildir, *file = NULL, *folder = NULL, *proc;
-    char buf[100], **ap, **argp, *arguments[MAXARGS];
+    char buf[BUFSIZ], **argp, **arguments;
     char *msgs[MAXARGS], *vec[MAXARGS];
     struct msgs *mp;
 
@@ -70,19 +73,15 @@ main (int argc, char **argv)
 #endif
     invo_name = r1bindex (argv[0], '/');
 
+    /* read user profile/context */
+    context_read();
+
     if (!strcasecmp (invo_name, "next")) {
 	mode = NEXT;
     } else if (!strcasecmp (invo_name, "prev")) {
 	mode = PREV;
     }
-
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -97,8 +96,9 @@ main (int argc, char **argv)
 		    continue;
 
 		case HELPSW: 
-		    sprintf (buf, "%s [+folder] %s[switches] [switches for showproc]",
-			    invo_name, mode == SHOW ? "[msgs] ": "");
+		    snprintf (buf, sizeof(buf),
+			"%s [+folder] %s[switches] [switches for showproc]",
+			invo_name, mode == SHOW ? "[msgs] ": "");
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -111,7 +111,7 @@ main (int argc, char **argv)
 		    draftsw++;
 		    if (mode == SHOW)
 			continue;
-	    usage:  ;
+usage:
 		    adios (NULL,
 			    "usage: %s [+folder] [switches] [switches for showproc]",
 			    invo_name);
@@ -229,7 +229,8 @@ main (int argc, char **argv)
     if (!(mp = folder_read (folder)))
 	adios (NULL, "unable to read folder %s", folder);
 
-    if (mp->hghmsg == 0)
+    /* check for empty folder */
+    if (mp->nummsg == 0)
 	adios (NULL, "no messages in %s", folder);
 
     /* parse all the message ranges/sequences and set SELECTED */
@@ -303,8 +304,9 @@ go_to_it: ;
 	m_putenv ("mhfolder", folder);
 
     /*
-     * If "proc" is mhn, then add "-show" option.
-     * Add "-file" if showing file or draft.
+     * For backward compatibility, if the "proc" is mhn,
+     * then add "-show" option.  Add "-file" if showing
+     * file or draft.
      */
     if (strcmp (r1bindex (proc, '/'), "mhn") == 0) {
 	if (draftsw || file) {
@@ -316,6 +318,14 @@ go_to_it: ;
 	vec[vecp] = NULL;
     }
 
+    /* If the "proc" is "mhshow", add "-file" if showing file or draft.
+     */
+    if (strcmp (r1bindex (proc, '/'), "mhshow") == 0 && (draftsw || file) ) {
+       vec[vecp] = vec[vecp - 1];
+       vec[vecp - 1] = "-file";
+       vec[++vecp] = NULL;
+    }
+
     /*
      * If "proc" is mhl, then run it internally
      * rather than exec'ing it.
@@ -346,6 +356,7 @@ go_to_it: ;
     vec[0] = r1bindex (proc, '/');
     execvp (proc, vec);
     adios (proc, "unable to exec");
+    return 0;  /* dead code to satisfy the compiler */
 }
 
 /*
