@@ -2,7 +2,7 @@
 /*
  * slocal.c -- asynchronously filter and deliver new mail
  *
- * $Id$
+ * $Id: slocal.c,v 1.4 1999/09/12 13:50:12 doug Exp $
  */
 
 /*
@@ -27,8 +27,14 @@
 #include <pwd.h>
 #include <signal.h>
 #include <sys/ioctl.h>
-#include <ndbm.h>
 #include <fcntl.h>
+#include <grp.h>
+
+#ifdef HAVE_DB1_NDBM_H
+#include <db1/ndbm.h>
+#else
+#include <ndbm.h>
+#endif
 
 #include <utmp.h>
 
@@ -74,10 +80,10 @@ static struct swit switches[] = {
     { NULL, 0 }
 };
 
-static int globbed = 0;
-static int parsed = 0;
-static int utmped = 0;
-static int suppressdup = 0;   /* are we suppressing duplicate messages? */
+static int globbed = 0;		/* have we built "vars" table yet?        */
+static int parsed = 0;		/* have we built header field table yet   */
+static int utmped = 0;		/* have we scanned umtp(x) file yet       */
+static int suppressdup = 0;	/* are we suppressing duplicate messages? */
 
 static int verbose = 0;
 static int debug = 0;
@@ -91,18 +97,17 @@ static char *envelope = NULL;	/* envelope information ("From " line)  */
 static char *mbox = NULL;
 static char *home = NULL;
 
-static struct passwd *pw;
+static struct passwd *pw;	/* passwd file entry */
 
-/* record the delivery date */
-static char ddate[BUFSIZ];
+static char ddate[BUFSIZ];	/* record the delivery date */
 struct tws *now;
 
 static jmp_buf myctx;
 
 /* flags for pair->p_flags */
 #define	P_NIL  0x00
-#define	P_ADR  0x01
-#define	P_HID  0x02
+#define	P_ADR  0x01	/* field is address     */
+#define	P_HID  0x02	/* special (fake) field */
 #define	P_CHK  0x04
 
 struct pair {
@@ -113,6 +118,11 @@ struct pair {
 
 #define	NVEC 100
 
+/*
+ * Lookup table for matching fields and patterns
+ * in messages.  The rest of the table is added
+ * when the message is parsed.
+ */
 static struct pair hdrs[NVEC + 1] = {
     { "source",          NULL, P_HID },
     { "addr",            NULL, P_HID },
@@ -127,17 +137,19 @@ static struct pair hdrs[NVEC + 1] = {
     { "Resent-Sender",   NULL, P_ADR },
     { "Resent-To",       NULL, P_ADR },
     { "Resent-cc",       NULL, P_ADR },
-
     { NULL, NULL, 0 }
 };
 
+/*
+ * The list of builtin variables to expand in a string
+ * before it is executed by the "pipe" or "qpipe" action.
+ */
 static struct pair vars[] = {
     { "sender",   NULL, P_NIL },
     { "address",  NULL, P_NIL },
     { "size",     NULL, P_NIL },
     { "reply-to", NULL, P_CHK },
     { "info",     NULL, P_NIL },
-
     { NULL, NULL, 0 }
 };
 
@@ -156,11 +168,14 @@ static struct pair *lookup (struct pair *, char *);
 static int logged_in (void);
 static int timely (char *, char *);
 static int usr_file (int, char *, int);
-static int usr_pipe (int, char *, char *, char **);
+static int usr_pipe (int, char *, char *, char **, int);
+static int usr_folder (int, char *);
 static RETSIGTYPE alrmser (int);
 static void get_sender (char *, char **);
 static int copy_message (int, char *, int);
+static void verbose_printf (char *fmt, ...);
 static void adorn (char *, char *, ...);
+static void debug_printf (char *fmt, ...);
 static int suppress_duplicates (int, char *);
 static char *trim (char *);
 
@@ -168,12 +183,11 @@ static char *trim (char *);
 int
 main (int argc, char **argv)
 {
-    int fd, result;
+    int fd, status;
     FILE *fp = stdin;
-    char *cp, *mdlvr = NULL;
-    char buf[100];
+    char *cp, *mdlvr = NULL, buf[BUFSIZ];
     char mailbox[BUFSIZ], tmpfil[BUFSIZ];
-    char **argp = argv + 1;
+    char **argp, **arguments;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
@@ -185,9 +199,12 @@ main (int argc, char **argv)
 	done (1);
 
     mts_init (invo_name);
+    arguments = getarguments (invo_name, argc, argv, 0);
+    argp = arguments;
 
+    /* Parse arguments */
     while ((cp = *argp++)) {
-	if (*cp == '-')
+	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
 		case AMBIGSW: 
 		    ambigsw (cp, switches);
@@ -196,7 +213,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [switches] [address info sender]", invo_name);
+		    snprintf (buf, sizeof(buf),
+			"%s [switches] [address info sender]", invo_name);
 		    print_help (buf, switches, 0);
 		    done (1);
 		case VERSIONSW:
@@ -257,6 +275,7 @@ main (int argc, char **argv)
 		    debug++;
 		    continue;
 	    }
+	}
 
 	switch (argp - (argv + 1)) {
 	    case 1: 
@@ -298,7 +317,7 @@ main (int argc, char **argv)
     /* Record the delivery time */
     if ((now = dlocaltimenow ()) == NULL)
 	adios (NULL, "unable to ascertain local time");
-    sprintf (ddate, "Delivery-Date: %s\n", dtimenow (0));
+    snprintf (ddate, sizeof(ddate), "Delivery-Date: %s\n", dtimenow (0));
 
     /*
      * Copy the message to a temporary file
@@ -308,36 +327,47 @@ main (int argc, char **argv)
 
 	/* getting message from file */
 	if ((tempfd = open (file, O_RDONLY)) == -1)
-	    adios(file, "unable to open");
+	    adios (file, "unable to open");
 	if (debug)
-	    fprintf (stderr, "retrieving message from file \"%s\"\n", file);
+	    debug_printf ("retrieving message from file \"%s\"\n", file);
 	if ((fd = copy_message (tempfd, tmpfil, 1)) == -1)
 	    adios (NULL, "unable to create temporary file");
 	close (tempfd);
     } else {
 	/* getting message from stdin */
 	if (debug)
-	    fprintf(stderr, "retrieving message from stdin\n");
+	    debug_printf ("retrieving message from stdin\n");
 	if ((fd = copy_message (fileno (stdin), tmpfil, 1)) == -1)
 	    adios (NULL, "unable to create temporary file");
     }
+
     if (debug)
-	fprintf (stderr, "temporary file=\"%s\"\n", tmpfil);
-    else
-	unlink (tmpfil);
+	debug_printf ("temporary file=\"%s\"\n", tmpfil);
+
+    /* Delete the temp file now or a copy of every single message passed through
+       slocal will be left in the /tmp directory until deleted manually!  This
+       unlink() used to be under an 'else' of the 'if (debug)' above, but since
+       some people like to always run slocal with -debug and log the results,
+       the /tmp directory would get choked over time.  Of course, now that we
+       always delete the temp file, the "temporary file=" message above is
+       somewhat pointless -- someone watching debug output wouldn't have a
+       chance to 'tail -f' or 'ln' the temp file before it's unlinked.  The best
+       thing would be to delay this unlink() until later if debug == 1, but I'll
+       leave that for someone who cares about the temp-file-accessing
+       functionality (they'll have to watch out for cases where we adios()). */
+    unlink (tmpfil);
 
     if (!(fp = fdopen (fd, "r+")))
 	adios (NULL, "unable to access temporary file");
 
     /*
      * If no sender given, extract it
-     * from envelope information.
-     */
+     * from envelope information.  */
     if (sender == NULL)
 	get_sender (envelope, &sender);
 
     if (mbox == NULL) {
-	sprintf (mailbox, "%s/%s",
+	snprintf (mailbox, sizeof(mailbox), "%s/%s",
 		mmdfldir[0] ? mmdfldir : pw->pw_dir,
 		mmdflfil[0] ? mmdflfil : pw->pw_name);
 	mbox = mailbox;
@@ -346,22 +376,28 @@ main (int argc, char **argv)
 	home = pw->pw_dir;
 
     if (debug) {
-	fprintf (stderr, "addr=\"%s\"\n", trim(addr));
-	fprintf (stderr, "user=\"%s\"\n", trim(user));
-	fprintf (stderr, "info=\"%s\"\n", trim(info));
-	fprintf (stderr, "sender=\"%s\"\n", trim(sender));
-	fprintf (stderr, "envelope=\"%s\"\n", envelope ? trim(envelope) : "");
-	fprintf (stderr, "mbox=\"%s\"\n", trim(mbox));
-	fprintf (stderr, "home=\"%s\"\n", trim(home));
-	fprintf (stderr, "ddate=\"%s\"\n", trim(ddate));
-	fprintf (stderr, "now=%02d:%02d\n\n", now->tw_hour, now->tw_min);
+	debug_printf ("addr=\"%s\"\n", trim(addr));
+	debug_printf ("user=\"%s\"\n", trim(user));
+	debug_printf ("info=\"%s\"\n", trim(info));
+	debug_printf ("sender=\"%s\"\n", trim(sender));
+	debug_printf ("envelope=\"%s\"\n", envelope ? trim(envelope) : "");
+	debug_printf ("mbox=\"%s\"\n", trim(mbox));
+	debug_printf ("home=\"%s\"\n", trim(home));
+	debug_printf ("ddate=\"%s\"\n", trim(ddate));
+	debug_printf ("now=%02d:%02d\n\n", now->tw_hour, now->tw_min);
     }
 
-    result = localmail (fd, mdlvr);
-    done (result != -1 ? RCV_MOK : RCV_MBX);
+    /* deliver the message */
+    status = localmail (fd, mdlvr);
+
+    return done (status != -1 ? RCV_MOK : RCV_MBX);
 }
 
 
+/*
+ * Main routine for delivering message.
+ */
+
 static int
 localmail (int fd, char *mdlvr)
 {
@@ -378,17 +414,10 @@ localmail (int fd, char *mdlvr)
     if (usr_delivery (fd, maildelivery, 1) != -1)
 	return 0;
 
-#if 0
     if (verbose)
-	printf ("(invoking hook)\n");
-    if (usr_hook (fd, mbox) != -1)
-	return 0;
-#endif
+	verbose_printf ("(delivering to standard mail spool)\n");
 
-    if (verbose)
-	printf ("(delivering to standard mail spool)\n");
-
-    /* deliver to standard mail spool */
+    /* last resort - deliver to standard mail spool */
 #ifdef SLOCAL_MBOX
     return usr_file (fd, mbox, MBOX_FORMAT);
 #else
@@ -399,74 +428,109 @@ localmail (int fd, char *mdlvr)
 
 #define	matches(a,b) (stringdex (b, a) >= 0)
 
+/*
+ * Parse the delivery file, and process incoming message.
+ */
+
 static int
 usr_delivery (int fd, char *delivery, int su)
 {
     int i, accept, status, won, vecp, next;
-    char *cp, *action, *field, *pattern, *string;
-    char buffer[BUFSIZ], tmpbuf[BUFSIZ], *vec[NVEC];
+    char *field, *pattern, *action, *result, *string;
+    char buffer[BUFSIZ], tmpbuf[BUFSIZ];
+    char *cp, *vec[NVEC];
     struct stat st;
     struct pair *p;
     FILE *fp;
 
+    /* open the delivery file */
     if ((fp = fopen (delivery, "r")) == NULL)
 	return -1;
+
+    /* check if delivery file has bad ownership or permissions */
     if (fstat (fileno (fp), &st) == -1
 	    || (st.st_uid != 0 && (su || st.st_uid != pw->pw_uid))
 	    || st.st_mode & (S_IWGRP|S_IWOTH)) {
 	if (verbose) {
-	    printf ("%s: ownership/modes bad (%d, %d,%d,0%o)\n",
+	    verbose_printf ("WARNING: %s has bad ownership/modes (su=%d,uid=%d,owner=%d,mode=0%o)\n",
 		    delivery, su, (int) pw->pw_uid, (int) st.st_uid, (int) st.st_mode);
-	    fflush (stdout);
 	}
 	return -1;
     }
 
     won = 0;
     next = 1;
+
+    /* read and process delivery file */
     while (fgets (buffer, sizeof(buffer), fp)) {
 	/* skip comments and empty lines */
 	if (*buffer == '#' || *buffer == '\n')
 	    continue;
+
+	/* zap trailing newline */
 	if ((cp = strchr(buffer, '\n')))
 	    *cp = 0;
-	if ((vecp = split (buffer, vec)) < 5) {
+
+	/* split buffer into fields */
+	vecp = split (buffer, vec);
+
+	/* check for too few fields */
+	if (vecp < 5) {
 	    if (debug)
-		fprintf (stderr, "WARNING: entry with only %d fields, skipping ...\n", vecp);
+		debug_printf ("WARNING: entry with only %d fields, skipping.\n", vecp);
 	    continue;
 	}
+
 	if (debug) {
 	    for (i = 0; vec[i]; i++)
-		fprintf (stderr, "vec[%d]: \"%s\"\n", i, trim(vec[i]));
+		debug_printf ("vec[%d]: \"%s\"\n", i, trim(vec[i]));
 	}
 
 	field   = vec[0];
 	pattern = vec[1];
 	action  = vec[2];
+	result  = vec[3];
+	string  = vec[4];
 
-	switch (vec[3][0]) {
+	/* find out how to perform the action */
+	switch (result[0]) {
 	    case 'N':
 	    case 'n':
+		/*
+		 * If previous condition failed, don't
+		 * do this - else fall through
+		 */
  		if (!next)
-		    continue;  /* if previous condition failed, don't
-				  do this - else fall through */
+		    continue;	/* else fall */
+
 	    case '?': 
+		/*
+		 * If already delivered, skip this action.  Else
+		 * consider delivered if action is successful.
+		 */
 		if (won)
 		    continue;	/* else fall */
+
 	    case 'A': 
 	    case 'a': 
+		/*
+		 * Take action, and consider delivered if
+		 * action is successful.
+		 */
 		accept = 1;
 		break;
 
 	    case 'R': 
 	    case 'r': 
 	    default: 
+		/*
+		 * Take action, but don't consider delivered, even
+		 * if action is successful
+		 */
 		accept = 0;
 		break;
 	}
 
-	string = vec[4];
-
 	if (vecp > 5) {
 	    if (!strcasecmp (vec[5], "select")) {
 		if (logged_in () != -1)
@@ -476,11 +540,17 @@ usr_delivery (int fd, char *delivery, int su)
 	    }
 	}
 
+	/* check if the field matches */
 	switch (*field) {
 	    case '*': 
+	    /* always matches */
 		break;
 
 	    case 'd': 
+	    /*
+	     * "default" matches only if the message hasn't
+	     * been delivered yet.
+	     */
 		if (!strcasecmp (field, "default")) {
 		    if (won)
 			continue;
@@ -488,21 +558,26 @@ usr_delivery (int fd, char *delivery, int su)
 		}		/* else fall */
 
 	    default: 
+		/* parse message and build lookup table */
 		if (!parsed && parse (fd) == -1) {
 		    fclose (fp);
 		    return -1;
 		}
-		if ((p = lookup (hdrs, field)) == NULL
-			|| (p->p_value == NULL)			/* XXX */
-			|| !matches (p->p_value, pattern)) {
-			next = 0;
-			continue;
-		}
-		else
+		/*
+		 * find header field in lookup table, and
+		 * see if the pattern matches.
+		 */
+		if ((p = lookup (hdrs, field)) && (p->p_value != NULL)
+			&& matches (p->p_value, pattern)) {
 		    next = 1;
+		} else {
+		    next = 0;
+		    continue;
+		}
 		break;
 	}
 
+	/* find out the action to perform */
 	switch (*action) {
 	    case 'q':
 		/* deliver to quoted pipe */
@@ -512,7 +587,7 @@ usr_delivery (int fd, char *delivery, int su)
 		expand (tmpbuf, string, fd);
 		if (split (tmpbuf, vec) < 1)
 		    continue;
-		status = usr_pipe (fd, tmpbuf, vec[0], vec);
+		status = usr_pipe (fd, tmpbuf, vec[0], vec, 0);
 		break;
 
 	    case 'p': 
@@ -525,7 +600,7 @@ usr_delivery (int fd, char *delivery, int su)
 		expand (tmpbuf, string, fd);
 		vec[4] = tmpbuf;
 		vec[5] = NULL;
-		status = usr_pipe (fd, tmpbuf, "/bin/sh", vec + 2);
+		status = usr_pipe (fd, tmpbuf, "/bin/sh", vec + 2, 0);
 		break;
 
 	    case 'f': 
@@ -534,20 +609,11 @@ usr_delivery (int fd, char *delivery, int su)
 		    status = usr_file (fd, string, MBOX_FORMAT);
 		    break;
 		}
-		/* deliver to folder */
+		/* deliver to nmh folder */
 		else if (strcasecmp (action, "folder"))
 		    continue;	/* else fall */
 	    case '+':
-		if (*string == '+')
-		    strcpy(tmpbuf, string);
-		else
-		    sprintf(tmpbuf, "+%s", string);
-		vec[2] = "rcvstore";
-		vec[3] = tmpbuf;
-		vec[4] = NULL;
-		if (verbose)
-		    printf("%s %s", rcvstoreproc, vec[3]);
-		status = usr_pipe(fd, "rcvstore", rcvstoreproc, vec + 2);
+		status = usr_folder (fd, string);
 		break;
 
 	    case 'm':
@@ -566,6 +632,7 @@ usr_delivery (int fd, char *delivery, int su)
 		break;
 
 	    case 'd': 
+		/* ignore message */
 		if (strcasecmp (action, "destroy"))
 		    continue;
 		status = 0;
@@ -583,6 +650,11 @@ usr_delivery (int fd, char *delivery, int su)
 
 #define	QUOTE	'\\'
 
+/*
+ * Split buffer into fields (delimited by whitespace or
+ * comma's).  Return the number of fields found.
+ */
+
 static int
 split (char *cp, char **vec)
 {
@@ -590,28 +662,42 @@ split (char *cp, char **vec)
     char *s;
 
     s = cp;
+
+    /* split into a maximum of NVEC fields */
     for (i = 0; i <= NVEC;) {
 	vec[i] = NULL;
+
+	/* zap any whitespace and comma's */
 	while (isspace (*s) || *s == ',')
 	    *s++ = 0;
+
+	/* end of buffer, time to leave */
 	if (*s == 0)
 	    break;
 
+	/* get double quote text as a single field */
 	if (*s == '"') {
-	    for (vec[i++] = ++s; *s && *s != '"'; s++)
+	    for (vec[i++] = ++s; *s && *s != '"'; s++) {
+		/*
+		 * Check for escaped double quote.  We need
+		 * to shift the string to remove slash.
+		 */
 		if (*s == QUOTE) {
 		    if (*++s == '"')
 			strcpy (s - 1, s);
 		    s--;
 		}
-	    if (*s == '"')
+	    }
+	    if (*s == '"')	/* zap trailing double quote */
 		*s++ = 0;
 	    continue;
 	}
+
 	if (*s == QUOTE && *++s != '"')
 	    s--;
 	vec[i++] = s++;
 
+	/* move forward to next field delimiter */
 	while (*s && !isspace (*s) && *s != ',')
 	    s++;
     }
@@ -621,6 +707,11 @@ split (char *cp, char **vec)
 }
 
 
+/*
+ * Parse the headers of a message, and build the
+ * lookup table for matching fields and patterns.
+ */
+
 static int
 parse (int fd)
 {
@@ -634,6 +725,7 @@ parse (int fd)
     if (parsed++)
 	return 0;
 
+    /* get a new FILE pointer to message */
     if ((fd1 = dup (fd)) == -1)
 	return -1;
     if ((in = fdopen (fd1, "r")) == NULL) {
@@ -642,11 +734,16 @@ parse (int fd)
     }
     rewind (in);
 
+    /* add special entries to lookup table */
     if ((p = lookup (hdrs, "source")))
 	p->p_value = getcpy (sender);
     if ((p = lookup (hdrs, "addr")))
 	p->p_value = getcpy (addr);
 
+    /*
+     * Scan the headers of the message and build
+     * a lookup table.
+     */
     for (i = 0, state = FLD;;) {
 	switch (state = m_getfld (state, name, field, sizeof(field), in)) {
 	    case FLD: 
@@ -657,23 +754,25 @@ parse (int fd)
 		    state = m_getfld (state, name, field, sizeof(field), in);
 		    lp = add (field, lp);
 		}
-		for (p = hdrs; p->p_name; p++)
+		for (p = hdrs; p->p_name; p++) {
 		    if (!strcasecmp (p->p_name, name)) {
 			if (!(p->p_flags & P_HID)) {
-			    if ((cp = p->p_value))
+			    if ((cp = p->p_value)) {
 				if (p->p_flags & P_ADR) {
 				    dp = cp + strlen (cp) - 1;
 				    if (*dp == '\n')
 					*dp = 0;
 				    cp = add (",\n\t", cp);
-				}
-				else
+				} else {
 				    cp = add ("\t", cp);
+				}
+			    }
 			    p->p_value = add (lp, cp);
 			}
 			free (lp);
 			break;
 		    }
+		}
 		if (p->p_name == NULL && i < NVEC) {
 		    p->p_name = getcpy (name);
 		    p->p_value = lp;
@@ -696,7 +795,7 @@ parse (int fd)
 		break;
 
 	    default: 
-		advise (NULL, "internal error");
+		advise (NULL, "internal error in m_getfld");
 		fclose (in);
 		return -1;
 	}
@@ -710,13 +809,14 @@ parse (int fd)
 	p->p_value = getcpy (q ? q->p_value : "");
 	p->p_flags &= ~P_CHK;
 	if (debug)
-	    fprintf (stderr, "vars[%d]: name=\"%s\" value=\"%s\"\n",
+	    debug_printf ("vars[%d]: name=\"%s\" value=\"%s\"\n",
 		    p - vars, p->p_name, trim(p->p_value));
     }
-    if (debug)
+    if (debug) {
 	for (p = hdrs; p->p_name; p++)
-	    fprintf (stderr, "hdrs[%d]: name=\"%s\" value=\"%s\"\n",
+	    debug_printf ("hdrs[%d]: name=\"%s\" value=\"%s\"\n",
 		p - hdrs, p->p_name, p->p_value ? trim(p->p_value) : "");
+    }
 
     return 0;
 }
@@ -725,6 +825,11 @@ parse (int fd)
 #define	LPAREN	'('
 #define	RPAREN	')'
 
+/*
+ * Expand any builtin variables such as $(sender),
+ * $(address), etc., in a string.
+ */
+
 static void
 expand (char *s1, char *s2, int fd)
 {
@@ -734,10 +839,10 @@ expand (char *s1, char *s2, int fd)
     if (!globbed)
 	glob (fd);
 
-    while ((c = *s2++))
-	if (c != '$' || *s2 != LPAREN)
+    while ((c = *s2++)) {
+	if (c != '$' || *s2 != LPAREN) {
 	    *s1++ = c;
-	else {
+	} else {
 	    for (cp = ++s2; *s2 && *s2 != RPAREN; s2++)
 		continue;
 	    if (*s2 != RPAREN) {
@@ -753,10 +858,18 @@ expand (char *s1, char *s2, int fd)
 		s1 += strlen (s1);
 	    }
 	}
+    }
     *s1 = 0;
 }
 
 
+/*
+ * Fill in the information missing from the "vars"
+ * table, which is necessary to expand any builtin
+ * variables in the string for a "pipe" or "qpipe"
+ * action.
+ */
+
 static void
 glob (int fd)
 {
@@ -772,19 +885,26 @@ glob (int fd)
     if ((p = lookup (vars, "address")))
 	p->p_value = getcpy (addr);
     if ((p = lookup (vars, "size"))) {
-	sprintf (buffer, "%d", fstat (fd, &st) != -1 ? (int) st.st_size : 0);
+	snprintf (buffer, sizeof(buffer), "%d",
+		fstat (fd, &st) != -1 ? (int) st.st_size : 0);
 	p->p_value = getcpy (buffer);
     }
     if ((p = lookup (vars, "info")))
 	p->p_value = getcpy (info);
 
-    if (debug)
+    if (debug) {
 	for (p = vars; p->p_name; p++)
-	    fprintf (stderr, "vars[%d]: name=\"%s\" value=\"%s\"\n",
+	    debug_printf ("vars[%d]: name=\"%s\" value=\"%s\"\n",
 		    p - vars, p->p_name, trim(p->p_value));
+    }
 }
 
 
+/*
+ * Find a matching name in a lookup table.  If found,
+ * return the "pairs" entry, else return NULL.
+ */
+
 static struct pair *
 lookup (struct pair *pairs, char *key)
 {
@@ -796,6 +916,11 @@ lookup (struct pair *pairs, char *key)
 }
 
 
+/*
+ * Check utmp(x) file to see if user is currently
+ * logged in.
+ */
+
 static int
 logged_in (void)
 {
@@ -806,19 +931,20 @@ logged_in (void)
 	return utmped;
 
     if ((uf = fopen (UTMP_FILE, "r")) == NULL)
-	return -1;
+	return NOTOK;
 
-    while (fread ((char *) &ut, sizeof(ut), 1, uf) == 1)
+    while (fread ((char *) &ut, sizeof(ut), 1, uf) == 1) {
 	if (ut.ut_name[0] != 0
-		&& strncmp (user, ut.ut_name, sizeof(ut.ut_name)) == 0) {
+	    && strncmp (user, ut.ut_name, sizeof(ut.ut_name)) == 0) {
 	    if (debug)
 		continue;
 	    fclose (uf);
 	    return (utmped = DONE);
 	}
+    }
 
     fclose (uf);
-    return (utmped = -1);
+    return (utmped = NOTOK);
 }
 
 
@@ -847,118 +973,118 @@ timely (char *t1, char *t2)
 }
 
 
+/*
+ * Deliver message by appending to a file.
+ */
+
 static int
 usr_file (int fd, char *mailbox, int mbx_style)
 {
     int	md, mapping;
 
     if (verbose)
-	printf ("\tdelivering to file \"%s\"", mailbox);
+	verbose_printf ("delivering to file \"%s\"", mailbox);
 
     if (mbx_style == MBOX_FORMAT) {
 	if (verbose)
-	    printf (" (mbox style)");
+	    verbose_printf (" (mbox style)");
 	mapping = 0;
     } else {
 	if (verbose)
-	    printf (" (mmdf style)");
+	    verbose_printf (" (mmdf style)");
 	mapping = 1;
     }
 
-    if (verbose)
-	fflush (stdout);
-
-    if ((md = mbx_open (mailbox, mbx_style, pw->pw_uid, pw->pw_gid, m_gmprot()))
-	    == -1) {
-	adorn ("", "unable to open:");
+    /* open and lock the file */
+    if ((md = mbx_open (mailbox, mbx_style, pw->pw_uid, pw->pw_gid, m_gmprot())) == -1) {
+	if (verbose)
+	    adorn ("", "unable to open:");
 	return -1;
     }
 
     lseek (fd, (off_t) 0, SEEK_SET);
+
+    /* append message to file */
     if (mbx_copy (mailbox, mbx_style, md, fd, mapping, NULL, verbose) == -1) {
-	adorn ("", "error writing to:");
+	if (verbose)
+	    adorn ("", "error writing to:");
 	return -1;
     }
 
+    /* close and unlock file */
     mbx_close (mailbox, md);
-    if (verbose) {
-	printf (", done.\n");
-	fflush (stdout);
-    }
+
+    if (verbose)
+	verbose_printf (", success.\n");
     return 0;
 }
 
 
-#if 0
+/*
+ * Deliver message to a nmh folder.
+ */
+
 static int
-usr_hook (int fd, char *mailbox)
+usr_folder (int fd, char *string)
 {
-    int i, vecp;
-    char receive[BUFSIZ], tmpfil[BUFSIZ], *vec[NVEC];
+    int status;
+    char folder[BUFSIZ], *vec[3];
 
-    if ((fd = copy_message (fd, tmpfil, 0)) == -1) {
-	if (verbose)
-	    adorn ("unable to copy message; skipping hook\n");
-	return -1;
+    /* get folder name ready */
+    if (*string == '+')
+	strncpy(folder, string, sizeof(folder));
+    else
+	snprintf(folder, sizeof(folder), "+%s", string);
+
+    if (verbose)
+	verbose_printf ("delivering to folder \"%s\"", folder + 1);
+
+    vec[0] = "rcvstore";
+    vec[1] = folder;
+    vec[2] = NULL;
+
+    /* use rcvstore to put message in folder */
+    status = usr_pipe (fd, "rcvstore", rcvstoreproc, vec, 1);
+
+#if 0
+    /*
+     * Currently, verbose status messages are handled by usr_pipe().
+     */
+    if (verbose) {
+	if (status == 0)
+	    verbose_printf (", success.\n");
+	else
+	    verbose_printf (", failed.\n");
     }
-    chown (tmpfil, pw->pw_uid, pw->pw_gid);
-
-    vecp = 1;
-    sprintf (receive, "%s/.mh_receive", pw->pw_dir);
-    switch (access (receive, X_OK)) {
-	case -1: 
-	    sprintf (receive, "%s/bin/rcvmail", pw->pw_dir);
-	    if (access (receive, X_OK) == -1) {
-		unlink (tmpfil);
-		if (verbose) {
-		    printf ("\tnot present\n");
-		    fflush (stdout);
-		}
-		return -1;
-	    }
-	    vec[vecp++] = addr;
-	    vec[vecp++] = tmpfil;
-	    vec[vecp++] = sender;
-	    break;
-
-	default: 
-	    vec[vecp++] = tmpfil;
-	    vec[vecp++] = mailbox;
-	    vec[vecp++] = home;
-	    vec[vecp++] = addr;
-	    vec[vecp++] = sender;
-	    break;
-    }
-    vec[0] = r1bindex (receive, '/');
-    vec[vecp] = NULL;
-
-    i = usr_pipe (fd, "rcvmail", receive, vec);
-    unlink (tmpfil);
-
-    return i;
-}
 #endif
 
+    return status;
+}
+
+/*
+ * Deliver message to a process.
+ */
 
 static int
-usr_pipe (int fd, char *cmd, char *pgm, char **vec)
+usr_pipe (int fd, char *cmd, char *pgm, char **vec, int suppress)
 {
     pid_t child_id;
     int i, bytes, seconds, status;
     struct stat st;
 
-    if (verbose) {
-	printf ("\tdelivering to pipe \"%s\"", cmd);
-	fflush (stdout);
-    }
+    if (verbose && !suppress)
+	verbose_printf ("delivering to pipe \"%s\"", cmd);
+
     lseek (fd, (off_t) 0, SEEK_SET);
 
     for (i = 0; (child_id = fork()) == -1 && i < 5; i++)
 	sleep (5);
+
     switch (child_id) {
 	case -1: 
 	    /* fork error */
-	    adorn ("fork", "unable to");
+	    if (verbose)
+		adorn ("fork", "unable to");
 	    return -1;
 
 	case 0: 
@@ -1014,13 +1140,12 @@ usr_pipe (int fd, char *cmd, char *pgm, char **vec)
 #endif
 		if (verbose) {
 		    if (status == 0)
-			printf (", wins.\n");
+			verbose_printf (", success.\n");
 		    else
 			if ((status & 0xff00) == 0xff00)
-			    printf (", system error\n");
+			    verbose_printf (", system error\n");
 			else
-			    pidstatus (status, stdout, ", loses");
-		    fflush (stdout);
+			    pidstatus (status, stdout, ", failed");
 		}
 		return (status == 0 ? 0 : -1);
 	    } else {
@@ -1029,10 +1154,8 @@ usr_pipe (int fd, char *cmd, char *pgm, char **vec)
 		 * else in its process group.
 		 */
 		KILLPG(child_id, SIGKILL);
-		if (verbose) {
-		    printf (", timed-out; terminated\n");
-		    fflush (stdout);
-		}
+		if (verbose)
+		    verbose_printf (", timed-out; terminated\n");
 		return -1;
 	    }
     }
@@ -1068,7 +1191,7 @@ get_sender (char *envelope, char **sender)
     }
 
     i = strlen ("From ");
-    strcpy (buffer, envelope + i);
+    strncpy (buffer, envelope + i, sizeof(buffer));
     if ((cp = strchr(buffer, '\n'))) {
 	*cp = 0;
 	cp -= 24;
@@ -1110,7 +1233,7 @@ copy_message (int qd, char *tmpfil, int fold)
     if (!fold) {
 	while ((i = read (qd, buffer, sizeof(buffer))) > 0)
 	    if (write (fd1, buffer, i) != i) {
-you_lose: ;
+you_lose:
 		close (fd1);
 		unlink (tmpfil);
 		return -1;
@@ -1186,10 +1309,12 @@ you_lose: ;
 		if (hp) {
 		    /* return path for UUCP style addressing */
 		    ep = strchr(++hp, '\n');
-		    sprintf (buffer, "Return-Path: %.*s!%.*s\n", ep - hp, hp, cp - fp, fp);
+		    snprintf (buffer, sizeof(buffer), "Return-Path: %.*s!%.*s\n",
+			ep - hp, hp, cp - fp, fp);
 		} else {
 		    /* return path for standard domain addressing */
-		    sprintf (buffer, "Return-Path: %.*s\n", cp - fp, fp);
+		    snprintf (buffer, sizeof(buffer), "Return-Path: %.*s\n",
+			cp - fp, fp);
 		}
 
 		/* Add Return-Path header to message */
@@ -1243,7 +1368,7 @@ trim (char *cp)
 	return NULL;
 
     /* copy string into temp buffer */
-    strcpy (buffer, cp);
+    strncpy (buffer, cp, sizeof(buffer));
     bp = buffer;
 
     /* skip over leading whitespace */
@@ -1267,25 +1392,68 @@ trim (char *cp)
     return getcpy(bp);
 }
 
+/*
+ * Function for printing `verbose' messages.
+ */
+
+static void
+verbose_printf (char *fmt, ...)
+{
+    va_list ap;
+
+    va_start(ap, fmt);
+    vfprintf (stdout, fmt, ap);
+    va_end(ap);
+
+    fflush (stdout);	/* now flush output */
+}
+
+
+/*
+ * Function for printing `verbose' delivery
+ * error messages.
+ */
 
 static void
 adorn (char *what, char *fmt, ...)
 {
     va_list ap;
-    char *cp;
+    int eindex;
+    char *s;
 
-    if (!verbose)
-	return;
-    printf (", ");
-
-    cp = invo_name;
-    invo_name = NULL;
+    eindex = errno;	/* save the errno */
+    fprintf (stdout, ", ");
 
     va_start(ap, fmt);
-    advertise (what, NULL, fmt, ap);
+    vfprintf (stdout, fmt, ap);
     va_end(ap);
 
-    invo_name = cp;
+    if (what) {
+	if (*what)
+	    fprintf (stdout, " %s: ", what);
+	if ((s = strerror (eindex)))
+	    fprintf (stdout, "%s", s);
+	else
+	    fprintf (stdout, "Error %d", eindex);
+    }
+
+    fputc ('\n', stdout);
+    fflush (stdout);
+}
+
+
+/*
+ * Function for printing `debug' messages.
+ */
+
+static void
+debug_printf (char *fmt, ...)
+{
+    va_list ap;
+
+    va_start(ap, fmt);
+    vfprintf (stderr, fmt, ap);
+    va_end(ap);
 }
 
 
@@ -1342,10 +1510,12 @@ suppress_duplicates (int fd, char *file)
 		    return -1;
 		}
 		/*
-		 * Since it is difficult to portable lock a ndbm
-		 * file, we will lock the Maildelivery file instead.
+		 * Since it is difficult to portable lock a ndbm file,
+		 * we will open and lock the Maildelivery file instead.
+		 * This will fail if your Maildelivery file doesn't
+		 * exist.
 		 */
-		if ((lockfd = lkopen(file, O_RDWR)) == -1) {
+		if ((lockfd = lkopen(file, O_RDWR, 0)) == -1) {
 		    advise (file, "unable to perform file locking on");
 		    free (cp);
 		    fclose (in);
@@ -1354,7 +1524,7 @@ suppress_duplicates (int fd, char *file)
 		value = dbm_fetch (db, key);
 		if (value.dptr) {
 		    if (verbose)
-		        printf ("Message-ID: %s\n            already received on %s",
+		        verbose_printf ("Message-ID: %s\n            already received on %s",
 				 cp, value.dptr);
 		    result = DONE;
 		} else {
