@@ -2,12 +2,19 @@
 /*
  * sortm.c -- sort messages in a folder by date/time
  *
- * $Id$
+ * $Id: sortm.c,v 1.2 1999/07/16 00:58:36 danh Exp $
  */
 
 #include <h/mh.h>
 #include <zotnet/tws/tws.h>
 
+/*
+ * We allocate space for messages (msgs array)
+ * this number of elements at a time.
+ */
+#define MAXMSGS  256
+
+
 static struct swit switches[] = {
 #define DATESW                 0
      { "datefield field", 0 },
@@ -53,7 +60,7 @@ typedef int (*qsort_comp) (const void *, const void *);
  * static prototypes
  */
 static int read_hdrs (struct msgs *, char *);
-static int getws (char *, int, struct smsg *);
+static int get_fields (char *, int, struct smsg *);
 static int dsort (struct smsg **, struct smsg **);
 static int subsort (struct smsg **, struct smsg **);
 static int txtsort (struct smsg **, struct smsg **);
@@ -64,10 +71,10 @@ static void rename_msgs (struct msgs *, struct smsg **);
 int
 main (int argc, char **argv)
 {
-    int	msgp = 0, i, msgnum;
+    int	nummsgs, maxmsgs, i, msgnum;
     char *cp, *maildir, *datesw = NULL;
-    char *folder = NULL, buf[100], **ap, **argp;
-    char *arguments[MAXARGS], *msgs[MAXARGS];
+    char *folder = NULL, buf[BUFSIZ], **argp;
+    char **arguments, **msgs;
     struct msgs *mp;
     struct smsg **dlist;
 
@@ -75,15 +82,25 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
+    /*
+     * Allocate the initial space to record message
+     * names and ranges.
+     */
+    nummsgs = 0;
+    maxmsgs = MAXMSGS;
+    if (!(msgs = (char **) malloc ((size_t) (maxmsgs * sizeof(*msgs)))))
+	adios (NULL, "unable to allocate storage");
+
+    /*
+     * Parse arguments
+     */
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
@@ -94,7 +111,8 @@ main (int argc, char **argv)
 		adios (NULL, "-%s unknown", cp);
 
 	    case HELPSW:
-		sprintf(buf, "%s [+folder] [msgs] [switches]", invo_name);
+		snprintf(buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
+			invo_name);
 		print_help (buf, switches, 1);
 		done (1);
 	    case VERSIONSW:
@@ -154,14 +172,24 @@ main (int argc, char **argv)
 	    else
 		folder = path (cp + 1, *cp == '+' ? TFOLDER : TSUBCWF);
 	} else {
-	    msgs[msgp++] = cp;
+	    /*
+	     * Check if we need to allocate more space
+	     * for message names/ranges.
+	     */
+	    if (nummsgs >= maxmsgs) {
+		maxmsgs += MAXMSGS;
+		if (!(msgs = (char **) realloc (msgs,
+			(size_t) (maxmsgs * sizeof(*msgs)))))
+		    adios (NULL, "unable to reallocate msgs storage");
+	    }
+	    msgs[nummsgs++] = cp;
 	}
     }
 
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
-    if (!msgp)
-	msgs[msgp++] = "all";
+    if (!nummsgs)
+	msgs[nummsgs++] = "all";
     if (!datesw)
 	datesw = "date";
     if (!folder)
@@ -175,11 +203,12 @@ main (int argc, char **argv)
     if (!(mp = folder_read (folder)))
 	adios (NULL, "unable to read folder %s", folder);
 
-    if (mp->hghmsg == 0)
+    /* check for empty folder */
+    if (mp->nummsg == 0)
 	adios (NULL, "no messages in %s", folder);
 
     /* parse all the message ranges/sequences and set SELECTED */
-    for (msgnum = 0; msgnum < msgp; msgnum++)
+    for (msgnum = 0; msgnum < nummsgs; msgnum++)
 	if (!m_convert (mp, msgs[msgnum]))
 	    done (1);
     seq_setprev (mp);	/* set the previous sequence */
@@ -197,13 +226,14 @@ main (int argc, char **argv)
 	dlist[i] = &smsgs[i];
     dlist[nmsgs] = 0;
 
-    if (verbose) 	/* announce what we're doing */
+    if (verbose) {	/* announce what we're doing */
 	if (subjsort)
 	    printf ("sorting by %s-major %s-minor\n", 
 		submajor ? subjsort : datesw,
 		submajor ? datesw : subjsort);
 	else
 	    printf ("sorting by datefield %s\n", datesw);
+    }
 
     /* first sort by date, or by subject-major, date-minor */
     qsort ((char *) dlist, nmsgs, sizeof(*dlist), 
@@ -275,7 +305,7 @@ main (int argc, char **argv)
     seq_save (mp);			/* synchronize message sequences */
     context_save ();			/* save the context file         */
     folder_free (mp);			/* free folder/message structure */
-    done (0);
+    return done (0);
 }
 
 static int 
@@ -289,14 +319,14 @@ read_hdrs (struct msgs *mp, char *datesw)
 
     smsgs = (struct smsg *)
 	calloc ((size_t) (mp->hghsel - mp->lowsel + 2),
-	    sizeof *smsgs);
+	    sizeof(*smsgs));
     if (smsgs == NULL)
 	adios (NULL, "unable to allocate sort storage");
 
     s = smsgs;
     for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
 	if (is_selected(mp, msgnum)) {
-	    if (getws (datesw, msgnum, s)) {
+	    if (get_fields (datesw, msgnum, s)) {
 		s->s_msg = msgnum;
 		s++;
 	    }
@@ -306,8 +336,14 @@ read_hdrs (struct msgs *mp, char *datesw)
     return(s - smsgs);
 }
 
+
+/*
+ * Parse the message and get the data or subject field,
+ * if needed.
+ */
+
 static int
-getws (char *datesw, int msg, struct smsg *smsg)
+get_fields (char *datesw, int msg, struct smsg *smsg)
 {
     register int state;
     int compnum;
@@ -321,7 +357,7 @@ getws (char *datesw, int msg, struct smsg *smsg)
 	return (0);
     }
     for (compnum = 1, state = FLD;;) {
-	switch (state = m_getfld (state, nam, buf, sizeof buf, in)) {
+	switch (state = m_getfld (state, nam, buf, sizeof(buf), in)) {
 	case FLD:
 	case FLDEOF:
 	case FLDPLUS:
@@ -329,7 +365,7 @@ getws (char *datesw, int msg, struct smsg *smsg)
 	    if (!strcasecmp (nam, datesw)) {
 		datecomp = add (buf, datecomp);
 		while (state == FLDPLUS) {
-		    state = m_getfld (state, nam, buf, sizeof buf, in);
+		    state = m_getfld (state, nam, buf, sizeof(buf), in);
 		    datecomp = add (buf, datecomp);
 		}
 		if (!subjsort || subjcomp)
@@ -337,7 +373,7 @@ getws (char *datesw, int msg, struct smsg *smsg)
 	    } else if (subjsort && !strcasecmp (nam, subjsort)) {
 		subjcomp = add (buf, subjcomp);
 		while (state == FLDPLUS) {
-		    state = m_getfld (state, nam, buf, sizeof buf, in);
+		    state = m_getfld (state, nam, buf, sizeof(buf), in);
 		    subjcomp = add (buf, subjcomp);
 		}
 		if (datecomp)
@@ -345,7 +381,7 @@ getws (char *datesw, int msg, struct smsg *smsg)
 	    } else {
 		/* just flush this guy */
 		while (state == FLDPLUS)
-		    state = m_getfld (state, nam, buf, sizeof buf, in);
+		    state = m_getfld (state, nam, buf, sizeof(buf), in);
 	    }
 	    continue;
 
@@ -483,7 +519,7 @@ rename_chain (struct msgs *mp, struct smsg **mlist, int msg, int endmsg)
 	mlist[msg] = (struct smsg *)0;
 	old = smsgs[nxt].s_msg;
 	new = smsgs[msg].s_msg;
-	strcpy (oldname, m_name (old));
+	strncpy (oldname, m_name (old), sizeof(oldname));
 	newname = m_name (new);
 	if (verbose)
 	    printf ("message %d becomes message %d\n", old, new);
@@ -512,7 +548,7 @@ rename_msgs (struct msgs *mp, struct smsg **mlist)
     char f1[BUFSIZ], tmpfil[BUFSIZ];
     struct smsg *sp;
 
-    strcpy (tmpfil, m_name (mp->hghmsg + 1));
+    strncpy (tmpfil, m_name (mp->hghmsg + 1), sizeof(tmpfil));
 
     for (i = 0; i < nmsgs; i++) {
 	if (! (sp = mlist[i])) 
@@ -529,7 +565,7 @@ rename_msgs (struct msgs *mp, struct smsg **mlist)
 	 */
 	old = smsgs[j].s_msg;
 	new = smsgs[i].s_msg;
-	strcpy (f1, m_name (old));
+	strncpy (f1, m_name (old), sizeof(f1));
 
 	if (verbose)
 	    printf ("renaming message chain from %d to %d\n", old, new);
