@@ -5,7 +5,7 @@
  * This is a simpler, faster, replacement for "post" for use
  * when "sendmail" is the transport system.
  *
- * $Id$
+ * $Id: spost.c,v 1.6 1999/09/12 13:50:12 doug Exp $
  */
 
 #include <h/mh.h>
@@ -14,6 +14,7 @@
 #include <h/aliasbr.h>
 #include <h/dropsbr.h>
 #include <zotnet/tws/tws.h>
+#include <zotnet/mts/mts.h>
 
 #define	uptolow(c)	((isalpha(c) && isupper (c)) ? tolower (c) : c)
 
@@ -199,21 +200,22 @@ int
 main (int argc, char **argv)
 {
     int state, i, pid, compnum;
-    char *cp, *msg = NULL, **argp = argv + 1;
+    char *cp, *msg = NULL, **argp, **arguments;
     char *sargv[16], buf[BUFSIZ], name[NAMESZ];
-    char *arguments[MAXARGS];
     FILE *in;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
+
+    /* foil search of user profile/context */
+    if (context_foil (NULL) == -1)
+	done (1);
+
     mts_init (invo_name);
-    if ((cp = context_find (invo_name)) != NULL) {
-	argp = copyip (brkstring (cp, " ", "\n"), arguments);
-	copyip (argv+1, argp);
-	argp = arguments;
-    }
+    arguments = getarguments (invo_name, argc, argv, 0);
+    argp = arguments;
 
     while ((cp = *argp++)) {
 	if (*cp == '-') {
@@ -225,7 +227,7 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [switches] file", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches] file", invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -310,8 +312,15 @@ main (int argc, char **argv)
 		    continue;
 
 		case LIBSW:
+		    if (!(cp = *argp++) || *cp == '-')
+			adios (NULL, "missing argument to %s", argp[-2]);
+		    /* create a minimal context */
+		    if (context_foil (cp) == -1)
+			done(1);
+		    continue;
+
 		case ANNOSW:
-		    /* -library & -idanno switch ignored */
+		    /* -idanno switch ignored */
 		    if (!(cp = *argp++) || *cp == '-')
 			adios (NULL, "missing argument to %s", argp[-2]);
 		    continue;
@@ -338,16 +347,21 @@ main (int argc, char **argv)
 	out = stdout;
     }
     else {
+#ifdef HAVE_MKSTEMP
+	    if ((out = fdopen( mkstemp (tmpfil), "w" )) == NULL )
+		adios (tmpfil, "unable to create");
+#else
 	    mktemp (tmpfil);
 	    if ((out = fopen (tmpfil, "w")) == NULL)
 		adios (tmpfil, "unable to create");
 	    chmod (tmpfil, 0600);
+#endif
 	}
 
     hdrtab = (msgstate == normal) ? NHeaders : RHeaders;
 
     for (compnum = 1, state = FLD;;) {
-	switch (state = m_getfld (state, name, buf, sizeof buf, in)) {
+	switch (state = m_getfld (state, name, buf, sizeof(buf), in)) {
 	    case FLD: 
 		compnum++;
 		putfmt (name, buf, out);
@@ -357,7 +371,7 @@ main (int argc, char **argv)
 		compnum++;
 		cp = add (buf, cp);
 		while (state == FLDPLUS) {
-		    state = m_getfld (state, name, buf, sizeof buf, in);
+		    state = m_getfld (state, name, buf, sizeof(buf), in);
 		    cp = add (buf, cp);
 		}
 		putfmt (name, cp, out);
@@ -369,7 +383,7 @@ main (int argc, char **argv)
 		fprintf (out, "\n%s", buf);
 		if(whomflg == 0)
 		    while (state == BODY) {
-			state = m_getfld (state, name, buf, sizeof buf, in);
+			state = m_getfld (state, name, buf, sizeof(buf), in);
 			fputs (buf, out);
 		    }
 		break;
@@ -391,7 +405,7 @@ main (int argc, char **argv)
 
     fclose (in);
     if (backflg && !whomflg) {
-	strcpy (buf, m_backup (msg));
+	strncpy (buf, m_backup (msg), sizeof(buf));
 	if (rename (msg, buf) == NOTOK)
 	    advise (buf, "unable to rename %s to", msg);
     }
@@ -442,6 +456,7 @@ main (int argc, char **argv)
     }
     execv ( sendmail, sargv);
     adios ( sendmail, "can't exec");
+    return 0;  /* dead code to satisfy the compiler */
 }
 
 /* DRAFT GENERATION */
@@ -477,7 +492,7 @@ putfmt (char *name, char *str, FILE *out)
     if (hdr->flags & HFCC) {
 	if ((cp = strrchr(str, '\n')))
 	    *cp = 0;
-	for (cp = pp = str; cp = strchr(pp, ','); pp = cp) {
+	for (cp = pp = str; (cp = strchr(pp, ',')); pp = cp) {
 	    *cp++ = 0;
 	    insert_fcc (hdr, pp);
 	}
@@ -492,7 +507,7 @@ putfmt (char *name, char *str, FILE *out)
     }
 #endif	/* notdef */
 
-    if (*str != '\n' && *str != '\0')
+    if (*str != '\n' && *str != '\0') {
 	if (aliasflg && hdr->flags & HTRY) {
 	    /* this header contains address(es) that we have to do
 	     * alias expansion on.  Because of the saved state in
@@ -519,6 +534,7 @@ putfmt (char *name, char *str, FILE *out)
 	} else {
 	    fprintf (out, "%s: %s", name, str );
 	}
+    }
 }
 
 
@@ -528,14 +544,14 @@ start_headers (void)
     char *cp;
     char sigbuf[BUFSIZ];
 
-    strcpy( from, getusername() );
+    strncpy(from, getusername(), sizeof(from));
 
     if ((cp = getfullname ()) && *cp) {
-	strcpy (sigbuf, cp);
-	sprintf (signature, "%s <%s>", sigbuf,  from);
+	strncpy (sigbuf, cp, sizeof(sigbuf));
+	snprintf (signature, sizeof(signature), "%s <%s>", sigbuf,  from);
     }
     else
-	sprintf (signature, "%s",  from);
+	snprintf (signature, sizeof(signature), "%s",  from);
 }
 
 
@@ -792,7 +808,7 @@ fcc (char *file, char *folder)
 	    break;
 
 	case OK: 
-	    sprintf (fold, "%s%s",
+	    snprintf (fold, sizeof(fold), "%s%s",
 		    *folder == '+' || *folder == '@' ? "" : "+", folder);
 	    execlp (fileproc, r1bindex (fileproc, '/'),
 		    "-link", "-file", file, fold, NULL);
