@@ -2,7 +2,7 @@
 /*
  * termsbr.c -- termcap support
  *
- * $Id$
+ * $Id: termsbr.c,v 1.3 1999/09/13 18:20:28 doug Exp $
  */
 
 #include <h/mh.h>
@@ -21,9 +21,14 @@
 # include <termcap.h>
 #endif
 
+/* <sys/ioctl.h> is need anyway for ioctl()
 #ifdef GWINSZ_IN_SYS_IOCTL
+*/
 # include <sys/ioctl.h>
+/*
 #endif
+*/
+
 #ifdef WINSIZE_IN_PTEM
 # include <sys/stream.h>
 # include <sys/ptem.h>
@@ -170,7 +175,7 @@ sc_length (void)
 static int
 outc (int c)
 {
-    putchar(c);
+    return putchar(c);
 }
 
 
