@@ -2,7 +2,7 @@
 /*
  * viamail.c -- send multiple files in a MIME message
  *
- * $Id$
+ * $Id: viamail.c,v 1.2 1999/07/16 00:57:56 danh Exp $
  */
 
 #include <h/mh.h>
@@ -13,7 +13,7 @@
 #include <signal.h>
 #include <zotnet/mts/mts.h>
 #include <zotnet/tws/tws.h>
-#include <h/mhn.h>
+#include <h/mime.h>
 #include <h/mhparse.h>
 
 #ifdef HAVE_SYS_WAIT_H
@@ -53,18 +53,18 @@ extern int debugsw;
 extern int splitsw;
 extern int verbsw;
 
-/*
- * prototypes
- */
+int ebcdicsw = 0;	/* hack for linking purposes */
 
 /* mhmisc.c */
 void set_endian (void);
 
+/* mhoutsbr.c */
+int writeBase64aux (FILE *, FILE *);
+
 /*
  * static prototypes
  */
 static int via_mail (char *, char *, char *, char *, char *, int, char *);
-static int writeBase64aux (FILE *, FILE *);
 
 
 int
@@ -73,27 +73,19 @@ main (int argc, char **argv)
     int delay = 0;
     char *f1 = NULL, *f2 = NULL, *f3 = NULL;
     char *f4 = NULL, *f5 = NULL, *f7 = NULL;
-    char *cp, buf[100], **ap;
-    char **argp, *arguments[MAXARGS];
+    char *cp, buf[BUFSIZ];
+    char **argp, **arguments;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
 
-    /*
-     * foil search of user profile/context
-     */
+    /* foil search of user profile/context */
     if (context_foil (NULL) == -1)
 	done (1);
 
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+    arguments = getarguments (invo_name, argc, argv, 0);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -106,7 +98,7 @@ main (int argc, char **argv)
 		adios (NULL, "-%s unknown", cp);
 
 	    case HELPSW: 
-		sprintf (buf, "%s [switches]", invo_name);
+		snprintf (buf, sizeof(buf), "%s [switches]", invo_name);
 		print_help (buf, switches, 1);
 		done (1);
 	    case VERSIONSW:
@@ -169,7 +161,7 @@ main (int argc, char **argv)
 	adios (NULL, "missing -viamail \"mailpath\" switch");
 
     via_mail (f1, f2, f3, f4, f5, delay, f7);
-    /* NOTREACHED */
+    return 0;  /* dead code to satisfy the compiler */
 }
 
 
@@ -189,7 +181,7 @@ via_mail (char *mailsw, char *subjsw, char *parmsw, char *descsw,
 
     umask (~m_gmprot ());
 
-    strcpy (tmpfil, m_tmpfil (invo_name));
+    strncpy (tmpfil, m_tmpfil (invo_name), sizeof(tmpfil));
     if ((fp = fopen (tmpfil, "w+")) == NULL)
 	adios (tmpfil, "unable to open for writing");
     chmod (tmpfil, 0600);
@@ -253,57 +245,5 @@ via_mail (char *mailsw, char *subjsw, char *parmsw, char *descsw,
     fclose (fp);
     if (unlink (tmpfil) == -1)
 	advise (NULL, "unable to remove temp file %s", tmpfil);
-    done (status);
-}
-
-
-static char nib2b64[0x40+1] =
-	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-
-
-static int
-writeBase64aux (FILE *in, FILE *out)
-{
-    int	cc, n;
-    char inbuf[3];
-
-    n = BPERLIN;
-    while ((cc = fread (inbuf, sizeof(*inbuf), sizeof(inbuf), in)) > 0) {
-	unsigned long bits;
-	char *bp;
-	char outbuf[4];
-
-	if (cc < sizeof(inbuf)) {
-	    inbuf[2] = 0;
-	    if (cc < sizeof(inbuf) - 1)
-		inbuf[1] = 0;
-	}
-	bits = (inbuf[0] & 0xff) << 16;
-	bits |= (inbuf[1] & 0xff) << 8;
-	bits |= inbuf[2] & 0xff;
-
-	for (bp = outbuf + sizeof(outbuf); bp > outbuf; bits >>= 6)
-	    *--bp = nib2b64[bits & 0x3f];
-	if (cc < sizeof(inbuf)) {
-	    outbuf[3] = '=';
-	    if (cc < sizeof inbuf - 1)
-		outbuf[2] = '=';
-	}
-
-	fwrite (outbuf, sizeof(*outbuf), sizeof(outbuf), out);
-
-	if (cc < sizeof(inbuf)) {
-	    putc ('\n', out);
-	    return OK;
-	}
-
-	if (--n <= 0) {
-	    n = BPERLIN;
-	    putc ('\n', out);
-	}
-    }
-    if (n != BPERLIN)
-	putc ('\n', out);
-
-    return OK;
+    return done (status);
 }
