@@ -2,7 +2,7 @@
 /*
  * vmh.c -- visual front-end to nmh
  *
- * $Id$
+ * $Id: vmh.c,v 1.2 1999/07/16 01:21:17 danh Exp $
  */
 
 #include <h/mh.h>
@@ -217,20 +217,17 @@ main (int argc, char **argv)
 {
     int vecp = 1, nprog = 0;
     char *cp, buffer[BUFSIZ];
-    char **ap, **argp;
-    char *arguments[MAXARGS], *vec[MAXARGS];
+    char **argp, **arguments, *vec[MAXARGS];
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name)) != NULL) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    }
-    else
-	ap = arguments;
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++))
@@ -244,8 +241,9 @@ main (int argc, char **argv)
 		    continue;
 
 		case HELPSW: 
-		    sprintf (buffer, "%s [switches for vmhproc]", invo_name);
-		    print_help (buffer, switches);
+		    snprintf (buffer, sizeof(buffer), "%s [switches for vmhproc]",
+			invo_name);
+		    print_help (buffer, switches, 1);
 		    done (1);
 		case VERSIONSW:
 		    print_version(invo_name);
@@ -280,7 +278,7 @@ main (int argc, char **argv)
 
     vmh ();
 
-    done (0);
+    return done (0);
 }
 
 
@@ -340,10 +338,10 @@ PEERinit (int vecp, char *vec[])
 	    close (pfd1[1]);
 
 	    vec[vecp++] = "-vmhread";
-	    sprintf (buf1, "%d", pfd1[0]);
+	    snprintf (buf1, sizeof(buf1), "%d", pfd1[0]);
 	    vec[vecp++] = buf1;
 	    vec[vecp++] = "-vmhwrite";
-	    sprintf (buf2, "%d", pfd0[1]);
+	    snprintf (buf2, sizeof(buf2), "%d", pfd0[1]);
 	    vec[vecp++] = buf2;
 	    vec[vecp] = NULL;
 
@@ -368,20 +366,28 @@ PEERinit (int vecp, char *vec[])
 static int
 pINI (void)
 {
-    register char *bp;
-    char buffer[BUFSIZ];
+    int len, buflen;
+    char *bp, buffer[BUFSIZ];
     struct record rcs;
     register struct record *rc = &rcs;
     register WINDOW **w;
 
     initrc (rc);
 
+    /* Get buffer ready to go */
     bp = buffer;
-    sprintf (bp, "%d %d", RC_VRSN, numwins);
-    bp += strlen (bp);
+    buflen = sizeof(buffer);
+
+    snprintf (bp, buflen, "%d %d", RC_VRSN, numwins);
+    len = strlen (bp);
+    bp += len;
+    buflen -= len;
+
     for (w = windows; *w; w++) {
-	sprintf (bp, " %d", (*w)->_maxy);
-	bp += strlen (bp);
+	snprintf (bp, buflen, " %d", (*w)->_maxy);
+	len = strlen (bp);
+	bp += len;
+	buflen -= len;
     }
 
     switch (str2rc (RC_INI, buffer, rc)) {
@@ -1426,13 +1432,14 @@ TSTPser (int sig)
 
 /* MISCELLANY */
 
-void
+int
 done (int status)
 {
     TTYoff ();
     pFIN ();
 
     exit (status);
+    return 1;  /* dead code to satisfy the compiler */
 }
 
 
@@ -1471,7 +1478,7 @@ advertise (char *what, char *tail, char *fmt, va_list ap)
 	iov++;
     }
     
-    vsprintf (buffer, fmt, ap);
+    vsnprintf (buffer, sizeof(buffer), fmt, ap);
     iov->iov_len = strlen (iov->iov_base = buffer);
     iov++;
     if (what) {
@@ -1484,7 +1491,7 @@ advertise (char *what, char *tail, char *fmt, va_list ap)
 	    iov++;
 	}
 	if (!(iov->iov_base = strerror (eindex))) {
-	    sprintf (err, "Error %d", eindex);
+	    snprintf (err, sizeof(err), "Error %d", eindex);
 	    iov->iov_base = err;
 	}
 	iov->iov_len = strlen (iov->iov_base);
