@@ -2,7 +2,7 @@
 /*
  * vmhsbr.c -- routines to help vmh along
  *
- * $Id$
+ * $Id: vmhsbr.c,v 1.3 1999/07/16 01:01:24 danh Exp $
  */
 
 /*
@@ -12,6 +12,7 @@
 
 #include <h/mh.h>
 #include <h/vmhsbr.h>
+#include <errno.h>
 
 static char *types[] = {
     "OK",
@@ -24,8 +25,6 @@ static	FILE *fp = NULL;
 static int PEERrfd = NOTOK;
 static int PEERwfd = NOTOK;
 
-extern int  errno;
-
 /*
  * static prototypes
  */
@@ -41,7 +40,7 @@ rcinit (int rfd, int wfd)
     PEERwfd = wfd;
 
     if ((cp = getenv ("MHVDEBUG")) && *cp) {
-	sprintf (buffer, "%s.out", invo_name);
+	snprintf (buffer, sizeof(buffer), "%s.out", invo_name);
 	if ((fp = fopen (buffer, "w"))) {
 	  fseek (fp, 0L, SEEK_END);
 	  fprintf (fp, "%d: rcinit (%d, %d)\n", (int) getpid(), rfd, wfd);
@@ -165,11 +164,13 @@ fmt2peer (char code, char *fmt, ...)
 int
 err2peer (char code, char *what, char *fmt, ...)
 {
+    int     return_value;
     va_list ap;
 
     va_start(ap, fmt);
-    verr2peer(code, what, fmt, ap);
+    return_value = verr2peer(code, what, fmt, ap);
     va_end(ap);
+    return return_value;  /* This routine returned garbage before 1999-07-15. */
 }
 
 
@@ -177,21 +178,32 @@ int
 verr2peer (char code, char *what, char *fmt, va_list ap)
 {
     int eindex = errno;
-    register char *bp, *s;
-    char buffer[BUFSIZ * 2];
+    int len, buflen;
+    char *bp, *s, buffer[BUFSIZ * 2];
+
+    /* Get buffer ready to go */
+    bp = buffer;
+    buflen = sizeof(buffer);
+
+    vsnprintf (bp, buflen, fmt, ap);
+    len = strlen (bp);
+    bp += len;
+    buflen -= len;
 
-    vsprintf (buffer, fmt, ap);
-    bp = buffer + strlen (buffer);
     if (what) {
 	if (*what) {
-	    sprintf (bp, " %s: ", what);
-	    bp += strlen (bp);
+	    snprintf (bp, buflen, " %s: ", what);
+	    len = strlen (bp);
+	    bp += len;
+	    buflen -= len;
 	}
 	if ((s = strerror (eindex)))
-	    strcpy (bp, s);
+	    strncpy (bp, s, buflen);
 	else
-	    sprintf (bp, "unknown error %d", eindex);
-	bp += strlen (bp);
+	    snprintf (bp, buflen, "unknown error %d", eindex);
+	len = strlen (bp);
+	bp += len;
+	buflen -= len;
     }
 
     return rc2peer (code, bp - buffer, buffer);
@@ -205,7 +217,7 @@ rclose (struct record *rc, char *fmt, ...)
     static char buffer[BUFSIZ * 2];
 
     va_start(ap, fmt);
-    vsprintf (buffer, fmt, ap);
+    vsnprintf (buffer, sizeof(buffer), fmt, ap);
     va_end(ap);
 
     rc->rc_len = strlen (rc->rc_data = getcpy (buffer));
