@@ -2,7 +2,7 @@
 /*
  * vmhtest.c -- test out vmh protocol
  *
- * $Id$
+ * $Id: vmhtest.c,v 1.2 1999/07/16 01:26:05 danh Exp $
  */
 
 #include <h/mh.h>
@@ -36,8 +36,8 @@ int
 main (int argc, char **argv)
 {
     int fd1, fd2;
-    char *cp, buffer[BUFSIZ], **ap;
-    char **argp = argv + 1, *arguments[MAXARGS];
+    char *cp, buffer[BUFSIZ];
+    char **argp, **arguments;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
@@ -48,6 +48,9 @@ main (int argc, char **argv)
     if (context_foil (NULL) == -1)
 	done (1);
 
+    arguments = getarguments (invo_name, argc, argv, 0);
+    argp = arguments;
+
     while ((cp = *argp++))
 	if (*cp == '-')
 	    switch (smatch (++cp, switches)) {
@@ -58,7 +61,7 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buffer, "%s [switches]", invo_name);
+		    snprintf (buffer, sizeof(buffer), "%s [switches]", invo_name);
 		    print_help (buffer, switches, 0);
 		    done (1);
 		case VERSIONSW:
@@ -85,7 +88,7 @@ main (int argc, char **argv)
     pINI ();
     pLOOP ();
 
-    done (0);
+    return done (0);
 }
 
 
@@ -241,7 +244,7 @@ char   *str;
 
     initrc (rc);
 
-    sprintf (buffer, "%d", selwin ());
+    snprintf (buffer, sizeof(buffer), "%d", selwin ());
     switch (str2rc (RC_WIN, buffer, rc)) {
 	case RC_ACK: 
 	    break;
