@@ -2,13 +2,13 @@
 /*
  * whatnowsbr.c -- the WhatNow shell
  *
- * $Id$
+ * $Id: whatnowsbr.c,v 1.2 1999/07/16 01:13:28 danh Exp $
  */
 
 #include <h/mh.h>
 #include <fcntl.h>
 #include <signal.h>
-#include <h/mhn.h>
+#include <h/mime.h>
 
 static struct swit whatnowswitches[] = {
 #define	DFOLDSW                 0
@@ -82,18 +82,16 @@ WhatNow (int argc, char **argv)
     int isdf = 0, nedit = 0, use = 0;
     char *cp, *dfolder = NULL, *dmsg = NULL;
     char *ed = NULL, *drft = NULL, *msgnam = NULL;
-    char buf[100], prompt[BUFSIZ];
-    char **ap, **argp, *arguments[MAXARGS];
+    char buf[BUFSIZ], prompt[BUFSIZ];
+    char **argp, **arguments;
     struct stat st;
 
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -106,7 +104,7 @@ WhatNow (int argc, char **argv)
 		adios (NULL, "-%s unknown", cp);
 
 	    case HELPSW: 
-		sprintf (buf, "%s [switches] [file]", invo_name);
+		snprintf (buf, sizeof(buf), "%s [switches] [file]", invo_name);
 		print_help (buf, whatnowswitches, 1);
 		done (1);
 	    case VERSIONSW:
@@ -170,7 +168,7 @@ WhatNow (int argc, char **argv)
     if (!nedit && editfile (&ed, NULL, drft, use, NULL, msgnam, NULL, 1) < 0)
 	done (1);
 
-    sprintf (prompt, myprompt, invo_name);
+    snprintf (prompt, sizeof(prompt), myprompt, invo_name);
     for (;;) {
 	if (!(argp = getans (prompt, aleqs))) {
 	    unlink (LINK);
@@ -294,13 +292,13 @@ editfile (char **ed, char **arg, char *file, int use, struct msgs *mp,
 
     if (altmsg) {
 	if (mp == NULL || *altmsg == '/' || cwd == NULL)
-	    strcpy (altpath, altmsg);
+	    strncpy (altpath, altmsg, sizeof(altpath));
 	else
-	    sprintf (altpath, "%s/%s", mp->foldpath, altmsg);
+	    snprintf (altpath, sizeof(altpath), "%s/%s", mp->foldpath, altmsg);
 	if (cwd == NULL)
-	    strcpy (linkpath, LINK);
+	    strncpy (linkpath, LINK, sizeof(linkpath));
 	else
-	    sprintf (linkpath, "%s/%s", cwd, LINK);
+	    snprintf (linkpath, sizeof(linkpath), "%s/%s", cwd, LINK);
     }
 
     if (altmsg) {
@@ -365,13 +363,14 @@ editfile (char **ed, char **arg, char *file, int use, struct msgs *mp,
 		else {
 #endif
 		if (((status & 0xff00) != 0xff00)
-			&& (!reedit || (status & 0x00ff)))
+		    && (!reedit || (status & 0x00ff))) {
 		    if (!use && (status & 0xff00) &&
 			    (rename (file, cp = m_backup (file)) != NOTOK)) {
 			advise (NULL, "problems with edit--draft left in %s", cp);
 		    } else {
 			advise (NULL, "problems with edit--%s preserved", file);
 		    }
+		}
 		status = -2;	/* maybe "reedit ? -2 : -1"? */
 		break;
 #ifdef ATTVIBUG
@@ -701,9 +700,9 @@ extern char *distfile;
 static void
 sendit (char *sp, char **arg, char *file, int pushed)
 {
-    int	vecp = 1;
-    char *cp, buf[100], **ap, **argp;
-    char *arguments[MAXARGS], *vec[MAXARGS];
+    int	vecp, n = 1;
+    char *cp, buf[BUFSIZ], **argp;
+    char **arguments, *vec[MAXARGS];
     struct stat st;
 
 #ifndef	lint
@@ -713,16 +712,37 @@ sendit (char *sp, char **arg, char *file, int pushed)
     FILE *fp;
 #endif
 
-    if (arg)
-	copyip (arg, vec);
-    if ((cp = context_find (sp))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
+    /*
+     * Make sure these are defined.  In particular, we need
+     * vec[1] to be NULL, in case "arg" is NULL below.  It
+     * doesn't matter what is the value of vec[0], but we
+     * set it to NULL, to help catch "off-by-one" errors.
+     */
+    vec[0] = NULL;
+    vec[1] = NULL;
+
+    /*
+     * Temporarily copy arg to vec, since the brkstring() call in
+     * getarguments() will wipe it out before it is merged in.
+     * Also, we skip the first element of vec, since getarguments()
+     * skips it.  Then we count the number of arguments
+     * copied.  The value of "n" will be one greater than
+     * this in order to simulate the standard argc/argv.
+     */
+    if (arg) {
+	char **bp;
+
+	copyip (arg, vec+1, MAXARGS-1);
+	bp = vec+1;
+	while (*bp++)
+	    n++;
     }
-    if (arg)
-	copyip (vec, ap);
+
+    /*
+     * Merge any arguments from command line (now in vec)
+     * and arguments from profile.
+     */
+    arguments = getarguments (sp, n, vec, 1);
     argp = arguments;
 
     debugsw = 0;
@@ -734,6 +754,7 @@ sendit (char *sp, char **arg, char *file, int pushed)
     annotext = NULL;
     distfile = NULL;
 
+    vecp = 1;			/* we'll get the zero'th element later */
     vec[vecp++] = "-library";
     vec[vecp++] = getcpy (m_maildir (""));
 
@@ -748,7 +769,7 @@ sendit (char *sp, char **arg, char *file, int pushed)
 		    return;
 
 		case SHELPSW: 
-		    sprintf (buf, "%s [switches]", sp);
+		    snprintf (buf, sizeof(buf), "%s [switches]", sp);
 		    print_help (buf, sendswitches, 1);
 		    return;
 		case SVERSIONSW:
@@ -841,9 +862,10 @@ sendit (char *sp, char **arg, char *file, int pushed)
 
     /* allow Aliasfile: profile entry */
     if ((cp = context_find ("Aliasfile"))) {
-	char *dp = NULL;
+	char **ap, *dp;
 
-	for (ap = brkstring(dp = getcpy(cp), " ", "\n"); ap && *ap; ap++) {
+	dp = getcpy (cp);
+	for (ap = brkstring (dp, " ", "\n"); ap && *ap; ap++) {
 	    vec[vecp++] = "-alias";
 	    vec[vecp++] = *ap;
 	}
@@ -854,7 +876,7 @@ sendit (char *sp, char **arg, char *file, int pushed)
 	    m_putenv ("SIGNATURE", cp);
 #ifdef UCI
 	else {
-	    sprintf (buf, "%s/.signature", mypath);
+	    snprintf (buf, sizeof(buf), "%s/.signature", mypath);
 	    if ((fp = fopen (buf, "r")) != NULL
 		&& fgets (buf, sizeof(buf), fp) != NULL) {
 		    fclose (fp);
@@ -882,12 +904,15 @@ sendit (char *sp, char **arg, char *file, int pushed)
 	distfile = getcpy (m_scratch (altmsg, invo_name));
 	if (link (altmsg, distfile) == NOTOK)
 	    adios (distfile, "unable to link %s to", altmsg);
-    }
-    else
+    } else {
 	distfile = NULL;
+    }
 
-    if (altmsg == NULL || stat (altmsg, &st) == NOTOK)
-	st.st_mtime = 0, st.st_dev = 0, st.st_ino = 0;
+    if (altmsg == NULL || stat (altmsg, &st) == NOTOK) {
+	st.st_mtime = 0;
+	st.st_dev = 0;
+	st.st_ino = 0;
+    }
     if ((pushsw = pushed))
 	push ();
 
