@@ -2,7 +2,7 @@
 /*
  * whom.c -- report to whom a message would be sent
  *
- * $Id$
+ * $Id: whom.c,v 1.2 1999/07/16 01:25:22 danh Exp $
  */
 
 #include <h/mh.h>
@@ -46,19 +46,17 @@ main (int argc, char **argv)
     int distsw = 0, vecp = 0;
     char *cp, *dfolder = NULL, *dmsg = NULL;
     char *msg = NULL, **ap, **argp, backup[BUFSIZ];
-    char buf[100], *arguments[MAXARGS], *vec[MAXARGS];
+    char buf[BUFSIZ], **arguments, *vec[MAXARGS];
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     vec[vecp++] = invo_name;
@@ -76,7 +74,7 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [switches] [file]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches] [file]", invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -187,5 +185,5 @@ main (int argc, char **argv)
 	    done (status);
     }
 
-    exit (-1);	/* NOT REACHED */
+    return 0;  /* dead code to satisfy the compiler */
 }
