@@ -2,7 +2,7 @@
 /*
  * wmh.c -- window front-end to nmh
  *
- * $Id$
+ * $Id: wmh.c,v 1.2 1999/07/16 01:21:39 danh Exp $
  */
 
 /*
@@ -139,20 +139,18 @@ int
 main (int argc, char **argv)
 {
     int vecp = 1, nprog = 0;
-    char *cp, buffer[BUFSIZ], **ap, **argp;
-    char *arguments[MAXARGS], *vec[MAXARGS];
+    char *cp, buffer[BUFSIZ], **argp;
+    char **arguments, *vec[MAXARGS];
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name)) != NULL) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    }
-    else
-	ap = arguments;
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc,argv, 1);
     argp = arguments;
 
     while ((cp = *argp++))
@@ -166,7 +164,8 @@ main (int argc, char **argv)
 		    continue;
 
 		case HELPSW: 
-		    sprintf (buffer, "%s [switches for vmhproc]", invo_name);
+		    snprintf (buffer, sizeof(buffer), "%s [switches for vmhproc]",
+			invo_name);
 		    print_help (buffer, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -201,7 +200,7 @@ main (int argc, char **argv)
 
     vmh ();
 
-    done (0);
+    return done (0);
 }
 
 
@@ -213,7 +212,7 @@ vmh (void)
     for (;;) {
 	pLOOP (RC_QRY, NULL);
 
-	sprintf (prompt, myprompt, invo_name);
+	snprintf (prompt, sizeof(prompt), myprompt, invo_name);
 
 	switch (WINgetstr (Command, prompt, buffer)) {
 	    case NOTOK: 
@@ -255,10 +254,10 @@ PEERinit (int vecp, char *vec[])
 	    close (pfd1[1]);
 
 	    vec[vecp++] = "-vmhread";
-	    sprintf (buf1, "%d", pfd1[0]);
+	    snprintf (buf1, sizeof(buf1), "%d", pfd1[0]);
 	    vec[vecp++] = buf1;
 	    vec[vecp++] = "-vmhwrite";
-	    sprintf (buf2, "%d", pfd0[1]);
+	    snprintf (buf2, sizeof(buf2), "%d", pfd0[1]);
 	    vec[vecp++] = buf2;
 	    vec[vecp] = NULL;
 
@@ -284,20 +283,28 @@ PEERinit (int vecp, char *vec[])
 static int
 pINI (void)
 {
-    register char *bp;
-    char buffer[BUFSIZ];
+    int len, buflen;
+    char *bp, buffer[BUFSIZ];
     struct record rcs, *rc;
     WINDOW **w;
 
     rc = &rcs;
     initrc (rc);
 
+    /* Get buffer ready to go */
     bp = buffer;
-    sprintf (bp, "%d %d", RC_VRSN, numwins);
-    bp += strlen (bp);
+    buflen = sizeof(buffer);
+
+    snprintf (bp, buflen, "%d %d", RC_VRSN, numwins);
+    len = strlen (bp);
+    bp += len;
+    buflen -= len;
+
     for (w = windows; *w; w++) {
-	sprintf (bp, " %d", (*w)->w_height);
-	bp += strlen (bp);
+	snprintf (bp, buflen, " %d", (*w)->w_height);
+	len = strlen (bp);
+	bp += len;
+	buflen -= len;
     }
 
     switch (str2rc (RC_INI, buffer, rc)) {
@@ -666,7 +673,7 @@ WINgetstr (WINDOW *w, char *prompt, char *buffer)
     if (ChangeWindowDepth (dfd, w->w_wd, 0) == NOTOK)
 	adios ("failed", "ChangeWindowDepth");
 
-    strcpy (image, prompt);
+    strncpy (image, prompt, sizeof(image));
     bp = ip = image + strlen (image);
 
     Redisplay (w, 0);
@@ -971,8 +978,8 @@ WINless (WINDOW *w)
 				break;
 
 			    case 7: /* Goto ... */
-				sprintf (mylineno, "%d",
-					w->w_top->l_no);
+				snprintf (mylineno, sizeof(mylineno),
+					"%d", w->w_top->l_no);
 				cancel[0] = 0;
 				if (PresentMenu (&mymenu, myanswers)
 					|| cancel[0])
@@ -1278,7 +1285,7 @@ SIGser (int sig)
 }
 
 
-void
+int
 done (int status)
 {
     if (dfd != NOTOK)
@@ -1287,6 +1294,7 @@ done (int status)
     pFIN ();
 
     exit (status);
+    return 1;  /* dead code to satisfy the compiler */
 }
 
 
@@ -1325,7 +1333,7 @@ advertise (char *what, char *tail, va_list ap)
 	iov++;
     }
     
-    vsprintf (buffer, fmt, ap);
+    vsnprintf (buffer, sizeof(buffer), fmt, ap);
     iov->iov_len = strlen (iov->iov_base = buffer);
     iov++;
     if (what) {
@@ -1338,7 +1346,7 @@ advertise (char *what, char *tail, va_list ap)
 	    iov++;
 	}
 	if (!(iov->iov_base = strerror (eindex))) {
-	    sprintf (err, "unknown error %d", eindex);
+	    snprintf (err, sizeof(err), "unknown error %d", eindex);
 	    iov->iov_base = err;
 	}
 	iov->iov_len = strlen (iov->iov_base);
@@ -1368,8 +1376,8 @@ DisplayVector (struct iovec *iov, int n)
     char buffer[BUFSIZ];
 
     for (i = 0, cp = NULL; i < n; i++, iov++) {
-	sprintf (buffer, "%*.*s", iov->iov_len, iov->iov_len,
-		iov->iov_base);
+	snprintf (buffer, sizeof(buffer), "%*.*s", iov->iov_len,
+		iov->iov_len, iov->iov_base);
 	cp = add (buffer, cp);
     }
 
