@@ -2,7 +2,7 @@
 /*
  * getbbent.c -- subroutines for accessing the BBoards file
  *
- * $Id$
+ * $Id: getbbent.c,v 1.2 1999/07/16 01:18:12 danh Exp $
  */
 
 #include <h/nmh.h>
@@ -100,7 +100,7 @@ setbbfile (char *file, int f)
     if (BBuid == -1)
 	return setbbinfo (BBOARDS, file, f);
 
-    strcpy (BBData, file);
+    strncpy (BBData, file, sizeof(BBData));
 
     BBflags = SB_NULL;
     endbbent ();
@@ -115,7 +115,7 @@ setbbinfo (char *user, char *file, int f)
     register struct passwd *pw;
 
     if ((pw = getpwnam (user)) == NULL) {
-	sprintf (BBErrors, "unknown user: %s", user);
+	snprintf (BBErrors, sizeof(BBErrors), "unknown user: %s", user);
 	return 0;
     }
 
@@ -142,7 +142,7 @@ setbbaux (char *name, char *file)
     register struct passwd *pw;
 
     if ((pw = getpwnam (name)) == NULL) {
-	sprintf (BBErrors, "unknown user: %s", name);
+	snprintf (BBErrors, sizeof(BBErrors), "unknown user: %s", name);
 	return 0;
     }
 
@@ -153,10 +153,10 @@ setbbaux (char *name, char *file)
 static int
 setpwaux (struct passwd *pw, char *file)
 {
-    strcpy (BBName, pw->pw_name);
+    strncpy (BBName, pw->pw_name, sizeof(BBName));
     BBuid = pw->pw_uid;
-    strcpy (BBDir, pw->pw_dir);
-    sprintf (BBData, "%s/%s",
+    strncpy (BBDir, pw->pw_dir, sizeof(BBDir));
+    snprintf (BBData, sizeof(BBData), "%s/%s",
 	    *file != '/' ? BBDir : "",
 	    *file != '/' ? file : file + 1);
 
@@ -174,7 +174,7 @@ setbbent (int f)
 	    return 0;
 
 	if ((BBfile = fopen (BBData, "r")) == NULL) {
-	    sprintf (BBErrors, "unable to open: %s", BBData);
+	    snprintf (BBErrors, sizeof(BBErrors), "unable to open: %s", BBData);
 	    return 0;
 	}
     }
@@ -208,15 +208,15 @@ getbbtime (void)
 	    return 0;
 
 	if (stat (BBData, &st) == NOTOK) {
-	    sprintf (BBErrors, "unable to stat: %s", BBData);
+	    snprintf (BBErrors, sizeof(BBErrors), "unable to stat: %s", BBData);
+	    return 0;
+	}
+    } else {
+	if (fstat (fileno (BBfile), &st) == NOTOK) {
+	    snprintf (BBErrors, sizeof(BBErrors), "unable to fstat: %s", BBData);
 	    return 0;
 	}
     }
-    else
-	if (fstat (fileno (BBfile), &st) == NOTOK) {
-	    sprintf (BBErrors, "unable to fstat: %s", BBData);
-	    return 0;
-	}
 
     return ((long) st.st_mtime);
 }
@@ -393,14 +393,15 @@ BBread (void)
     r = strchr(bb->bb_addr, '@');
     BBRequest[0] = 0;
 
-    if (*bb->bb_request == '-')
+    if (*bb->bb_request == '-') {
 	if (p == NULL && r && *r == '@')
-	    sprintf (BBRequest, "%s%s%s", bb->bb_name, bb->bb_request, r);
+	    snprintf (BBRequest, sizeof(BBRequest), "%s%s%s", bb->bb_name, bb->bb_request, r);
 	else
-	    sprintf (BBRequest, "%s%s", bb->bb_name, bb->bb_request);
+	    snprintf (BBRequest, sizeof(BBRequest), "%s%s", bb->bb_name, bb->bb_request);
+    }
     else
 	if (p == NULL && r && *r == '@' && *bb->bb_request)
-	    sprintf (BBRequest, "%s%s", bb->bb_request, r);
+	    snprintf (BBRequest, sizeof(BBRequest), "%s%s", bb->bb_request, r);
 
     if (BBRequest[0])
 	bb->bb_request = BBRequest;
@@ -410,7 +411,7 @@ BBread (void)
 		: bb->bb_leader[0];
 
     if (*bb->bb_addr == '@') {
-	sprintf (BBAddr, "%s%s", bb->bb_name, bb->bb_addr);
+	snprintf (BBAddr, sizeof(BBAddr), "%s%s", bb->bb_name, bb->bb_addr);
 	bb->bb_addr = BBAddr;
     }
     else
@@ -420,22 +421,24 @@ BBread (void)
     if (*bb->bb_file == 0)
 	return;
     if (*bb->bb_file != '/') {
-	sprintf (BBFile, "%s/%s", BBDir, bb->bb_file);
+	snprintf (BBFile, sizeof(BBFile), "%s/%s", BBDir, bb->bb_file);
 	bb->bb_file = BBFile;
     }
 
-    if ((cp = strrchr(bb->bb_file, '/')) == NULL || *++cp == 0)
-	strcpy (prf, ""), cp = bb->bb_file;
-    else
-	sprintf (prf, "%.*s", cp - bb->bb_file, bb->bb_file);
+    if ((cp = strrchr(bb->bb_file, '/')) == NULL || *++cp == 0) {
+	strcpy (prf, "");
+	cp = bb->bb_file;
+    } else {
+	snprintf (prf, sizeof(prf), "%.*s", cp - bb->bb_file, bb->bb_file);
+    }
     if ((dp = strchr(cp, '.')) == NULL)
 	dp = cp + strlen (cp);
 
-    sprintf (BBArchive, "%s%s/%s", prf, ARCHIVE, cp);
+    snprintf (BBArchive, sizeof(BBArchive), "%s%s/%s", prf, ARCHIVE, cp);
     bb->bb_archive = BBArchive;
-    sprintf (BBInfo, "%s.%.*s%s", prf, dp - cp, cp, CNTFILE);
+    snprintf (BBInfo, sizeof(BBInfo), "%s.%.*s%s", prf, dp - cp, cp, CNTFILE);
     bb->bb_info = BBInfo;
-    sprintf (BBMap, "%s.%.*s%s", prf, dp - cp, cp, MAPFILE);
+    snprintf (BBMap, sizeof(BBMap), "%s.%.*s%s", prf, dp - cp, cp, MAPFILE);
     bb->bb_map = BBMap;
 
     if ((info = fopen (bb->bb_info, "r")) == NULL)
@@ -444,7 +447,7 @@ BBread (void)
     if (fgets (line, sizeof line, info) && (i = atoi (line)) > 0)
 	bb->bb_maxima = (unsigned) i;
     if (!feof (info) && fgets (line, sizeof line, info)) {
-	strcpy (BBDate, line);
+	strncpy (BBDate, line, sizeof(BBData));
 	if ((cp = strchr(BBDate, NEWLINE)))
 	    *cp = 0;
 	bb->bb_date = BBDate;
@@ -473,7 +476,7 @@ ldrbb (struct bboard *b)
 	if ((pw = getpwuid (uid = getuid ())) == NULL)
 	    return 0;
 	gid = getgid ();
-	strcpy (username, pw->pw_name);
+	strncpy (username, pw->pw_name, sizeof(username));
     }
 
     if (uid == BBuid)
@@ -623,18 +626,20 @@ getbbitem (struct bboard *bb, char *item, int (*action)())
 		    break;
 
 		case 0: 
-		    if ((cp = strrchr(bb->bb_file, '/')) == NULL || *++cp == 0)
-			strcpy (prf, ""), cp = bb->bb_file;
-		    else
-			sprintf (prf, "%.*s", cp - bb->bb_file, bb->bb_file);
+		    if ((cp = strrchr(bb->bb_file, '/')) == NULL || *++cp == 0) {
+			strcpy (prf, "");
+			cp = bb->bb_file;
+		    } else {
+			snprintf (prf, sizeof(prf), "%.*s", cp - bb->bb_file, bb->bb_file);
+		    }
 		    if ((dp = strchr(cp, '.')) == NULL)
 			dp = cp + strlen (cp);
-		    sprintf (file, "%s.%.*s%s", prf, dp - cp, cp, DSTFILE);
+		    snprintf (file, sizeof(file), "%s.%.*s%s", prf, dp - cp, cp, DSTFILE);
 		    hp = file;
 		    break;
 
 		default: 
-		    sprintf (file, "%s/%s", BBDir, item);
+		    snprintf (file, sizeof(file), "%s/%s", BBDir, item);
 		    hp = file;
 		    break;
 	    }
@@ -656,13 +661,13 @@ getbbitem (struct bboard *bb, char *item, int (*action)())
 	default: 
 	    if ((hp = strrchr(item, '@'))) {
 		*hp++ = 0;
-		strcpy (mbox, item);
-		strcpy (host, hp);
+		strncpy (mbox, item, sizeof(mbox));
+		strncpy (host, hp, sizeof(host));
 		*--hp = '@';
 	    }
 	    else {
-		sprintf (mbox, "%s%s", DISTADR, bb->bb_name);
-		strcpy (host, item);
+		snprintf (mbox, sizeof(mbox), "%s%s", DISTADR, bb->bb_name);
+		strncpy (host, item, sizeof(host));
 	    }
 	    if ((result = (*action) (mbox, host)))
 		bblose ("action (%s, %s) returned 0%o", mbox, host, result);
@@ -678,7 +683,7 @@ bblose (char *fmt, ...)
 
     va_start(ap, fmt);
     if (BBErrors[0] == 0)
-	vsprintf (BBErrors, fmt, ap);
+	vsnprintf (BBErrors, sizeof(BBErrors), fmt, ap);
 
     va_end(ap);
     return NOTOK;
