@@ -2,7 +2,7 @@
 /*
  * mf.c -- mail filter subroutines
  *
- * $Id$
+ * $Id: mf.c,v 1.3 1999/09/12 06:44:02 doug Exp $
  */
 
 #include <mf.h>
@@ -31,7 +31,9 @@ getcpy (char *s)
     register char *p;
 
     if (!s) {
-	_cleanup();
+/* causes compiles to blow up because the symbol _cleanup is undefined 
+   where did this ever come from? */
+	/* _cleanup(); */
 	abort();
 	for(;;)
 	    pause();
@@ -71,8 +73,8 @@ lequal (char *a, char *b)
 	if (*b == 0)
 	    return FALSE;
 	else {
-	    char    c1 = islower (*a) ? toupper (*a) : *a;
-	    char    c2 = islower (*b) ? toupper (*b) : *b;
+	    char c1 = islower (*a) ? toupper (*a) : *a;
+	    char c2 = islower (*b) ? toupper (*b) : *b;
 	    if (c1 != c2)
 		return FALSE;
 	}
@@ -97,14 +99,14 @@ static char *punctuators = ";<>.()[]";
 static char *vp = NULL;
 static char *tp = NULL;
 
-static struct adrx  adrxs1;
+static struct adrx adrxs1;
 
 
 struct adrx *
 seekadrx (char *addrs)
 {
     static int state = CHKADR;
-    register char   *cp;
+    register char *cp;
     register struct adrx *adrxp;
 
     if (state == CHKADR)
@@ -163,7 +165,7 @@ uucpadrx (char *addrs)
 	return NULL;
     }
 
-    if ((wp = strchr(cp, ',')) == NULL)
+    if ((wp = strchr(cp, ',')) == NULL) {
 	if ((wp = strchr(cp, ' ')) != NULL) {
 	    xp = wp;
 	    while (isspace (*xp))
@@ -172,11 +174,12 @@ uucpadrx (char *addrs)
 		yp = xp + 4;
 		while (isspace (*yp))
 		    yp++;
-		if (*yp != 0)
+		if (*yp != 0) {
 		    if ((zp = strchr(yp, ' ')) != NULL)
 			*zp = 0, tp = ++zp;
 		    else
 			tp = NULL;
+		}
 		else
 		    *wp = 0, tp = ++wp;
 	    }
@@ -185,6 +188,7 @@ uucpadrx (char *addrs)
 	}
 	else
 	    tp = NULL;
+    }
     else
 	*wp = 0, tp = ++wp;
 
@@ -297,46 +301,46 @@ isat (char *p)
  *
  */
 
-
 #define	QUOTE	'\\'
 
-#define	LX_END	0
-#define	LX_ERR	1
-#define	LX_ATOM	2
-#define	LX_QSTR	3
-#define	LX_DLIT	4
-#define	LX_SEMI	5
-#define	LX_COMA	6
-#define	LX_LBRK	7
-#define	LX_RBRK	8
-#define	LX_COLN	9
+#define	LX_END	 0
+#define	LX_ERR	 1
+#define	LX_ATOM	 2
+#define	LX_QSTR	 3
+#define	LX_DLIT	 4
+#define	LX_SEMI	 5
+#define	LX_COMA	 6
+#define	LX_LBRK	 7
+#define	LX_RBRK	 8
+#define	LX_COLN	 9
 #define	LX_DOT	10
 #define	LX_AT	11
 
-static struct {
-    char    lx_chr;
-    int     lx_val;
-}               special[] = {
-                    ';', LX_SEMI,
-                    ',', LX_COMA,
-                    '<', LX_LBRK,
-                    '>', LX_RBRK,
-                    ':', LX_COLN,
-                    '.', LX_DOT,
-                    '@', LX_AT,
-                    '(', LX_ERR,
-                    ')', LX_ERR,
-                    QUOTE, LX_ERR,
-                    '"', LX_ERR,
-                    '[', LX_ERR,
-                    ']', LX_ERR,
-                    0, 0
+struct specials {
+    char lx_chr;
+    int  lx_val;
 };
 
+static struct specials special[] = {
+    { ';',   LX_SEMI },
+    { ',',   LX_COMA },
+    { '<',   LX_LBRK },
+    { '>',   LX_RBRK },
+    { ':',   LX_COLN },
+    { '.',   LX_DOT },
+    { '@',   LX_AT },
+    { '(',   LX_ERR },
+    { ')',   LX_ERR },
+    { QUOTE, LX_ERR },
+    { '"',   LX_ERR },
+    { '[',   LX_ERR },
+    { ']',   LX_ERR },
+    { 0,     0 }
+};
 
-static int  glevel = 0;
-static int  ingrp = 0;
-static int  last_lex = LX_END;
+static int glevel = 0;
+static int ingrp = 0;
+static int last_lex = LX_END;
 
 static char *dp = NULL;
 static char *cp = NULL;
@@ -356,7 +360,7 @@ static struct adrx  adrxs2;
 struct adrx *
 getadrx (char *addrs)
 {
-    register char   *bp;
+    register char *bp;
     register struct adrx *adrxp = &adrxs2;
 
     if (pers)
@@ -449,7 +453,7 @@ getadrx (char *addrs)
 static int
 parse_address (void)
 {
-    char    buffer[BUFSIZ];
+    char buffer[BUFSIZ];
 
 again: ;
     ap = cp;
@@ -616,7 +620,7 @@ phrase (char *buffer)
 static int
 route_addr (char *buffer)
 {
-    register char   *pp = cp;
+    register char *pp = cp;
 
     if (my_lex (buffer) == LX_AT) {
 	if (route (buffer) == NOTOK)
@@ -885,8 +889,8 @@ got_atom: ;
 char *
 legal_person (char *p)
 {
-    int     i;
-    register char   *cp;
+    int i;
+    register char *cp;
     static char buffer[BUFSIZ];
 
     if (*p == '"')
@@ -951,7 +955,7 @@ mfgets (FILE *in, char **bp)
 		*cp++ = i;
 		break;
 	}
-	if (cp >= ep)
+	if (cp >= ep) {
 	    if (!(dp = realloc (pp, (size_t) (len += BUFSIZ)))) {
 		free (pp);
 		pp = NULL;
@@ -959,5 +963,6 @@ mfgets (FILE *in, char **bp)
 	    }
 	    else
 		cp += dp - pp, ep = (pp = cp) + len - 2;
+	}
     }
 }
