@@ -2,7 +2,7 @@
 /*
  * client.c -- connect to a server
  *
- * $Id$
+ * $Id: client.c,v 1.3 1999/10/15 22:38:56 doug Exp $
  */
 
 #include <h/mh.h>
@@ -35,8 +35,6 @@
 #define	MAXNETS      5
 #define	MAXHOSTS    25
 
-extern int errno;
-
 struct addrent {
     int a_addrtype;		/* assumes AF_INET for inet_netof () */
     union {
@@ -63,7 +61,7 @@ char krb_realm[REALM_SZ];
 char *PrincipalHostname();
 #endif /* KPOP */
 
-#if defined(BIND) && !defined(h_addr)
+#if !defined(h_addr)
 # define h_addr h_addr_list[0]
 #endif
 
@@ -73,60 +71,66 @@ char *PrincipalHostname();
 /*
  * static prototypes
  */
-static int rcaux (struct servent *, struct hostent *, int, char *);
-static int getport (int, int, char *);
+static int rcaux (struct servent *, struct hostent *, int, char *, int);
+static int getport (int, int, char *, int);
 static int inet (struct hostent *, int);
 struct hostent *gethostbystring (char *s);
 
 /* client's own static version of several nmh subroutines */
 static char **client_brkstring (char *, char *, char *);
 static int client_brkany (char, char *);
-static char **client_copyip (char **, char **);
+static char **client_copyip (char **, char **, int);
 static char *client_getcpy (char *);
 
 
 int
-client (char *args, char *protocol, char *service, int rproto, char *response)
+client (char *args, char *protocol, char *service, int rproto,
+		char *response, int len_response)
 {
     int sd;
     register char **ap;
     char *arguments[MAXARGS];
     register struct hostent *hp;
     register struct servent *sp;
-#ifndef	BIND
+/* we assume netent and getnetbyaddr come with gethostbyname */
+#ifdef	HAVE_GETHOSTBYNAME
     register struct netent *np;
 #endif
 
 #ifdef KPOP
     char *cp;
 
-    if (cp = strchr (kservice = service, '/')) {	/* "pop/kpop" */
+    kservice = service;
+    if (cp = strchr (service, '/')) {	/* "pop/kpop" */
 	*cp++ = '\0';		/* kservice = "pop" */
 	service = cp;		/* service  = "kpop" */
-    }
-    else
+    } else {
 	kservice = NULL;	/* not using KERBEROS */
+    }
 #endif	/* KPOP */
     
 
     if ((sp = getservbyname (service, protocol)) == NULL) {
 #ifdef HESIOD
 	if ((sp = hes_getservbyname (service, protocol)) == NULL) {
-	    sprintf (response, "%s/%s: unknown service", protocol, service);
+	    snprintf (response, len_response, "%s/%s: unknown service", protocol, service);
 	    return NOTOK;
 	}
 #else
-	sprintf (response, "%s/%s: unknown service", protocol, service);
+	snprintf (response, len_response, "%s/%s: unknown service", protocol, service);
 	return NOTOK;
 #endif
     }
 
     ap = arguments;
-    if (args != NULL && *args != 0)
-	ap = client_copyip (client_brkstring (client_getcpy (args), " ", "\n"), ap);
-    else
+    if (args != NULL && *args != 0) {
+	ap = client_copyip (client_brkstring (client_getcpy (args), " ", "\n"),
+		ap, MAXARGS);
+    } else {
 	if (servers != NULL && *servers != 0)
-	    ap = client_copyip (client_brkstring (client_getcpy (servers), " ", "\n"), ap);
+	    ap = client_copyip (client_brkstring (client_getcpy (servers), " ", "\n"),
+		ap, MAXARGS);
+    }
     if (ap == arguments) {
 	*ap++ = client_getcpy ("localhost");
 	*ap = NULL;
@@ -140,13 +144,22 @@ client (char *args, char *protocol, char *service, int rproto, char *response)
 
     for (ap = arguments; *ap; ap++) {
 	if (**ap == '\01') {
-#ifndef	BIND
+/*
+ * the assumption here is that if the system doesn't have a
+ * gethostbyname() function, it must not use DNS. So we need to look
+ * into the /etc/hosts using gethostent(). There probablly aren't any
+ * systems still like this, but you never know. On every system I have
+ * access to, this section is ignored.
+ */
+#ifndef	HAVE_GETHOSTBYNAME
 	    if ((np = getnetbyname (*ap + 1))) {
+#ifdef HAVE_SETHOSTENT
 		sethostent (1);
+#endif /* HAVE_SETHOSTENT */
 		while ((hp = gethostent()))
 		    if (np->n_addrtype == hp->h_addrtype
 			    && inet (hp, np->n_net)) {
-			switch (sd = rcaux (sp, hp, rproto, response)) {
+			switch (sd = rcaux (sp, hp, rproto, response, len_response)) {
 			    case NOTOK: 
 				continue;
 			    case OOPS1: 
@@ -160,12 +173,12 @@ client (char *args, char *protocol, char *service, int rproto, char *response)
 			break;
 		    }
 	    }
-#endif
+#endif /* don't HAVE_GETHOSTBYNAME */
 	    continue;
 	}
 
 	if ((hp = gethostbystring (*ap))) {
-	    switch (sd = rcaux (sp, hp, rproto, response)) {
+	    switch (sd = rcaux (sp, hp, rproto, response, len_response)) {
 		case NOTOK: 
 		case OOPS1: 
 		    break;
@@ -179,13 +192,14 @@ client (char *args, char *protocol, char *service, int rproto, char *response)
 	}
     }
 
-    strcpy (response, "no servers available");
+    strncpy (response, "no servers available", len_response);
     return NOTOK;
 }
 
 
 static int
-rcaux (struct servent *sp, struct hostent *hp, int rproto, char *response)
+rcaux (struct servent *sp, struct hostent *hp, int rproto,
+		char *response, int len_response)
 {
     int sd;
     struct in_addr in;
@@ -206,10 +220,10 @@ rcaux (struct servent *sp, struct hostent *hp, int rproto, char *response)
 		&& memcmp(ap->a_addr, hp->h_addr, hp->h_length) == 0)
 	    return NOTOK;
 
-    if ((sd = getport (rproto, hp->h_addrtype, response)) == NOTOK)
+    if ((sd = getport (rproto, hp->h_addrtype, response, len_response)) == NOTOK)
 	return OOPS2;
 
-    memset(isock, 0, sizeof(*isock));
+    memset (isock, 0, sizeof(*isock));
     isock->sin_family = hp->h_addrtype;
     inaddr_copy (hp, isock);
     isock->sin_port = sp->s_port;
@@ -241,17 +255,25 @@ rcaux (struct servent *sp, struct hostent *hp, int rproto, char *response)
 
 #ifdef KPOP
     if (kservice) {	/* "pop" */
-	ticket = (KTEXT)malloc( sizeof(KTEXT_ST) );
-	rem = krb_sendauth(0L, sd, ticket, kservice, hp->h_name,
-			   (char *) krb_realmofhost(hp->h_name),
-			   (unsigned long)0, &msg_data, &cred, schedule,
-			   (struct sockaddr_in *)NULL,
-			   (struct sockaddr_in *)NULL,
+	char *instance;
+
+	if ((instance = strdup (hp->h_name)) == NULL) {
+	    close (sd);
+	    strncpy (response, "Out of memory.", len_response);
+	    return OOPS2;
+	}
+	ticket = (KTEXT) malloc (sizeof(KTEXT_ST));
+	rem = krb_sendauth (0L, sd, ticket, kservice, instance,
+			   (char *) krb_realmofhost (instance),
+			   (unsigned long) 0, &msg_data, &cred, schedule,
+			   (struct sockaddr_in *) NULL,
+			   (struct sockaddr_in *) NULL,
 			   "KPOPV0.1");
+	free (instance);
 	if (rem != KSUCCESS) {
-	    close(sd);
-	    strcpy(response, "Post office refused connection: ");
-	    strcat(response, krb_err_txt[rem]);
+	    close (sd);
+	    strncpy (response, "Post office refused connection: ", len_response);
+	    strncat (response, krb_err_txt[rem], len_response - strlen(response));
 	    return OOPS2;
 	}
     }
@@ -262,14 +284,14 @@ rcaux (struct servent *sp, struct hostent *hp, int rproto, char *response)
 
 
 static int
-getport (int rproto, int addrtype, char *response)
+getport (int rproto, int addrtype, char *response, int len_response)
 {
     int sd, port;
     struct sockaddr_in in_socket, *isock;
 
     isock = &in_socket;
     if (rproto && addrtype != AF_INET) {
-	sprintf (response, "reserved ports not supported for af=%d", addrtype);
+	snprintf (response, len_response, "reserved ports not supported for af=%d", addrtype);
 	errno = ENOPROTOOPT;
 	return NOTOK;
     }
@@ -278,9 +300,9 @@ getport (int rproto, int addrtype, char *response)
 	char *s;
 
 	if ((s = strerror (errno)))
-	    sprintf (response, "unable to create socket: %s", s);
+	    snprintf (response, len_response, "unable to create socket: %s", s);
 	else
-	    sprintf (response, "unable to create socket: unknown error");
+	    snprintf (response, len_response, "unable to create socket: unknown error");
 	return NOTOK;
     }
 #ifdef KPOP
@@ -303,16 +325,16 @@ getport (int rproto, int addrtype, char *response)
 	    case EADDRINUSE: 
 	    case EADDRNOTAVAIL: 
 		if (--port <= IPPORT_RESERVED / 2) {
-		    strcpy (response, "ports available");
+		    strncpy (response, "ports available", len_response);
 		    return NOTOK;
 		}
 		break;
 
 	    default: 
 		if ((s = strerror (errno)))
-		    sprintf (response, "unable to bind socket: %s", s);
+		    snprintf (response, len_response, "unable to bind socket: %s", s);
 		else
-		    sprintf (response, "unable to bind socket: unknown error");
+		    snprintf (response, len_response, "unable to bind socket: unknown error");
 		return NOTOK;
 	}
     }
@@ -426,12 +448,13 @@ client_brkany (char chr, char *strg)
  * copy a string array and return pointer to end
  */
 static char **
-client_copyip (char **p, char **q)
+client_copyip (char **p, char **q, int len_q)
 {
-    while (*p)
+    while (*p && --len_q > 0)
 	*q++ = *p++;
-    *q = 0;
- 
+
+    *q = NULL;
+
     return q;
 }
 
