@@ -2,7 +2,7 @@
 /*
  * mts.c -- definitions for the mail transport system
  *
- * $Id$
+ * $Id: mts.c,v 1.2 1999/10/15 22:38:56 doug Exp $
  */
 
 #include <h/nmh.h>
@@ -275,22 +275,22 @@ LocalName (void)
 
     /* check if the mts.conf file specifies a "localname" */
     if (*localname) {
-	strcpy (buffer, localname);
+	strncpy (buffer, localname, sizeof(buffer));
     } else {
 #ifdef HAVE_UNAME
 	/* first get our local name */
 	uname (&name);
-	strcpy (buffer, name.nodename);
+	strncpy (buffer, name.nodename, sizeof(buffer));
 #else
 	/* first get our local name */
 	gethostname (buffer, sizeof(buffer));
 #endif
-#ifndef BIND
+#ifdef HAVE_SETHOSTENT
 	sethostent (1);
-#endif
+#endif 
 	/* now fully qualify our name */
 	if ((hp = gethostbyname (buffer)))
-	    strcpy (buffer, hp->h_name);
+	    strncpy (buffer, hp->h_name, sizeof(buffer));
     }
 
     /*
@@ -328,13 +328,13 @@ SystemName (void)
 
     /* check if mts.conf file specifies a "systemname" */
     if (*systemname) {
-	strcpy (buffer, systemname);
+	strncpy (buffer, systemname, sizeof(buffer));
 	return buffer;
     }
 
 #ifdef HAVE_UNAME
     uname (&name);
-    strcpy (buffer, name.nodename);
+    strncpy (buffer, name.nodename, sizeof(buffer));
 #else
     gethostname (buffer, sizeof(buffer));
 #endif
@@ -389,7 +389,7 @@ getuserinfo (void)
     uid = getuid ();
     if (uid == geteuid () && (cp = getenv ("USER")) != NULL
 	&& (pw = getpwnam (cp)) != NULL)
-      strcpy (username, cp);
+      strncpy (username, cp, sizeof(username));
     else if ((pw = getpwuid (uid)) == NULL
 	     || pw->pw_name == NULL
 	     || *pw->pw_name == '\0') {
@@ -399,8 +399,9 @@ getuserinfo (void)
 	    || *pw->pw_name == '\0') {
 #endif /* KPOP */
 
-	strcpy (username, "unknown");
-	sprintf (fullname, "The Unknown User-ID (%d)", (int) getuid ());
+	strncpy (username, "unknown", sizeof(username));
+	snprintf (fullname, sizeof(fullname), "The Unknown User-ID (%d)",
+		(int) getuid ());
 	return;
     }
 
@@ -416,7 +417,7 @@ getuserinfo (void)
 #else
     for (cp = fullname; *np && *np != (MMailids ? '<' : ','); ) {
 	if (*np == '&')	{	/* blech! */
-	    strcpy(cp, pw->pw_name);
+	    strcpy (cp, pw->pw_name);
 	    *cp = toupper(*cp);
 	    while (*cp)
 		cp++;
@@ -436,17 +437,17 @@ getuserinfo (void)
 	*cp = '\0';
     }
     if (MMailids == 0 || *np == '\0')
-	strcpy (username, pw->pw_name);
+	strncpy (username, pw->pw_name, sizeof(username));
 
     if ((cp = getenv ("SIGNATURE")) && *cp)
-	strcpy (fullname, cp);
+	strncpy (fullname, cp, sizeof(fullname));
 
     if (strchr(fullname, '.')) {		/*  quote any .'s */
 	char tmp[BUFSIZ];
 
 	/* should quote "'s too */
-	sprintf (tmp, "\"%s\"", fullname);
-	strcpy (fullname, tmp);
+	snprintf (tmp, sizeof(tmp), "\"%s\"", fullname);
+	strncpy (fullname, tmp, sizeof(fullname));
     }
 
     return;
