@@ -2,7 +2,7 @@
 /*
  * mts.h -- definitions for the mail system
  *
- * $Id$
+ * $Id: mts.h,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 /*
@@ -20,11 +20,11 @@ extern char *uucpldir;
 extern char *uucplfil;
 
 #define	MAILDIR	(mmdfldir && *mmdfldir ? mmdfldir : getenv ("HOME"))
-#define	MAILFIL	(mmdflfil && *mmdflfil ? mmdflfil : getusr ())
+#define	MAILFIL	(mmdflfil && *mmdflfil ? mmdflfil : getusername ())
 #define	UUCPDIR	(uucpldir && *uucpldir ? uucpldir : getenv ("HOME"))
-#define	UUCPFIL	(uucplfil && *uucplfil ? uucplfil : getusr ())
+#define	UUCPFIL	(uucplfil && *uucplfil ? uucplfil : getusername ())
 
-char *getusr(void);
+char *getusername(void);
 char *getfullname(void);
 
 /*
