@@ -2,7 +2,7 @@
 /*
  * dtime.c -- time/date routines
  *
- * $Id$
+ * $Id: dtime.c,v 1.2 1999/05/13 01:13:58 dan Exp $
  */
 
 #include <h/nmh.h>
@@ -33,9 +33,6 @@ extern char *tzname[];
 # define abs(a) (a >= 0 ? a : -a)
 #endif
 
-/* The time "epoch" */
-#define CENTURY 1900
-
 /*
  * The number of days in the year, accounting for leap years
  */
@@ -154,7 +151,12 @@ dlocaltime (time_t *clock)
     tw.tw_hour = tm->tm_hour;
     tw.tw_mday = tm->tm_mday;
     tw.tw_mon  = tm->tm_mon;
-    tw.tw_year = tm->tm_year + CENTURY;
+
+    /*
+     * tm_year is always "year - 1900".
+     * So we correct for this.
+     */
+    tw.tw_year = tm->tm_year + 1900;
     tw.tw_wday = tm->tm_wday;
     tw.tw_yday = tm->tm_yday;
 
@@ -209,7 +211,12 @@ dgmtime (time_t *clock)
     tw.tw_hour = tm->tm_hour;
     tw.tw_mday = tm->tm_mday;
     tw.tw_mon  = tm->tm_mon;
-    tw.tw_year = tm->tm_year + CENTURY;
+
+    /*
+     * tm_year is always "year - 1900"
+     * So we correct for this.
+     */
+    tw.tw_year = tm->tm_year + 1900;
     tw.tw_wday = tm->tm_wday;
     tw.tw_yday = tm->tm_yday;
 
@@ -240,15 +247,15 @@ dgmtime (time_t *clock)
 char *
 dctime (struct tws *tw)
 {
-    static char buffer[25];
+    static char buffer[26];
 
     if (!tw)
 	return NULL;
 
-    sprintf (buffer, "%.3s %.3s %02d %02d:%02d:%02d %.4d\n",
+    snprintf (buffer, sizeof(buffer), "%.3s %.3s %02d %02d:%02d:%02d %.4d\n",
 	    tw_dotw[tw->tw_wday], tw_moty[tw->tw_mon], tw->tw_mday,
 	    tw->tw_hour, tw->tw_min, tw->tw_sec,
-	    tw->tw_year >= 100 ? tw->tw_year : CENTURY + tw->tw_year);
+	    tw->tw_year < 100 ? tw->tw_year + 1900 : tw->tw_year);
 
     return buffer;
 }
@@ -319,20 +326,20 @@ dasctime (struct tws *tw, int flags)
     if ((tw->tw_flags & TW_SZONE) == TW_SZNIL)
 	result[0] = '\0';
     else
-	sprintf(result, " %s", dtimezone(tw->tw_zone, tw->tw_flags | flags));
+	snprintf(result, sizeof(result), " %s", dtimezone(tw->tw_zone, tw->tw_flags | flags));
 
-    sprintf(buffer, "%02d %s %0*d %02d:%02d:%02d%s",
+    snprintf(buffer, sizeof(buffer), "%02d %s %0*d %02d:%02d:%02d%s",
 	    tw->tw_mday, tw_moty[tw->tw_mon],
 	    tw->tw_year < 100 ? 2 : 4, tw->tw_year,
 	    tw->tw_hour, tw->tw_min, tw->tw_sec, result);
 
     if ((tw->tw_flags & TW_SDAY) == TW_SEXP)
-	sprintf (result, "%s, %s", tw_dotw[tw->tw_wday], buffer);
+	snprintf (result, sizeof(result), "%s, %s", tw_dotw[tw->tw_wday], buffer);
     else
 	if ((tw->tw_flags & TW_SDAY) == TW_SNIL)
-	    strcpy (result, buffer);
+	    strncpy (result, buffer, sizeof(result));
 	else
-	    sprintf (result, "%s (%s)", buffer, tw_dotw[tw->tw_wday]);
+	    snprintf (result, sizeof(result), "%s (%s)", buffer, tw_dotw[tw->tw_wday]);
 
     return result;
 }
@@ -372,7 +379,8 @@ dtimezone (int offset, int flags)
     if (flags & TW_DST)
 	hours += 1;
 #endif /* defined(DSTXXX) */
-    sprintf (buffer, "%s%02d%02d", offset < 0 ? "-" : "+", abs (hours), abs (mins));
+    snprintf (buffer, sizeof(buffer), "%s%02d%02d",
+		offset < 0 ? "-" : "+", abs (hours), abs (mins));
     return buffer;
 }
 
@@ -404,7 +412,8 @@ dmktime (struct tws *tw)
 
     result = 0;
     if (year < 100)
-	year += CENTURY;
+	year += 1900;
+
     for (i = 1970; i < year; i++)
 	result += dysize (i);
     if (dysize (year) == 366 && mon >= 3)
@@ -425,8 +434,8 @@ dmktime (struct tws *tw)
 
 /*
  * Simple calculation of day of the week.  Algorithm
- * used is Zeller's congruence.  Currently, we assume
- * if tw->tw_year < 100, then the century is CENTURY.
+ * used is Zeller's congruence.  We assume that
+ * if tw->tw_year < 100, then the century = 19.
  */
 
 void
@@ -437,7 +446,7 @@ set_dotw (struct tws *tw)
     month = tw->tw_mon - 1;
     day = tw->tw_mday;
     year = tw->tw_year % 100;
-    century = tw->tw_year >= 100 ? tw->tw_year / 100 : (CENTURY / 100);
+    century = tw->tw_year < 100 ? 19 : tw->tw_year / 100;
 
     if (month <= 0) {
 	month += 12;
